
/****************************************************************************

  Copyright (C) Cambridge Silicon Radio April 2000

FILE
        BCSPStack.h  -  make BCSPStack public

MODIFICATION HISTORY
	1.3  27-apr-00 npm  - added boilerplating
	1.4  17-jan-01 mr01 - added detection of remote reset (BC01-S-010-e)

*/




#ifndef _BCSPSTACK_H_
#define _BCSPSTACK_H_

#ifdef	RCS_STRINGS
static  char    bcspstack_h_id[]
  = "$Id: //depot/bc/main/devHost/HostStack/BCSP/Generic/BCSPStack.h#7 $";
#endif



#include <setjmp.h>

#include "BCSP/BCSPLinkage.h"

#include "Timer.h"

#include "PacketBuffer.h"

#include "XMITWindow.h"
#include "SLIP.h"
#include "bcsp_signal.h"
#include "TransferRequest.h"
#include "ByteBuffer.h"
#include "PacketPool.h"
#include "ChannelControl.h"
#include "StackConfiguration.h"
#include "API.h"
#include "BTRACE.h"
#include "scheduler.h"
#include "Environment.h"



typedef struct _BCSPStack
{
  jmp_buf schedulerState ;
  BCSPEnvironment * environment ;
  Queue incomingReadRequests ;
  Queue incomingWriteRequests ;
  PacketBuffer MUXUnreliableInput  ;
  PacketBuffer SEQInput  ;
  PacketBuffer RCVInputBuffer ;
  PacketBuffer PacketDelivererInput ;
  PacketBuffer LinkEstablishmentInput ;
  Packet *  SLIPInputPacket ;
  Signal SLIPSendAvailableSignal ;
  Signal MUXHasPacketForSLIPSignal ;
  PacketPool thePacketPool ;
  ByteBuffer SLIPByteOutput ;
  ByteBuffer SLIPByteInput ;
  StackConfiguration * configuration ;
  uint32 lastPacketSentAt ;
  uint32 lastSyncSentAt ;
  uint32 timeNow ;
  uint32 wakeupTime ;
  Timer resendTimer ;
  Timer LinkEstablishmentTimer ;

  XMITWindow theXMITWindow ;
  BCTransferRequestQueue  requestRouterInput ;
  BCTransferRequestQueue  unreliableWriteRequests ;
  BCTransferRequestQueue  readRequests[16] ;
  ChannelControl channels[16] ;
  SLIPState theSLIPState ;
  Signal txackSignal ;
  Signal unchokeSignal ;
  uint32 currentTaskId ;
  Packet theAckPacket ;
  uint16 confCnt ;
  uint16 waitMask ;
  uint8 XMITretries ;
  uint8 txack ;
  uint8 rxack ;
  uint8 seq ;
  bool isUnchoked ; // Note that if we're choked, then we're in the shy state.
  bool isGarrulous ;
  bool ackRequired ;

} BCSPStack ;

#include "shutdown.h"

BCSP_API void  initialiseStack(struct _BCSPStack * stack) ;
BCSP_API uint16 numBytesInTransmitBuffer(BCSPStack * stack) ;
BCSP_API uint16 numFreeSlotsInReceiveBuffer(BCSPStack * stack) ;
BCSP_API uint8 readByteFromTransmitBuffer(BCSPStack * stack) ;
BCSP_API void writeByteToReceiveBuffer(BCSPStack * stack,uint8 data) ;
BCSP_API void readFromTransmitBuffer(BCSPStack * stack,uint8* dest,uint8 len) ;
BCSP_API void writeToReceiveBuffer(BCSPStack * stack,uint8 *src,uint8 len) ;
BCSP_API bool BCSPLinkEstablished(BCSPStack * stack) ;
BCSP_API void BCSPaddPacket(struct _BCSPStack * theStack,Packet * pkt) ;

extern void ReleaseXMITWindowAcknowledgedSlots(BCSPStack * theStack) ;

#endif
