/****************************************************************************

  Copyright (C) Cambridge Silicon Radio April 2000

FILE
        PacketBuffer.c  -  buffer for packets

MODIFICATION HISTORY
	1.2  27-apr-00 npm  - added boilerplating
$Log:$

*/


#ifdef	RCS_STRINGS
static  char    packetbuffer_c_id[]
  = "$Id: //depot/bc/main/devHost/HostStack/BCSP/Generic/PacketBuffer.c#4 $";
#endif

#ifndef _WINCE
#include <stddef.h>
#endif

#include "PacketBuffer.h"

#include "BCSPStack.h"

/****************************************************************************
NAME
	addPacketToBuffer

FUNCTION
	wrapper function to add a packet to the buffer

IMPLEMENTATION NOTE
	uses underlying queue functionality

****************************************************************************/


void addPacketToBuffer(struct _BCSPStack * stack,PacketBuffer * buf,Packet * pkt) 
{
	addNodeToQueue(stack,&buf->queue,&pkt->node) ;
}

/****************************************************************************
NAME
	SYNCBLKgetPacketFromBuffer

FUNCTION
	attempts to obtain a packet from the buffer.  If none is available, block
	immediately.

IMPLEMENTATION NOTE
	uses underlying queue functionality

****************************************************************************/


Packet *  SYNCBLKgetPacketFromBuffer(struct _BCSPStack * stack,PacketBuffer * buf) 
{
	Packet * pkt ;
	pkt = (Packet*) SYNCBLKgetNodeFromQueue(stack,&buf->queue) ;
	return pkt ; //this will block rather than return NULL
}

/****************************************************************************
NAME
	ASYNCBLKgetPacketFromBuffer

FUNCTION
	attempts to obtain a packet from the buffer.  If none is available, return
	null and block later.

IMPLEMENTATION NOTE
	uses underlying queue functionality

****************************************************************************/


Packet *  ASYNCBLKgetPacketFromBuffer(struct _BCSPStack * stack,PacketBuffer * buf) 
{
	Packet * pkt ;
	pkt = (Packet*) ASYNCBLKgetNodeFromQueue(stack,&buf->queue) ;
	return pkt ; 
}

/****************************************************************************
NAME
	peekAtPacketInBuffer

FUNCTION
	return pointer to first packet in buffer without actually removing it or
	null if no packet present

IMPLEMENTATION NOTE
	uses underlying queue functionality

****************************************************************************/


Packet * peekAtPacketInBuffer(PacketBuffer * buf) 
{
	return (Packet*) peekAtNodeInQueue(&buf->queue) ;
}

/****************************************************************************
NAME
	SYNCBLKpeekAtPacketInBuffer

FUNCTION
	return pointer to first packet in buffer without actually removing it.
	If no packet present, block immediately

IMPLEMENTATION NOTE
	uses underlying queue functionality

****************************************************************************/

Packet * SYNCBLKpeekAtPacketInBuffer(struct _BCSPStack * stack,PacketBuffer * buf) 
{
	return (Packet*) SYNCBLKpeekAtNodeInQueue(stack,&buf->queue) ;
}

