/****************************************************************************

	Copyright (C) Cambridge Silicon Radio April 2000

FILE
	bcsp_Signal.h  -  simple signal functionality

MODIFICATION HISTORY
	1.2  27-apr-00 npm  - added boilerplating
	1.3  12-mar-02 pws    Fix _SIGNAL_H conflict with standard header.

*/

#ifndef _BCSP_SIGNAL_H_
#define _BCSP_SIGNAL_H_

#ifdef	RCS_STRINGS
static  char    bcsp_signal_h_id[]
  = "$Id: //depot/bc/main/devHost/HostStack/BCSP/Generic/bcsp_signal.h#4 $";
#endif




/*
STRUCTURE
	Signal
FUNCTION
	Provides a way of waiting for some condition to become true.
	The semantics on 'wait' and 'set' are that it is assumed that
	wait will only be called if the condition is false and set will
	only be called when the condition becomes true.  Therefore, 
	waiting on a signal causes the 'set' flag to be cleared, ie 
	waiting on a set signal will _not_ result in an immediate return
	but will require the signal to be set again.  (Ie, signals 
	are edge-triggered)

FIELDS
	waitMask - ids of tasks waiting on the signal.
*/

#include "common/types.h"

struct _BCSPStack ;

typedef struct _Signal
{
	uint16 waitMask ;
} Signal ;

extern void setSignal(struct _BCSPStack * stack,Signal * sig);
extern void SYNCBLKwaitSignal(struct _BCSPStack * stack,Signal * sig) ;
extern void ASYNCBLKwaitSignal(struct _BCSPStack * stack,Signal * sig) ;
#endif
