#ifndef POSIXBCSP_H
#define POSIXBCSP_H


#include "BCSPStack.h"

#include <pthread.h>


class PosixBCSP
{
private: 

	/* Miscellaneous */
	BCSPStack	*mStack;
	BCSPEnvironment	mBCSPenv;
	int		fd;		/* serial port file descriptor */
	bool		close ();
	void		(*userOnLinkFail) (void *);
	void		*userOnLinkFailArg;

	/* Threads spawned by start () */
	void		thread (),
			txThread (),
			rxThread ();
	pthread_t	threadID,
			txThreadID,
			rxThreadID;

	/* Buffers used for serial I/O */
	uint8		txbuf[BYTE_BUFFERSIZE],
			rxbuf[BYTE_BUFFERSIZE];
	size_t		txlen,
			rxlen;

	/* Mutexes used for inter-thread synchronisation */
	pthread_mutex_t	mMutexEnv,	/* For user critical sections */
			mMutexThread;	/* For threads' critical sections */

	/* Flags used for inter-thread communication */
	bool		txbusy,	/* txThread asked by thread to tx some data */
			txdone,	/* txThread has txed the data */
			rxbusy,	/* rxThread asked by thread to rx something */
			rxdone,	/* rxThread has rxed something */
			signalled,	/* signaller wants thread to awake */
			die;	/* destructor wants thread to die */

	/* Conditions used for inter-thread communication */
	pthread_cond_t	mCondDone,	/* txThread, rxThread, signaller wakes thread */
			mCondDoTx,	/* thread wakes txThread */
			mCondDoRx;	/* thread wakes rxThread */

public:

	PosixBCSP ();
	bool		open (const char *portName, uint32 bps);
	void		start (bool useLinkEstablishment = true);
	bool		connect (uint32 timeout);
	BCSPStack	*getBCSPStack() { return mStack; }
	void		setOnLinkFail (void (*fn) (void *), void *arg);
	~PosixBCSP ();

	friend void	*threadFromC (void *),
			*txThreadFromC (void *),
			*rxThreadFromC (void *);
	friend void	PosixEnterCS (void *),
			PosixLeaveCS (void *),
			PosixOnLinkFailure (void *),
			PosixSignal (void *);
};


#endif
