// Copyright October 2000 Cambridge Silicon Radio

// a simple application which shows how to use the bcsp port
// The application prints out the BD_ADDR of the device. 
// This code is unsupported and is intended only as an example.

#include "stdafx.h"

#include "bcprotocols/BCProtocols.h"
#include "bcsp/win32bcsp.h"

#ifdef _WINCE
#include "wincefuncs.h"
#include "winbase.h" /* added for dcb work - 15/01/01 */
#include <stdlib.h>
#include <malloc.h>
#endif

#include <stdio.h>

HANDLE hReadEvent, hFlowEvent;
__int64 CardAddress;

#define  BT_CMD_BD_ADDR  0x1009
#define  BT_COMMAND_COMPLETE  0x0E
#define  BT_COMMAND_STATUS  0x0F

#ifndef _WINCE
#define TRACE iprintf
#else
#define TRACE wincedebug 
#endif

/*use wincedebug above to print to log*/

#ifdef _WINCE
static void winCEbuildcommdcb(ichar *);
static void wincedebug(ichar * fmt,...); 

static ichar* tokentable [] = {II("baud"),II("parity"),II("data"),II("stop"),II("rts"),II("dtr")};
static int toktabsize = 6;
static FILE *debugfilep;
#endif

void MGTcontinueRead(BCTransferRequest * req, BCTransferStatus status, void * param)
{
	static uint8 rcvBuf [100] ;
	switch (status)
	{
	case transferComplete:
		{
			// ignore this packet, set up the next (or first)
			TRACE(II("BC_CHIP_MGT_CH_CHAN received\n"));
			BCreloadTransferRequest(req, rcvBuf, sizeof(rcvBuf)) ;
			BCreadPacket(BC_CHIP_MGT_CH_CHAN,req) ;
		}
		break ;
	case transferCancelled:
		break ;
	default :
		break ;
	}
}


void continueRead(BCTransferRequest * req, BCTransferStatus status, void * param)
{
	static uint8 rcvBuf [100] ;
	uint8 *pBlock = req->data;

	TRACE ( II("HCI_CMD_EVT Block recieved: pBlock is %u\n") , pBlock );
	switch (status)
	{
	case transferComplete:
		{
			if (pBlock != NULL)
			{
				TRACE ( II("Event %u\n") , pBlock[0] );
				if (pBlock[0] == BT_COMMAND_COMPLETE) 
				{
				    if ((pBlock[3] + 256*pBlock[4]) == BT_CMD_BD_ADDR)
					{  // Its the reply!
						CardAddress = 0;
						for (int i = 10; i >= 5; i--)
						{
							CardAddress *= 256;
							CardAddress += pBlock[i];
						}
						SetEvent(hReadEvent);
						return;
					}
					else
					{  // Its a command complete, but not the one we're after.
						TRACE(II("Length = %02X, Packets = %02x, Opcode = %04x\n"), pBlock[1], pBlock[2], pBlock[3] + 256 * pBlock[4]);
					}

				}
				else if (pBlock[0] == BT_COMMAND_STATUS) 
				{  // Its a status block.
					TRACE(II("Status block received.\n"));
					TRACE(II("Length = %02x, Code = %02x, Packets = %02x, Opcode = %04x\n"), pBlock[1], pBlock[2], pBlock[3], pBlock[4] + 256 * pBlock[5]);
 					SetEvent(hFlowEvent);
				}

				else
				{  // ignore this packet
					TRACE(II("Block ignored.\n"));
				}	
			}
			// set up the next (or first) packet to be read
			BCreloadTransferRequest(req, rcvBuf, sizeof(rcvBuf)) ;
			BCreadPacket(BC_HCI_CMD_EVT,req) ;
		}
		break ;
	case transferCancelled:
		iprintf(II("Error - transfer cancelled\n")) ;
		break ;
	default :
		break ;
	}
}


void startRead()
{
	// Initialize the  CHIP_MGT_CH_CHAN callback...
	hFlowEvent = CreateEvent(0, 0, 0, 0);
	// Allocate the request block  THIS IS NOT FREEd
	BCTransferRequest * req = (BCTransferRequest*) calloc(1,sizeof(BCTransferRequest)) ;
	
	// Set up the call back - will be called when data is received.
	BCinitTransferRequest(req,0,0, MGTcontinueRead, NULL) ;
	// Use the call back to set up buffers and get things going.
	MGTcontinueRead(req, transferComplete, NULL);

	// Initialize the main callback...	
	// Initialize the wait semaphore
	hReadEvent = CreateEvent(0, 0, 0, 0);
	// Allocate the request block
	req = (BCTransferRequest*) calloc(1,sizeof(BCTransferRequest));
	// Set up the call back - will be called when data is received.
	BCinitTransferRequest(req,0,0,continueRead, NULL) ;
	// Use the call back to set up buffers and get things going.
	continueRead(req, transferComplete, NULL);
}


int	waitForFlow()
{
	if (WaitForSingleObject(hFlowEvent,  1000) != WAIT_TIMEOUT )
		return TRUE;
	else
	{
		CloseHandle(hFlowEvent);
		return FALSE;
	}
}


int	waitForRead()
{
	if (WaitForSingleObject(hReadEvent,  1000) != WAIT_TIMEOUT )
		return TRUE;
	else
	{
		CloseHandle(hReadEvent);
		return FALSE;
	}
}


//continue write
//called every time a write callback is received

void continueWrite(BCTransferRequest * req, BCTransferStatus status,void * param)
{
	iprintf ( II("called back from send\n") );
	static uint8 xmitBuf [100] ;
	switch (status)
	{
	case transferComplete:
	    free(param);
		free(req);
		break ;
	case transferCancelled:
		iprintf(II("Error - transfer cancelled\n")) ;
		free(param);
		free(req);
 		break ;
	default :
		break ;
	}
}


void doWrite()
{
	iprintf (II("sending command\n"));
	// Allocate the request block
	BCTransferRequest * req = (BCTransferRequest*) calloc(1,sizeof(BCTransferRequest)) ;
	uint8 *pBlock = (uint8 *)calloc(1, 3);
	// Allocate the data block

	*(uint16 *)(&pBlock[0]) = BT_CMD_BD_ADDR;

//	pBlock[0] = BT_CMD_BD_ADDR & 0xFF;
//	pBlock[1] = BT_CMD_BD_ADDR >> 8;
	pBlock[2] = 0;

	TRACE( II("Sending %02x%02x%02x\n"), pBlock[0] , pBlock[1] , pBlock[2] );

	// Set up the call back - will be called when data is received.
	BCinitTransferRequest(req, pBlock, 3, continueWrite, NULL);
	BCwritePacket(BC_HCI_CMD_EVT, req);
}


extern "C" int imain(int argc, ichar* argv[])
{
	if (argc ==1) 
	{
		iprintf(II("usage is %s com \n"), argv[0]) ;
		iprintf(II("Eg %s com2\n"), argv[0]) ;
		iprintf(II("\n"));
		iprintf(II("Will return the address of the instrument\n"));
		return 0 ;
	}
	Win32BCSP winStack ;
	ichar * cfgStr = II("baud=115200 parity=e stop=1 data=8 rts=on dtr=on") ;
	if (!winStack.open(argv[1],cfgStr))
	{
		iprintf(II("Error - couldn't open port '%s'\n"),argv[1]) ;
		return 0 ;
	}

	BTRACEDISABLE(0xffffffff) ;

	winStack.start() ;
	if (!winStack.connect(5000))
	{
		iprintf(II("Couldn't establish link with peer\n")) ;
		return 1;
	}

	initialiseBCProtocols(winStack.getBCSPStack()) ;

	if (!BCopenChannel(BC_HCI_CMD_EVT))
	{
		iprintf(II("Couldn't open HCI_CMD_EVT channel\n")) ;
		return 2 ;
	}

	if (!BCopenChannel(BC_CHIP_MGT_CH_CHAN))
	{
		iprintf(II("Couldn't open CHIP_MGT_CH_CHAN channel\n")) ;
		return 3 ;
	}

	uint32 count =100 ;
	
	CardAddress = 0xDEADBEEFDEADBEEFI64;
	// Set off a process to wait for the read event
	startRead();
	waitForFlow();
	// Write the packet
    doWrite();

	// Wait on the result of the read
	int readResult = waitForRead();
	if (readResult)
		iprintf(II("Address is 0x%012I64X\n"), CardAddress);
    else
	{
		iprintf(II("Cannot read address - no reply. (Device may need resetting)\n"));
		return 4;
	}

//	if (argv[2]) doWrite(&count);
//	else doRead(&count) ;
//  while (1) Sleep(100) ;
	return 0;
}

#ifdef _WINCE

static void winCEbuildcommdcb(ichar *strin)
{
    DCB dcb;
    INT i;

    HANDLE hLocal;
	ichar *pdest;
	ichar *token;
	ichar seps[]   = II(" ,\t\n");
	ichar *string1 = II("A string\tof ,,tokens\nand some  more tokens"); /* works ? */

	ichar *string = II("baud=1200 parity=N data=8 stop=1");
	int intfound = 0;



	iprintf(II("todo = %s\n\nTokens:\n"),strin);
    /* Establish string and get the first token: */	
    token = istrtok( strin, seps );
   while( token != NULL )
   {
      /* While there are tokens in "string" */
	  iprintf(II("new token =%s\n\r"), token);
      iprintf( II(" %s\n"), token );
	  intfound = 0;
 
	  for (i=0; i<toktabsize; i++)
	  {
		iprintf(II("look for %s\n\r"),tokentable[i]);
		if (istrstr(token, tokentable[i]))
		{
			iprintf(II("found %s in %s\n"),tokentable[i], token);

			iprintf(II("val = %s\n\r"),token + 1 + istrlen(tokentable[i]));

			intfound = 1;

			switch (i)
			{
			case 0: /* baud */
				iprintf(II("Setting the dcb baud val = %d\n\r"), iatoi(token + 1 + istrlen(tokentable[i])));
				dcb.BaudRate = iatoi(token + 1 + istrlen(tokentable[i]));
				break;


			case 1: /* parity e,o,n,m,s*/
				iprintf(II("Setting the parity val .... "));
				dcb.fParity = TRUE;
				if (*(token + 1 + istrlen(tokentable[i])) == II('e'))
				{
					iprintf(II("even\n\r"));
					dcb.Parity = EVENPARITY;
				}
				else if (*(token + 1 + istrlen(tokentable[i])) == II('o'))
				{
					iprintf(II("odd\n\r"));

					dcb.Parity = ODDPARITY;
				}
				else if (*(token + 1 + istrlen(tokentable[i])) == II('m'))
				{
					iprintf(II("mark\n\r"));

					dcb.Parity = MARKPARITY;
				}
				else if (*(token + 1 + istrlen(tokentable[i])) == II('s'))
				{
					iprintf(II("space\n\r"));
					dcb.Parity = SPACEPARITY;
				}
				else
				{
					iprintf(II("none\n\r"));
					dcb.Parity = NOPARITY;
					dcb.fParity = FALSE;
				}
				break;

			case 2:	/* data */
				iprintf(II("data set to %d\n\r"),iatoi(token + 1 + istrlen(tokentable[i])));
				dcb.ByteSize = iatoi(token + 1 + istrlen(tokentable[i]));
				break;
			case 3: /* stop */
				if (*(token + 1 + istrlen(tokentable[i])) == II('1'))
				{
					iprintf(II("one stop bit\n\r"));

					dcb.StopBits = ONESTOPBIT;
				}
				else
				{
					iprintf(II("two stop bits\n\r"));
					dcb.StopBits = TWOSTOPBITS;
				}
				break;
			case 4: /* rts */
				if (*(token + 2 + istrlen(tokentable[i])) == II('n'))
				{
					iprintf(II("rts on\n\r"));

					dcb.fRtsControl  = RTS_CONTROL_HANDSHAKE;
				}
				else
				{
					iprintf(II("rts off\n\r"));

					dcb.fRtsControl  = RTS_CONTROL_ENABLE;
				}
				break;



			case 5: /* DTR */
 				if (*(token + 2 + istrlen(tokentable[i])) == II('n'))
				{
					iprintf(II("dtr on\n\r"));
					dcb.fDtrControl  = DTR_CONTROL_HANDSHAKE;
				}
				else
				{
					iprintf(II("dtr off\n\r"));
					dcb.fDtrControl  = DTR_CONTROL_ENABLE;
				}
				break; 



			} /* end of switch */


		} /* end of if */
	} /* end of for */

	/* if we could not find the token print out an error message and quit */
//	if (i == toktabsize)
	if (intfound == 0)
	{
		iprintf(II("Can not find the token\n\r"));
		exit(1);
	}


     /* Get next token: */
      token = istrtok( NULL, seps );

   } /* end of while */


}

void wincedebug(ichar * fmt,...) 
{
	va_list arglist ;

	va_start(arglist,fmt) ;


	ivfprintf(debugfilep,fmt,arglist) ;
	fflush(debugfilep) ;

}

#endif
