//Copyright May 2000 Cambridge Silicon Radio


#include "stdafx.h"
#include "BCProtocols.h"
#include "win32bcsp.h"
#include "API.h"

#include <stdio.h>
#include <string>

BCSPChannel channels[2][16]; //channels for both stacks
uint8 buf[2][16][4096] ;     //buffers for both stacks


ichar *comnames [2] ;

uint8 instances[2] = {0,1} ;

bool opt_printfDebug = 0 ;

FILE * fout  ;

bool dumpPacket(int instance,uint8 chan,uint8 * buf,uint16 len)			
{
	bool retval ;
	int c ;
	ifprintf(fout,II("%06ld:FROM %s:%2d:%4d:"),GetTickCount(),comnames[instance],chan,len) ;

	if ((chan == 11) && (opt_printfDebug))
	{
		for (c = 0 ; c < len ;c ++) ifprintf(fout,II(" %02x"),buf[c]) ;
		retval = 1 ;
	}
	else
	{
		for (c = 0 ; c < len ;c ++) ifprintf(fout,II(" %02x"),buf[c]) ;
		retval = 0 ;
	}
	ifprintf(fout,II("\n")) ;
	return retval ;
}


void completeRead(BCTransferRequest * req, BCTransferStatus status,void * inst) ;


//completeWrite
//after we've finished writing to the destination instance, we can 
//rearm the read on the source instance


void completeWrite(BCTransferRequest * req, BCTransferStatus status,void * inst)
{
	uint8 instance = *(uint8*) inst ;
	switch (status)
	{
	case transferComplete:
		{
			//ok - now we can restart the read on the other stack			
			int len = req->length ;
			int chan = req->channel ;
			instance = instance ?0 :1 ;
			BCinitTransferRequest(req,buf[instance][chan],sizeof(buf[instance][chan]),completeRead,&instances[instance]) ;
			BCSPreadPacket(&channels[instance][chan],req) ;
		}
		break ;
	case transferCancelled:
		iprintf(II("Error - transfer cancelled\n")) ;
		break ;
	default :
		break ;
	}
}

//completeRead
//called whenever we receive a packet 
//The first thing we do is print it out, then we set up a write to the destination instance

void completeRead(BCTransferRequest * req, BCTransferStatus status,void * inst)
{
	uint8 instance = *(uint8*) inst ;
	switch (status)
	{
	case transferComplete:
		{
			
			//we now need to print out this packet and send it to the other stack...
			int len = req->length ;
			int chan = req->channel ;
			uint8 writeInstance = (instance) ?0 :1 ;
			if (dumpPacket(instance,chan,req->data,len))
			{
				
				//we need to manually complete the write to retrigger the read 
				completeWrite(req,transferComplete,&instances[writeInstance]) ;
			}
			else			
			{
				BCinitTransferRequest(req,buf[instance][chan],len,completeWrite,&instances[writeInstance]) ;
				BCSPwritePacket(&channels[writeInstance][chan],req) ;
			}
		}
		break ;
	case transferCancelled:
		iprintf(II("Error - transfer cancelled\n")) ;
		break ;
	default :
		break ;
	}
}



void setupRead(int instance,uint8 chan)
{
	BCTransferRequest * req = (BCTransferRequest*) calloc(1,sizeof(BCTransferRequest)) ;
	BCinitTransferRequest(req,buf[instance][chan],sizeof(buf[instance][chan]),completeRead,&instances[instance]) ;
	BCSPreadPacket(&channels[instance][chan],req) ;
}

//open a BCSP stack with the specified port and baud rate

void startReading(Win32BCSP * stack,int instance)
{
	for (uint8 c = 2 ; c < 16 ; c ++)
	{
	
		BCSPopenChannel(&channels[instance][c],stack->getBCSPStack(),c,
							BCSPChannelReliable,BCSPCallbackDeferred) ;
		setupRead(instance,c) ;
	}

}


bool openBCSP(Win32BCSP * stack,ichar * port, ichar * baud,int instance) 
{
    istring cfgStr = istring(II("baud=")) + baud + istring(II(" parity=e stop=1 data=8 rts=on dtr=on"));
	
	comnames[instance] = port ;

	if (!stack->open(port,cfgStr.c_str()))
	{
		iprintf(II("Error - couldn't open port '%s'\n"),port) ;
		return 0 ;
	}
	
	BTRACEDISABLE(0xffffffff) ;
	stack->start() ;
		
	iprintf(II("Waiting to establish link on %s..."),port) ;
	if (!stack->connect(50000))
	{
		iprintf(II("FAILED\nERROR - couldn't establish link with peer on %s\n"),port) ;
		return 0;
	}
	iprintf(II(" SUCCEEDED\n")) ;

	return 1 ;

}


extern "C" int imain(int argc, ichar* argv[])
{
	
	if (argc <5) 
	{
		iprintf(II("bcsp_spy\n")) ;
		iprintf(II("This utility allows you to eavesdrop on the bcsp traffic between\n")) ;
		iprintf(II("two devices.  Only valid packets are reported.\n\n")) ;
		iprintf(II("The printf option may be used to report on (and swallow) channel 11\n" ));
		iprintf(II("printf statements.\n")) ;
		iprintf(II("Usage:   bcsp_spy com baud com baud [printf]\n")) ;
		iprintf(II("Example: bcsp_spy com1 38400 com2 115200 printf\n")) ;
		return 0 ;
	}

	fout = ifopen(II("\\spy.log"),II("w")) ;	
	if (( argc >= 6) && (!istricmp(II("printf"),argv[5]))) opt_printfDebug = 1 ;
	Win32BCSP winStack1 ;
	Win32BCSP winStack2 ;

	if (!openBCSP(&winStack1,argv[1],argv[2],0)) return 0 ;
	if (!openBCSP(&winStack2,argv[3],argv[4],1)) return 0 ;
	startReading(&winStack1,0) ;
	startReading(&winStack2,1) ;

	while(1) Sleep(100) ;
	fclose(fout) ;
}
