/******************************************************************************

Project 	:	Blue Chat
File Name	:	BlueChatHciDefs.h
Description	:	Type definitions and usefull include stuff

Dependences	
-----------
			 
			 
Revision History
----------------
Date		Author				Information
01/10/00	Trevor Dowling		Created


******************************************************************************/

// Include only once!
#ifndef	_BLUE_CHAT_HCI_DEFS_H
#define	_BLUE_CHAT_HCI_DEFS_H


#include "common/types.h"          

/****************************** Macro definitions   **************************/

// User defined ON_MESSAGE message ID's
#define	UDM_ON_HCI_INQUIRY_EVENT				(WM_USER + 1)
#define	UDM_ON_HCI_CONNECTION_COMPLETE_EVENT	(WM_USER + 2)
#define	UDM_ON_HCI_DISCONNECT_EVENT				(WM_USER + 3)
#define UDM_SEND_CHAR 							(WM_USER + 4)
#define UDM_RECEIVE_HCI_DATA_PACKET 			(WM_USER + 5)
#define UDM_ON_HCI_INQUIRY_COMPLETE_EVENT		(WM_USER + 6)
#define UDM_ON_HCI_COMMAND_COMPLETE_EVENT		(WM_USER + 7)
#define UDM_ON_HCI_EVENT						(WM_USER + 8)
#define UDM_HCI_FILE_MORE						(WM_USER + 9)
#define UDM_HCI_FILE_PROGRESS					(WM_USER + 10)
#define UDM_SEND_FILE							(WM_USER + 11)	


/****************************** Type definitions    **************************/

typedef	struct	TAG_BTDEVICE_ADDRESS
{
	unsigned int			Lap;		/* Lower Address Part 00..23 */
	unsigned char			Uap;		/* upper Address Part 24..31 */
	unsigned short			Nap;		/* Non-significant	  32..47 */		

} BTDEVICE_ADDRESS;


typedef	struct	TAG_HCI_INQUIRY_RESULT_EVENT
{
	unsigned char			NumberResponces;
	BTDEVICE_ADDRESS		BTDeviceAddress;
	unsigned char			PScanRepititionMode;
	unsigned char			PScanPeriodMode;
	unsigned char			PScanMode;
	unsigned int			ClassOfDevice;
	unsigned short			ClockOffset;
} HCI_INQUIRY_RESULT_EVENT;


typedef	struct	TAG_HCI_CONNECTION_COMPLETE_EVENT
{
	unsigned char			Status;
	unsigned int			ConnectionHandle;
	BTDEVICE_ADDRESS		BTDeviceAddress;
	unsigned char			LinkType;
	unsigned char			EncryptionMode;
} HCI_CONNECTION_COMPLETE_EVENT;


typedef	struct	TAG_HCI_SEND_DATA
{
	unsigned int			ConnectionHandle;
	bool					PacketBoundaryFlag;
	unsigned char			BroadcastFlag;
	uint8*		   		    DataStart;
	unsigned int			DataLength;
	CString *				FileName;
	unsigned int			FileLength;
} HCI_SEND_DATA;



typedef	struct	TAG_HCI_INQUIRY_COMPLETE_EVENT
{
	unsigned char	Status;
	unsigned char	NumberResponces;
} HCI_INQUIRY_COMPLETE_EVENT;


typedef	struct	TAG_HCI_CMD_COMPLETE_READ_BD_ADDR_EVENT
{
	unsigned char		Status;
	BTDEVICE_ADDRESS	BTDeviceAddress;
} HCI_CMD_COMPLETE_READ_BD_ADDR_EVENT;


typedef	struct	TAG_HCI_CMD_COMPLETE_READ_BUFFER_SIZE_EVENT
{
	unsigned char		Status;
	unsigned int		HcAclDataPacketLength;
	unsigned char		HcScoDataPacketLength;
	unsigned int		HcTotalNumAclDataPackets;
	unsigned int		HcTotalNumScoDataPackets;
} HCI_CMD_COMPLETE_READ_BUFFER_SIZE_EVENT;


typedef	struct	TAG_HCI_COMMAND_STATUS_EVENT
{
	unsigned char	Status;
	unsigned char	NumHciCommandPackets;
	unsigned int	CommandOpCode;
} HCI_COMMAND_STATUS_EVENT;


typedef	struct	TAG_HCI_DISCONNECTION_COMPLETE_EVENT
{
	unsigned char			Status;
	unsigned int			ConnectionHandle;
	unsigned char			Reason;
} HCI_DISCONNECTION_COMPLETE_EVENT;


typedef	struct	TAG_HCI_CONNECTION_REQUEST_EVENT
{
	BTDEVICE_ADDRESS		BTDeviceAddress;
	unsigned int			ClassOfDevice;
	unsigned char			LinkType;
} HCI_CONNECTION_REQUEST_EVENT;

/*  Data Packet Handling  */
typedef struct
{
    uint16 channel;
    uint16 length;
    uint16 nextByteToRecv;
    uint8 *Data;
}
L2CAP_PACKET_T;

/****************************** Function prototypes **************************/

/****************************** Data declarations   **************************/


#endif