// BlueChatEngine.cpp: implementation of the CBlueChatEngine class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#include "BlueChatEngine.h"

#include "hci/bc01DeviceController.h"
#include "hci/HCIFactory.h"
#include "hci_def.h"
#include "bccmd/bccmdpdu.h"
#include "hq/hqpdu.h"
#include "BlueChat2HciDefs.h"
#include "reportStrings.h"
  

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif





//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBlueChatEngine::CBlueChatEngine()
{
    mFactory		= NULL;
    mChip			= NULL;
    
	::InitializeCriticalSection (&sequenceProtection);

	sequenceNumber	= 0;
}


CBlueChatEngine::~CBlueChatEngine()
{
    resetChip();
	
	delete ComPortID;

    ::DeleteCriticalSection (&sequenceProtection);
}



//  Set the factory.
void CBlueChatEngine::setFactory(BlueCoreControllerFactory * aFactory)
{
	mFactory = aFactory;
}

//  AJH HACKING //
BlueCoreDeviceController * CBlueChatEngine::getDC ()
{
    return mChip;
}
    

//  Set up BCSP and talk to the chip
bool CBlueChatEngine::startChip()
{
	ASSERT(mFactory != NULL);
	mChip = mFactory->spawn();

    mChip->setCallBackOnData  ( 0 , onHCIData  , this );
    mChip->setCallBackOnEvent ( 0 , onHCIEvent , this );


    if ( mChip->getTransport()->start() == OK )
    {
		mChip->createWorker();

        ComPortID = mFactory->getDeviceName();
        return true;
    }
    else
    {
        delete mChip;
        mChip = NULL;
    }
 
	return false;
}



 
//  Is there a chip?
bool CBlueChatEngine::connected()
{
    bool Ok = false;

    if(mChip) Ok = mChip->isTransportReady();

    if(Ok)
        TRACE(_T("\n\rCONNECTED TO CHIP"));
    else
        TRACE(_T("\n\rNO CHIP"));
    return Ok;
}



//  Provide stubs for callbacks.
void __stdcall CBlueChatEngine::onHCIData(uint16 connectionHandle,	bool	packetBoundaryFlag, uint8	broadcastFlag, 
										uint8  *dataStart,			uint16	dataLength,			void   *aThis)
{
	CString			Name;

	
	// Memory Clean Up done by dialog box code
	HCI_SEND_DATA * DataToSend = new HCI_SEND_DATA;

	// Put data into structure for easy moving to dialog code
	DataToSend->ConnectionHandle		= connectionHandle;
	DataToSend->PacketBoundaryFlag		= packetBoundaryFlag;
	DataToSend->BroadcastFlag			= broadcastFlag;
	DataToSend->DataStart				= dataStart;
	DataToSend->DataLength				= dataLength;
	
	PostMessage( ((CBlueChatEngine*)aThis)->BC2Dialog, UDM_RECEIVE_HCI_DATA_PACKET, 0, (LPARAM)DataToSend);
}


void __stdcall CBlueChatEngine::onHCIEvent(uint8 EventCode, PARAMETER_ARRAY EventParameters, void *aThis)
{
	// Send this data to the dialog box thats interested in it.
	// Memory Clean Up done by dialog box code
	PostMessage( ((CBlueChatEngine*)aThis)->BC2Dialog, UDM_ON_HCI_EVENT, (WPARAM)EventCode, (LPARAM)EventParameters);
}
	


void CBlueChatEngine::resetChip ()
{
	TRACE(_T("\n\rResetChip"));
    if (mChip != NULL)
    {
        //  Send a hardware reset.
        PARAMETER_ARRAY reset = new PARAMETER_VALUE[4];

        reset[0] = 4;
        reset[1] = BCCMDPDU_SETREQ;
        reset[2] = nextSequence();
        reset[3] = BCCMDVARID_COLD_RESET;

        mChip->sendChipMgtHostToChip(reset);
        ::Sleep (20);
		
		delete [] reset ; //NPM

        //  The chip sends no acknowledgement!
        delete mChip;

        mChip = NULL;
    }
}


void CBlueChatEngine::onBCCMD(PARAMETER_ARRAY eventParameters, void * aThis)
{
	TRACE(_T("\n\rOnBCCMD"));
    ((CBlueChatEngine*) aThis)->processBCCMD ( eventParameters );
}


void CBlueChatEngine::processBCCMD(PARAMETER_ARRAY eventParameters)
{
    // decode the packet
    uint8 reportCode = 255;
    uint16 lLength = (uint16)eventParameters[0];

    delete [] eventParameters;
}


void CBlueChatEngine::onLinkFail ( void * aThis )
{
	TRACE(_T("\n\rOnLinkFail"));
}


void CBlueChatEngine::onDebugData ( uint8 *aBuffer , uint32 aLength , void *aThis )
{
	TRACE(_T("\n\rOnDebugData"));
}


uint16 CBlueChatEngine::nextSequence()
{
    //  Generate a unique sequence number.
	::EnterCriticalSection(&sequenceProtection);
	
	sequenceNumber ++;
	
	::LeaveCriticalSection(&sequenceProtection);

    return sequenceNumber;
}


bool CBlueChatEngine::SendHCICommand(PARAMETER_ARRAY Parameters)
{
//	TRACE("\n\rSendHCICommand");

	return mChip->sendHCICommand(Parameters);
}


bool CBlueChatEngine::SendHCIData(uint16 ConnectionHandle, bool PacketBoundaryFlag, uint8 BroadcastFlag, uint8 *DataStart, uint16 DataLength)
{
//    TRACE("\n\rSendHCIData");

	return mChip->sendHCIData(ConnectionHandle, PacketBoundaryFlag, BroadcastFlag, DataStart, DataLength);
}



uint32 CBlueChatEngine::getSizeOfBufferContent()
{
//	TRACE("\n\rGetSizeOfBuffer");

	return mChip->getSizeOfBufferContent();
}
