/*******************************************************************************
**   FILE	tehci_def.h
**   
**   CONTAINS
**           Defines for the HCI command and event codes
******************************************************************************/


#ifndef _HCI_H    /* Once is enough */
#define _HCI_H

/*****************************************************************************/
typedef uint16 hci_packet_boundary_flag_t;
typedef uint16 hci_connection_handle_t;
typedef uint16 HCI_OPCODE;            /* used for commands  */
typedef uint8  HCI_EVTCODE;         /* used for events    */
typedef uint8  hci_return_t;             /* return value (error if !success) */
typedef uint8  hci_error_t;
typedef uint16 hci_pkt_type_t;
typedef uint8  hci_role_t;               /* whether host controller is master or slave */
typedef uint8  hci_key_flag_t;           /* type of link key */
typedef uint8  hci_link_enc_t;           /* whether link encryption on or off */
typedef uint8  hci_qos_type_t;           /* service type for QoS */
typedef uint32 hci_event_mask_t;         /* event mask type.  This is 8 bytes in fact
                                              declared as an array[2].  Only 4 ls bytes used */
typedef uint8  hci_version_t;            /* version info */
typedef uint8  hci_bt_mode_t;            /* used for indicating state change */
typedef uint8  hci_reason_t;             /* Reason for command */
typedef uint8  page_scan_rep_mode_t;     /* Page scan repetition mode */
typedef uint16 clock_offset_t;           /* Clock offset */
typedef uint16 link_policy_settings_t;   /* Link policy settings */
typedef uint8  filter_type_t;            /* Event filter type */
typedef uint8  filter_condition_type_t;  /* Inquiry Result Filter Condition Type */
typedef uint8  pin_type_t;               /* Pin Type */
typedef uint8  read_all_flag_t;
typedef uint8  delete_all_flag_t;
typedef uint8  auto_accept_t;

/******************************************************************************
   Defines for the OGF opcodes group - these occupy the 6 msb of the opcode 
   hence the values 
 *****************************************************************************/
#define HCI_OGF_BIT_OFFSET             ((uint8)10) /* Number of bit shifts */

#define HCI_LINK                       ((HCI_OPCODE)0x0400)
#define HCI_POLICY                     ((HCI_OPCODE)0x0800)
#define HCI_HOST_BB                    ((HCI_OPCODE)0x0C00)
#define HCI_INFO                       ((HCI_OPCODE)0x1000)
#define HCI_STATUS                     ((HCI_OPCODE)0x1400)
#define HCI_TEST                       ((HCI_OPCODE)0x1800)
#define HCI_MAX_OGF                    ((HCI_OPCODE)0x1C00)
#define HCI_DEBUG                      ((HCI_OPCODE)0xFC00)
#define HCI_OPCODE_GROUP_MASK          ((HCI_OPCODE)0xFC00)
#define HCI_OPCODE_MASK                ((HCI_OPCODE)0x03FF)

/*---------------------------------------------------------------------------*\
   These defines are needed so that the hci generator can identify
   the reply (_RET) macros.
\*---------------------------------------------------------------------------*/
#define HCI_LINK_RET                   HCI_LINK
#define HCI_POLICY_RET                 HCI_POLICY
#define HCI_HOST_BB_RET                HCI_HOST_BB
#define HCI_INFO_RET                   HCI_INFO
#define HCI_STATUS_RET                 HCI_STATUS
#define HCI_TEST_RET                   HCI_TEST
 
/******************************************************************************
   Miscellaneous
 *****************************************************************************/
#define HCI_NOP                         (HCI_OPCODE) 0x0000

/******************************************************************************
   OCF opcode defines - Link Control Commands
 *****************************************************************************/
#define HCI_INQUIRY								((HCI_OPCODE)HCI_LINK | 0x0001)
#define HCI_INQUIRY_CANCEL						((HCI_OPCODE)HCI_LINK | 0x0002)
#define HCI_PERIODIC_INQUIRY_MODE				((HCI_OPCODE)HCI_LINK | 0x0003)
#define HCI_EXIT_PERIODIC_INQUIRY_MODE			((HCI_OPCODE)HCI_LINK | 0x0004)
#define HCI_CREATE_CONNECTION					((HCI_OPCODE)HCI_LINK | 0x0005)
#define HCI_DISCONNECT							((HCI_OPCODE)HCI_LINK | 0x0006)
#define HCI_ADD_SCO_CONNECTION					((HCI_OPCODE)HCI_LINK | 0x0007)
#define HCI_ACCEPT_CONNECTION_REQ				((HCI_OPCODE)HCI_LINK | 0x0009)
#define HCI_ACCEPT_CONNECTION_REQUEST			((HCI_OPCODE)HCI_LINK | 0x0009)
#define HCI_REJECT_CONNECTION_REQ				((HCI_OPCODE)HCI_LINK | 0x000A)
#define HCI_REJECT_CONNECTION_REQUEST			((HCI_OPCODE)HCI_LINK | 0x000A)
#define HCI_LINK_KEY_REQ_REPLY					((HCI_OPCODE)HCI_LINK | 0x000B)
#define HCI_LINK_KEY_REQUEST_REPLY				((HCI_OPCODE)HCI_LINK | 0x000B)
#define HCI_LINK_KEY_REQ_NEG_REPLY				((HCI_OPCODE)HCI_LINK | 0x000C)
#define HCI_LINK_KEY_REQUEST_NEGATIVE_REPLY		((HCI_OPCODE)HCI_LINK | 0x000C)
#define HCI_PIN_CODE_REQ_REPLY					((HCI_OPCODE)HCI_LINK | 0x000D)
#define HCI_PIN_CODE_REQUEST_REPLY				((HCI_OPCODE)HCI_LINK | 0x000D)
#define HCI_PIN_CODE_REQ_NEG_REPLY				((HCI_OPCODE)HCI_LINK | 0x000E)
#define HCI_PIN_CODE_REQUEST_NEGATIVE_REPLY		((HCI_OPCODE)HCI_LINK | 0x000E)
#define HCI_CHANGE_CONN_PKT_TYPE				((HCI_OPCODE)HCI_LINK | 0x000F)
#define HCI_CHANGE_CONNECTION_PACKET_TYPE       ((HCI_OPCODE)HCI_LINK | 0x000F)
#define HCI_AUTH_REQ							((HCI_OPCODE)HCI_LINK | 0x0011)
#define HCI_AUTHENTICATION_REQUEST              ((HCI_OPCODE)HCI_LINK | 0x0011)
#define HCI_SET_CONN_ENCRYPTION					((HCI_OPCODE)HCI_LINK | 0x0013)
#define HCI_SET_CONNECTION_ENCRYPTION			((HCI_OPCODE)HCI_LINK | 0x0013)
#define HCI_CHANGE_CONN_LINK_KEY				((HCI_OPCODE)HCI_LINK | 0x0015)
#define HCI_CHANGE_CONNECTION_LINK_KEY			((HCI_OPCODE)HCI_LINK | 0x0015)
#define HCI_MASTER_LINK_KEY						((HCI_OPCODE)HCI_LINK | 0x0017)
#define HCI_REMOTE_NAME_REQ						((HCI_OPCODE)HCI_LINK | 0x0019)
#define HCI_REMOTE_NAME_REQUEST					((HCI_OPCODE)HCI_LINK | 0x0019)
#define HCI_READ_REMOTE_SUPP_FEATURES			((HCI_OPCODE)HCI_LINK | 0x001B)
#define HCI_READ_REMOTE_SUPPORTED_FEATURES		((HCI_OPCODE)HCI_LINK | 0x001B)
#define HCI_READ_REMOTE_VER_INFO				((HCI_OPCODE)HCI_LINK | 0x001D)
#define HCI_READ_REMOTE_VERSION_INFORMATION		((HCI_OPCODE)HCI_LINK | 0x001D)
#define HCI_READ_CLOCK_OFFSET					((HCI_OPCODE)HCI_LINK | 0x001F)
#define HCI_MAX_LINK_OCF						((HCI_OPCODE)HCI_LINK | 0x0020)

/******************************************************************************
   OP CODE defines - Link Policy Commands
 *****************************************************************************/
#define HCI_HOLD_MODE                  ((HCI_OPCODE)HCI_POLICY | 0x0001)
#define HCI_SNIFF_MODE                 ((HCI_OPCODE)HCI_POLICY | 0x0003)
#define HCI_EXIT_SNIFF_MODE            ((HCI_OPCODE)HCI_POLICY | 0x0004)
#define HCI_PARK_MODE                  ((HCI_OPCODE)HCI_POLICY | 0x0005)
#define HCI_EXIT_PARK_MODE             ((HCI_OPCODE)HCI_POLICY | 0x0006)
#define HCI_QOS_SETUP                  ((HCI_OPCODE)HCI_POLICY | 0x0007)
#define HCI_ROLE_DISCOVERY             ((HCI_OPCODE)HCI_POLICY | 0x0009)
#define HCI_SWITCH_ROLE                ((HCI_OPCODE)HCI_POLICY | 0x000B)
#define HCI_READ_LINK_POLICY_SETTINGS  ((HCI_OPCODE)HCI_POLICY | 0x000C)
#define HCI_WRITE_LINK_POLICY_SETTINGS ((HCI_OPCODE)HCI_POLICY | 0x000D)
#define HCI_MAX_POLICY_OCF             ((HCI_OPCODE)HCI_POLICY | 0x000E)

/******************************************************************************
   OP CODE defines - Host Controller and Baseband Commands
 *****************************************************************************/
#define HCI_SET_EVENT_MASK						((HCI_OPCODE)HCI_HOST_BB | 0x0001)
#define HCI_RESET								((HCI_OPCODE)HCI_HOST_BB | 0x0003)
#define HCI_SET_EVENT_FILTER					((HCI_OPCODE)HCI_HOST_BB | 0x0005)
#define HCI_FLUSH								((HCI_OPCODE)HCI_HOST_BB | 0x0008)
#define HCI_READ_PIN_TYPE						((HCI_OPCODE)HCI_HOST_BB | 0x0009)
#define HCI_WRITE_PIN_TYPE						((HCI_OPCODE)HCI_HOST_BB | 0x000A)
#define HCI_CREATE_NEW_UNIT_KEY					((HCI_OPCODE)HCI_HOST_BB | 0x000B)
#define HCI_READ_STORED_LINK_KEY				((HCI_OPCODE)HCI_HOST_BB | 0x000D)
#define HCI_WRITE_STORED_LINK_KEY				((HCI_OPCODE)HCI_HOST_BB | 0x0011)
#define HCI_DELETE_STORED_LINK_KEY				((HCI_OPCODE)HCI_HOST_BB | 0x0012)
#define HCI_CHANGE_LOCAL_NAME					((HCI_OPCODE)HCI_HOST_BB | 0x0013)
#define HCI_READ_LOCAL_NAME						((HCI_OPCODE)HCI_HOST_BB | 0x0014)
#define HCI_READ_CONN_ACCEPT_TIMEOUT			((HCI_OPCODE)HCI_HOST_BB | 0x0015)
#define HCI_READ_CONNECTION_ACCEPT_TIMEOUT		((HCI_OPCODE)HCI_HOST_BB | 0x0015)
#define HCI_WRITE_CONN_ACCEPT_TIMEOUT			((HCI_OPCODE)HCI_HOST_BB | 0x0016)
#define HCI_WRITE_CONNECTION_ACCEPT_TIMEOUT		((HCI_OPCODE)HCI_HOST_BB | 0x0016)
#define HCI_READ_PAGE_TIMEOUT					((HCI_OPCODE)HCI_HOST_BB | 0x0017)
#define HCI_WRITE_PAGE_TIMEOUT					((HCI_OPCODE)HCI_HOST_BB | 0x0018)
#define HCI_READ_SCAN_ENABLE					((HCI_OPCODE)HCI_HOST_BB | 0x0019)
#define HCI_WRITE_SCAN_ENABLE					((HCI_OPCODE)HCI_HOST_BB | 0x001A)
#define HCI_READ_PAGESCAN_ACTIVITY				((HCI_OPCODE)HCI_HOST_BB | 0x001B)
#define HCI_WRITE_PAGESCAN_ACTIVITY				((HCI_OPCODE)HCI_HOST_BB | 0x001C)
#define HCI_READ_INQUIRYSCAN_ACTIVITY			((HCI_OPCODE)HCI_HOST_BB | 0x001D)
#define HCI_WRITE_INQUIRYSCAN_ACTIVITY			((HCI_OPCODE)HCI_HOST_BB | 0x001E)
#define HCI_READ_AUTH_ENABLE					((HCI_OPCODE)HCI_HOST_BB | 0x001F)
#define HCI_READ_AUTHENTICATION_ENABLE					((HCI_OPCODE)HCI_HOST_BB | 0x001F)
#define HCI_WRITE_AUTH_ENABLE					((HCI_OPCODE)HCI_HOST_BB | 0x0020)
#define HCI_WRITE_AUTHENTICATION_ENABLE					((HCI_OPCODE)HCI_HOST_BB | 0x0020)
#define HCI_READ_ENC_MODE						((HCI_OPCODE)HCI_HOST_BB | 0x0021)
#define HCI_READ_ENCRYPTION_MODE						((HCI_OPCODE)HCI_HOST_BB | 0x0021)
#define HCI_WRITE_ENC_MODE						((HCI_OPCODE)HCI_HOST_BB | 0x0022)
#define HCI_WRITE_ENCRYPTION_MODE						((HCI_OPCODE)HCI_HOST_BB | 0x0022)
#define HCI_READ_CLASS_OF_DEVICE				((HCI_OPCODE)HCI_HOST_BB | 0x0023)
#define HCI_WRITE_CLASS_OF_DEVICE				((HCI_OPCODE)HCI_HOST_BB | 0x0024)
#define HCI_READ_VOICE_SETTING					((HCI_OPCODE)HCI_HOST_BB | 0x0025)
#define HCI_WRITE_VOICE_SETTING					((HCI_OPCODE)HCI_HOST_BB | 0x0026)
#define HCI_READ_AUTO_FLUSH_TIMEOUT				((HCI_OPCODE)HCI_HOST_BB | 0x0027)
#define HCI_WRITE_AUTO_FLUSH_TIMEOUT			((HCI_OPCODE)HCI_HOST_BB | 0x0028)
#define HCI_READ_NUM_BCAST_RETXS				((HCI_OPCODE)HCI_HOST_BB | 0x0029)
#define HCI_READ_NUM_BROADCAST_RETRANSMISSIONS				((HCI_OPCODE)HCI_HOST_BB | 0x0029)
#define HCI_WRITE_NUM_BCAST_RETXS				((HCI_OPCODE)HCI_HOST_BB | 0x002A)
#define HCI_WRITE_NUM_BROADCAST_RETRANSMISSIONS				((HCI_OPCODE)HCI_HOST_BB | 0x002A)
#define HCI_READ_HOLD_MODE_ACTIVITY				((HCI_OPCODE)HCI_HOST_BB | 0x002B)
#define HCI_WRITE_HOLD_MODE_ACTIVITY			((HCI_OPCODE)HCI_HOST_BB | 0x002C)
#define HCI_READ_TX_POWER_LEVEL					((HCI_OPCODE)HCI_HOST_BB | 0x002D)
#define HCI_READ_TRANSMIT_POWER_LEVEL					((HCI_OPCODE)HCI_HOST_BB | 0x002D)
#define HCI_READ_SCO_FLOW_CON_ENABLE			((HCI_OPCODE)HCI_HOST_BB | 0x002E)
#define HCI_READ_SCO_FLOW_CONTROL_ENABLE			((HCI_OPCODE)HCI_HOST_BB | 0x002E)
#define HCI_WRITE_SCO_FLOW_CON_ENABLE			((HCI_OPCODE)HCI_HOST_BB | 0x002F)
#define HCI_WRITE_SCO_FLOW_CONTROL_ENABLE			((HCI_OPCODE)HCI_HOST_BB | 0x002F)
#define HCI_SET_HCTOHOST_FLOW_CONTROL			((HCI_OPCODE)HCI_HOST_BB | 0x0031)
#define HCI_SET_HC_TO_H_FLOW_CONTROL			((HCI_OPCODE)HCI_HOST_BB | 0x0031)
#define HCI_HOST_BUFFER_SIZE					((HCI_OPCODE)HCI_HOST_BB | 0x0033)
#define HCI_HOST_NUM_COMPLETED_PACKETS			((HCI_OPCODE)HCI_HOST_BB | 0x0035)
#define HCI_HOST_NUMBER_COMPLETED_PACKETS			((HCI_OPCODE)HCI_HOST_BB | 0x0035)
#define HCI_READ_LINK_SUPERV_TIMEOUT			((HCI_OPCODE)HCI_HOST_BB | 0x0036)
#define HCI_READ_LINK_SUPERVISION_TIMEOUT			((HCI_OPCODE)HCI_HOST_BB | 0x0036)
#define HCI_WRITE_LINK_SUPERV_TIMEOUT			((HCI_OPCODE)HCI_HOST_BB | 0x0037)
#define HCI_WRITE_LINK_SUPERVISION_TIMEOUT			((HCI_OPCODE)HCI_HOST_BB | 0x0037)
#define HCI_READ_NUM_SUPPORTED_IAC				((HCI_OPCODE)HCI_HOST_BB | 0x0038)
#define HCI_READ_NUMBER_OF_SUPPORTED_IAC				((HCI_OPCODE)HCI_HOST_BB | 0x0038)
#define HCI_READ_CURRENT_IAC_LAP				((HCI_OPCODE)HCI_HOST_BB | 0x0039)
#define HCI_WRITE_CURRENT_IAC_LAP				((HCI_OPCODE)HCI_HOST_BB | 0x003A)
#define HCI_READ_PAGESCAN_PERIOD_MODE			((HCI_OPCODE)HCI_HOST_BB | 0x003B)
#define HCI_WRITE_PAGESCAN_PERIOD_MODE			((HCI_OPCODE)HCI_HOST_BB | 0x003C)
#define HCI_READ_PAGESCAN_MODE					((HCI_OPCODE)HCI_HOST_BB | 0x003D)
#define HCI_WRITE_PAGESCAN_MODE					((HCI_OPCODE)HCI_HOST_BB | 0x003E)
#define HCI_MAX_HOST_BB_OCF						((HCI_OPCODE)HCI_HOST_BB | 0x003F)

/******************************************************************************
   OP CODE defines - Informational Parameters
 *****************************************************************************/
#define HCI_READ_LOCAL_VER_INFO        ((HCI_OPCODE)HCI_INFO | 0x0001)
#define HCI_READ_LOCAL_VERSION_INFORMATION        ((HCI_OPCODE)HCI_INFO | 0x0001)
#define HCI_READ_LOCAL_SUPP_FEATURES   ((HCI_OPCODE)HCI_INFO | 0x0003)
#define HCI_READ_LOCAL_SUPPORTED_FEATURES   ((HCI_OPCODE)HCI_INFO | 0x0003)
#define HCI_READ_BUFFER_SIZE           ((HCI_OPCODE)HCI_INFO | 0x0005)
#define HCI_READ_COUNTRY_CODE          ((HCI_OPCODE)HCI_INFO | 0x0007)
#define HCI_READ_BD_ADDR               ((HCI_OPCODE)HCI_INFO | 0x0009)
#define HCI_MAX_INFO_OCF               ((HCI_OPCODE)HCI_INFO | 0x000A)

/******************************************************************************
   OP CODE defines - Status Parameters
 *****************************************************************************/
#define HCI_READ_FAILED_CONTACT_COUNT  ((HCI_OPCODE)HCI_STATUS | 0x0001)
#define HCI_READ_FAILED_CONTACT_COUNTER  ((HCI_OPCODE)HCI_STATUS | 0x0001)
#define HCI_RESET_FAILED_CONTACT_COUNT ((HCI_OPCODE)HCI_STATUS | 0x0002)
#define HCI_RESET_FAILED_CONTACT_COUNTER ((HCI_OPCODE)HCI_STATUS | 0x0002)
#define HCI_GET_LINK_QUALITY           ((HCI_OPCODE)HCI_STATUS | 0x0003)
#define HCI_READ_RSSI                  ((HCI_OPCODE)HCI_STATUS | 0x0005)
#define HCI_MAX_STATUS_OCF             ((HCI_OPCODE)HCI_STATUS | 0x0006)

/******************************************************************************
   OP CODE defines - Testing Parameters
 *****************************************************************************/
#define HCI_READ_LOOPBACK_MODE         ((HCI_OPCODE)HCI_TEST | 0x0001)
#define HCI_WRITE_LOOPBACK_MODE        ((HCI_OPCODE)HCI_TEST | 0x0002)
#define HCI_ENABLE_DUT_MODE            ((HCI_OPCODE)HCI_TEST | 0x0003)
#define HCI_ENABLE_DEVICE_UNDER_TEST_MODE            ((HCI_OPCODE)HCI_TEST | 0x0003)
#define HCI_MAX_TEST_OCF               ((HCI_OPCODE)HCI_TEST | 0x0004)

/******************************************************************************
   OP CODE defines - VENDOR SPECIFIC DEBUG Parameters
 *****************************************************************************/
#define HCI_DEBUG_STATS                ((HCI_OPCODE)HCI_DEBUG | 0x0001)
#define HCI_DEBUG_STATS_ACL            ((HCI_OPCODE)HCI_DEBUG | 0x0002)
#define HCI_DEBUG_STATS_SCO            ((HCI_OPCODE)HCI_DEBUG | 0x0003)
#define HCI_DEBUG_MEM                  ((HCI_OPCODE)HCI_DEBUG | 0x0004)
#define HCI_DEBUG_RAND                 ((HCI_OPCODE)HCI_DEBUG | 0x0005)
#define HCI_DEBUG_KEY                  ((HCI_OPCODE)HCI_DEBUG | 0x0006)
#define HCI_DEBUG_SRES                 ((HCI_OPCODE)HCI_DEBUG | 0x0007)


/******************************************************************************
        Event defines - these are events as per the HCI definition
 *****************************************************************************/
#define HCI_EV_INQUIRY_COMPLETE								((HCI_EVTCODE)0x01)
#define HCI_EV_INQUIRY_RESULT								((HCI_EVTCODE)0x02)
#define HCI_EV_CONN_COMPLETE								((HCI_EVTCODE)0x03)
#define HCI_EV_CONNECTION_COMPLETE							((HCI_EVTCODE)0x03)
#define HCI_EV_CONN_REQUEST									((HCI_EVTCODE)0x04)
#define HCI_EV_CONNECTION_REQUEST							((HCI_EVTCODE)0x04)
#define HCI_EV_DISCONNECT_COMPLETE							((HCI_EVTCODE)0x05)
#define HCI_EV_DISCONNECTION_COMPLETE						((HCI_EVTCODE)0x05)
#define HCI_EV_AUTH_COMPLETE								((HCI_EVTCODE)0x06)
#define HCI_EV_AUTHENTICATION_COMPLETE						((HCI_EVTCODE)0x06)
#define HCI_EV_REMOTE_NAME_REQ_COMPLETE						((HCI_EVTCODE)0x07)
#define HCI_EV_REMOTE_NAME_REQUEST_COMPLETE					((HCI_EVTCODE)0x07)
#define HCI_EV_ENCRYPTION_CHANGE							((HCI_EVTCODE)0x08)
#define HCI_EV_CHANGE_CONN_LINK_KEY_COMPLETE				((HCI_EVTCODE)0x09)
#define HCI_EV_CHANGE_CONNECTION_LINK_KEY_COMPLETE			((HCI_EVTCODE)0x09)
#define HCI_EV_MASTER_LINK_KEY_COMPLETE						((HCI_EVTCODE)0x0A)
#define HCI_EV_READ_REM_SUPP_FEATURES_COMPLETE				((HCI_EVTCODE)0x0B)
#define HCI_EV_READ_REMOTE_SUPPORTED_FEATURES_COMPLETE		((HCI_EVTCODE)0x0B)
#define HCI_EV_READ_REMOTE_VER_INFO_COMPLETE				((HCI_EVTCODE)0x0C)
#define HCI_EV_READ_REMOTE_VERSION_INFORMATION_COMPLETE		((HCI_EVTCODE)0x0C)
#define HCI_EV_QOS_SETUP_COMPLETE							((HCI_EVTCODE)0x0D)
#define HCI_EV_COMMAND_COMPLETE								((HCI_EVTCODE)0x0E)
#define HCI_EV_COMMAND_STATUS								((HCI_EVTCODE)0x0F)
#define HCI_EV_HARDWARE_ERROR								((HCI_EVTCODE)0x10)
#define HCI_EV_FLUSH_OCCURRED								((HCI_EVTCODE)0x11)
#define HCI_EV_ROLE_CHANGE									((HCI_EVTCODE)0x12)
#define HCI_EV_NUMBER_COMPLETED_PKTS						((HCI_EVTCODE)0x13)
#define HCI_EV_NUMBER_COMPLETED_PACKETS						((HCI_EVTCODE)0x13)
#define HCI_EV_MODE_CHANGE									((HCI_EVTCODE)0x14)
#define HCI_EV_RETURN_LINK_KEYS								((HCI_EVTCODE)0x15)
#define HCI_EV_PIN_CODE_REQ									((HCI_EVTCODE)0x16)
#define HCI_EV_PIN_CODE_REQUEST								((HCI_EVTCODE)0x16)
#define HCI_EV_LINK_KEY_REQ									((HCI_EVTCODE)0x17)
#define HCI_EV_LINK_KEY_REQUEST								((HCI_EVTCODE)0x17)
#define HCI_EV_LINK_KEY_NOTIFICATION						((HCI_EVTCODE)0x18)
#define HCI_EV_LOOPBACK_COMMAND								((HCI_EVTCODE)0x19)
#define HCI_EV_DATA_BUFFER_OVERFLOW							((HCI_EVTCODE)0x1A)
#define HCI_EV_MAX_SLOTS_CHANGE								((HCI_EVTCODE)0x1B)
#define HCI_EV_READ_CLOCK_OFFSET_COMPLETE					((HCI_EVTCODE)0x1C)
#define HCI_EV_CONN_PACKET_TYPE_CHANGED						((HCI_EVTCODE)0x1D)
#define HCI_EV_CONNECTION_PACKET_TYPE_CHANGED				((HCI_EVTCODE)0x1D)
#define HCI_EV_QOS_VIOLATION								((HCI_EVTCODE)0x1E)
#define HCI_EV_PAGE_SCAN_MODE_CHANGE						((HCI_EVTCODE)0x1F)
#define HCI_EV_PAGE_SCAN_REP_MODE_CHANGE					((HCI_EVTCODE)0x20)
#define HCI_EV_PAGE_SCAN_REPETITION_MODE_CHANGE				((HCI_EVTCODE)0x20)
#define HCI_MAX_EVENT_OPCODE								((HCI_EVTCODE)0x21)

/******************************************************************************
        HCI_COMMAND_COMPLETE defines - these are events as per the HCI definition
 *****************************************************************************/

/******************************************************************************
   OCF opcode defines - Link Control Commands
 *****************************************************************************/
#define HCI_LINK_KEY_REQ_REPLY_RET         ((HCI_OPCODE)HCI_LINK_RET | 0x000B)
#define HCI_LINK_KEY_REQ_NEG_REPLY_RET     ((HCI_OPCODE)HCI_LINK_RET | 0x000C)
#define HCI_PIN_CODE_REQ_REPLY_RET         ((HCI_OPCODE)HCI_LINK_RET | 0x000D)
#define HCI_PIN_CODE_REQ_NEG_REPLY_RET     ((HCI_OPCODE)HCI_LINK_RET | 0x000E)
#define HCI_MAX_LINK_RET_OCF               ((HCI_OPCODE)HCI_LINK_RET | 0x000F)

/******************************************************************************
   OP CODE defines - Link Policy Commands
 *****************************************************************************/
#define HCI_ROLE_DISCOVERY_RET             ((HCI_OPCODE)HCI_POLICY_RET | 0x0009)
#define HCI_READ_LINK_POLICY_SETTINGS_RET  ((HCI_OPCODE)HCI_POLICY_RET | 0x000C)
#define HCI_WRITE_LINK_POLICY_SETTINGS_RET ((HCI_OPCODE)HCI_POLICY_RET | 0x000D)
#define HCI_MAX_POLICY_RET_OCF             ((HCI_OPCODE)HCI_POLICY_RET | 0x000E)

/******************************************************************************
   OP CODE defines - Host Controller and Baseband Commands
 *****************************************************************************/
#define HCI_FLUSH_RET                      ((HCI_OPCODE)HCI_HOST_BB_RET | 0x0008)
#define HCI_READ_PIN_TYPE_RET              ((HCI_OPCODE)HCI_HOST_BB_RET | 0x0009)
#define HCI_READ_STORED_LINK_KEY_RET       ((HCI_OPCODE)HCI_HOST_BB_RET | 0x000D)
#define HCI_WRITE_STORED_LINK_KEY_RET      ((HCI_OPCODE)HCI_HOST_BB_RET | 0x0011)
#define HCI_DELETE_STORED_LINK_KEY_RET     ((HCI_OPCODE)HCI_HOST_BB_RET | 0x0012)
#define HCI_READ_LOCAL_NAME_RET            ((HCI_OPCODE)HCI_HOST_BB_RET | 0x0014)
#define HCI_READ_CONN_ACCEPT_TIMEOUT_RET   ((HCI_OPCODE)HCI_HOST_BB_RET | 0x0015)
#define HCI_READ_PAGE_TIMEOUT_RET          ((HCI_OPCODE)HCI_HOST_BB_RET | 0x0017)
#define HCI_READ_SCAN_ENABLE_RET           ((HCI_OPCODE)HCI_HOST_BB_RET | 0x0019)
#define HCI_READ_PAGESCAN_ACTIVITY_RET     ((HCI_OPCODE)HCI_HOST_BB_RET | 0x001B)
#define HCI_READ_INQUIRYSCAN_ACTIVITY_RET  ((HCI_OPCODE)HCI_HOST_BB_RET | 0x001D)
#define HCI_READ_AUTH_ENABLE_RET           ((HCI_OPCODE)HCI_HOST_BB_RET | 0x001F)
#define HCI_READ_ENC_MODE_RET              ((HCI_OPCODE)HCI_HOST_BB_RET | 0x0021)
#define HCI_READ_CLASS_OF_DEVICE_RET       ((HCI_OPCODE)HCI_HOST_BB_RET | 0x0023)
#define HCI_READ_VOICE_SETTING_RET         ((HCI_OPCODE)HCI_HOST_BB_RET | 0x0025)
#define HCI_READ_AUTO_FLUSH_TIMEOUT_RET    ((HCI_OPCODE)HCI_HOST_BB_RET | 0x0027)
#define HCI_WRITE_AUTO_FLUSH_TIMEOUT_RET   ((HCI_OPCODE)HCI_HOST_BB_RET | 0x0028)
#define HCI_READ_NUM_BCAST_RETXS_RET       ((HCI_OPCODE)HCI_HOST_BB_RET | 0x0029)
#define HCI_READ_HOLD_MODE_ACTIVITY_RET    ((HCI_OPCODE)HCI_HOST_BB_RET | 0x002B)
#define HCI_READ_TX_POWER_LEVEL_RET        ((HCI_OPCODE)HCI_HOST_BB_RET | 0x002D)
#define HCI_READ_SCO_FLOW_CON_ENABLE_RET   ((HCI_OPCODE)HCI_HOST_BB_RET | 0x002E)
#define HCI_READ_LINK_SUPERV_TIMEOUT_RET   ((HCI_OPCODE)HCI_HOST_BB_RET | 0x0036)
#define HCI_WRITE_LINK_SUPERV_TIMEOUT_RET  ((HCI_OPCODE)HCI_HOST_BB_RET | 0x0037)
#define HCI_READ_NUM_SUPPORTED_IAC_RET     ((HCI_OPCODE)HCI_HOST_BB_RET | 0x0038)
#define HCI_READ_CURRENT_IAC_LAP_RET       ((HCI_OPCODE)HCI_HOST_BB_RET | 0x0039)
#define HCI_READ_PAGESCAN_PERIOD_MODE_RET  ((HCI_OPCODE)HCI_HOST_BB_RET | 0x003B)
#define HCI_READ_PAGESCAN_MODE_RET         ((HCI_OPCODE)HCI_HOST_BB_RET | 0x003D)
#define HCI_MAX_HOST_BB_RET_OCF            ((HCI_OPCODE)HCI_HOST_BB_RET | 0x003E)

/******************************************************************************
   OP CODE defines - Informational Parameters
 *****************************************************************************/
#define HCI_READ_LOCAL_VER_INFO_RET        ((HCI_OPCODE)HCI_INFO_RET | 0x0001)
#define HCI_READ_LOCAL_SUPP_FEATURES_RET   ((HCI_OPCODE)HCI_INFO_RET | 0x0003)
#define HCI_READ_BUFFER_SIZE_RET           ((HCI_OPCODE)HCI_INFO_RET | 0x0005)
#define HCI_READ_COUNTRY_CODE_RET          ((HCI_OPCODE)HCI_INFO_RET | 0x0007)
#define HCI_READ_BD_ADDR_RET               ((HCI_OPCODE)HCI_INFO_RET | 0x0009)
#define HCI_MAX_INFO_RET_OCF               ((HCI_OPCODE)HCI_INFO_RET | 0x000A)

/******************************************************************************
   OP CODE defines - Status Parameters
 *****************************************************************************/
#define HCI_READ_FAILED_CONTACT_COUNT_RET  ((HCI_OPCODE)HCI_STATUS_RET | 0x0001)
#define HCI_RESET_FAILED_CONTACT_COUNT_RET ((HCI_OPCODE)HCI_STATUS_RET | 0x0002)
#define HCI_GET_LINK_QUALITY_RET           ((HCI_OPCODE)HCI_STATUS_RET | 0x0003)
#define HCI_READ_RSSI_RET                  ((HCI_OPCODE)HCI_STATUS_RET | 0x0005)
#define HCI_MAX_STATUS_RET_OCF             ((HCI_OPCODE)HCI_STATUS_RET | 0x0006)

/******************************************************************************
   OP CODE defines - Testing Parameters
 *****************************************************************************/
#define HCI_READ_LOOPBACK_MODE_RET         ((HCI_OPCODE)HCI_TEST_RET | 0x0001)
#define HCI_MAX_TEST_RET_OCF               ((HCI_OPCODE)HCI_TEST_RET | 0x0004)
/*#define HCI_MAX_TEST_RET_OCF               ((HCI_OPCODE)HCI_TEST_RET | 0x0002)*/

/******************************************************************************
   HCI_COMMAND_COMPLETE, Argument Length Definitions (Full length)
   3 From Command Complete + Return Parameters.
   When an argument length is dependant on the number of elements in the array
   the defined length contains the constant parameter lengths only. The full 
   array length must be calculated.
 *****************************************************************************/
#define HCI_INQUIRY_CANCEL_ARG_LEN             ((uint8) 4)
#define HCI_PERIODIC_INQ_MODE_ARG_LEN          ((uint8) 4)
#define HCI_EXIT_PERIODIC_INQ_MODE_ARG_LEN     ((uint8) 4)
#define HCI_LINK_KEY_REQ_REPLY_ARG_LEN         ((uint8) 10)
#define HCI_LINK_KEY_REQ_NEG_REPLY_ARG_LEN     ((uint8) 10)
#define HCI_PIN_CODE_REQ_REPLY_ARG_LEN         ((uint8) 10)
#define HCI_PIN_CODE_REQ_NEG_REPLY_ARG_LEN     ((uint8) 10)
#define HCI_ROLE_DISCOVERY_ARG_LEN             ((uint8) 7)
#define HCI_READ_LINK_POLICY_SETTINGS_ARG_LEN  ((uint8) 8)
#define HCI_WRITE_LINK_POLICY_SETTINGS_ARG_LEN ((uint8) 6)
#define HCI_SET_EVENT_MASK_ARG_LEN             ((uint8) 4)
#define HCI_RESET_ARG_LEN                      ((uint8) 4)
#define HCI_SET_EVENT_FILTER_ARG_LEN           ((uint8) 4)
#define HCI_FLUSH_ARG_LEN                      ((uint8) 6)
#define HCI_READ_PIN_TYPE_ARG_LEN              ((uint8) 5)
#define HCI_WRITE_PIN_TYPE_ARG_LEN             ((uint8) 4)
#define HCI_CREATE_NEW_UNIT_KEY_ARG_LEN        ((uint8) 4)
#define HCI_READ_STORED_LINK_KEY_ARG_LEN       ((uint8) 8)
#define HCI_WRITE_STORED_LINK_KEY_ARG_LEN      ((uint8) 5)
#define HCI_DELETE_STORED_LINK_KEY_ARG_LEN     ((uint8) 6)
#define HCI_CHANGE_LOCAL_NAME_ARG_LEN          ((uint8) 4)
#define HCI_READ_LOCAL_NAME_ARG_LEN            ((uint8) 252)
#define HCI_READ_CONN_ACCEPT_TIMEOUT_ARG_LEN   ((uint8) 6)
#define HCI_WRITE_CONN_ACCEPT_TIMEOUT_ARG_LEN  ((uint8) 4)
#define HCI_READ_PAGE_TIMEOUT_ARG_LEN          ((uint8) 6)
#define HCI_WRITE_PAGE_TIMEOUT_ARG_LEN         ((uint8) 4)
#define HCI_READ_SCAN_ENABLE_ARG_LEN           ((uint8) 5)
#define HCI_WRITE_SCAN_ENABLE_ARG_LEN          ((uint8) 4)
#define HCI_READ_PAGESCAN_ACTIVITY_ARG_LEN     ((uint8) 8)
#define HCI_WRITE_PAGESCAN_ACTIVITY_ARG_LEN    ((uint8) 4)
#define HCI_READ_INQUIRYSCAN_ACTIVITY_ARG_LEN  ((uint8) 8)
#define HCI_WRITE_INQUIRYSCAN_ACTIVITY_ARG_LEN ((uint8) 4)
#define HCI_READ_AUTH_ENABLE_ARG_LEN           ((uint8) 5)
#define HCI_WRITE_AUTH_ENABLE_ARG_LEN          ((uint8) 4)
#define HCI_READ_ENC_MODE_ARG_LEN              ((uint8) 5)
#define HCI_WRITE_ENC_MODE_ARG_LEN             ((uint8) 4)
#define HCI_READ_CLASS_OF_DEVICE_ARG_LEN       ((uint8) 7)
#define HCI_WRITE_CLASS_OF_DEVICE_ARG_LEN      ((uint8) 4)
#define HCI_READ_VOICE_SETTING_ARG_LEN         ((uint8) 6)
#define HCI_WRITE_VOICE_SETTING_ARG_LEN        ((uint8) 4)
#define HCI_READ_AUTO_FLUSH_TIMEOUT_ARG_LEN    ((uint8) 8)
#define HCI_WRITE_AUTO_FLUSH_TIMEOUT_ARG_LEN   ((uint8) 6)
#define HCI_READ_NUM_BCASTXS_ARG_LEN           ((uint8) 5)
#define HCI_WRITE_NUM_BCASTXS_ARG_LEN          ((uint8) 4)
#define HCI_READ_HOLD_MODE_ACTIVITY_ARG_LEN    ((uint8) 5)
#define HCI_WRITE_HOLD_MODE_ACTIVITY_ARG_LEN   ((uint8) 4)
#define HCI_READ_TX_POWER_LEVEL_ARG_LEN        ((uint8) 7)
#define HCI_READ_SCO_FLOW_CON_ENABLE_ARG_LEN   ((uint8) 5)
#define HCI_WRITE_SCO_FLOW_CON_ENABLE_ARG_LEN  ((uint8) 4)
#define HCI_SET_HC_TO_H_FLOW_CONTROL_ARG_LEN   ((uint8) 4)
#define HCI_HOST_BUFFER_SIZE_ARG_LEN           ((uint8) 4)
#define HCI_HOST_NUM_COMPLETED_PKTS_ARG_LEN    ((uint8) 5) /* Variable */
#define HCI_READ_LINK_SUPERV_TIMEOUT_ARG_LEN   ((uint8) 8)
#define HCI_WRITE_LINK_SUPERV_TIMEOUT_ARG_LEN  ((uint8) 6)
#define HCI_READ_NUM_SUPPORTED_IAC_ARG_LEN     ((uint8) 5)
#define HCI_READ_CURRENT_IAC_LAP_ARG_LEN       ((uint8) 5) /* Variable */
#define HCI_WRITE_CURRENT_IAC_LAP_ARG_LEN      ((uint8) 4)
#define HCI_READ_PAGESCAN_PERIOD_MODE_ARG_LEN  ((uint8) 5)
#define HCI_WRITE_PAGESCAN_PERIOD_MODE_ARG_LEN ((uint8) 4)
#define HCI_READ_PAGESCAN_MODE_ARG_LEN         ((uint8) 5)
#define HCI_WRITE_PAGESCAN_MODE_ARG_LEN        ((uint8) 4)
#define HCI_READ_LOCAL_VER_INFO_ARG_LEN        ((uint8) 12)
#define HCI_READ_LOCAL_FEATURES_LEN            ((uint8) 12)
#define HCI_READ_BUFFER_SIZE_ARG_LEN           ((uint8) 11)
#define HCI_READ_COUNTRY_CODE_ARG_LEN          ((uint8) 5)
#define HCI_READ_BD_ADDR_ARG_LEN               ((uint8) 10)
#define HCI_READ_FAILED_CONTACT_COUNT_ARG_LEN  ((uint8) 8)
#define HCI_RESET_FAILED_CONTACT_COUNT_ARG_LEN ((uint8) 6)
#define HCI_GET_LINK_QUALITY_ARG_LEN           ((uint8) 7)
#define HCI_READ_RSSI_ARG_LEN                  ((uint8) 7)
#define HCI_READ_LOOPBACK_MODE_ARG_LEN         ((uint8) 5)
#define HCI_WRITE_LOOPBACK_MODE_ARG_LEN        ((uint8) 4)
#define HCI_ENABLE_DUT_ARG_LEN                 ((uint8) 4)

/******************************************************************************
   Broadcast flag defines.  Active broadcast and piconet broadcast are
   not for SCO use
 *****************************************************************************/
#define HCI_BROADCAST_FLAG_ACTIVE   ((hci_connection_handle_t)0x4000)
#define HCI_BROADCAST_FLAG_PICONET  ((hci_connection_handle_t)0x8000)
#define HCI_BROADCAST_FLAG_MASK     ((hci_connection_handle_t)0xC000)

/******************************************************************************
   Packet boundary flag defines, these are already 'bit shifted'
   These are for use with ACL only, for SCO use 00 (reserved)
 *****************************************************************************/
#define HCI_PKT_BOUNDARY_FLAG_RESERVED ((hci_packet_boundary_flag_t)0)
#define HCI_PKT_BOUNDARY_FLAG_CONT     ((hci_packet_boundary_flag_t)0x1000)
#define HCI_PKT_BOUNDARY_FLAG_FIRST    ((hci_packet_boundary_flag_t)0x2000)
#define HCI_PKT_BOUNDARY_MASK          ((hci_packet_boundary_flag_t)0x3000)

/******************************************************************************
    HCI Successful return value 
 *****************************************************************************/
#define HCI_SUCCESS                         ((hci_return_t)0x00)
#define HCI_COMMAND_CURRENTLY_PENDING       ((hci_return_t)0x00)

/******************************************************************************
    HCI Error codes 
 *****************************************************************************/
#define HCI_ERROR_ILLEGAL_COMMAND           ((hci_error_t)0x01)
#define HCI_ERROR_NO_CONNECTION             ((hci_error_t)0x02)
#define HCI_ERROR_HARDWARE_FAIL             ((hci_error_t)0x03)
#define HCI_ERROR_PAGE_TIMEOUT              ((hci_error_t)0x04)
#define HCI_ERROR_AUTH_FAIL                 ((hci_error_t)0x05)
#define HCI_ERROR_KEY_MISSING               ((hci_error_t)0x06)
#define HCI_ERROR_MEMORY_FULL               ((hci_error_t)0x07)
#define HCI_ERROR_CONN_TIMEOUT              ((hci_error_t)0x08)
#define HCI_ERROR_MAX_NR_OF_CONNS           ((hci_error_t)0x09)
#define HCI_ERROR_MAX_NR_OF_SCO             ((hci_error_t)0x0A)
#define HCI_ERROR_MAX_NR_OF_ACL             ((hci_error_t)0x0B)
#define HCI_ERROR_COMMAND_DISALLOWED        ((hci_error_t)0x0C)
#define HCI_ERROR_REJ_BY_REMOTE_NO_RES      ((hci_error_t)0x0D)  /* no resource */
#define HCI_ERROR_REJ_BY_REMOTE_SEC         ((hci_error_t)0x0E)  /* security violation */
#define HCI_ERROR_REJ_BY_REMOTE_PERS        ((hci_error_t)0x0F)  /* personal device */
#define HCI_ERROR_HOST_TIMEOUT              ((hci_error_t)0x10) 
#define HCI_ERROR_UNSUPPORTED_FEATURE       ((hci_error_t)0x11)  /* or incorrect param value */
#define HCI_ERROR_ILLEGAL_FORMAT            ((hci_error_t)0x12) 
#define HCI_ERROR_OETC_USER                 ((hci_error_t)0x13)  /* other end terminated */
#define HCI_ERROR_OETC_LOW_RESOURCE         ((hci_error_t)0x14)  /* other end terminated */
#define HCI_ERROR_OETC_POWERING_OFF         ((hci_error_t)0x15)  /* other end terminated */
#define HCI_ERROR_CONN_TERM_LOCAL_HOST      ((hci_error_t)0x16)  /* local host terminated */
#define HCI_ERROR_AUTH_REPEATED             ((hci_error_t)0x17)  
#define HCI_ERROR_PAIRING_NOT_ALLOWED       ((hci_error_t)0x18) 
#define HCI_ERROR_UNKNOWN_LMP_PDU           ((hci_error_t)0x19) 
#define HCI_ERROR_UNSUPPORTED_REM_FEATURE   ((hci_error_t)0x1A) 
#define HCI_ERROR_SCO_OFFSET_REJECTED       ((hci_error_t)0x1B) 
#define HCI_ERROR_SCO_INTERVAL_REJECTED     ((hci_error_t)0x1C) 
#define HCI_ERROR_SCO_AIR_MODE_REJECTED     ((hci_error_t)0x1D) 
#define HCI_ERROR_INVALID_LMP_PARAMETERS    ((hci_error_t)0x1E) 
#define HCI_ERROR_UNSPECIFIED               ((hci_error_t)0x1F) 
#define HCI_ERROR_UNSUPP_LMP_PARAM          ((hci_error_t)0x20) 
#define HCI_ERROR_ROLE_CHANGE_NOT_ALLOWED   ((hci_error_t)0x21) 
#define HCI_ERROR_LMP_RESPONSE_TIMEOUT      ((hci_error_t)0x22)
#define HCI_ERROR_LMP_TRANSACTION_COLLISION ((hci_error_t)0x23)
#define HCI_ERROR_LMP_PDU_NOT_ALLOWED       ((hci_error_t)0x24) 

/*****************************************************************************
 *
 *  HCI miscellaneous common fields 
 *
 *****************************************************************************/

#define HCI_DEFAULT        ((uint8)0)     /* 0 in a message field means default */ 
                                            /* however, the interpretation     
                                               varies depending on the message */
#define HCI_MASTER         ((hci_role_t)0)
#define HCI_SLAVE          ((hci_role_t)1)

#define HCI_LINK_KEY_SEMIPERM  ((hci_key_flag_t)0)
#define HCI_LINK_KEY_TEMP      ((hci_key_flag_t)1)

#define HCI_LINK_ENC_OFF       ((hci_link_enc_t)0)
#define HCI_LINK_ENC_ON        ((hci_link_enc_t)1)

#define HCI_QOS_NO_TRAFFIC     ((hci_qos_type_t)0)
#define HCI_QOS_BEST_EFFORT    ((hci_qos_type_t)1)
#define HCI_QOS_GUARANTEED     ((hci_qos_type_t)2)

/*----------------------------------------------------------------------------*
 *
 *  HCI packet types
 *
 *----------------------------------------------------------------------------*/
#define HCI_PKT_DM1       ((hci_pkt_type_t)0x0008)    /* SCO and ACL */
#define HCI_PKT_DH1       ((hci_pkt_type_t)0x0010)    /* ACL only */
#define HCI_PKT_HV1       ((hci_pkt_type_t)0x0020)    /* SCO only */
#define HCI_PKT_HV2       ((hci_pkt_type_t)0x0040)    /* SCO only */
#define HCI_PKT_HV3       ((hci_pkt_type_t)0x0080)    /* SCO only */
#define HCI_PKT_DV        ((hci_pkt_type_t)0x0100)    /* SCO only */
#define HCI_PKT_AUX1      ((hci_pkt_type_t)0x0200)    /* ACL only NOT L2CAP */
#define HCI_PKT_DM3       ((hci_pkt_type_t)0x0400)    /* ACL only */
#define HCI_PKT_DH3       ((hci_pkt_type_t)0x0800)    /* ACL only */
#define HCI_PKT_DM5       ((hci_pkt_type_t)0x4000)    /* ACL only */
#define HCI_PKT_DH5       ((hci_pkt_type_t)0x8000)    /* ACL only */


/*-----------------------------------------------------------------------------*
 *
 *  HCI event mask values 
 *  As 8 byte quantity split into two uint32 so require high and low
 *  part masks.
 *
 *------------------------------------------------------------------------------*/
#define HCI_EV_MASK_MSB                     ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_NONE_HI                 ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_NONE_LO                 ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_INQUIRY_COMPLETE_HI     ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_INQUIRY_COMPLETE_LO     ((hci_event_mask_t)0x00000001)
#define HCI_EV_MASK_INQUIRY_RESULT_HI       ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_INQUIRY_RESULT_LO       ((hci_event_mask_t)0x00000002)
#define HCI_EV_MASK_CONN_COMPLETE_HI        ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_CONN_COMPLETE_LO        ((hci_event_mask_t)0x00000004)
#define HCI_EV_MASK_CONN_REQ_HI             ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_CONN_REQ_LO             ((hci_event_mask_t)0x00000008)
#define HCI_EV_MASK_DISC_COMPLETE_HI        ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_DISC_COMPLETE_LO        ((hci_event_mask_t)0x00000010)
#define HCI_EV_MASK_AUTH_COMPLETE_HI        ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_AUTH_COMPLETE_LO        ((hci_event_mask_t)0x00000020)
#define HCI_EV_MASK_REM_NAME_REQ_COMP_HI    ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_REM_NAME_REQ_COMP_LO    ((hci_event_mask_t)0x00000040)
#define HCI_EV_MASK_CHG_CONN_ENC_EN_HI      ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_CHG_CONN_ENC_EN_LO      ((hci_event_mask_t)0x00000080)
#define HCI_EV_MASK_CHG_CONN_LINK_KEY_HI    ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_CHG_CONN_LINK_KEY_LO    ((hci_event_mask_t)0x00000100)
#define HCI_EV_MASK_MASTER_LINK_KEY_HI      ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_MASTER_LINK_KEY_LO      ((hci_event_mask_t)0x00000200)
#define HCI_EV_MASK_READ_REM_SUPP_FEAT_HI   ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_READ_REM_SUPP_FEAT_LO   ((hci_event_mask_t)0x00000400)
#define HCI_EV_MASK_READ_REM_VER_INFO_HI    ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_READ_REM_VER_INFO_LO    ((hci_event_mask_t)0x00000800)
#define HCI_EV_MASK_QOS_SETUP_COMP_HI       ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_QOS_SETUP_COMP_LO       ((hci_event_mask_t)0x00001000)
#define HCI_EV_MASK_COMMAND_COMPLETE_HI     ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_COMMAND_COMPLETE_LO     ((hci_event_mask_t)0x00002000)
#define HCI_EV_MASK_COMMAND_STATUS_HI       ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_COMMAND_STATUS_LO       ((hci_event_mask_t)0x00004000)
#define HCI_EV_MASK_HARDWARE_ERROR_HI       ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_HARDWARE_ERROR_LO       ((hci_event_mask_t)0x00008000)
#define HCI_EV_MASK_FLUSH_OCCURRED_EV_HI    ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_FLUSH_OCCURRED_EV_LO    ((hci_event_mask_t)0x00010000)
#define HCI_EV_MASK_ROLE_CHANGE_HI          ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_ROLE_CHANGE_LO          ((hci_event_mask_t)0x00020000)
#define HCI_EV_MASK_NUM_HCI_DATA_PKTS_HI    ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_NUM_HCI_DATA_PKTS_LO    ((hci_event_mask_t)0x00040000)
#define HCI_EV_MASK_MODE_CHANGE_HI          ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_MODE_CHANGE_LO          ((hci_event_mask_t)0x00080000)
#define HCI_EV_MASK_RETURN_LINK_KEYS_HI     ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_RETURN_LINK_KEYS_LO     ((hci_event_mask_t)0x00100000)
#define HCI_EV_MASK_PIN_CODE_REQ_HI         ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_PIN_CODE_REQ_LO         ((hci_event_mask_t)0x00200000)
#define HCI_EV_MASK_LINK_KEY_REQ_HI         ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_LINK_KEY_REQ_LO         ((hci_event_mask_t)0x00400000)
#define HCI_EV_MASK_LINK_KEY_NOTIFY_HI      ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_LINK_KEY_NOTIFY_LO      ((hci_event_mask_t)0x00800000)
#define HCI_EV_MASK_LOOPBACK_COMMAND_HI     ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_LOOPBACK_COMMAND_LO     ((hci_event_mask_t)0x01000000)
#define HCI_EV_MASK_DATA_BUFFER_OVERFLOW_HI ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_DATA_BUFFER_OVERFLOW_LO ((hci_event_mask_t)0x02000000)
#define HCI_EV_MASK_MAX_SLOTS_CHANGE_HI     ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_MAX_SLOTS_CHANGE_LO     ((hci_event_mask_t)0x04000000)
#define HCI_EV_MASK_READ_CLOCK_OFFSET_HI    ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_READ_CLOCK_OFFSET_LO    ((hci_event_mask_t)0x08000000)
#define HCI_EV_MASK_CONN_PKT_TYPE_HI        ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_CONN_PKT_TYPE_LO        ((hci_event_mask_t)0x10000000)
#define HCI_EV_MASK_QOS_VIOLATION_HI        ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_QOS_VIOLATION_LO        ((hci_event_mask_t)0x20000000)
#define HCI_EV_MASK_PAGE_SCAN_MODE_HI       ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_PAGE_SCAN_MODE_LO       ((hci_event_mask_t)0x40000000)
#define HCI_EV_MASK_PAGE_SCAN_REP_MODE_HI   ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_PAGE_SCAN_REP_MODE_LO   ((hci_event_mask_t)0x80000000)

#define HCI_EV_MASK_DEFAULT_HI              ((hci_event_mask_t)0x00000000)
#define HCI_EV_MASK_DEFAULT_LO              ((hci_event_mask_t)0xFFFFFFFF)

/*-----------------------------------------------------------------------------*
 *
 *   Auto accept values 
 *
 *------------------------------------------------------------------------------*/
#define HCI_AUTO_ACCEPT_OFF             ((uint8)0x01)
#define HCI_AUTO_ACCEPT_ON              ((uint8)0x02)

/*-----------------------------------------------------------------------------*
 *
 *   Inquiry Values :   HCI_INQUIRY, HCI_PERIODIC_INQUIRY_MODE,
 *                      HCWI_WRITE_INQUIRYSCAN_ACTIVITY
 *
 *------------------------------------------------------------------------------*/
#define HCI_INQUIRY_LENGTH_MIN              ((uint8)0x01)
#define HCI_INQUIRY_LENGTH_MAX              ((uint8)0x30)
#define HCI_INQUIRY_RESPONSES_MIN           ((uint8)0x01)
#define HCI_INQUIRY_RESPONSES_MAX           ((uint8)0xFF)
#define HCI_INQUIRY_MAX_PERIOD_MIN          ((uint16)0x0003)
#define HCI_INQUIRY_MAX_PERIOD_MAX          ((uint16)0xFFFF)
#define HCI_INQUIRY_MIN_PERIOD_MIN          ((uint16)0x0002)
#define HCI_INQUIRY_MIN_PERIOD_MAX          ((uint16)0xFFFE)
#define HCI_INQUIRYSCAN_INTERVAL_MIN        ((uint16)0x0012)
#define HCI_INQUIRYSCAN_INTERVAL_DEFAULT    ((uint16)0x0800)
#define HCI_INQUIRYSCAN_INTERVAL_MAX        ((uint16)0x1000)
#define HCI_INQUIRYSCAN_WINDOW_MIN          ((uint16)0x0012)
#define HCI_INQUIRYSCAN_WINDOW_DEFAULT      ((uint16)0x0012)
#define HCI_INQUIRYSCAN_WINDOW_MAX          ((uint16)0x1000)

/*-----------------------------------------------------------------------------*
 *
 *   Scan enable values 
 *
 *------------------------------------------------------------------------------*/
#define HCI_SCAN_ENABLE_OFF             ((uint8)0x00)
#define HCI_SCAN_ENABLE_INQ             ((uint8)0x01)
#define HCI_SCAN_ENABLE_PAGE            ((uint8)0x02)
#define HCI_SCAN_ENABLE_INQ_AND_PAGE    ((uint8)0x03)    /* default */

/*-----------------------------------------------------------------------------*
 *
 *   Authentication enable values 
 *
 *------------------------------------------------------------------------------*/
#define HCI_AUTH_ENABLE_OFF             ((uint8)0x00)
#define HCI_AUTH_ENABLE_ON              ((uint8)0x01)    /* for all connections */

/*-----------------------------------------------------------------------------*
 *
 *   Encryption mode values 
 *
 *------------------------------------------------------------------------------*/
#define HCI_ENC_OFF                     ((uint8)0x00)
#define HCI_ENC_ON                      ((uint8)0x01)

#define HCI_ENC_MODE_OFF                ((uint8)0x00)
#define HCI_ENC_MODE_PT_TO_PT           ((uint8)0x01)
#define HCI_ENC_MODE_PT_TO_PT_AND_BCAST ((uint8)0x02)

/*-----------------------------------------------------------------------------*
 *
 *   Voice setting mask values
 *
 *------------------------------------------------------------------------------*/
#define HCI_VOICE_INPUT_MASK            ((uint16)0x0300)
#define HCI_VOICE_INPUT_LIN             ((uint16)0x0000)
#define HCI_VOICE_INPUT_MU_LAW          ((uint16)0x0100)    
#define HCI_VOICE_INPUT_A_LAW           ((uint16)0x0200)
#define HCI_VOICE_FORMAT_MASK           ((uint16)0x00C0)
#define HCI_VOICE_FORMAT_1SCOMP         ((uint16)0x0000)
#define HCI_VOICE_FORMAT_2SCOMP         ((uint16)0x0040)
#define HCI_VOICE_FORMAT_SMAG           ((uint16)0x0080)
#define HCI_VOICE_SAMP_SIZE_MASK        ((uint16)0x0020)
#define HCI_VOICE_SAMP_SIZE_8BIT        ((uint16)0x0000)
#define HCI_VOICE_SAMP_SIZE_16BIT       ((uint16)0x0020)
#define HCI_VOICE_LINEAR_PCM_MASK       ((uint16)0x001C)
#define HCI_VOICE_AIR_CODING_MASK       ((uint16)0x0003)
#define HCI_VOICE_AIR_CODING_CVSD       ((uint16)0x0000)
#define HCI_VOICE_AIR_CODING_MU_LAW     ((uint16)0x0001)
#define HCI_VOICE_AIR_CODING_A_LAW      ((uint16)0x0002)

#define HCI_VOICE_SETTINGS_DEFAULT      ((uint16)0x0060)

/*-----------------------------------------------------------------------------*
 *
 *   Hold Mode Activity Values 
 *
 *------------------------------------------------------------------------------*/
#define HCI_HOLD_CURR_PWR_STATE         ((uint8)0x00)
#define HCI_HOLD_SUSPEND_PAGE_SCAN      ((uint8)0x01)
#define HCI_HOLD_SUSPEND_INQ_SCAN       ((uint8)0x02)
#define HCI_HOLD_SUSPEND_PER_INQ        ((uint8)0x04)

#define HCI_HOLD_MIN_PERIOD             ((uint16)0x01)

/*-----------------------------------------------------------------------------*
 *
 *   HCI Version Information Values 
 *
 *------------------------------------------------------------------------------*/
#define HCI_VER_1_0               ((hci_version_t)0x00)

/*-----------------------------------------------------------------------------*
 *
 *   HCI Country Code Values 
 *
 *------------------------------------------------------------------------------*/
#define HCI_CO_CODE_NA_AND_EUR     ((uint8)0x00)
#define HCI_CO_CODE_FRANCE         ((uint8)0x01)
#define HCI_CO_CODE_SPAIN          ((uint8)0x02)
#define HCI_CO_CODE_JAPAN          ((uint8)0x03)

/*-----------------------------------------------------------------------------*
 *
 *   HCI Current device mode Values 
 *
 *------------------------------------------------------------------------------*/
#define HCI_BT_MODE_ACTIVE         ((hci_bt_mode_t)0x00)
#define HCI_BT_MODE_HOLD           ((hci_bt_mode_t)0x01)
#define HCI_BT_MODE_SNIFF          ((hci_bt_mode_t)0x02)
#define HCI_BT_MODE_PARK           ((hci_bt_mode_t)0x03)

/*-----------------------------------------------------------------------------*
 *
 *   HCI Test Values 
 *
 *------------------------------------------------------------------------------*/
#define HCI_GEN_SELF_TEST          ((uint8)0x00)
#define HCI_LOOPBACK_OFF           ((uint8)0x00)
#define HCI_LOCAL_LOOPBACK         ((uint8)0x01)
#define HCI_REMOTE_LOOPBACK        ((uint8)0x02)
#define HCI_LOOPBACK_MODE_MAX      ((uint8)0x03)

/*-----------------------------------------------------------------------------*
 *
 *   HCI Link Type Values 
 *
 *------------------------------------------------------------------------------*/
#define HCI_LINK_TYPE_SCO          ((uint8)0x00)
#define HCI_LINK_TYPE_ACL          ((uint8)0x01)


/*-----------------------------------------------------------------------------*
 *
 *   HCI Page Scan Repetition Mode Values 
 *
 *------------------------------------------------------------------------------*/
#define HCI_PAGE_SCAN_REP_MODE_R0     ((uint8)0x00)
#define HCI_PAGE_SCAN_REP_MODE_R1     ((uint8)0x01)
#define HCI_PAGE_SCAN_REP_MODE_R2     ((uint8)0x02)

/*-----------------------------------------------------------------------------*
 *
 *   HCI Page Scan Mode Values 
 *
 *------------------------------------------------------------------------------*/
#define HCI_PAGE_SCAN_MODE_MANDATORY      ((uint8)0x00)
#define HCI_PAGE_SCAN_MODE_OPTIONAL_1     ((uint8)0x01)
#define HCI_PAGE_SCAN_MODE_OPTIONAL_2     ((uint8)0x02)
#define HCI_PAGE_SCAN_MODE_OPTIONAL_3     ((uint8)0x03)
#define HCI_PAGE_SCAN_MODE_DEFAULT        HCI_PAGE_SCAN_MODE_MANDATORY    

/*-----------------------------------------------------------------------------*
 *
 *   HCI Page Scan Interval : HCI_WRITE_PAGESCAN_ACTIVITY 
 *
 *------------------------------------------------------------------------------*/
#define HCI_PAGESCAN_INTERVAL_MIN       ((uint16) 0x12)
#define HCI_PAGESCAN_INTERVAL_DEFAULT   ((uint16) 0x800)
#define HCI_PAGESCAN_INTERVAL_MAX       ((uint16) 0x1000)

/*-----------------------------------------------------------------------------*
 *
 *   HCI Page Scan Window : HCI_WRITE_PAGESCAN_ACTIVITY 
 *
 *------------------------------------------------------------------------------*/
#define HCI_PAGESCAN_WINDOW_MIN       ((uint16) 0x12)
#define HCI_PAGESCAN_WINDOW_DEFAULT   ((uint16) 0x12)
#define HCI_PAGESCAN_WINDOW_MAX       ((uint16) 0x1000)

/*-----------------------------------------------------------------------------*
 *
 *   HCI Page Scan PERIOD : HCI_WRITE_PAGESCAN_PERIOD_MODE 
 *
 *------------------------------------------------------------------------------*/
#define HCI_PAGESCAN_PERIOD_MODE_P0         ((uint8) 0x00)
#define HCI_PAGESCAN_PERIOD_MODE_P1         ((uint8) 0x01)
#define HCI_PAGESCAN_PERIOD_MODE_P2         ((uint8) 0x02)
#define HCI_PAGESCAN_PERIOD_MODE_DEFAULT    HCI_PAGESCAN_PERIOD_MODE_P0

/*-----------------------------------------------------------------------------*
 *
 *   HCI Clock offset Values 
 *
 *------------------------------------------------------------------------------*/
#define HCI_CLOCK_OFFSET_MASK       ((clock_offset_t)0x7fff)
#define HCI_CLOCK_OFFSET_VALID_MASK ((clock_offset_t)0x8000)
#define HCI_CLOCK_OFFSET_INVALID    ((clock_offset_t)0x0000)
#define HCI_CLOCK_OFFSET_VALID      ((clock_offset_t)0x8000)

/*-----------------------------------------------------------------------------*
 *
 *   HCI Link Policy Settings
 *
 *------------------------------------------------------------------------------*/
#define DISABLE_ALL_LM_MODES         ((link_policy_settings_t)0x0000)
#define ENABLE_MS_SWITCH             ((link_policy_settings_t)0x0001)
#define ENABLE_HOLD                  ((link_policy_settings_t)0x0002)
#define ENABLE_SNIFF                 ((link_policy_settings_t)0x0004)
#define ENABLE_PARK                  ((link_policy_settings_t)0x0008)

/*-----------------------------------------------------------------------------*
 *
 *   HCI Filter types
 *
 *------------------------------------------------------------------------------*/
#define CLEAR_ALL_FILTERS            ((filter_type_t)0x00)
#define INQUIRY_RESULT_FILTER        ((filter_type_t)0x01)
#define CONNECTION_FILTER            ((filter_type_t)0x02)

/*-----------------------------------------------------------------------------*
 *
 *   HCI Filter condition types
 *
 *------------------------------------------------------------------------------*/
#define NEW_DEVICE_RESPONDED         ((filter_condition_type_t)0x00)
#define ALL_CONNECTION               ((filter_condition_type_t)0x00)
#define CLASS_OF_DEVICE_RESPONDED    ((filter_condition_type_t)0x01)
#define ADDRESSED_DEVICE_RESPONDED   ((filter_condition_type_t)0x02)

/*-----------------------------------------------------------------------------*
 *
 *   HCI Pin types
 *
 *------------------------------------------------------------------------------*/
#define HCI_VARIABLE_PIN    ((pin_type_t)0x00)
#define HCI_FIXED_PIN       ((pin_type_t)0x01)

/*-----------------------------------------------------------------------------*
 *
 *   Size of link keys
 *
 *------------------------------------------------------------------------------*/
#define SIZE_LINK_KEY   0x10

/*-----------------------------------------------------------------------------*
 *
 *   HCI Read stored link key read all flag types
 *
 *------------------------------------------------------------------------------*/
#define RETURN_BDADDR   ((read_all_flag_t)0x00)
#define RETURN_ALL      ((read_all_flag_t)0x01)

/*-----------------------------------------------------------------------------*
 *
 *   HCI Delete stored link key read all flag types
 *
 *------------------------------------------------------------------------------*/
#define DELETE_BDADDR   ((delete_all_flag_t)0x00)
#define DELETE_ALL      ((delete_all_flag_t)0x01)

/*-----------------------------------------------------------------------------*
 *
 *   HCI IAC LAP Boundary values and Other Values
 *
 *------------------------------------------------------------------------------*/
#define HCI_NUM_CURRENT_IAC_MIN   ((uint8)0x01)
#define HCI_NUM_CURRENT_IAC_MAX   ((uint8)0x40)
#define HCI_IAC_LAP_MIN           ((uint32)0x9E8B00)
#define HCI_IAC_LAP_MAX           ((uint32)0x9E8B3F)

#define HCI_INQ_CODE_GIAC  ((uint8)0)     /* use to set all three bytes to 0 */
#define HCI_IAC_LAP_PTRS        4
#define HCI_IAC_LAP_PER_PTR     16


/*-----------------------------------------------------------------------------*
 *
 *   HCI Connection Accept Timeout values
 *
 *------------------------------------------------------------------------------*/
#define HCI_CONNECTION_ACCEPT_TIMEOUT_MIN   ((uint16)0x01)

/*-----------------------------------------------------------------------------*
 *
 *   HCI Handle Range
 *
 *------------------------------------------------------------------------------*/
#define HCI_HANDLE_MAX   ((uint16)0x0EFF)

/*-----------------------------------------------------------------------------*
 *
 *   HCI Link Supervision Timeout Values
 *
 *------------------------------------------------------------------------------*/
#define HCI_LINK_SUPERVISION_MIN        ((uint16)0x0001)
#define HCI_LINK_SUPERVISION_DEFAULT    ((uint16)0x7D00)
#define HCI_LINK_SUPERVISION_MAX        ((uint16)0xFFFF)

/*-----------------------------------------------------------------------------*
 *
 *   HCI Transmit Power Type Values
 *
 *------------------------------------------------------------------------------*/
#define HCI_READ_CURRENT_TX_POWER       ((uint8)0x00)
#define HCI_READ_MAX_TX_POWER           ((uint8)0x01)

/*-----------------------------------------------------------------------------*
 *
 *   HCI SCO Flow Control Type Values
 *
 *------------------------------------------------------------------------------*/
#define HCI_SCO_FLOW_CONTROL_DISABLED   ((uint8)0x00)
#define HCI_SCO_FLOW_CONTROL_ENABLED    ((uint8)0x01)

/*-----------------------------------------------------------------------------*
 *
 *   HCI HC to H Flow Control Type Values
 *
 *------------------------------------------------------------------------------*/
#define HCI_HCITOH_FLOW_CONTROL_DISABLED   ((uint8)0x00)
#define HCI_HCITOH_FLOW_CONTROL_ENABLED    ((uint8)0x01)


/*-----------------------------------------------------------------------------*
 *
 *   HCI Write Stored Link Key, HCI Return Link Keys Event, 
 *   maximum number of keys that can be present
 *   in a write stored link key. Max command parameter size= 255 bytes.
 *      Num keys parameter = 1 byte
 *      (BD_ADDR + Key) = (6+16) = X
 *      Max Keys = (255-1)/X = 11
 *
 *------------------------------------------------------------------------------*/
#define HCI_STORED_LINK_KEY_MAX      ((uint8)11)

#endif /* ndef _TEHCI_DEF_H */
