///////////////////////////////////////////////////////////////////////
//
//  FILE   :  CBluetestEngine.h
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  CBluetestEngine
//
//  PURPOSE:  Provide the interface as presented in James' Memo.
//
///////////////////////////////////////////////////////////////////////

#ifndef __CBLUETESTENGINE_H__
#define __CBLUETESTENGINE_H__

#include "common/types.h"
#include "hci/packer_parameters.h"

#define TEST_SIZE (4)

class BlueCoreDeviceController;
class BlueCoreControllerFactory;

class CBluetestEngine
{
public:
    CBluetestEngine();
    ~CBluetestEngine();

    //  Set up callback for string
    void setOnString( void (*onString) ( bool verboseNotConcise , TCHAR * aString , void * aParameters ) , void * aParameters );

    //  Set the factory.
    void setFactory ( BlueCoreControllerFactory * aFactory );

    //  Set PSSetting tool if defined.
    void setPSSetFunction ( void (*aPSSetFunction) ( PARAMETER_ARRAY , void * ) , void * aParameters );

    //  Set up BCSP and talk to the chip
    bool startChip ();

    //  Hard reset over bccmd, and take down BCSP.
    void resetChip ( void );

    //  Is there a chip?
    bool connected ( void );

    //  RF test functions.
    //  All seem to require hardware reset to end them.
    //  All return void and have up to 3 arguments.
    bool runTest ( uint16 aTest[TEST_SIZE] );
    //  and 4 aren't really RF tests.
    bool readPIOLines ( void );
    bool setPIOLines ( uint16 mask , uint16 settings );
    bool provokeFault ( uint16 aFaultId );
    bool readFWVersion ( void );

    //  additional commands
    bool sendChipMgtHostToChip ( PARAMETER_ARRAY aParameters );

    //  Generate a bccmd sequence number.
    uint16 nextSequence ( void );

    //  Provide stubs for callbacks.
    static void discardHCIData ( uint16 connectionHandle , 
                                 bool packetBoundaryFlag , 
                                 uint8 broadcastFlag , 
                                 uint8 *dataStart , 
                                 uint16 dataLength, 
                                 void * aThis);
    static void discardHCIEvent ( uint8 eventcode, 
                                  PARAMETER_ARRAY eventParameters ,
                                  void * aThis);
    static void onHQ ( PARAMETER_ARRAY eventParameters , 
                       void * aThis );
    static void onBCCMD ( PARAMETER_ARRAY eventParameters , 
                          void * aThis );
    static void onLinkFail ( void * aThis );

    static void onDebugData ( uint8* aBuffer , uint32 aLength , void * aThis );

    void processBCCMD ( PARAMETER_ARRAY eventParameters );
    void processHQ    ( PARAMETER_ARRAY eventParameters );

private:
    BlueCoreControllerFactory * mFactory;
    BlueCoreDeviceController * mChip;

    HANDLE testStarted;
    HANDLE pioMask;
    HANDLE pioLines;

    void (*mPSSetFunction) ( PARAMETER_ARRAY , void * );
    void * mPSSetParameters;

    void sendReset();

    void report ( bool verboseNotConcise , TCHAR * aString );
    void report ( bool verboseNotConcise , uint8 aStringNumber );
    void report ( bool verboseNotConcise , uint8 aStringNumber , uint32 aNumberParameter1 );
    void report ( bool verboseNotConcise , uint8 aStringNumber , uint32 aNumberParameter1 , uint32 aNumberParameter2 );
    void report ( bool verboseNotConcise , uint8 aStringNumber , uint32 aNumberParameter1 , TCHAR * aString2 );
    void report ( bool verboseNotConcise , uint8 aStringNumber , uint32 aNumberParameter1 , uint32 aNumberParameter2 , uint32 aNumberParameter3 );
    void report ( bool verboseNotConcise , uint8 aStringNumber , uint32 aNumberParameter1 , uint32 aNumberParameter2 , uint32 aNumberParameter3 , uint32 aNumberParameter4 );
    void report ( bool verboseNotConcise , uint8 aStringNumber , uint32 aNumberParameter1 , uint32 aNumberParameter2 , uint32 aNumberParameter3 , uint32 aNumberParameter4 , uint32 aNumberParameter5 );
    void report ( bool verboseNotConcise , uint8 aStringNumber , uint32 aNumberParameter1 , uint32 aNumberParameter2 , uint32 aNumberParameter3 , uint32 aNumberParameter4 , TCHAR * aStringParameter5 );
    void report ( bool verboseNotConcise , uint8 aStringNumber , TCHAR * aString );
    void report ( bool verboseNotConcise , uint8 aStringNumber , TCHAR * aString1 , uint32 aNumberParameter2 , uint32 aNumberParameter3 );
    void report ( bool verboseNotConcise , uint8 aStringNumber , double aNumberParameter1 );
    void report ( bool verboseNotConcise , uint8 aStringNumber , double aNumberParameter1 , uint32 aNumberParameter2 , uint32 aNumberParameter3 , uint32 aNumberParameter4 , uint32 aStringParameter5 , uint32 aStringParameter6 , uint32 aStringParameter7 , uint32 aStringParameter8 );
    void * mStringCallBackParameters;
    void (*mStringCallBackFn) ( bool , TCHAR* , void* );

    CRITICAL_SECTION sequenceProtection;
    uint16 sequenceNumber;

    //  Interpretation methods and variables
    void interpretBitErr ( uint32 type , uint32 periodValue , uint32 cumulativeValue );
    uint8 reportLines;
    bool firstTime;
    bool printHeader;
    uint32 bitCount;
    uint32 bitErrors;
    uint32 cumulativeBitCount;
    uint32 cumulativeBitErrors;
    uint32 cumulativeExpPkts;
    uint32 cumulativeRcdPkts;
    uint32 cumulativeSyncErr;
    uint32 cumulativeHdrErr;
    uint32 cumulativeCRCErr;
    uint32 cumulativeUncorrErr;
};

#endif//__CBLUETESTENGINE_H__
