///////////////////////////////////////////////////////////////////////
//
//  FILE   :  testNames.h
//
//  AUTHOR :  Adam Hughes
//
//  TABLE  :  testNames
//
//  PURPOSE:  Provide changeable strings interface.
//
///////////////////////////////////////////////////////////////////////

#include "hq/hqpdu.h"
#include "bccmd/bccmdpdu.h"

#define NOT_RADIOTEST_SET_PIO (RADIOTEST_LAST_COMMAND + 1)
#define NOT_RADIOTEST_READ_PIO (RADIOTEST_LAST_COMMAND + 2)
#define NOT_RADIOTEST_PROVOKE_FAULT (RADIOTEST_LAST_COMMAND + 3)
#define NOT_RADIOTEST_NOTHING (RADIOTEST_LAST_COMMAND + 4)
#define NOT_RADIOTEST_READ_FW_VERSION (RADIOTEST_LAST_COMMAND + 5)



#ifndef _WINCE



struct
{
    uint16 test;
    char * string;
}
testNames[] =
{
        // Acceptable values for "test".
        {RADIOTEST_PAUSE , "PAUSE"},
        {NOT_RADIOTEST_NOTHING ,"------------"},
        //  Transmitter tests:
        {RADIOTEST_TXSTART	,"TXSTART"},
        {RADIOTEST_TXDATA1	,"TXDATA1"},
        {RADIOTEST_TXDATA2	,"TXDATA2"},
        {RADIOTEST_TXDATA3	,"TXDATA3"},
        {RADIOTEST_TXDATA4	,"TXDATA4"},
        {NOT_RADIOTEST_NOTHING ,"------------"},

        //  Receiver tests
        {RADIOTEST_RXSTART1	,"RXSTART1"},
        {RADIOTEST_RXSTART2	,"RXSTART2"},
        {RADIOTEST_RXDATA1	,"RXDATA1"},
        {RADIOTEST_RXDATA2	,"RXDATA2"},
        {RADIOTEST_BER1		,"BIT ERR1"},
        {RADIOTEST_BER2		,"BIT ERR2"},
        {NOT_RADIOTEST_NOTHING ,"------------"},

        //  Loop back tests
        {RADIOTEST_LOOP_BACK     ,"LOOP BACK"},
        {RADIOTEST_RX_LOOP_BACK  ,"RX LOOP BACK"},
        {RADIOTEST_BER_LOOP_BACK ,"BER LOOP BACK"},
        {NOT_RADIOTEST_NOTHING ,"------------"},

        //  Configuration
        {RADIOTEST_CFG_FREQ	,"CFG FREQ"},
        {RADIOTEST_CFG_PKT	,"CFG PKT"},
        {RADIOTEST_CFG_BIT_ERR	,"CFG BIT ERR"},
        {RADIOTEST_CONFIG_TX_IF  ,"CFG TX IF"},
        {RADIOTEST_CFG_XTAL_FTRIM,"CFG XTAL FTRIM"},
        {RADIOTEST_CFG_UAP_LAP  ,"CFG UAP/LAP"},
        {RADIOTEST_CFG_ACC_ERRS, "CFG ACC ERRS"},
        {RADIOTEST_CFG_IQ_TRIM,"CFG IQ TRIM"}, 
        {RADIOTEST_CFG_TX_TRIM,"CFG TX TRIM"}, 
        {NOT_RADIOTEST_NOTHING ,"------------"},

        // Built-in self tests
        {RADIOTEST_DEEP_SLEEP	,"Deep Sleep"},
        {RADIOTEST_PCM_LB	,"PCM LB"},
        {RADIOTEST_PCM_EXT_LB   ,"PCM EXT LB" },
        {RADIOTEST_SETTLE	,"SETTLE"},
        {RADIOTEST_IF_RESP	,"IF RESP"},
        {RADIOTEST_RF_IQ_MATCH	,"RF IQ MATCH"},
        {RADIOTEST_IF_IQ_MATCH	,"IF IQ MATCH"},
        {RADIOTEST_VCOTRIM	,"VCOTRIM"},
        {NOT_RADIOTEST_SET_PIO  ,"Set PIO"},
        {NOT_RADIOTEST_READ_PIO ,"Read PIO"},
        {RADIOTEST_BUILD_LUT	,"BUILD LUT"},
        {RADIOTEST_READ_LUT	,"READ LUT"},
        // Extras
        {NOT_RADIOTEST_PROVOKE_FAULT	,"Provoke Fault"},
        {NOT_RADIOTEST_READ_FW_VERSION ,"Read FW version" },
        {RADIOTEST_RADIO_STATUS ,"RADIO STATUS" }
};

const uint32 TESTNAMES_SIZE = sizeof (testNames) / sizeof(testNames[0]);

#else
struct
{
    uint16 test;
    TCHAR * string;
}
testNames[] =
{
        // Acceptable values for "test".
        {RADIOTEST_PAUSE , _T("PAUSE")},
        {NOT_RADIOTEST_NOTHING ,_T("------------")},
        //  Transmitter tests:
        {RADIOTEST_TXSTART	,_T("TXSTART")},
        {RADIOTEST_TXDATA1	,_T("TXDATA1")},
        {RADIOTEST_TXDATA2	,_T("TXDATA2")},
        {RADIOTEST_TXDATA3	,_T("TXDATA3")},
        {RADIOTEST_TXDATA4	,_T("TXDATA4")},
        {NOT_RADIOTEST_NOTHING ,_T("------------")},

        //  Receiver tests
        {RADIOTEST_RXSTART1	,_T("RXSTART1")},
        {RADIOTEST_RXSTART2	,_T("RXSTART2")},
        {RADIOTEST_RXDATA1	,_T("RXDATA1")},
        {RADIOTEST_RXDATA2	,_T("RXDATA2")},
        {RADIOTEST_BER1		,_T("BIT ERR1")},
        {RADIOTEST_BER2		,_T("BIT ERR2")},
        {NOT_RADIOTEST_NOTHING ,_T("------------")},

        //  Loop back tests
        {RADIOTEST_LOOP_BACK     ,_T("LOOP BACK")},
        {RADIOTEST_RX_LOOP_BACK  ,_T("RX LOOP BACK")},
        {RADIOTEST_BER_LOOP_BACK ,_T("BER LOOP BACK")},
        {NOT_RADIOTEST_NOTHING ,_T("------------")},

        //  Configuration
        {RADIOTEST_CFG_FREQ	,_T("CFG FREQ")},
        {RADIOTEST_CFG_PKT	,_T("CFG PKT")},
        {RADIOTEST_CFG_BIT_ERR	,_T("CFG BIT ERR")},
        {RADIOTEST_CONFIG_TX_IF  ,_T("CFG TX IF")},
        {RADIOTEST_CFG_XTAL_FTRIM,_T("CFG XTAL FTRIM")},
        {RADIOTEST_CFG_UAP_LAP  ,_T("CFG UAP/LAP")},
        {RADIOTEST_CFG_ACC_ERRS, _T("CFG ACC ERRS")},
        {RADIOTEST_CFG_IQ_TRIM,_T("CFG IQ TRIM")}, 
        {RADIOTEST_CFG_TX_TRIM,_T("CFG TX TRIM")}, 
        {NOT_RADIOTEST_NOTHING ,_T("------------")},

        // Built-in self tests
        {RADIOTEST_DEEP_SLEEP	,_T("Deep Sleep")},
        {RADIOTEST_PCM_LB	,_T("PCM LB")},
        {RADIOTEST_PCM_EXT_LB   ,_T("PCM EXT LB") },
        {RADIOTEST_SETTLE	,_T("SETTLE")},
        {RADIOTEST_IF_RESP	,_T("IF RESP")},
        {RADIOTEST_RF_IQ_MATCH	,_T("RF IQ MATCH")},
        {RADIOTEST_IF_IQ_MATCH	,_T("IF IQ MATCH")},
        {RADIOTEST_VCOTRIM	,_T("VCOTRIM")},
        {NOT_RADIOTEST_SET_PIO  ,_T("Set PIO")},
        {NOT_RADIOTEST_READ_PIO ,_T("Read PIO")},
        {RADIOTEST_BUILD_LUT	,_T("BUILD LUT")},
        {RADIOTEST_READ_LUT	,_T("READ LUT")},
        // Extras
        {NOT_RADIOTEST_PROVOKE_FAULT	,_T("Provoke Fault")},
        {NOT_RADIOTEST_READ_FW_VERSION ,_T("Read FW version")},
        {RADIOTEST_RADIO_STATUS ,_T("RADIO STATUS") }
};

const uint32 TESTNAMES_SIZE = sizeof (testNames) / sizeof(testNames[0]);

#endif
