/*****************************************************************************\

This file was generated from Win32BCSPTransport.cpp revision 1.43.

How to generate PosixBCSPTransport.cpp from Win32BCSPTransport.cpp :

- Change all occurrences of 'Win32' to 'Posix' (e.g. sed 's/Win32/Posix/g').

- Remove the header boilerplate, and replace with this one.

- Remove the entire #ifndef _WINCE and replace with just
#include "cassert"

- Make the following change in comportPrep ():

<         isprintf (mConfigString , II("baud=%d parity=e data=8 stop=1 dtr=on rts=on") , aDataRate );
---
>         isprintf (mConfigString , II("%lu"), (unsigned long) aDataRate);

- Make the following change in start ():

<             if ( mComportHandle == INVALID_HANDLE_VALUE
<                                ? !mStack->open( mComportName , mConfigString )
<                                : !mStack->open( mComportHandle) )
---
>             uint32 bps = 38400;
>             if ( mConfigString != NULL )
>                 bps = iatol (mConfigString); // HERE for now, expecting config to be a datarate in bps
>             if ( mComportHandle != INVALID_HANDLE_VALUE ) // HERE not currently supported
>                 ok = serialProblem ;
>             else if ( !mStack->open( mComportName , bps ) ) // HERE was mConfigString ) )

- Update the first line in this boilerplate.

MODIFICATION HISTORY
	1.6  12:feb:02  pws     Added dummy set_sco_bandwidth().

\*****************************************************************************/

#include "HCIDeviceController.h"
#include "cassert"
#include "PosixBCSPTransport.h"
#include <stdio.h>
#include <stdlib.h>
#include "memfuncs.h"

PosixBCSPTransport::PosixBCSPTransport( ichar* comPort , uint32 aDataRate )
{
    creationInit ();
    comportPrep ( comPort , aDataRate , 0 );
}

PosixBCSPTransport::PosixBCSPTransport( ichar* comPort , ichar * configString )
{
    creationInit ();
    comportPrep ( comPort , 0 , configString );
}

PosixBCSPTransport::PosixBCSPTransport( HANDLE comPort )
{
    creationInit ();
	mComportHandle = comPort;
}

void PosixBCSPTransport::creationInit()
{
    //  Do the creation initialisation, in a procedure so it can be called
    //  from all starting points.
    mStack = NULL;
    mBCSPStack = 0;
    mComportName = NULL;
    mConfigString = NULL;

    mComportHandle = INVALID_HANDLE_VALUE;
    tunnel = false;
    mBufferSize = 4096;
    mName = II("PosixBCSPTransport");
    mUserOnLinkFailFn = NULL;
	mStackCfg = *::getDefaultStackConfiguration();
}

void PosixBCSPTransport::comportPrep  ( ichar* comPort , uint32 aDataRate , ichar * configString )
{
    if ( comPort != NULL )
    {
        mComportName = new ichar[istrlen(comPort) + 1];
        istrcpy ( mComportName , comPort );
    }
    if ( aDataRate != 0 )
    {
        mConfigString = new ichar[60];
        isprintf (mConfigString , II("%lu") , (unsigned long) aDataRate );
    }
    else if ( configString != NULL )
    {
        int length = istrlen(configString) + 1;
        mConfigString = new ichar[length];
        istrcpy ( mConfigString , configString );
    }
}

PosixBCSPTransport::~PosixBCSPTransport()
{
    stop();
    delete[] mComportName;
    delete[] mConfigString;
}

startTransportReturnValue PosixBCSPTransport::start()
{
    startTransportReturnValue ok = OK;
    if (mStack != NULL)
        ok = otherProblem;
    else
    {
        //set up bcsp stack and callbacks on all channels
        mStack = new PosixBCSP;
	mBCSPStack = mStack->getBCSPStack() ;

        if ( (mComportName == NULL) && (mComportHandle == INVALID_HANDLE_VALUE) )
            ok = serialProblem;
        else
        {    
            uint32 bps = 38400;
            if ( mConfigString != NULL )
                bps = iatol (mConfigString); // HERE for now, expecting config to be a datarate in bps
            if ( mComportHandle != INVALID_HANDLE_VALUE ) // HERE not currently supported
                ok = serialProblem ;
            else if ( !mStack->open( mComportName , bps ) ) // HERE was mConfigString ) )
                ok = serialProblem ;
            else
            {
                mStack->setOnLinkFail(onLinkFail,(void*) this) ;
		::setStackConfiguration(mBCSPStack, &mStackCfg) ;
                mStack->start() ;

                //  Set up all the channels to receive.
                for (uint8 i = 0 ; i < BLUECORE_CHANNEL_COUNT ; i ++ )
                {
                    buffers[i] = new uint8[mBufferSize];
                    if ( i == hciSCO )
                        BCSPopenChannel ( &channels[i] , mBCSPStack , i , BCSPChannelUnreliable , BCSPCallbackDeferred ) ;
                    else
                        BCSPopenChannel ( &channels[i] , mBCSPStack , i , BCSPChannelReliable , BCSPCallbackDeferred ) ;
                    CallBackInfo *info = new CallBackInfo;
                    info->object = this;
                    info->channelNumber = (BLUECORE_CHANNELS)i;
                    BCinitTransferRequest( &recvReqs[i], buffers[i], mBufferSize, onPacketFromChip , info );
                    BCSPreadPacket( &channels[i] , &recvReqs[i] ) ;
                }
            }
        }
    }
    return ok ;
}

void PosixBCSPTransport::stop()
{
    if (mStack)
		delete mStack;
	mStack = 0 ;
    // deleting the stack causes cancelled requests on all the channels.
}

bool PosixBCSPTransport::ready ( uint32 timeout )
{
    //  if the link has failed return false
    return ( linkFailed ? false : mStack->connect( timeout ) );
}

void PosixBCSPTransport::getStackConfiguration(StackConfiguration &cfg)
{
	cfg = mStackCfg;
}

void PosixBCSPTransport::setStackConfiguration(const StackConfiguration &cfg)
{
	mStackCfg = cfg;
}

uint16 PosixBCSPTransport::getBufferSize()
{
    return mBufferSize;
}

void PosixBCSPTransport::resetStack()
{
    stop();

    start () ;
}


void PosixBCSPTransport::sendpdu ( BLUECORE_CHANNELS channel, uint8 * apdu , uint32 aLength, CallBack *aCallBack )
{
    if (!linkFailed)
    {
        assert ( (II("pdu bigger than uint16") , aLength < 65536) );
        BCTransferRequest *req = new BCTransferRequest;
        CallBackInfo *info = new CallBackInfo;
        info->callBack = aCallBack;
        info->channelNumber = channel;
        info->object = this;
        BCinitTransferRequest( req , apdu , (uint16)aLength , onPacketToChip , info );
        BCSPwritePacket ( &channels[channel], req );
    }
    else
		if (aCallBack)
			aCallBack->diedIncomplete();
}

void PosixBCSPTransport::onPacketToChip ( BCTransferRequest * req, BCTransferStatus status, void * info )
{
    CallBackInfo* lInfo = (CallBackInfo*)info;
    CallBack * lCallBack = lInfo->callBack;
	if ( lCallBack != NULL)
	{
	    if (status != transferCancelled)
	    {
			lCallBack->execute();
		}
	    else
	    {
			lCallBack->diedIncomplete();
		}
	}

	delete lCallBack;
    delete req;
    delete lInfo;
}

void PosixBCSPTransport::onPacketFromChip (BCTransferRequest *req , BCTransferStatus status, void *info)
{
    CallBackInfo* lInfo = (CallBackInfo*)info;
    switch (status)
    {
    case transferCancelled:
        // delete the buffer
        delete[] req->data;
        delete lInfo;
        break;
    case transferBufferTooSmall:
        assert ( 0 );
        break;
    default:
        lInfo->object->processDataFromChip(req,lInfo->channelNumber);
        break;
    }
    return;
}

void PosixBCSPTransport::processDataFromChip ( BCTransferRequest * req, BLUECORE_CHANNELS ch )
{
    //  collect the data.
    //  Take a copy to pass up to the next level, so as not to expose our request->data buffer.
    uint8* buffer = (uint8*)getpdu(req->length);
    memcpy ( buffer , req->data , req->length );

    recvData (ch, buffer, req->length);

    //  reset the transfer request to the right size.
    BCreloadTransferRequest( req , req->data , mBufferSize) ;
    BCSPreadPacket( &channels[ch] , req );
}

#ifdef MGRDIAG
void PosixBCSPTransport::printAndClearRetryLog (void)
{
    mStack->printAndClearRetryLog ();
}
#endif

void PosixBCSPTransport::set_sco_bandwidth(int, void *, int)
{
}
