/*****************************************************************************\

This file was last compared with Win32BCSPTransport.h revision 1.21.

MODIFICATION HISTORY
	1.5  12:feb:02  pws     Added dummy set_sco_bandwidth().

\*****************************************************************************/

#ifndef POSIXBCSPTRANSPORT_H
#define POSIXBCSPTRANSPORT_H

#include "blueCoreTransport.h"
#include "PosixBCSP.h"

typedef void *HANDLE; // HERE just enough to make it build...
#define INVALID_HANDLE_VALUE (HANDLE)-1

class PosixBCSPTransport : public BlueCoreTransport
{
public:
    PosixBCSPTransport( ichar* comPort , uint32 dataRate );
    PosixBCSPTransport( ichar* comPort , ichar * configString );
    PosixBCSPTransport( HANDLE comPort );
    virtual ~PosixBCSPTransport();

    static void onPacketFromChip ( BCTransferRequest * req, BCTransferStatus status, void * info );
    static void onPacketToChip ( BCTransferRequest * req, BCTransferStatus status, void * info );
    void processDataFromChip ( BCTransferRequest * req, BLUECORE_CHANNELS ch );

    startTransportReturnValue start( void );
    bool ready ( uint32 timeout );
    void stop();

    void getStackConfiguration(StackConfiguration &cfg);
    void setStackConfiguration(const StackConfiguration &cfg);

    uint16 getBufferSize();

    void resetStack ();

    void set_sco_bandwidth (int, void *, int);

private:
    void creationInit ();
    void comportPrep  ( ichar* comPort , uint32 dataRate , ichar * configString );


    uint16 mBufferSize;

    ichar * mComportName;
    ichar * mConfigString;
    HANDLE mComportHandle;

    PosixBCSP * mStack;
    BCSPStack * mBCSPStack;
    StackConfiguration mStackCfg;
    BCSPChannel       channels[BLUECORE_CHANNEL_COUNT];
    BCTransferRequest recvReqs[BLUECORE_CHANNEL_COUNT];
    uint8*            buffers [BLUECORE_CHANNEL_COUNT];
    void sendpdu     ( BLUECORE_CHANNELS channel, uint8 *apdu, uint32 aLength , CallBack *aCallBack ) ;
};

typedef struct
{
    BLUECORE_CHANNELS channelNumber;
    PosixBCSPTransport * object;
    CallBack * callBack;
}
CallBackInfo;

#endif // POSIXBCSPTRANSPORT_H
