///////////////////////////////////////////////////////////////////////
//
//  FILE   :  PosixH4Transport.h
//
//            (C) CSR 2000
//
//  AUTHOR :  Mark Marshall
//
//  CLASS  :  PosixH4Transport
//
//  PURPOSE:  transparent class inheriting top API from the
//            BlueCoreTransport class, and owning a H4Stack which does
//            all the work.
//
//            BlueCoreTransport also provides BCSP-specific private channels
//            L2CAP etc which are tunneled through HCI vendor specific
//            commands and puts the debug channel messages to a file
//            in the debug build version.
//
//  MODIFICATION HISTORY:
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/Posix/H4/PosixH4Transport.h#4 $
//
///////////////////////////////////////////////////////////////////////

#ifndef __POSIXH4TRANSPORT_H__
#define __POSIXH4TRANSPORT_H__

#include "blueCoreTransport.h"

class PosixH4Base;

class DEVICECONTROLLER_API_CLASS PosixH4TransportBase
    : public BlueCoreTransport
{
 public:
    virtual ~PosixH4TransportBase();

    startTransportReturnValue start(void);
    bool ready(uint32 timeout);
    void stop();

    static void onPacketFromChip(uint8 *apdu, uint32 aLength,
				 void * parameters);
    void processPacketFromChip(enum BLUECORE_CHANNELS channel,
			       uint8 *apdu, uint32 aLength);

 protected:
    PosixH4TransportBase();
    void creationInit(const ichar *comPort = 0);

 private:
    void sendpdu(enum BLUECORE_CHANNELS channel, uint8 *apdu,
		 uint32 aLength, CallBack *aCallBack);

    void set_sco_bandwidth(int bytes_per_second, void *additional_info,
			   int additional_info_size);

    virtual PosixH4Base *MakeStack() = 0;

    PosixH4Base *mStack;
    ichar *mComportName;

 protected:
    uint32 mDataRate;
    int mComportHandle;

 private:

    struct CallBackInfo
    {
	PosixH4TransportBase *mObject;
	BLUECORE_CHANNELS channel;
    };

    CallBackInfo mCommandCallBackInfo;
    CallBackInfo mACLCallBackInfo;
    CallBackInfo mSCOCallBackInfo;
};

class DEVICECONTROLLER_API_CLASS PosixH4Transport
    : public PosixH4TransportBase
{
 public:
    PosixH4Transport(const ichar *comPort, uint32 dataRate);

    PosixH4Transport(int comPort);

 private:
    virtual PosixH4Base *MakeStack();
};

#endif // __POSIXH4TRANSPORT_H__
