///////////////////////////////////////////////////////////////////////
//
//  FILE   :  Win32H4Transport.h
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  Win32H4Transport
//
//  PURPOSE:  transparent class inheriting top API from the
//            BlueCoreTransport class, and owning a H4Stack which does
//            all the work.
//
//            BlueCoreTransport also provides BCSP-specific private channels
//            L2CAP etc which are tunneled through HCI vendor specific
//            commands and puts the debug channel messages to a file
//            in the debug build version.
//
//  MODIFICATION HISTORY:
//  1.5   7:Oct:00  at  Added modification history.
//                      Added intercept of send callback to delete the
//                      pdu buffer for consistency with the BCSP
//                      transport.
//  1.6   16:Oct:00 at  Restored previous callback behaviour - the
//                      memory deallocation problem was in
//                      BlueCoreTransport.
//  1.8   16:Nov:00 at  Added ability to open and close COM port
//                      externally.
//	1.10  28:Mar:01 ckl Removed stdafx.h
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/Win32/H4/Win32H4Transport.h#16 $
//
///////////////////////////////////////////////////////////////////////

#ifndef __WIN32H4TRANSPORT_H__
#define __WIN32H4TRANSPORT_H__

#include "blueCoreTransport.h"
#include "h4/Win32H4.h"

class Win32H4Transport;

typedef struct
{
    Win32H4Transport * mObject;
    BLUECORE_CHANNELS channel;
}
CallBackInfoH4;

class DEVICECONTROLLER_API_CLASS Win32H4Transport : public BlueCoreTransport
{
public:
    Win32H4Transport( ichar* comPort , uint32 dataRate );
    Win32H4Transport( HANDLE comPort );
    virtual ~Win32H4Transport();

    startTransportReturnValue start( void );
    bool ready ( uint32 timeout );
    void stop();

    static void onPacketFromChip ( uint8 * apdu , uint32 aLength , void * parameters );
    void processPacketFromChip ( enum BLUECORE_CHANNELS channel , uint8 * apdu , uint32 aLength );

private:
    void creationInit ();
    void comportPrep  ( ichar* comPort , uint32 dataRate );

    void sendpdu ( enum BLUECORE_CHANNELS channel , uint8 * apdu , uint32 aLength , CallBack * aCallBack );
	void set_sco_bandwidth(int bytes_per_second, void *additional_info, int additional_info_size);

    Win32H4 * mStack;
    ichar * mComportName;
	HANDLE mComportHandle;
    uint32 mDataRate;
    
    CallBackInfoH4 mCommandCallBackInfo;
    CallBackInfoH4 mACLCallBackInfo;
    CallBackInfoH4 mSCOCallBackInfo;
};

#endif // __WIN32H4TRANSPORT_H__
