///////////////////////////////////////////////////////////////////////
//
//  FILE   :  Win32USBTransport.h
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  Win32USBTransport
//
//  PURPOSE:  transparent class inheriting top API from the
//            HCITransport class, and owning a USBStack which does
//            all the work.
//
//            Start-up sequence:
//            new Win32USBTransport;
//            setupCallBacks (HCITransport functions);
//            start stack;
//            
//            If the stack is started with out the callbacks in place,
//            then data arriving in the intervening time will be lost.
//            Since this is likely to be the HCI initialisation packet,
//            that would be bad...
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/Win32/USB/Win32USBTransport.h#10 $
//
//  MODIFICATION HISTORY:
//  1.7   7:Sep:00  at  Added modification history.
//                      Removed the inappropriate duplication of
//                      BlueCoreTransport::recvData().
//	1.3  28:Mar:01 ckl Removed stdafx.h
//  1.9   4:Nov:02 at  Added ability to use an existing handle.
//
///////////////////////////////////////////////////////////////////////

#ifndef __WIN32USBTRANSPORT_H__
#define __WIN32USBTRANSPORT_H__

#include "blueCoreTransport.h"

class USBStack;

class DEVICECONTROLLER_API_CLASS Win32USBTransport : public BlueCoreTransport
{
public:
    Win32USBTransport( ichar* deviceName , uint32 dataRate = 0 );
    Win32USBTransport( ichar* deviceName , ichar * configString );
    Win32USBTransport( HANDLE deviceHandle , uint32 dataRate = 0 );
    virtual ~Win32USBTransport();

    static void onLinkFail(void * aObject);

    static void onEventPacketFromChip ( void * aData , unsigned long aLength , void * info );
    static void onDataPacketFromChip  ( void * aData , unsigned long aLength , void * info );
    static void onScoPacketFromChip  ( void * aData , unsigned long aLength , void * info );
    void processDataFromChip ( enum BLUECORE_CHANNELS channel , void * aData, unsigned long aLength );

    startTransportReturnValue start( void );
    bool ready ( uint32 timeout );
    void stop();

private:
    void creationInit ( ichar* comPort , uint32 dataRate , ichar * configString );

    USBStack * mStack;
    ichar * mDeviceName;
    HANDLE mDeviceHandle;
    ichar * mConfigString;
    
    void sendpdu     ( BLUECORE_CHANNELS channel, uint8 *apdu, uint32 aLength , CallBack *aCallBack ) ;
	void set_sco_bandwidth ( int bytes_per_second, void *additional_info, int additional_info_size );


    bool linkFailed;
};

typedef struct
{
    BLUECORE_CHANNELS channelNumber;
    Win32USBTransport * object;
    CallBack * callBack;
}
USBCallBackInfo;

#endif // __WIN32USBTRANSPORT_H__
