/*++

Copyright (c) 1997-1998  Microsoft Corporation

Module Name:

    CSRBC01.h

Abstract:

    Driver-defined special IOCTL's    

Environment:

    Kernel & user mode

Notes:

  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
  PURPOSE.

  Copyright (c) 1997-1998 Microsoft Corporation.  All Rights Reserved.
Revision History:

	11/17/97: created

--*/


// Module to handle USB transport.
// Copyright (c) 2000-2001 Cambridge Silicon Radio.  All Rights Reserved.


#ifndef CSRBC01H_INC
#define CSRBC01H_INC

#include "unicode/ichar.h"

#define CSRBC01_IOCTL_INDEX  0x0000


#define IOCTL_CSRBC01_SEND_HCI_COMMAND			CTL_CODE(FILE_DEVICE_UNKNOWN, \
													CSRBC01_IOCTL_INDEX, \
													METHOD_BUFFERED, \
													FILE_ANY_ACCESS)

#define IOCTL_CSRBC01_GET_HCI_EVENT				CTL_CODE(FILE_DEVICE_UNKNOWN, \
													CSRBC01_IOCTL_INDEX+1, \
													METHOD_BUFFERED, \
													FILE_ANY_ACCESS)

#define IOCTL_CSRBC01_GET_VERSION				CTL_CODE(FILE_DEVICE_UNKNOWN, \
													CSRBC01_IOCTL_INDEX+2, \
													METHOD_BUFFERED, \
													FILE_ANY_ACCESS)

#define IOCTL_CSRBC01_GET_DRIVER_NAME			CTL_CODE(FILE_DEVICE_UNKNOWN, \
													CSRBC01_IOCTL_INDEX+3, \
													METHOD_BUFFERED, \
													FILE_ANY_ACCESS)

#define IOCTL_CSRBC01_GET_CONFIG_DESCRIPTOR     CTL_CODE(FILE_DEVICE_UNKNOWN,  \
                                                   CSRBC01_IOCTL_INDEX+4,\
                                                   METHOD_BUFFERED,  \
                                                   FILE_ANY_ACCESS)
                                                   
#define IOCTL_CSRBC01_GET_DEVICE_DESCRIPTOR     CTL_CODE(FILE_DEVICE_UNKNOWN,  \
                                                   CSRBC01_IOCTL_INDEX+5,\
                                                   METHOD_BUFFERED,  \
                                                   FILE_ANY_ACCESS)
                                                   
#define IOCTL_CSRBC01_RESET_DEVICE				CTL_CODE(FILE_DEVICE_UNKNOWN,  \
                                                   CSRBC01_IOCTL_INDEX+6,\
                                                   METHOD_BUFFERED,  \
                                                   FILE_ANY_ACCESS)                                                              
                                                   
#define IOCTL_CSRBC01_RESET_PIPE				CTL_CODE(FILE_DEVICE_UNKNOWN,  \
                                                   CSRBC01_IOCTL_INDEX+7,\
                                                   METHOD_BUFFERED,  \
                                                   FILE_ANY_ACCESS)                                                           

#define IOCTL_CSRBC01_BLOCK_HCI_EVENT			CTL_CODE(FILE_DEVICE_UNKNOWN, \
													CSRBC01_IOCTL_INDEX+12, \
													METHOD_BUFFERED, \
													FILE_ANY_ACCESS)

#define IOCTL_CSRBC01_BLOCK_HCI_DATA			CTL_CODE(FILE_DEVICE_UNKNOWN, \
													CSRBC01_IOCTL_INDEX+13, \
													METHOD_BUFFERED, \
													FILE_ANY_ACCESS)

#define IOCTL_CSRBC01_SEND_CONTROL_TRANSFER		CTL_CODE(FILE_DEVICE_UNKNOWN, \
													CSRBC01_IOCTL_INDEX+14,\
													METHOD_BUFFERED, \
													FILE_ANY_ACCESS)

#define IOCTL_CSRBC01_SELECT_ALTERNATE_INTERFACE CTL_CODE(FILE_DEVICE_UNKNOWN, \
													CSRBC01_IOCTL_INDEX+15, \
													METHOD_BUFFERED, \
													FILE_ANY_ACCESS)

#define IOCTL_CSRBC01_START_SCO_DATA			CTL_CODE(FILE_DEVICE_UNKNOWN, \
													CSRBC01_IOCTL_INDEX+16, \
													METHOD_BUFFERED, \
													FILE_ANY_ACCESS)

#define IOCTL_CSRBC01_SEND_SCO_DATA				CTL_CODE(FILE_DEVICE_UNKNOWN, \
													CSRBC01_IOCTL_INDEX+17, \
													METHOD_BUFFERED, \
													FILE_ANY_ACCESS)

#define IOCTL_CSRBC01_RECV_SCO_DATA				CTL_CODE(FILE_DEVICE_UNKNOWN, \
													CSRBC01_IOCTL_INDEX+18, \
													METHOD_BUFFERED, \
													FILE_ANY_ACCESS)

class USBStack
{

private:

	HANDLE	handle;
	HANDLE	wait_for_event;
	HANDLE	wait_for_data;
	HANDLE	wait_for_acl;
	HANDLE	wait_for_recv_sco; 
	HANDLE	wait_for_send_sco; 
	HANDLE	wait_for_start_sco;

	HANDLE	wait_for_sco;
	HANDLE	wait_for_command;
	HANDLE	kill_threads_event;

	CRITICAL_SECTION close_handle_crit;

	bool monitor_events_thread_terminate;
	bool monitor_data_thread_terminate;
	bool monitor_sco_thread_terminate;
	bool monitor_sco_enabled;

	int	event_paused;
	int	data_paused;

	int	sco_packet_size;

	char *event_buffer;
	char *data_buffer;
	char *sco_buffer;

	HANDLE monitor_events_thread_handle;
	HANDLE monitor_data_thread_handle;
	HANDLE monitor_sco_thread_handle;

	void (*event_func) (void *data, unsigned long length, void *context);
	void (*data_func) (void *data, unsigned long length, void *context);
	void (*sco_func) (void *data, unsigned long length, void *context);

	void *event_func_context;
	void *data_func_context;
	void *sco_func_context;

	int USBStack::get_hci_event(void *buffer, int length);
	static DWORD WINAPI USBStack::monitor_events_thread(LPVOID arg);

	int USBStack::get_hci_data(void *buffer, int length);
	int USBStack::get_sco_data(void *buffer, unsigned long length);
	static DWORD WINAPI USBStack::monitor_data_thread(LPVOID arg);
	static DWORD WINAPI USBStack::monitor_sco_thread(LPVOID arg);

	void monitor_events(void);
	void monitor_data(void);
	void monitor_sco(void);
	void close_handle(void);
	void close_connection_on_error(void);
	void set_alternate_setting(int num, void *additonal_info, int additonal_info_size);

public:

	USBStack();
	~USBStack();

	bool open_connection(ichar *device_name);
	bool open_connection(HANDLE device_handle);
	void close_connection(void);

	bool is_open(void);

	int send_hci_command(void *data, unsigned long length);
	int send_acl_data(void *data, unsigned long length);
	int send_sco_data(void *buffer, unsigned long length);

	void set_sco_bandwidth(int bytes_per_second, void *additional_info, int additional_info_size);
	void set_event_callback(void (*func) (void *, unsigned long, void *), void *context);
	void set_data_callback(void (*func) (void *, unsigned long, void *), void *context);
	void set_sco_callback(void (*func)(void *, unsigned long, void *), void *context);
};



#endif // end, #ifndef CSRBC01H_INC

