///////////////////////////////////////////////////////////////////////
//
//  FILE   :  Win32\signalFunctions.cpp
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  PURPOSE:  Provide "wait until we get something to do"
//            platform specific functions, wrapped to be general.
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/Win32/signalFunctions.cpp#8 $
//
///////////////////////////////////////////////////////////////////////

#include "windows.h"

#ifdef _WINCE
#include <windef.h>
#include <winbase.h>
#include <dbgapi.h>
#endif

//  include the class definitions.
#include "signalBox.h"
#include "signalFunctions.h"

void * createSignal()
{
#ifdef _WINCE
    return ::CreateEvent ( (SECURITY_ATTRIBUTES *)NULL , true , false , (unsigned short *)NULL );
#else
    return ::CreateEvent ( NULL , true , false , NULL );
#endif
}

void killSignal ( void * Signal )
{
    ::CloseHandle ( Signal ) ;
}

void setSignal ( void * Signal )
{
    ::SetEvent ( Signal );
}

void unsetSignal ( void * Signal )
{
    ::ResetEvent ( Signal );
}

uint8 waitOnSignal ( void * Signal )
{
    bool continuing = true;
    uint8 returnValue = S_SIGNALLED;

    while ( continuing )
    {
        DWORD nResponse = ::WaitForSingleObject ( Signal , 1000 );

        continuing = false;
        switch (nResponse)
        {
        //  Good things...
        case WAIT_OBJECT_0:
            //  Yay...  leave the exit value as "SIGNALLED" and go.
            break;

        case WAIT_TIMEOUT:
            //  Wait more.
            continuing = true;
            break;

        //  Bad things...
        case WAIT_ABANDONED:
            //  The specified object is a mutex object that was not released
            //  by the thread that owned the mutex object before the owning
            //  thread terminated. Ownership of the mutex object is granted
            //  to the calling thread, and the mutex is set to nonsignaled.
        case WAIT_FAILED:
            //  If the function fails, the return value is WAIT_FAILED.
        default:
            //  Drop out.
            returnValue = S_ERROR;
            break;
        }
    }
    return returnValue;
}

