///////////////////////////////////////////////////////////////////////
//
//  FILE   :  Win32\transportCreatorFunctions.cpp
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  PURPOSE:  Win32 Platform specific platform specific transport
//            creators.
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/Win32/transportCreatorFunctions.cpp#18 $
//
///////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "bcsp/bcspstack.h"

#include "hci/HCIDeviceController.h"
#include "hci/Win32BCSPTransport.h"
#ifndef _WINCE
#include "hci/Win32H4Transport.h"
#include "hci/Win32USBTransport.h"
#include "hci/Win32USBWidTransport.h"
#endif

#include "transportCreatorFunctions.h"


#include "ProtocolDlg.h"
#ifdef _WINCE
#include <windef.h>
#include <winbase.h>
#include <dbgapi.h>
#define assert ASSERT
#else
#include <stdio.h>
#include <cassert>
#include <stdarg.h>

#endif

BlueCoreTransport * createBCSP ( ichar * aDeviceName , uint32 aDataRate , bool tunnelling )
{
    Win32BCSPTransport * t = new Win32BCSPTransport ( aDeviceName , aDataRate );
    t->tunnel = tunnelling;
	ASSERT(t);
    return t;
}

BlueCoreTransport * createH4   ( ichar * aDeviceName , uint32 aDataRate )
{
#ifndef _NO_H4
#ifndef _WINCE
    return new Win32H4Transport ( aDeviceName , aDataRate );
#else
	return NULL;
#endif
#else
    return NULL;
#endif
}
BlueCoreTransport * createUSB  ( ichar * aDeviceName , uint32 aDataRate )
{
#ifndef _WINCE
	if (toupper(aDeviceName[4]) == 'C')
		return new Win32USBTransport ( aDeviceName );
	else if (toupper(aDeviceName[6]) == 'W')
		return new Win32USBWidTransport ( aDeviceName );
	else
		return NULL;
#else
	return NULL;
#endif
}

//  Some platform specific defaults.
ConfigurationSettings initialSettings[3] =
{
    { BCSP , II("com1") , 115200 , true },
    { H4   , II("com1") , 115200 , false },
    { USB  , II("\\\\.\\CSR0") , 0   , false }
};

void filterSettings ( ConfigurationSettings * partialData , ConfigurationSettings * defaultData )
{
    //  IGNORE ANY PARTIAL DATA TO START WITH, IF I HAVE TIME, I'LL DO THIS PROPERLY.


    //  if the types are null, then assume the rest is rubbish too.
//    if ( ( partialData[0].type | partialData[1].type | partialData[2].type ) == 0 )
    {
        memcpy ( partialData , defaultData , 3 * sizeof ( ConfigurationSettings ) );
        for ( int i = 0 ; i < 3 ; i++ )
        {
            partialData[i].transportDevice = new ichar[istrlen(defaultData[i].transportDevice) + 1];
            istrcpy ( partialData[i].transportDevice , defaultData[i].transportDevice );
        }
        return;
    }
/*
    //  sort the ordering on the partial data to match the default.
    int partialIndex[3] = { -1, -1, -1 };
    for ( int j = 0 ; i < 3 ; i ++ )
        for ( int i = 0 ; i < 3 ; i ++ )
            if ( defaultData[j].type == partialData[i] )
                partialIndex[j] = i;
    //  Now defaultData[i].type = partialData[partialIndex[i]].type, if it is there at all.
    //  So fill in the other values:

            NEED A FRESH MIND TO DO THIS....
    int lastfree = -1;
    for ( int i = 0 ; i < 3 ; i ++ )
    {
        if ( partialIndex[i] = -1 )
            lastfree = i;
        else if ( partialIndex[i] = 0 )
            break;
    }
    if ( i == 3 )
        partialIndex[lastfree] = 0;
*/
}

/*
HWND FindConsoleHandle()
{
    static LPCTSTR temptitle = II("{98C1C303-2A9E-11d4-9FF5-006067718D04}");
    ichar title[512];
    if(GetConsoleTitle(title, sizeof(title)/sizeof(ichar)) == 0)
        return NULL;
    SetConsoleTitle(temptitle);
    HWND wnd = FindWindow(NULL, temptitle);
    SetConsoleTitle(title);
    return wnd;
}
*/

ControllerDefinition * getLocalConfiguration( bool configurable )
{
    //  Some random initial settings.
    ConfigurationSettings * CurrentSettings = new ConfigurationSettings[3];
    ConfigurationSettings * selectedConfig = CurrentSettings;
    memset ( CurrentSettings , 0 , 3 * sizeof ( ConfigurationSettings ) );

    //  Go to the registry and extract values from there.
    if ( !readLocalConfiguration ( CurrentSettings ) )
    {
        filterSettings ( CurrentSettings , initialSettings );
        createLocalConfiguration ( CurrentSettings );
    }

    //  if we allow the dialog box, then the user may cancel...
    bool cancelled = false;
    bool tunnelled = false;

    //  If configuarable, let the user change the settings.
	CProtocolDlg cdlg;
    if ( configurable && afxCurrentResourceHandle != NULL )
    {
        cdlg.setDefaults ( CurrentSettings );
        int nResponse = cdlg.DoModal();

        if ( nResponse == IDOK )
        {
            selectedConfig = cdlg.getSelectedConfiguration();
            updateLocalConfiguration ( selectedConfig );
            tunnelled = cdlg.getTunnel();
        }
		else
        {
	        if (nResponse == -1)
			{
				MessageBox(NULL, II("Failed to display factory dialog"), II("Dialog Error"), MB_OK);
			}
            cancelled = true;
        }
    }
    else
    {
        for ( int i = 0 ; i < 3 ; i++ )
            if ( CurrentSettings [i].mostRecent == true )
                selectedConfig = CurrentSettings + i ;
    }

    //  Create and return the ControllerDefinition
    ControllerDefinition * d = NULL ;
    if ( !cancelled )
    {
        if ( tunnelled )
            d = new ControllerDefinition ( selectedConfig->type ,
                                           selectedConfig->transportDevice ,
                                           selectedConfig->dataRate , true );
        else
            d = new ControllerDefinition ( selectedConfig->type ,
                                           selectedConfig->transportDevice ,
                                           selectedConfig->dataRate );
    }

    delete[] CurrentSettings;
    return d;
}

#define SOFTWARE           II("Software")
#define COMPANY            II("Cambridge Silicon Radio")
#define KIT                II("Developement Kit")
#define SETTINGS_FOR_USB   II("USB Configuration")
#define SETTINGS_FOR_BCSP  II("BCSP Configuration")
#define SETTINGS_FOR_H4    II("H4 Configuration")
#define MOST_RECENT        II("Most Recently Used")
#define TRANSPORT_DEVICE   II("Name of Device")
#define DATA_RATE          II("Data Rate for Device")





HKEY openAkey ( HKEY from , ichar * name )
{

    HKEY newkey = NULL;

    if ( ERROR_SUCCESS == ::RegOpenKeyEx ( from , name , NULL , KEY_ALL_ACCESS , &newkey ) )
        return newkey;
    else
        return NULL;
}

HKEY createOrOpenAkey ( HKEY from , ichar * name )
{


    HKEY newkey = NULL;
    unsigned long nDisposition;

    if ( ERROR_SUCCESS == ::RegCreateKeyEx ( from , name , NULL , NULL , REG_OPTION_NON_VOLATILE , KEY_ALL_ACCESS , NULL , &newkey , &nDisposition ) )
        return newkey;
    else
        return NULL;
}

HKEY sw = NULL ;
HKEY csr = NULL ;

bool getToTheCoalFace ( HKEY * kit )
{
    sw = openAkey ( HKEY_CURRENT_USER , SOFTWARE );
    if ( sw != NULL )
    {
        //  Successful
        csr = openAkey ( sw , COMPANY );
        if ( csr != NULL )
        {
            //  Been here before
            *kit = openAkey ( csr , KIT );
            if ( *kit != NULL )
            {
                return true;
            }
        }
    }
    return false;
}

void closeTheCoalFace ( HKEY kit )
{
    if ( kit )
        ::RegCloseKey ( kit );
    if ( csr )
        ::RegCloseKey ( csr );
    if ( sw )
        ::RegCloseKey ( sw );
}

void setAllMostRecent( HKEY where )
{
    //  Set all the "most recent" fields to false.
    unsigned long int lFalse = 0;
    HKEY datapoint = ::openAkey ( where , SETTINGS_FOR_BCSP );
    ::RegSetValueEx ( datapoint , MOST_RECENT , NULL , REG_DWORD , (unsigned char *)&lFalse , sizeof (lFalse) );
    ::RegCloseKey ( datapoint );
    datapoint = ::openAkey ( where , SETTINGS_FOR_H4 );
    ::RegSetValueEx ( datapoint , MOST_RECENT , NULL , REG_DWORD , (unsigned char *)&lFalse , sizeof (lFalse) );

    ::RegCloseKey ( datapoint );
    datapoint = ::openAkey ( where , SETTINGS_FOR_USB );
    ::RegSetValueEx ( datapoint , MOST_RECENT , NULL , REG_DWORD , (unsigned char *)&lFalse , sizeof (lFalse) );
    ::RegCloseKey ( datapoint );
}

void getLocalSetting (  HKEY where , bool * aMostRecent , ichar ** aTransportDevice , uint32 *aDataRate )
{
    unsigned long int lDataType;
    unsigned long int lSize = 20 ;
    unsigned char * buffer = new unsigned char [lSize];
    if ( ERROR_SUCCESS == ::RegQueryValueEx ( where , MOST_RECENT , NULL , &lDataType , buffer , &lSize ) )
    {
        if ( lDataType == REG_DWORD )
            *aMostRecent = ( *(unsigned long *)buffer != 0 );
        else
            *aMostRecent = false;
    }
    lSize = 20;
    if ( ERROR_SUCCESS == ::RegQueryValueEx ( where , TRANSPORT_DEVICE , NULL , &lDataType , buffer , &lSize ) )
    {
        if ( lDataType == REG_SZ )
        {
            *aTransportDevice = new ichar[lSize];
            istrcpy ( *aTransportDevice , (ichar*) buffer );
        }
        else
            *aTransportDevice = NULL;
    }
    if ( ERROR_SUCCESS == ::RegQueryValueEx ( where , DATA_RATE , NULL , &lDataType , buffer , &lSize ) )
    {
        if ( lDataType == REG_DWORD )
            *aDataRate = ( *(unsigned long *)buffer );
        else
            *aDataRate = 0;
    }
    delete buffer;
}

void changeLocalSetting( HKEY where , ConfigurationSettings * aSingleConfig )
{
    ichar * keyName = NULL;
    switch ( aSingleConfig->type )
    {
    case BCSP:
        keyName = SETTINGS_FOR_BCSP;
        break;
    case H4:
        keyName = SETTINGS_FOR_H4;
        break;
    case USB:
        keyName = SETTINGS_FOR_USB;
        break;
    default:
        break;
    }
    HKEY datapoint = createOrOpenAkey ( where , keyName );
    unsigned long int lValue = 0;
    lValue = aSingleConfig->mostRecent;
    ::RegSetValueEx ( datapoint , MOST_RECENT , NULL , REG_DWORD , (unsigned char *) &lValue , sizeof ( lValue ) );
    if ( aSingleConfig->type != USB )
    {
        ::RegSetValueEx ( datapoint , TRANSPORT_DEVICE , NULL , REG_SZ , (unsigned char *) aSingleConfig->transportDevice , (istrlen(aSingleConfig->transportDevice) + 1) * sizeof(ichar) );
        ::RegSetValueEx ( datapoint , DATA_RATE , NULL , REG_DWORD , (unsigned char *) &(aSingleConfig->dataRate) , sizeof ( aSingleConfig->dataRate ) );
    }
    else
        ::RegSetValueEx ( datapoint , TRANSPORT_DEVICE , NULL , REG_SZ , (unsigned char *) aSingleConfig->transportDevice , (istrlen(aSingleConfig->transportDevice) + 1) * sizeof(ichar) );
    RegCloseKey ( datapoint );
}

//  database functions.
bool readLocalConfiguration ( ConfigurationSettings * aConfigList )
{
    bool ok = true;
    HKEY writeHere = NULL;
    if ( getToTheCoalFace ( &writeHere ) )
    {
        HKEY bcsp = openAkey ( writeHere , SETTINGS_FOR_BCSP );
        if (bcsp != NULL)
        {
            getLocalSetting (  bcsp ,
                         &(aConfigList[0].mostRecent) ,
                         &(aConfigList[0].transportDevice) ,
                         &(aConfigList[0].dataRate) );
            if ( aConfigList[0].transportDevice != NULL )
                aConfigList[0].type = BCSP;
            else
                aConfigList[0].type = NONE;
        }
        else
            ok = false;
        HKEY h4 = openAkey ( writeHere , SETTINGS_FOR_H4 );
        if ( h4 != NULL )
        {
            getLocalSetting (  h4 ,
                         &(aConfigList[1].mostRecent) ,
                         &(aConfigList[1].transportDevice) ,
                         &(aConfigList[1].dataRate) );
            if ( aConfigList[1].transportDevice != NULL )
                aConfigList[1].type = H4;
            else
                aConfigList[1].type = NONE;
        }
        else
            ok = false;
        HKEY usb = openAkey ( writeHere , SETTINGS_FOR_USB );
        if ( usb != NULL )
        {
            getLocalSetting (  usb ,
                         &(aConfigList[2].mostRecent) ,
                         &(aConfigList[2].transportDevice) ,
                         &(aConfigList[2].dataRate) );
            aConfigList[2].type = USB;
            if ( aConfigList[2].transportDevice != NULL )
                aConfigList[2].type = USB;
            else
                aConfigList[2].type = NONE;
        }
        else
            ok = false;
    }
    else
        ok = false;

    closeTheCoalFace ( writeHere );
    return ok;
}

void updateLocalConfiguration ( ConfigurationSettings * aSingleConfig )
{
    aSingleConfig->mostRecent = true;
    HKEY kit;
    if ( getToTheCoalFace ( &kit ) )
    {
        setAllMostRecent( kit );
        changeLocalSetting( kit , aSingleConfig );
    }
    closeTheCoalFace ( kit );
}

//  create the defaults in the database, if this is a first time on that local space.
void createLocalConfiguration ( ConfigurationSettings * aConfigList )
{
    HKEY kit;
    sw = createOrOpenAkey ( HKEY_CURRENT_USER , SOFTWARE );
    if ( sw != NULL )
    {
        //  Successful
        csr = createOrOpenAkey ( sw , COMPANY );
        if ( csr != NULL )
        {
            //  Been here before
            kit = createOrOpenAkey ( csr , KIT );
            if ( kit != NULL )
            {
                //  insert the data
                for ( int i = 0 ; i < 3 ; i++ )
                    changeLocalSetting( kit , & aConfigList[i] );
            }
        }
    }
    ::closeTheCoalFace(kit);
}
