///////////////////////////////////////////////////////////////////////
//
//  FILE   :  ACLHandler.h
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  ACLHandler
//
//  PURPOSE:  handle the throughput of ACL data.
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/DeviceDriver/ACLHandler.h#19 $
//
///////////////////////////////////////////////////////////////////////

#ifndef __ACLHANDLER_H__
#define __ACLHANDLER_H__

#include "common/types.h"
#include "common/bool.h"
#include "common/linkage.h"

#include "handler.h"
#include "connectionList.h"
#include "pduList.h"
#include "dataBuffer.h"

#include <stdlib.h> /* size_t */

//  include the callback controlling classes
#include "sendPathway.h"
#include "callBack.h"

class CriticalSection;

#define ACTIVE_BROADCAST_HANDLE  0x0bac
#define PICONET_BROADCAST_HANDLE 0x0bec

class DEVICECONTROLLER_API_CLASS ACLHandler : public Handler
{
public:
    ACLHandler();
    ~ACLHandler();
    void initialise ( ConnectionList * aConnList );
    bool queue (uint16 aHandle, uint8 *aData , uint32 aLength , bool aPB, uint8 aBT );
    size_t flush ( uint16 aHandle );
    size_t flush ( Connection * aConn );
    void initialiseFlowControl( uint16 maxPackets , uint16 packetSize );
    uint16 getIncomingData( uint16 * aHandle , uint8 ** aDataPtr , uint8 * aPB , uint8 * aBT );
    uint16 getBufferSize();
    size_t getHowMuchInBuffer ();
    static void recvData ( uint8 * apdu , uint32 aLength , void * aThis );
    void doneSend ( Genericpdu * apdu );
    void checkCollisionsWithBroadcast ( uint16 handle );
    void disconnect ( Connection * aConn );
private:
    ConnectionList *pConnectionList;
    //  use a size_t to ensure this is a BIG number!
    size_t mTotalDataInBuffer;
    CriticalSection * protectTotalDataInBuffer;
    bool initialised;
    uint16 mSizeOfBuffer;
    uint16 mNumberOfBuffers;
    //  Attempting to move per connection: DataBuffer mOutBoundData;
    Connection * nextConnection;
    Connection * setNextConnection();
    NEXT_PACKET_INFO peekAtNext ( void );
    Genericpdu * getpdu( void );
    void   send   ( Genericpdu * apdu );
    Connection * ActiveBroadcastHandle;
    Connection * PiconetBroadcastHandle;
};

#endif // __ACLHANDLER_H__
