///////////////////////////////////////////////////////////////////////
//
//  FILE   :  DMHandler.cpp
//
//            (C) CSR 2000
//
//  AUTHOR :  Jiny Bradshaw
//
//  CLASS  :  DMHandler
//
//  PURPOSE:  handle the throughput of dm channel data
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/DeviceDriver/DMHandler.cpp#13 $
//
///////////////////////////////////////////////////////////////////////


#include "DMHandler.h"

#include "PrimConverter.h"


DMHandler::DMHandler()
  : DM_MyPrimConvert(*new PrimConverter)
{
	//initialised = false;
    //mTotalDataInBuffer = 0;
}

DMHandler::~DMHandler()
{
	delete &DM_MyPrimConvert;
}


void DMHandler::setup(BlueCoreTransport *aTransport,
                      SignalBox::Item *aOutSignal,
		      SignalBox::Item *aInSignal,
		      SignalBox::Item *aStopSignal)
{
    pTransport = aTransport;
    Handler::setup (aTransport, aOutSignal, aInSignal, aStopSignal);
	pTransport->setupRecvData(deviceMgt, (void*)this, DMPrimFromChip);
}

void DMHandler::DMPrimFromChip(uint8 *rx_prim, uint32 len, void *thisObj)
{
	((DMHandler *) thisObj)->AddPrimToIncomingQueue(rx_prim, len);
}

void DMHandler::AddPrimToIncomingQueue(uint8 *rx_prim, uint32 len)
{
	DMpdu *lpdu = new DMpdu;
	lpdu->mBuffer = rx_prim;   
	lpdu->mLength = len;

//uint16 prim_type = (uint16)(lpdu->mBuffer[0] + (lpdu->mBuffer[1] <<8));
//iprintf(II("Received on devMgt channel: prim type %02x \n"), prim_type);

	//iprintf(II("DATA: "));
	//for (int tmp=0; tmp<len; tmp++)
	//{
	//	iprintf(II(" %02x "), lpdu->mBuffer[tmp]);
	//}
	//iprintf(II(" LENGTH %d\n"), len);


	if (lpdu->mLength != 0)
	{
		// send to the chip via pdu queue serviced by checkqueues
        onPduReceipt(lpdu);
	}
}


bool DMHandler::queue(uint16 prim_type, void *prim)
{
	bool ok = true;
	
	// convert into right format
	DMpdu *lpdu = new DMpdu;

	lpdu->mBuffer = (uint8 *) DM_MyPrimConvert.conv_DM_UPRIM_T(toXAP, prim_type, prim);
	lpdu->mLength = DM_MyPrimConvert.getXAPoffset(); 

	if (lpdu->mLength != 0)
	{
		// send to the chip via pdu queue serviced by checkqueues
        mSentpdu.Add(lpdu);
		mSentSignal->set();
	}
	else
		ok = false;

    //delete prim;
    return ok;
}	


void DMHandler::send(Genericpdu *apdu)
{
	//  set up the tidying object
    SendPathway *lSendPathway = new SendPathway;
    lSendPathway->initialise(this, apdu);

    //  set up the callback.
    CallBack *lCallBack = new CallBack;
    lCallBack->initialise(lSendPathway, lSendPathway->doneSend);

	pTransport->senddeviceMgt(apdu->mBuffer, (uint16) apdu->mLength, lCallBack);
}


void DMHandler::doneSend(Genericpdu *apdu)
{
	delete apdu;
}


NEXT_PACKET_INFO DMHandler::peekAtNext(void)
{
	return mSentpdu.peekAtNext();
}


Genericpdu *DMHandler::getpdu(void)
{
	return mSentpdu.Get();
}


uint8 *DMHandler::getIncomingDMprim()
{
	// allocate a large buffer for the unpacked prim, if necessary 
	// the PrimConverter will extend this buffer
	uint8 *dm_prim = 0;

	// get the pud from the incoming queue
	DMpdu *rx_prim = 0;
	if ((rx_prim = (DMpdu *) mIncomingpdu.Get()) != NULL)
	{
		// get the prim type
		uint16 prim_type = (uint16)(rx_prim->mBuffer[0] + (rx_prim->mBuffer[1] <<8));

		// receive prim should be alloced as DM_UPRIM_T and xapPrim contains 
		// the raw data from chip
		DM_MyPrimConvert.updateXAPprim((uint8_t *) rx_prim->mBuffer, (uint32)rx_prim->mLength);

		dm_prim = new uint8[500*sizeof(uint8)];
		DM_MyPrimConvert.conv_DM_UPRIM_T(fromXAP, prim_type, (void *) dm_prim);
	}
	else
	{
		// Nothing in the queue
	}

	delete rx_prim;
	return dm_prim; 
}
