///////////////////////////////////////////////////////////////////////
//
//  FILE   :  L2CAPHandler.cpp
//
//            (C) CSR 2000
//
//  AUTHOR :  Jiny Bradshaw
//
//  CLASS  :  L2CAPHandler
//
//  PURPOSE:  handle the throughput of L2CAP channel data
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/DeviceDriver/L2CAPHandler.cpp#6 $
//
///////////////////////////////////////////////////////////////////////


#include "L2CAPHandler.h"

#include "PrimConverter.h"


L2CAPHandler::L2CAPHandler()
  : L2CAP_MyPrimConvert(*new PrimConverter)
{
	//initialised = false;
    //mTotalDataInBuffer = 0;
}

L2CAPHandler::~L2CAPHandler()
{
	delete &L2CAP_MyPrimConvert;
}


void L2CAPHandler::setup(BlueCoreTransport *aTransport,
						SignalBox::Item *aOutSignal,
						SignalBox::Item *aInSignal,
						SignalBox::Item *aStopSignal)
{
    pTransport = aTransport;
    Handler::setup (aTransport, aOutSignal, aInSignal, aStopSignal);
	pTransport->setupRecvData(l2cap, (void*)this, L2CAPPrimFromChip);
}

void L2CAPHandler::L2CAPPrimFromChip(uint8 *rx_prim, uint32 len, void *thisObj)
{
	((L2CAPHandler *) thisObj)->AddPrimToIncomingQueue(rx_prim, len);
}

void L2CAPHandler::AddPrimToIncomingQueue(uint8 *rx_prim, uint32 len)
{
	L2CAPpdu *lpdu = new L2CAPpdu;
	lpdu->mBuffer = rx_prim;   
	lpdu->mLength = len;

	//iprintf(II("DATA: "));
	//for (int tmp=0; tmp<len; tmp++)
	//{
	//	iprintf(II(" %02x "), lpdu->mBuffer[tmp]);
	//}
	//iprintf(II(" LENGTH %d\n"), len);


	if (lpdu->mLength != 0)
	{
		// send to the chip via pdu queue serviced by checkqueues
        onPduReceipt(lpdu);
	}
}


bool L2CAPHandler::queue(uint16 prim_type, void *prim)
{
	bool ok = true;
	
	// convert into right format
	L2CAPpdu *lpdu = new L2CAPpdu;

	lpdu->mBuffer = (uint8 *) L2CAP_MyPrimConvert.conv_L2CA_UPRIM_T(toXAP, prim_type, prim);
	lpdu->mLength = L2CAP_MyPrimConvert.getXAPoffset(); 

	if (lpdu->mLength != 0)
	{
		// send to the chip via pdu queue serviced by checkqueues
        mSentpdu.Add(lpdu);
		mSentSignal->set();
	}
	else
		ok = false;

    //delete prim;
    return ok;
}	


void L2CAPHandler::send(Genericpdu *apdu)
{
	//  set up the tidying object
    SendPathway *lSendPathway = new SendPathway;
    lSendPathway->initialise(this, apdu);

    //  set up the callback.
    CallBack *lCallBack = new CallBack;
    lCallBack->initialise(lSendPathway, lSendPathway->doneSend);

	pTransport->sendl2cap(apdu->mBuffer, (uint16) apdu->mLength, lCallBack);
}


void L2CAPHandler::doneSend(Genericpdu *apdu)
{
	delete apdu;
}


NEXT_PACKET_INFO L2CAPHandler::peekAtNext(void)
{
	return mSentpdu.peekAtNext();
}


Genericpdu *L2CAPHandler::getpdu(void)
{
	return mSentpdu.Get();
}


uint8 *L2CAPHandler::getIncomingL2CAPprim()
{
	// allocate a large buffer for the unpacked prim, if necessary 
	// the PrimConverter will extend this buffer
	uint8 *l2cap_prim = 0;

	// get the pud from the incoming queue
	L2CAPpdu *rx_prim = 0;
	if ((rx_prim = (L2CAPpdu *) mIncomingpdu.Get()) != NULL)
	{
		// get the prim type
		uint16 prim_type = (uint16)(rx_prim->mBuffer[0] + (rx_prim->mBuffer[1] <<8));

		// receive prim should be alloced as L2CA_UPRIM_T and xapPrim contains 
		// the raw data from chip
		L2CAP_MyPrimConvert.updateXAPprim((uint8_t *) rx_prim->mBuffer, (uint32)rx_prim->mLength);

		l2cap_prim = new uint8[500*sizeof(uint8)];
		L2CAP_MyPrimConvert.conv_L2CA_UPRIM_T(fromXAP, prim_type, (void *) l2cap_prim);
	}
	else
	{
		// Nothing in the queue
	}

	delete rx_prim;
	return l2cap_prim;
}
