///////////////////////////////////////////////////////////////////////
//
//  FILE   :  PCHandler.cpp
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  PrivateChannelHandler
//
//  PURPOSE:  handle the throughput of private channel commands
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/DeviceDriver/PCHandler.cpp#19 $
//
///////////////////////////////////////////////////////////////////////

#include "PCHandler.h"
#ifndef _WINCE
#include <cassert>
#include <stdlib.h> /* NULL */
#else
#include <windef.h>
#include <winbase.h>
#include <dbgapi.h>
#define assert ASSERT
#endif
PCHandler::PCHandler()
{
    bccmd = false;
    return;
}

PCHandler::~PCHandler()
{
    return;
}

void PCHandler::makeHostToChip( void )
{
    bccmd = true;
}

void PCHandler::setup( BlueCoreTransport * aTransport ,
                       SignalBox::Item * aOutSignal ,
					   SignalBox::Item * aInSignal ,
					   SignalBox::Item * aStopSignal )
{
    pTransport = aTransport;
    Handler::setup ( aTransport , aOutSignal , aInSignal , aStopSignal);
}


bool PCHandler::queue ( PARAMETER_ARRAY parameters )
{
    bool ok = true;

    if ( parameters[0] != 0 )
    {
        if (bccmd)
        {
            Commandpdu * lpdu = new Commandpdu;
            lpdu->mLength = bccmdGenerate( &lpdu->mBuffer , parameters );

            if (lpdu->mLength != 0)
            {
#ifdef _INDEBUG
                // release debugging
                mDebug->unstring ( II("BCCMD Sent: ") );
                mDebug->unload ( lpdu->mBuffer , lpdu->mLength );
#endif
                send ( lpdu );
            }
            else
                ok = false;
        }
        else
            //  no hq from host.
            ok = false;
    }
    else
        ok = false;

    return ok;
}

void PCHandler::recvEvent ( uint8 * apdu , uint32 aLength , void * aThis )
{
    PCHandler * lObject = (PCHandler*)aThis;

    Eventpdu * data = new Eventpdu;
    data->mBuffer = apdu;
    data->mLength = aLength;

    lObject->onPduReceipt (data);
}

PARAMETER_ARRAY PCHandler::getIncomingEvent ( )
{
    PARAMETER_ARRAY lAnswer = NULL;
    Eventpdu * lpdu = NULL;

    if ( NULL != ( lpdu = (Eventpdu *) mIncomingpdu.Get() ) )
    {
        if (bccmd)
        {
            lAnswer = bccmdAnalyse ( lpdu->mBuffer , lpdu->mLength );
        }
        else
        {
            lAnswer = hqAnalyse ( lpdu->mBuffer , lpdu->mLength );
            Commandpdu * cmd = new Commandpdu (lpdu);
#ifdef _INDEBUG
            // release debugging
            mDebug->unstring ( II(" HQ Answer: ") );
            mDebug->unloadPA ( lAnswer );
            mDebug->unstring ( II("   HQ Sent: ") );
            mDebug->unload ( cmd->mBuffer , cmd->mLength );
#endif
            this -> send ( cmd );
        }
    }
    //  the pdu was passed up in deletable memory, so delete it.
    delete lpdu;

    return lAnswer;
}

NEXT_PACKET_INFO PCHandler::peekAtNext ( void )
{
    return nothingMore;
}

Genericpdu * PCHandler::getpdu( )
{
    return NULL;
}

void PCHandler::send (Genericpdu * apdu)
{
    //  set up the tidying object
    SendPathway * lSendPathway = new SendPathway;
    lSendPathway->initialise ( this , apdu );

    //  set up the callback.
    CallBack * lCallBack = new CallBack;
    lCallBack->initialise ( lSendPathway , lSendPathway->doneSend );

    assert ( (II("pdu Length > 65535") , apdu->mLength <= 0xFFFF ) );

    if (bccmd)
        pTransport->sendbccmd ( apdu->mBuffer , (uint16) apdu->mLength , lCallBack );
    else
        pTransport->sendhq ( apdu->mBuffer , (uint16) apdu->mLength , lCallBack );
}

void PCHandler::doneSend ( Genericpdu * apdu )
{
    delete apdu;
}
