///////////////////////////////////////////////////////////////////////
//
//  FILE   :  RFCOMMHandler.cpp
//
//            (C) CSR 2000
//
//  AUTHOR :  Jiny Bradshaw
//
//  CLASS  :  RFCOMMHandler
//
//  PURPOSE:  handle the throughput of rfcomm channel data
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/DeviceDriver/RFCOMMHandler.cpp#9 $
//
///////////////////////////////////////////////////////////////////////


#include "RFCOMMHandler.h"
#include "PrimConverter.h"


RFCOMMHandler::RFCOMMHandler()
	: RFCOMM_MyPrimConvert(*new PrimConverter)
{
}

RFCOMMHandler::~RFCOMMHandler()
{
	delete &RFCOMM_MyPrimConvert;
}


void RFCOMMHandler::setup(BlueCoreTransport *aTransport,
                          SignalBox::Item *aOutSignal,
						  SignalBox::Item *aInSignal,
						  SignalBox::Item *aStopSignal)
{
    pTransport = aTransport;
    Handler::setup (aTransport, aOutSignal, aInSignal, aStopSignal);
	pTransport->setupRecvData(rfcomm, (void*)this, RFCOMMPrimFromChip);
}

void RFCOMMHandler::RFCOMMPrimFromChip(uint8 *rx_prim, uint32 len, void *thisObj)
{
	((RFCOMMHandler *) thisObj)->AddPrimToIncomingQueue(rx_prim, len);
}

void RFCOMMHandler::AddPrimToIncomingQueue(uint8 *rx_prim, uint32 len)
{
	RFCOMMpdu *lpdu = new RFCOMMpdu;
	lpdu->mBuffer = rx_prim;
	lpdu->mLength = len;

	if (lpdu->mLength != 0)
	{
		// send to the chip via pdu queue serviced by checkqueues
        onPduReceipt(lpdu);
	}
}


bool RFCOMMHandler::queue(uint16 prim_type, void *prim)
{
	bool ok = true;
	
	// convert into right format
	RFCOMMpdu *lpdu = new RFCOMMpdu;

	lpdu->mBuffer = (uint8 *) RFCOMM_MyPrimConvert.conv_RFCOMM_UPRIM_T(toXAP, prim_type, prim);
	lpdu->mLength = RFCOMM_MyPrimConvert.getXAPoffset(); 

	if (lpdu->mLength != 0)
	{
		// send to the chip via pdu queue serviced by checkqueues
        mSentpdu.Add(lpdu);
		mSentSignal->set();
	}
	else
		ok = false;

    // delete prim;
    return ok;
}	


void RFCOMMHandler::send(Genericpdu *apdu)
{
	//  set up the tidying object
    SendPathway *lSendPathway = new SendPathway;
    lSendPathway->initialise(this, apdu);

    //  set up the callback.
    CallBack *lCallBack = new CallBack;
    lCallBack->initialise(lSendPathway, lSendPathway->doneSend);

	pTransport->sendrfcomm(apdu->mBuffer, (uint16) apdu->mLength, lCallBack);
}


void RFCOMMHandler::doneSend(Genericpdu *apdu)
{
	delete apdu;
}


NEXT_PACKET_INFO RFCOMMHandler::peekAtNext(void)
{
	return mSentpdu.peekAtNext();
}


Genericpdu *RFCOMMHandler::getpdu(void)
{
	return mSentpdu.Get();
}


uint8 *RFCOMMHandler::getIncomingRFCOMMprim()
{
	// allocate a large buffer for the unpacked prim, if necessary 
	// the PrimConverter will extend this buffer
	uint8 *rfcomm_prim = 0;

	// get the pud from the incoming queue
	RFCOMMpdu *rx_prim = 0;
	if ((rx_prim = (RFCOMMpdu *) mIncomingpdu.Get()) != NULL)
	{
		// get the prim type
		uint16 prim_type = (uint16)(rx_prim->mBuffer[0] + (rx_prim->mBuffer[1] <<8));

		// receive prim should be alloced as RFCOMM_UPRIM_T and xapPrim contains 
		// the raw data from chip
		RFCOMM_MyPrimConvert.updateXAPprim((uint8_t *) rx_prim->mBuffer, (uint32)rx_prim->mLength);

		rfcomm_prim = new uint8[1000*2*sizeof(uint8)];
		RFCOMM_MyPrimConvert.conv_RFCOMM_UPRIM_T(fromXAP, prim_type, (void *) rfcomm_prim);
	}
	else
	{
		// Nothing in the queue
	}

	delete rx_prim;
	return rfcomm_prim;
}
