///////////////////////////////////////////////////////////////////////
//
//  FILE   :  SDPHandler.cpp
//
//            (C) CSR 2000
//
//  AUTHOR :  Jiny Bradshaw
//
//  CLASS  :  SDPHandler
//
//  PURPOSE:  handle the throughput of sdp channel data
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/DeviceDriver/SDPHandler.cpp#10 $
//
///////////////////////////////////////////////////////////////////////


#include "SDPHandler.h"
#include "PrimConverter.h"


SDPHandler::SDPHandler()
	: SDP_MyPrimConvert(*new PrimConverter)
{
}

SDPHandler::~SDPHandler()
{
	delete &SDP_MyPrimConvert;
}


void SDPHandler::setup(BlueCoreTransport *aTransport,
                       SignalBox::Item *aOutSignal,
					   SignalBox::Item *aInSignal,
					   SignalBox::Item *aStopSignal)
{
    pTransport = aTransport;
    Handler::setup (aTransport, aOutSignal, aInSignal, aStopSignal);
	pTransport->setupRecvData(sdp, (void*)this, SDPPrimFromChip);
}

void SDPHandler::SDPPrimFromChip(uint8 *rx_prim, uint32 len, void *thisObj)
{
	((SDPHandler *) thisObj)->AddPrimToIncomingQueue(rx_prim, len);
}

void SDPHandler::AddPrimToIncomingQueue(uint8 *rx_prim, uint32 len)
{
	SDPpdu *lpdu = new SDPpdu;
	lpdu->mBuffer = rx_prim;
	lpdu->mLength = len;

	if (lpdu->mLength != 0)
	{
		// send to the chip via pdu queue serviced by checkqueues
        onPduReceipt(lpdu);
	}
}

uint8 *SDPHandler::getIncomingSDPprim()
{
	// allocate a large buffer for the unpacked prim, if necessary 
	// the PrimConverter will extend this buffer
	uint8 *sdp_prim = 0;

	// get the pud from the incoming queue
	SDPpdu *rx_prim = 0;
	if ((rx_prim = (SDPpdu *) mIncomingpdu.Get()) != NULL)
	{
		// get the prim type
		uint16 prim_type = (uint16)(rx_prim->mBuffer[0] + (rx_prim->mBuffer[1] <<8));

		// receive prim should be alloced as SDP_UPRIM_T and xapPrim contains 
		// the raw data from chip
		SDP_MyPrimConvert.updateXAPprim((uint8_t *) rx_prim->mBuffer, (uint32)rx_prim->mLength);

		sdp_prim = new uint8[1000*sizeof(uint8)];
		SDP_MyPrimConvert.conv_SDC_UPRIM_T(fromXAP, prim_type, (void *) sdp_prim);
	}
	else
	{
		// Nothing in the queue
	}

	delete rx_prim;
	return sdp_prim; 
}


bool SDPHandler::queue(uint16 prim_type, void *prim)
{
	bool ok = true;
	
	// convert into right format
	SDPpdu *lpdu = new SDPpdu;

	lpdu->mBuffer = (uint8 *) SDP_MyPrimConvert.conv_SDC_UPRIM_T(toXAP, prim_type, prim);
	lpdu->mLength = SDP_MyPrimConvert.getXAPoffset(); 

	if (lpdu->mLength != 0)
	{
		// send to the chip via pdu queue serviced by checkqueues
        mSentpdu.Add(lpdu);
		mSentSignal->set();
	}
	else
		ok = false;

    //delete prim;
    return ok;
}	


void SDPHandler::send(Genericpdu *apdu)
{
	//  set up the tidying object
    SendPathway *lSendPathway = new SendPathway;
    lSendPathway->initialise(this, apdu);

    //  set up the callback.
    CallBack *lCallBack = new CallBack;
    lCallBack->initialise(lSendPathway, lSendPathway->doneSend);

	pTransport->sendsdp(apdu->mBuffer, (uint16) apdu->mLength, lCallBack);
}


void SDPHandler::doneSend(Genericpdu *apdu)
{
	delete apdu;
}


NEXT_PACKET_INFO SDPHandler::peekAtNext(void)
{
	return mSentpdu.peekAtNext();
}


Genericpdu *SDPHandler::getpdu(void)
{
	return mSentpdu.Get();
}
