///////////////////////////////////////////////////////////////////////
//
//  FILE   :  bc01DeviceController.cpp
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  bc01DeviceController
//
//  PURPOSE:  Provide a C++ object API.
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/DeviceDriver/bc01/bc01DeviceController.cpp#31 $
//
///////////////////////////////////////////////////////////////////////

#include "HCIDeviceController.h"
#include "bc01DeviceController.h"
#include "PCHandler.h"
#ifndef NOPRIMCONVERTER
// JB support for BlueStack
#include "DMHandler.h"
#include "RFCOMMHandler.h"
#include "SDPHandler.h"
#include "L2CAPHandler.h"
#endif
#include "DCThread.h"

BlueCoreDeviceController::BlueCoreDeviceController (BlueCoreTransport *aTransport, bool aWithBlueStack) 
: HCIDeviceController (aTransport, aWithBlueStack) ,
  mhcHandler ( *new PCHandler() ) ,
  mchHandler ( *new PCHandler() ) 
#ifndef NOPRIMCONVERTER
                                  ,
  dmHandler  ( *new DMHandler() ) ,
  rfcommHandler ( *new RFCOMMHandler() ) ,
  sdpHandler ( *new SDPHandler() ) ,
  l2capHandler ( *new L2CAPHandler() )
#endif
{
	withBlueStack = aWithBlueStack;
    creationInit ();
    if ( aTransport != NULL )
        setTransport ( aTransport );
}


void BlueCoreDeviceController::creationInit ()
{
#ifdef _INDEBUG
    // release debugging
    mhcHandler.mDebug = mDebug;
    mchHandler.mDebug = mDebug;
#endif

    mBCSP = NULL ;
    mOnTransportFailFn = NULL;
    mHostToChipCallBackFn = NULL ;
    mChipToHostCallBackFn = NULL ;
	mDebugDataCallBackFn = NULL ;

	// JB
	if (withBlueStack)
	{
#ifndef _WINCE
		mDMprimsFromChipCallBackFn = NULL;
		mRFCOMMprimsFromChipCallBackFn = NULL;
		mSDPprimsFromChipCallBackFn = NULL;
		mL2CAPprimsFromChipCallBackFn = NULL;
#endif
	}
    mhcHandler.makeHostToChip();

    //mHCSentSignal = new SignalBox::Item(mSignals); 
    mHCRecvSignal = new SignalBox::Item(mSignals);
    mCHSentSignal = new SignalBox::Item(mSignals); 
    mCHRecvSignal = new SignalBox::Item(mSignals);
	// JB
	if (withBlueStack)
	{
		mDMSentSignal = new SignalBox::Item(mSignals);
		mDMRecvSignal = new SignalBox::Item(mSignals);
		mRFCOMMSentSignal = new SignalBox::Item(mSignals);
		mRFCOMMRecvSignal = new SignalBox::Item(mSignals);
		mSDPSentSignal = new SignalBox::Item(mSignals);
		mSDPRecvSignal = new SignalBox::Item(mSignals);
		mL2CAPSentSignal = new SignalBox::Item(mSignals);
		mL2CAPRecvSignal = new SignalBox::Item(mSignals);
	}
}

BlueCoreDeviceController::~BlueCoreDeviceController( )
{
#ifdef _INDEBUG
    mDebug->unstring ( II("Deleting bc01DC\n") );
#endif
    mBeingDeletedSignal->set();

    while ( mThreadExitedSignal->getState() == false )
        mThreadCloseDown->wait();

    //delete mHCSentSignal ;
    delete mHCRecvSignal ;
    delete mCHSentSignal ;
    delete mCHRecvSignal ;
	// JB
	if (withBlueStack)
	{
		delete mDMSentSignal ;
		delete mDMRecvSignal ;
		delete mRFCOMMSentSignal ;
		delete mRFCOMMRecvSignal ;
		delete mSDPSentSignal ;
		delete mSDPRecvSignal ;
		delete mL2CAPSentSignal ;
		delete mL2CAPRecvSignal ;
	}
    delete &mhcHandler;
    delete &mchHandler;
#ifndef NOPRIMCONVERTER
    delete &dmHandler;
    delete &rfcommHandler;
    delete &sdpHandler;
    delete &l2capHandler;
#endif
}

void BlueCoreDeviceController::setTransport ( BlueCoreTransport * aTransport )
{
    //  do this once and only once.
    if ( aTransport != NULL && mBCSP == NULL )
    {
        mBCSP = aTransport;

        mBCSP->setupRecvCHEvent ( &mchHandler , mchHandler.recvEvent );
        mBCSP->setupRecvHCReply ( &mhcHandler , mhcHandler.recvEvent );

        mchHandler.setup( mBCSP , mCHSentSignal , mCHRecvSignal , mBeingDeletedSignal );
        mhcHandler.setup( mBCSP , /*mHCSentSignal*/0 , mHCRecvSignal , mBeingDeletedSignal );
	
#ifndef NOPRIMCONVERTER
		// JB
		if (withBlueStack)
		{
			dmHandler.setup(mBCSP, mDMSentSignal, mDMRecvSignal, mBeingDeletedSignal);
			rfcommHandler.setup(mBCSP, mRFCOMMSentSignal, mRFCOMMRecvSignal, mBeingDeletedSignal);
			sdpHandler.setup(mBCSP, mSDPSentSignal, mSDPRecvSignal, mBeingDeletedSignal);
			l2capHandler.setup(mBCSP, mL2CAPSentSignal, mL2CAPRecvSignal, mBeingDeletedSignal);
		}
#endif
        if ( mOnTransportFailFn != NULL )
            mBCSP->setOnLinkFail ( mOnTransportFailFn , mOnTransportFailParameters );

        if ( mDebugDataCallBackFn != NULL )
            mBCSP->setupRecvDebugData ( this , onDebugData ) ;
    }
}

BlueCoreTransport * BlueCoreDeviceController::getTransport( void )
{
    return mBCSP;
}

void BlueCoreDeviceController::setupOnTransportFailure ( void (*onLinkFailFn) ( void * ) , void * onLinkFailParameters )
{
    this->mOnTransportFailFn = onLinkFailFn;
    this->mOnTransportFailParameters = onLinkFailParameters;

    if ( mBCSP != NULL )
        mBCSP->setOnLinkFail ( onLinkFailFn , onLinkFailParameters );
}

void BlueCoreDeviceController::setupRecvHostToChipResponses ( void (*hostToChipCallBackFn) (PARAMETER_ARRAY eventParameters , void * parameters), void * hostToChipCallBackParameters )
{
    mHostToChipCallBackParameters = hostToChipCallBackParameters;
    mHostToChipCallBackFn = hostToChipCallBackFn;
}

void BlueCoreDeviceController::setupRecvChipToHostRequests ( void (*chipToHostCallBackFn) (PARAMETER_ARRAY eventParameters , void * parameters),void * chipToHostCallBackParameters = NULL )
{
    mChipToHostCallBackParameters = chipToHostCallBackParameters;
    mChipToHostCallBackFn = chipToHostCallBackFn;
}

void BlueCoreDeviceController::setupRecvDebug ( void (*DebugDataFn) ( uint8 * , uint32 , void * ) , void * DebugDataParameters )
{
    mDebugDataCallBackFn = DebugDataFn ;
    mDebugDataCallBackParameters = DebugDataParameters ;
    if ( mBCSP != NULL )
        mBCSP->setupRecvDebugData ( this , onDebugData ) ;
}


void BlueCoreDeviceController::doWork ( void )
{
    if ( mBeingDeletedSignal->getState() )
        return;

    checkQueues();
}

void BlueCoreDeviceController::checkQueues ()
{
#ifdef _INDEBUG
    mDebug->unstring ( II("Checking BC queues.\n") );
#endif

    //  Check there is any point in trying
    if ( mBCSP==NULL )
    {
#ifdef _INDEBUG
        mDebug->unstring ( II("No Transport.\n") );
#endif
        return;
    }

    bool done = false ;

    //  Check Host to Chip
    if ( mHCRecvSignal->getState() )
    {
        bool gotOne = true;
        done = mHCRecvSignal->unset();
        PARAMETER_ARRAY lAnswers;
        while (gotOne)
        {
#ifdef _INDEBUG
            mDebug->unstring ( II("Checking BCCMD\n") );
#endif
            lAnswers = mhcHandler.getIncomingEvent ();
            if ( lAnswers != NULL )
                onChipMgtHostToChipResponse( lAnswers );
            else
                gotOne = false;
        }
    }
    
    //  Check Chip To Host
    if ( !done && mCHRecvSignal->getState() )
    {
        bool gotOne = true;
        done = mCHRecvSignal->unset();
        PARAMETER_ARRAY lAnswers;
        while (gotOne)
        {
            lAnswers = mchHandler.getIncomingEvent ( );
            if ( lAnswers != NULL )
            {
                onChipMgtChipToHostRequest( lAnswers );
            }
            else
            {
                gotOne = false;
            }
        }
    }

    if ( !done )
        HCIDeviceController::checkQueues();

#ifndef NOPRIMCONVERTER
	// JB BlueStack support
	if (!done && withBlueStack)
        checkDMqueues();

	if (!done && withBlueStack)
		checkRFCOMMqueues();
	
	if (!done && withBlueStack)
		checkSDPqueues();

	if (!done && withBlueStack)
		checkL2CAPqueues();
#endif

#ifdef _INDEBUG
    mDebug->unstring ( II("Finished checking BC queues.\n") );
#endif
}

void BlueCoreDeviceController::createWorker ()
{
    if ( mThread == 0 )
    {
        mThread = new BCThread ( this );
        mThread->Start();
    }
}

void BlueCoreDeviceController::workerThread ( void * aObject )
{
    ( (BlueCoreDeviceController *) aObject )->waitForWork();
}

void BlueCoreDeviceController::waitForWork ()
{
    bool continuing = true;
    mThreadExitedSignal->unset();

    while (continuing)
    {
        mSignals->wait();

        //  Check we aren't trying to delete the object.
        if ( mBeingDeletedSignal->getState() )
        {
            continuing = false;
        }
        else
            checkQueues();

    }

    mThreadExitedSignal->set();
}

bool BlueCoreDeviceController::sendChipMgtHostToChip ( PARAMETER_ARRAY parameters )
{
    return mhcHandler.queue( parameters );
}

void BlueCoreDeviceController::onChipMgtHostToChipResponse ( PARAMETER_ARRAY parameters )
{
    if ( mHostToChipCallBackFn != NULL )
    {
#ifdef _INDEBUG
        mDebug->unstring ( II("Calling Application for BCCMD.\n") );
#endif
        mHostToChipCallBackFn (parameters , mHostToChipCallBackParameters );
    }
    else
        delete[] parameters;
}

void BlueCoreDeviceController::onChipMgtChipToHostRequest ( PARAMETER_ARRAY parameters  )
{
    if ( mChipToHostCallBackFn != NULL )
    {
#ifdef _INDEBUG
        mDebug->unstring ( II("Calling Application for HQ.\n") );
#endif
        mChipToHostCallBackFn ( parameters , mChipToHostCallBackParameters );
#ifdef _INDEBUG
        mDebug->unstring ( II("Called Application for HQ.\n") );
#endif
    }
    else
        delete[] parameters;
}

void BlueCoreDeviceController::onDebugData ( uint8* aBuffer , uint32 aLength , void * aObject )
{
    if (((BlueCoreDeviceController*)aObject)->mDebugDataCallBackFn != NULL)
        ((BlueCoreDeviceController*)aObject)->mDebugDataCallBackFn ( aBuffer , aLength , ((BlueCoreDeviceController*)aObject)->mDebugDataCallBackParameters);
}


// JB BlueStack support
void BlueCoreDeviceController::setupRecvDMPrimsFromChip(void (*DMprimsFromChipCallBackFn)(uint16 type, void *prim, void *parameters), void *DMprimChipToHostCallBackParameters=NULL)
{
	mDMprimChipToHostCallBackParameters = DMprimChipToHostCallBackParameters;
	mDMprimsFromChipCallBackFn = DMprimsFromChipCallBackFn;
}

void BlueCoreDeviceController::setupRecvRFCOMMPrimsFromChip(void (*RFCOMMprimsFromChipCallBackFn)(uint16 type, void *prim, void *parameters), void *RFCOMMprimChipToHostCallBackParameters=NULL)
{
	mRFCOMMprimChipToHostCallBackParameters = RFCOMMprimChipToHostCallBackParameters;
	mRFCOMMprimsFromChipCallBackFn = RFCOMMprimsFromChipCallBackFn;
}

void BlueCoreDeviceController::setupRecvSDPPrimsFromChip(void (*SDPprimsFromChipCallBackFn)(uint16 prim_type, void *prim, void *parameters), void *SDPprimChipToHostCallBackParameters=NULL)
{
	mSDPprimChipToHostCallBackParameters = SDPprimChipToHostCallBackParameters;
	mSDPprimsFromChipCallBackFn = SDPprimsFromChipCallBackFn;
}

void BlueCoreDeviceController::setupRecvL2CAPPrimsFromChip(void (*L2CAPprimsFromChipCallBackFn)(uint16 type, void *prim, void *parameters), void *L2CAPprimChipToHostCallBackParameters=NULL)
{
	mL2CAPprimChipToHostCallBackParameters = L2CAPprimChipToHostCallBackParameters;
	mL2CAPprimsFromChipCallBackFn = L2CAPprimsFromChipCallBackFn;
}

// send dm prim to the out queue
bool BlueCoreDeviceController::sendDMprim(uint16 prim_type, void *prim)
{
#ifndef NOPRIMCONVERTER
	if (dmHandler.queue(prim_type, prim))
	{
		return true;
	}
	else
#endif
		return false;
}

bool BlueCoreDeviceController::sendRFCOMMprim(uint16 prim_type, void *prim)
{
#ifndef NOPRIMCONVERTER
	if (rfcommHandler.queue(prim_type, prim))
	{
		return true;
	}
	else
#endif
		return false;
}

bool BlueCoreDeviceController::sendSDPprim(uint16 prim_type, void *prim)
{
#ifndef NOPRIMCONVERTER
	if (sdpHandler.queue(prim_type, prim))
	{
		return true;
	}
	else
#endif
		return false;
}

bool BlueCoreDeviceController::sendL2CAPprim(uint16 prim_type, void *prim)
{
#ifndef NOPRIMCONVERTER
	if (l2capHandler.queue(prim_type, prim))
	{
		return true;
	}
	else
#endif
		return false;
}


#ifndef NOPRIMCONVERTER
// JB check the DM queues fro stuff to or from chip
void BlueCoreDeviceController::checkDMqueues(void)
{  
    bool done = false ;

	//  Check for DM prims to send to chip
    if (!done && mDMSentSignal->getState())
    {
        done = mDMSentSignal->unset();
        while (dmHandler.trySend());
    }

    //  Check Chip to Host
	if (mDMRecvSignal->getState())
    {
        bool gotOne = true;
		uint8 *next_prim;

        done = mDMRecvSignal->unset();
        while (gotOne)
        {
            next_prim = dmHandler.getIncomingDMprim();
            if (next_prim != NULL)
			{
				if (mDMprimsFromChipCallBackFn != NULL)
				{
					uint16 *prim_type = (uint16 *)next_prim;
					mDMprimsFromChipCallBackFn(*prim_type, next_prim, mDMprimChipToHostCallBackParameters);
					delete [] next_prim;
				}
				else
					delete [] next_prim;
			}
            else
                gotOne = false;
        }
    }
}


void BlueCoreDeviceController::checkRFCOMMqueues(void)
{
    bool done = false ;

	//  Check for RFCOMM prims to send to chip
    if (!done && mRFCOMMSentSignal->getState())
    {
        done = mRFCOMMSentSignal->unset();
        while (rfcommHandler.trySend());
    }

    //  Check Chip to Host
	if (mRFCOMMRecvSignal->getState())
    {
        bool gotOne = true;
		uint8 *next_prim;

        done = mRFCOMMRecvSignal->unset();
        while (gotOne)
        {
            next_prim = rfcommHandler.getIncomingRFCOMMprim();
            if (next_prim != NULL)
			{
				if (mRFCOMMprimsFromChipCallBackFn != NULL)
				{
					uint16 *prim_type = (uint16 *)next_prim;
					mRFCOMMprimsFromChipCallBackFn(*prim_type, next_prim, mRFCOMMprimChipToHostCallBackParameters);
					delete [] next_prim;
				}
				else
					delete [] next_prim;
			}
            else
                gotOne = false;
        }
    }
}


void BlueCoreDeviceController::checkSDPqueues(void)
{  
    bool done = false ;

	//  Check for SDP prims to send to chip
    if (!done && mSDPSentSignal->getState())
    {
        done = mSDPSentSignal->unset();
        while (sdpHandler.trySend());
    }

    //  Check Chip to Host
	if (mSDPRecvSignal->getState())
    {
        bool gotOne = true;
		uint8 *next_prim;
		
        done = mSDPRecvSignal->unset();
        while (gotOne)
        {
            next_prim = sdpHandler.getIncomingSDPprim();
            if (next_prim != NULL)
			{
				if (mSDPprimsFromChipCallBackFn != NULL)
				{
					uint16 *prim_type = (uint16 *)next_prim;
					mSDPprimsFromChipCallBackFn(*prim_type, next_prim, mSDPprimChipToHostCallBackParameters);					
					delete [] next_prim;
				}
				else
					delete [] next_prim; 
			}
            else
                gotOne = false;
        }
    }
}


void BlueCoreDeviceController::checkL2CAPqueues(void)
{  
    bool done = false ;

	//  Check for L2CAP prims to send to chip
    if (!done && mL2CAPSentSignal->getState())
    {
        done = mL2CAPSentSignal->unset();
        while (l2capHandler.trySend());
    }

    //  Check Chip to Host
	if (mL2CAPRecvSignal->getState())
    {
        bool gotOne = true;
		uint8 *next_prim;
        
        done = mL2CAPRecvSignal->unset();
		while (gotOne)
        {
            next_prim = l2capHandler.getIncomingL2CAPprim();
            if (next_prim != NULL)
			{
				if (mL2CAPprimsFromChipCallBackFn != NULL)
				{
					uint16 *prim_type = (uint16 *)next_prim;
					mL2CAPprimsFromChipCallBackFn(*prim_type, next_prim, mL2CAPprimChipToHostCallBackParameters);					
					delete [] next_prim;
				}
				else
					delete [] next_prim;
			}
            else
                gotOne = false;
        }
    }
}
#endif
