///////////////////////////////////////////////////////////////////////
//
//  FILE   :  commandHandler.cpp
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  CommandHandler
//
//  PURPOSE:  handle sending HCI commands and receiving.events.
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/DeviceDriver/commandHandler.cpp#27 $
//
///////////////////////////////////////////////////////////////////////
#ifdef _WINCE
#include <windef.h>
#include <winbase.h>
#include <dbgapi.h>
#define assert ASSERT
#else
#include <assert.h>
#endif

#include "commandHandler.h"
#include "HCIPacker.h"
#include "connection.h"
#include "ACLHandler.h"
#include "hci_def.h"

CommandHandler::CommandHandler()
{
    //  begin with the flow control one command.
    //  This may break H4, but USB and BCSP will survive.
    mFlowControl.setFree(1);
    return;
}

CommandHandler::~CommandHandler()
{
    return;
}

bool CommandHandler::queueCommand ( PARAMETER_ARRAY aParameters , bool aInternallyGeneratedCommand)
{
    if ( aParameters[0] == 0 )
        return false;

    //  New Command
    Commandpdu * cmd = new Commandpdu ;
    cmd->HCIopCode = (uint16) aParameters[1] ;

    if ( cmd->HCIopCode == HCI_LINK_KEY_REQUEST_REPLY ||
         cmd->HCIopCode == HCI_LINK_KEY_REQUEST_NEGATIVE_REPLY ||
         cmd->HCIopCode == HCI_PIN_CODE_REQUEST_REPLY ||
         cmd->HCIopCode == HCI_PIN_CODE_REQUEST_NEGATIVE_REPLY ||
         cmd->HCIopCode == HCI_RESET ||
         cmd->HCIopCode == HCI_HOST_NUM_COMPLETED_PACKETS ||
         cmd->HCIopCode == HCI_TUNNEL_COMMAND )
        cmd->flowControlled = false;

    cmd->internallyGeneratedCommand = aInternallyGeneratedCommand;
    cmd->mLength = generateHCICommand ( &cmd->mBuffer , aParameters ) ;
    if (cmd->mLength != 0)
    {
        mCommandQueue.Add(cmd);
        mSentSignal->set();
        return true;
    }
    else
        return false;
}

void CommandHandler::recvEvent ( uint8 *apdu , uint32 aLength , void * aThis )
{
    CommandHandler * lObject = (CommandHandler*)aThis;

    Eventpdu * event = new Eventpdu;
    event->HCIeventCode = apdu[0];
    event->mBuffer = apdu;
    event->mLength = aLength;

    lObject->onPduReceipt (event);
}

PARAMETER_ARRAY CommandHandler::getEvent ()
{
    PARAMETER_ARRAY parameterPtr = NULL ;
    Eventpdu * lpdu = NULL;

    lpdu = (Eventpdu *) mIncomingpdu.Get();
    if ( lpdu != NULL )
    {
        parameterPtr = analyseHCIEvent( lpdu->mBuffer , lpdu->mLength );
        //  the pdu was passed up in deletable memory, so delete it.
        delete lpdu;
    }
    return parameterPtr;
}


bool CommandHandler::matchCommandToEvent( uint16 opCode )
{
    //  returns true if you are supposed to pass the event back up to the top layer.
    //  if there is no matching command it returns it to the top layer...
    //  maybe they can figure out where it came from there...
    bool toSendOrNotToSend = true;
    pduList holding;
    Commandpdu * lpdu = NULL;
    while (lpdu = (Commandpdu *) mSentpdu.Get() )
    {
        if (lpdu->HCIopCode == opCode)
            break;
        holding.Add(lpdu);
    }

    if (lpdu)
    {
        toSendOrNotToSend = !(lpdu->internallyGeneratedCommand);
        delete lpdu;
    }

    while (lpdu = (Commandpdu *) holding.Get() )
    {
        mSentpdu.Add( lpdu );
    }

    return toSendOrNotToSend;
}

NEXT_PACKET_INFO CommandHandler::peekAtNext ( void )
{
    return mCommandQueue.peekAtNext();
}

Genericpdu * CommandHandler::getpdu( )
{
    return mCommandQueue.Get();
}

void CommandHandler::send (Genericpdu * apdu)
{
    //  set up the tidying object
    SendPathway * lSendPathway = new SendPathway;
    lSendPathway->initialise ( this , apdu );

    //  set up the callback.
    CallBack * lCallBack = new CallBack;
    lCallBack->initialise ( lSendPathway , lSendPathway->doneSend );

    assert ( ( II("packet too big"), apdu->mLength < 65536 ) );
    pTransport->sendCommand( apdu->mBuffer , (uint16)apdu->mLength , lCallBack );
}

void CommandHandler::doneSend ( Genericpdu * apdu )
{
    if ( !mStopSignal->getState() && apdu->flowControlled )
    {
        mSentpdu.Add ( apdu ) ;
    }
    else
    {
        delete apdu;
    }
}
