///////////////////////////////////////////////////////////////////////
//
//  FILE   :  connection.cpp
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  Connection
//
//  PURPOSE:  To hold the relevant information about an
//            HCI connection handle.
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/DeviceDriver/connection.cpp#16 $
//
///////////////////////////////////////////////////////////////////////

#include "connection.h"

Connection::Connection()
{
    cl = 0;
    connectionHandle = 0;
    connectionType = conn_type_unspecified;
    incompleteData = 0;
    outstandingPackets = 0;
}

Connection::~Connection()
{
    //  clear the connectionList entry.  If it can't find the entry
    //  it doesn't really matter...
    (void) cl->removeConnection ( connectionHandle );
}

bool Connection::initialise(uint16 handle, CONNECTION_TYPE type,
		       ConnectionList *clPointer,
//		       DeviceList *dlPointer,
		       bdaddr *addr
		       )
{
    connectionHandle = handle;
    connectionType = type;
    cl = clPointer;

    //  Add the conncetionList and device list entries.
    return cl->addConnection ( handle , this );
}

uint16 Connection::getHandle ()
{
    return connectionHandle;
}


ichar *Connection::getDeviceName ()
{
    return NULL; // cl->mDeviceList.getDeviceName (device);
}

CONNECTION_TYPE Connection::getType ()
{
    return connectionType;
}

uint16 Connection::getOutstandingPackets ( void )
{
    return outstandingPackets;
}

void Connection::addOutstandingPackets ( uint16 aOutstandingPackets )
{
    outstandingPackets += aOutstandingPackets;
}

void Connection::reduceOutstandingPackets ( uint16 aOutstandingPackets )
{
    outstandingPackets -= aOutstandingPackets;
}

size_t Connection::getIncompleteData ( void )
{
    return incompleteData;
}

void Connection::addIncompleteData ( size_t aData )
{
    incompleteData += aData;
}

void Connection::reduceIncompleteData ( size_t aData )
{
    incompleteData -= aData;
}

void Connection::add ( const uint8 *aDataPacket , uint32 aPacketLength , bool aPB , uint8 aBT )
{
    mOutBoundData.add( connectionHandle , aDataPacket , aPacketLength , aPB , aBT );
    addIncompleteData ( aPacketLength );
}

size_t Connection::flushData ()
{
    size_t threwOut = mOutBoundData.flush();
    reduceIncompleteData ( threwOut );
    return threwOut;
}

Datapdu * Connection::getpdu ()
{
    Datapdu *lpdu = mOutBoundData.get();
    reduceIncompleteData ( lpdu->mLength - ( connectionType == conn_type_acl ? HCI_DATA_HEADER_SIZE : 3 ) );
    return lpdu;
}

NEXT_PACKET_INFO Connection::peekAtNext ()
{
    return mOutBoundData.peekAtNext();
}

void Connection::setBufferSize ( uint16 bufferSize )
{
    mOutBoundData.initialise( bufferSize , connectionType );
}