///////////////////////////////////////////////////////////////////////
//
//  FILE   :  connection.h
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  Connection
//
//  PURPOSE:  To hold the relevant information about an
//            HCI connection handle.
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/DeviceDriver/connection.h#12 $
//
///////////////////////////////////////////////////////////////////////

#ifndef __CONNECTION_H__
#define __CONNECTION_H__

#include "common/types.h"
#include "common/bool.h"
#include "common/linkage.h"
#include "unicode/ichar.h"

#include "driver_types.h"
#include "connectionList.h"
#include "dataBuffer.h"

#include <stdlib.h> /* size_t */

class DEVICECONTROLLER_API_CLASS Connection
{
public:
    Connection ();
    ~Connection ();
    bool initialise (uint16 handle,
                     CONNECTION_TYPE type,
		     ConnectionList *clPointer,
		     bdaddr *addr);
    uint16 getHandle ();
    void setBufferSize ( uint16 buffersize );
    ichar * getDeviceName ();
    CONNECTION_TYPE getType ();
    uint16 getOutstandingPackets ( void );
    size_t getIncompleteData ( void );
    void add ( const uint8 *aDataPacket , uint32 aPacketLength , bool aPB , uint8 aBT );
    size_t flushData ();
    Datapdu * getpdu ();
    NEXT_PACKET_INFO peekAtNext ();
    void addOutstandingPackets ( uint16 aOutstandingPackets );
    void reduceOutstandingPackets ( uint16 aOutstandingPackets );
    void addIncompleteData ( size_t aData );
    void reduceIncompleteData ( size_t aData );

private:
    uint16 connectionHandle;
    CONNECTION_TYPE connectionType;
    uint16 outstandingPackets;
    size_t incompleteData;
    DataBuffer mOutBoundData;
    ConnectionList *cl;
};

#endif  // once only
