///////////////////////////////////////////////////////////////////////
//
//  FILE   :  connectionList.h
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  ConnectionList
//
//  PURPOSE:  A "secure" array of connection handles and pointers
//            to connections.
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/DeviceDriver/connectionList.h#9 $
//
///////////////////////////////////////////////////////////////////////

#ifndef __CONNECTIONLIST_H__
#define __CONNECTIONLIST_H__

#include "common/types.h"
#include "common/bool.h"
#include "common/linkage.h"
#include "HCI/HCILinkage.h"

class Connection;
class CriticalSection;

typedef struct _ConnectionListEntry
{
    uint16 handle;
    Connection *connection;
    struct _ConnectionListEntry *next;
}
ConnectionListEntry;

class DEVICECONTROLLER_API_CLASS ConnectionList
{
public:
    ConnectionList();
    ~ConnectionList();
    void useBufferSizes ( uint16 aclSize , uint8 scoSize , uint16 aclMax , uint16 scoMax );
    bool addConnection (uint16 connectionHandle, Connection *cpointer);
    void removeConnection (uint16 connectionHandle);
    bool validateConnection (uint16 connectionHandle);
    Connection * getConnection (uint16 connectionHandle);
    //  Keep track of which connection should be next
    Connection * getNextConnectionWithACL ();
    void setNextConnectionWithACL ( Connection * cpointer );
    // The device list servers no purpose, and is buggy. lets not use it for the moment.
//    DeviceList mDeviceList;

private:
    ConnectionListEntry *Start;
    ConnectionListEntry *find(uint16 connectionHandle);
    ConnectionListEntry *circle ( ConnectionListEntry * cpointer );
    ConnectionListEntry *nextACLConnection;
    uint16 connectionCount;
    uint16 aclMaxTokens;
    uint16 scoMaxTokens;
    uint16 aclBufferSize;
    uint8  scoBufferSize;


    CriticalSection * protection;
};

#endif // __CONNECTIONLIST_H__
