///////////////////////////////////////////////////////////////////////
//
//  FILE   :  dataBuffer.h
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  DataBuffer
//
//  PURPOSE:  Hold data for connections.  Copy data into the buffer.
//            Pass it out as a reference.
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/DeviceDriver/dataBuffer.h#12 $
//
///////////////////////////////////////////////////////////////////////

#ifndef __DATABUFFER_H__
#define __DATABUFFER_H__

#include "common/types.h"
#include "common/bool.h"
#include "common/linkage.h"

#include "localTypes.h"
#include "driver_types.h"

#define DATA_HEADER_SIZE (4)
#define VOICE_HEADER_SIZE (3)
#define INITIAL_BUFFER_SIZE (4096)
#define MAX_BUFFER_SIZE (65536)

class CriticalSection;

typedef struct _dbListElement
{
    struct _dbListElement *next;
    Datapdu * pdu;
}
dbListElement;

class DEVICECONTROLLER_API_CLASS DataBuffer
{
public:
    DataBuffer();
    ~DataBuffer();
    void initialise ( uint16 packetSize, CONNECTION_TYPE type );
    bool   add ( uint16 connectionHandle, const uint8 *data , uint32 length , bool packetBoundary , uint8 broadcastType );
    size_t flush ();
    NEXT_PACKET_INFO peekAtNext ( void );
    Datapdu *get ( void );

private:
//  Stuff to make the streaming circular buffer work!
//  uint16 bufferSize;
//  uint8 *buffer, *addPoint, *exposePoint, *top, *removePoint;
//  uint16 dataLength;

    CONNECTION_TYPE dataPacketType;
    uint16 dataPacketSize;
    uint8 dataHeaderSize;
    CriticalSection *dataProtection;

//  stuff to make the linked list work.
    dbListElement *Start;
    dbListElement *End;
};

#endif // __DATABUFFER_H__
