///////////////////////////////////////////////////////////////////////
//
//  FILE   :  flowControl.cpp
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  FlowControl
//
//  PURPOSE:  count them in and count them out...
//            you may not unless there are some spaces.
//            Call mayI() to ask if you can start a process
//            which there can be a fixed number of at most.
//            When the process is complete, clear (n).  You may
//            clear several processes at once (if they all complete
//            at the same time).
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/DeviceDriver/flowControl.cpp#6 $
//
///////////////////////////////////////////////////////////////////////

#include "flowControl.h"
#include "thread/critical_section.h"

FlowControl::FlowControl() : protection(new CriticalSection())
{
    countOut = 0;
    countTokens = 0;
}

FlowControl::~FlowControl()
{
    delete protection;
}

bool FlowControl::mayI()
{
    //  if there is no flow control, then you can't...
    //  so, check there is space...
    bool status = false;

    // Enter Critical Section
    CriticalSection::Lock here (protection);

    if ( countOut != countTokens )
    {
        countOut ++;
	status = true;
    }

#ifdef BREAK_ALL_FLOW_CONTROL
    return true;
#else
    return status;
#endif
}

void FlowControl::clear(uint16 n)
{
    //  Give us n more tokens

    // Enter Critical Section
    CriticalSection::Lock here (protection);

    countTokens += n ;
}

void FlowControl::setFree(uint16 n)
{
    //  Ensure we have exactly n free tokens.

    // Enter Critical Section
    CriticalSection::Lock here (protection);

    countTokens = countOut + n;
}

