///////////////////////////////////////////////////////////////////////
//
//  FILE   :  flowControl.h
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  FlowControl
//
//  PURPOSE:  count them in and count them out...
//            you may not unless there are some spaces.
//            Call mayI() to ask if you can start a process
//            which there can be a fixed number of at most.
//            When the process is complete, clear (n).  You may
//            clear several processes at once (if they all complete
//            at the same time).
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/DeviceDriver/flowControl.h#7 $
//
///////////////////////////////////////////////////////////////////////

#ifndef __FLOWCONTROL_H__
#define __FLOWCONTROL_H__

#include "common/types.h"
#include "common/linkage.h"
#include "common/bool.h"
#include "HCI/HCILinkage.h"

class CriticalSection;

class DEVICECONTROLLER_API_CLASS FlowControl
{
private:
    //  Two variables which will eventually wrap.  If they are equal, then flow stops.
    //  countOut is only incremented when it is less than countTokens.
    //  countTokens can be incremented freely.
    uint16 countOut;
    uint16 countTokens;

    CriticalSection* protection;

public:
    FlowControl();
    ~FlowControl();

    //  Ask nicely if you want to do something which is controlled.
    //  If you can, it returns true and increments countOut.
    //  If you can't it returns false.
    bool mayI();

    //  Gives n extra tokens, but incrementing countTokens by n.
    void clear(uint16 n);

    //  Given exactly n tokens (setting countTokens = countOut + n).
    void setFree(uint16 n);
};

#endif // once only
