///////////////////////////////////////////////////////////////////////
//
//  FILE   :  handler.cpp
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  Handler
//
//  PURPOSE:  Base class for ACL, command and SCO handlers.
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/DeviceDriver/handler.cpp#16 $
//
///////////////////////////////////////////////////////////////////////

#include "handler.h"

Handler::~Handler()
{
}

void Handler::setup ( HCITransport *aTransport ,
                      SignalBox::Item * aOutSignal,
		      SignalBox::Item * aInSignal ,
                      SignalBox::Item * aStopEveryThing )
{
    pTransport = aTransport;
    transportReady = false;
    mSentSignal = aOutSignal;
    mRecvSignal = aInSignal;
    mStopSignal = aStopEveryThing;
}

bool Handler::trySend()
{
    //  is it worth even trying?
    bool ok = !mStopSignal->getState();
    if ( ok && !transportReady)
        ok = transportReady = pTransport->ready(0);

    
    if ( ok )
    {
		NEXT_PACKET_INFO next = peekAtNext();

        if ( next != nothingMore )
        {
			if ( next == flowControlled )
			{
				ok = mFlowControl.mayI();
			}

            if ( ok )
            {
                Genericpdu * pdu = getpdu();
                send ( pdu );
            }
        }
        else
			ok = false;
    }

    return ok;
}

void Handler::onPduReceipt ( Genericpdu * apdu )
{
    if ( !mStopSignal->getState() )
    {
		mIncomingpdu.Add( apdu ) ;
#ifdef _INDEBUG
        // release debugging
        mDebug->unstring ( II("  Received: ") );
        mDebug->unload ( apdu->mBuffer , apdu->mLength );
#endif

        mRecvSignal->set();
    }
}

