///////////////////////////////////////////////////////////////////////
//
//  FILE   :  pduList.cpp
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  pduList
//
//  PURPOSE:  Hold an ordered list of pdus
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/DeviceDriver/pduList.cpp#8 $
//
///////////////////////////////////////////////////////////////////////

#include "pduList.h"
#include "thread/critical_section.h"

pduList::pduList() : mDataProtection(new CriticalSection)
{
    mStart = 0;
}

pduList::~pduList()
{
    while (mStart)
    {
	pduListEntry *tbd = mStart;
	mStart = mStart->next;
	delete tbd->pdu;
	delete tbd;
    }

    delete mDataProtection;
}

void pduList::Add ( pduListEntry * node )
{
    node->next = 0;

    CriticalSection::Lock here (mDataProtection);

    if (mStart)
       mEnd->next = node;
    else
       mStart = node;
    mEnd = node;
}

void pduList::Add ( Genericpdu *apdu )
{
    pduListEntry *node = new pduListEntry;
    node->pdu = apdu;
    Add (node);
}

NEXT_PACKET_INFO pduList::peekAtNext ()
{
    NEXT_PACKET_INFO cheat = nothingMore;

    CriticalSection::Lock here (mDataProtection);

    if ( mStart != 0 )
        cheat = ( ( mStart->pdu->flowControlled ) ? flowControlled : notFlowControlled );

    return cheat;
}

Genericpdu *pduList::Get ()
{
    pduListEntry *data;
    Genericpdu * pdu = 0;

    {
        CriticalSection::Lock here (mDataProtection);

	data = mStart;
	if (data)
	    mStart = data->next;
    }

    if (data)
    {
        pdu = data->pdu;
        delete data;
    }

    return pdu;
}







