///////////////////////////////////////////////////////////////////////
//
//  FILE   :  HCIFactory.h
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  ControllerFactory,
//            HCIControllerFactory,
//            BlueCoreControllerFactory,
//            ControllerDefinition
//
//  PURPOSE:  A set of classes to create and initialise a Host Side HCI/Serial
//            stack.
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/Factory/HCIFactory.h#10 $
//
///////////////////////////////////////////////////////////////////////

#ifndef __HCIFACTORY_H__
#define __HCIFACTORY_H__

#include "common/types.h"
#include "unicode/ichar.h"
#include "FactoryLinkage.h"


//  dummy define for the significant classes

class HCIDeviceController ;
class BlueCoreDeviceController ;
class BlueCoreTransport ;

//  enumerate the variations available
typedef enum TransportType
{
    //  Bluetooth standard protocols:
    USB = 2,  // H2 unsupported yet
    //  H3 ,  unsupported.
    H4 = 4,
    //  CSR proprietary protocol:
    BCSP = 1,
    //  a null value which is not allowed.
    NONE = 0
};

//  the set of last used configurations, to be retrieved from a local database.
//  The database holds one struct per transport type, and when ever they are used,
//  all the configurations are loaded, and the one used is updated with its current
//  settings and the "mostRecent" flag.  All the others are updated with only a false
//  "mostRecent" flag.
typedef struct 
{
    TransportType type;
    ichar * transportDevice;
    uint32 dataRate;
    bool mostRecent;
}
ConfigurationSettings;

//  Put the definition of a controller interface in a class...
//  for no good reason but OO.

class FACTORY_API_CLASS ControllerDefinition
{
public:
    ControllerDefinition ( TransportType t , ichar * TransportDevice , uint32 DataRate , bool Tunnel = false );
    virtual ~ControllerDefinition();
private:
    const TransportType mType;
    ichar * const mTransportDevice;
    const uint32 mDataRate;
    bool mTunnel;

    friend class ControllerFactory;
    friend class HCIControllerFactory;
    friend class BlueCoreControllerFactory;
};

//
//  Initially we are going to make it that a factory can probuce only one kind of
//  controller.  We need to be able to pass the factory around so we can make more of them,
//  all the same...  ie, reset.
//

class FACTORY_API_CLASS ControllerFactory
{
public:
    ControllerFactory ( bool configurable );
    ControllerFactory ( ControllerDefinition * mDefinition );
    virtual ~ControllerFactory ();

    bool ready ();
    ichar * getDeviceName ();
protected:
    ControllerDefinition * mDefinition;
    BlueCoreTransport * getTransport ();
};

class FACTORY_API_CLASS HCIControllerFactory : public ControllerFactory
{
public:
    HCIControllerFactory ( bool configurable );
    HCIControllerFactory ( ControllerDefinition * mDefinition );
    virtual ~HCIControllerFactory ();
    HCIDeviceController * spawn();
};

class FACTORY_API_CLASS BlueCoreControllerFactory : public ControllerFactory
{
public:
    BlueCoreControllerFactory ( bool configurable );
    BlueCoreControllerFactory ( ControllerDefinition * mDefinition );
    virtual ~BlueCoreControllerFactory ();
    BlueCoreDeviceController * spawn( bool BlueStack = false );
};




#endif//__HCIFACTORY_H__
