///////////////////////////////////////////////////////////////////////
//
//  FILE   :  transportCreatorFunctions.h
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  PURPOSE:  Generic header file to platform specific transport
//            creators.
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/Factory/transportCreatorFunctions.h#7 $
//
///////////////////////////////////////////////////////////////////////

#include "BlueCoreTransport.h"
#include "HCIFactory.h"

//  Create the transport layer objects
BlueCoreTransport * createBCSP ( ichar * aDeviceName , uint32 aDataRate , bool tunneling = false );
BlueCoreTransport * createH4   ( ichar * aDeviceName , uint32 aDataRate );
BlueCoreTransport * createUSB  ( ichar * aDeviceName , uint32 aDataRate );

//  Create the configuration.  On each platform there will be a database
//  and a method for overriding the database settings if the
//  "configurable" flag is true.
ControllerDefinition * getLocalConfiguration( bool configurable );

/////////////////////////////////////////////////////////////////////
//  database functions.
/////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////
//  readLocalConfiguration
//
//  Pass it a pointer to a bloack of memory which it fills in from
//  the database (atm it requires 3 configuration settings)
//
//  Returns true if it finds some values, false if it doesn't find
//  anything.
/////////////////////////////////////////////////////////////////////
bool readLocalConfiguration ( ConfigurationSettings * aConfigList );

/////////////////////////////////////////////////////////////////////
//  updateLocalConfiguration
//
//  pass it a pointer to a single configuration setting, which it
//  uses to update the local database.
//
//  Returns nothing
/////////////////////////////////////////////////////////////////////
void updateLocalConfiguration ( ConfigurationSettings * aSingleConfig );

/////////////////////////////////////////////////////////////////////
//  create the defaults in the database, if this is a first time on
//  this local space.
//  pass it a pointer to 3 configs.
/////////////////////////////////////////////////////////////////////
void createLocalConfiguration ( ConfigurationSettings * aConfigList );
