/****************************************************************************
**
**  File: BCPack_Tables.h
**
**            (C) CSR 2000
**
**  Purpose:  Define the protocol for and the data in the tables necessary
**            to run the bluecore Private channel packet generation and
**            analysis.
**
**  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/Packer/BCPacker/BCPack_tables.h#22 $
**
****************************************************************************/

/****************************************************************************
**  Modification history
***************************************************************************** 
**  1.1   17:may:00	ajh	Created.
**  #22   01:mar:04     pws     B-2096: Support BCCMDVARID_REMOTE_SNIFF_LIMITS
*/

#ifndef	__BCPACK_TABLES_H__
#define	__BCPACK_TABLES_H__

/*
**  Define the formatting for reading and writing parameters to
**  buffers.  This is in the "format" element of the tables.
**
**  0 = null filler for ends.
**  1 = byte
**  2 = uint16
**  3 = uint24 (which we store in 4 bytes)
**  4 = uint32
**  5 = uint32 in bluecore friendly format.  used in bccmd stuff only.
**  6 = six byte BD_ADDR
**  7 = 16 byte link keys and PIN codes
**  8 = 8 byte event mask.
**  9 = A two byte length field for use with the following "B" array marker.
**  A = end the analysis here, and send a pointer to the rest of the
**      buffer, and its length. (or the next two parameters are a buffer and its length...)
**  B = Use the last number as the length of an array of the
**      data structure following the B (zero ended).
**  C = command parameters follow, use the last parameter as the
**      command op code to process command complete return parameters.
**  D = Event Filter type.  This is something of a git.  See the Spec for details.
**  E = 2 byte dummy variable.
*/

/*
**  Populate the chipmanagement H/HC (bccmd) table with all the types of command.
*/

#define BCCMD_TBL_SIZE (23)
PACKER_PDU_TBL bccmd_tbl[BCCMD_TBL_SIZE] =
{
    /* {varid,format}, */
    {BCCMDVARID_COMMANDS_START, 0x00000000},
    {BCCMDVARID_UINT8_START,     0x00000001},
    {BCCMDVARID_UINT16_START,    0x00000002},
    {BCCMDVARID_UINT32_START,    0x00000005},
    /* Specials */
    {BCCMDVARID_MEMORY,         0x0002BE92},
    {BCCMDVARID_BUFFER,         0x0002B922},
    {BCCMDVARID_PS,             0x0002B292},
    {BCCMDVARID_RADIOTEST,      0x00002222},
    {BCCMDVARID_PSSIZE,         0x00000222},
    {BCCMDVARID_PSNEXT,         0x00000222},
    {BCCMDVARID_ADCRES,         0x00000222},
    {BCCMDVARID_CRYPTKEYLEN,    0x00000022},
    {BCCMDVARID_PICONET_INSTANT,0x00000052},
    {BCCMDVARID_GET_CLR_EVT,    0x00000022},
    {BCCMDVARID_GET_NEXT_BUILDDEF, 0x00000022},
    {BCCMDVARID_PSCLRS,         0x00000022},
    {BCCMDVARID_I2C_TRANSFER,   0x02B22992},
    {BCCMDVARID_E2_DEVICE,      0x00000022},
    {BCCMDVARID_E2_APP_DATA,    0x0002BE92},
    {BCCMDVARID_ENABLE_AFH,     0x00000022},
    {BCCMDVARID_BER_THRESHOLD,  0x00000022},
    {BCCMDVARID_PS_MEMORY_TYPE, 0x00000022},
    {BCCMDVARID_REMOTE_SNIFF_LIMITS, 0x00222222}
};


/*
**  Populate the chip management HC/H (hq) table with all the types of command.
*/

#define HQ_TBL_SIZE (13)
PACKER_PDU_TBL hq_tbl[HQ_TBL_SIZE] =
{
    /* {varid,format}, */
    {HQVARID_COMMANDS_START,     0x00000000},
    {HQVARID_UINT8_START,        0x00000001},
    {HQVARID_UINT16_START,       0x00000002},
    {HQVARID_UINT32_START,       0x00000005},
    /* Specials */
    {HQVARID_FAULT,              0x00000252},
    {HQVARID_RXPKTSTATS,         0x00022222},
    {HQVARID_LUT_ENTRY,          0x00000022},
    {HQVARID_SYNTH_SETTLE,       0x00000022},
    {HQVARID_FILTER_RESP,        0x00000222},
    {HQVARID_IQ_MATCH,           0x00002222},
    {HQVARID_BIT_ERR,            0x00000552},
    {HQVARID_RADIO_STATUS,       0x02222222},
    {HQVARID_RADIO_STATUS_ARRAY, 0x00000022}
};




#endif /* __BCPACK_TABLES_H__ */
