/******************************************************************************
**
**   File: hci_def.h
**
**            (C) CSR 2000
**   
**   Purpose:  Defines for the HCI command and event codes
**
**  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/Packer/HCIPacker/hci_def.h#10 $
**
******************************************************************************/

/****************************************************************************
**  Modification history
***************************************************************************** 
**  1.1   8:may:00	ajh	Created.
**  1.3  30:nov:00  ajh Added tunnel.h
*/

#ifndef _HCI_DEF_H    /* Once is enough */
#define _HCI_DEF_H

#include "tunnel.h"

typedef uint16 HCI_OPCODE;
typedef uint8 HCI_EVTCODE;

/******************************************************************************
   Masks for OGF, OCF.
******************************************************************************/
#define HCI_OPCODE_GROUP_MASK                   ((HCI_OPCODE)0xFC00)
#define HCI_OPCODE_MASK                         ((HCI_OPCODE)0x03FF)
#define HCI_OGF_BIT_OFFSET                      ((uint8)10) /* Number of bit shifts */

/******************************************************************************
   Defines for the OGF opcodes group - these occupy the 6 msb of the opcode 
   hence the values 
******************************************************************************/
#define HCI_LINK                                ((HCI_OPCODE)0x0400)
#define HCI_POLICY                              ((HCI_OPCODE)0x0800)
#define HCI_HOST_BB                             ((HCI_OPCODE)0x0C00)
#define HCI_INFO                                ((HCI_OPCODE)0x1000)
#define HCI_STATUS                              ((HCI_OPCODE)0x1400)
#define HCI_TEST                                ((HCI_OPCODE)0x1800)
#define HCI_DEBUG                               ((HCI_OPCODE)0xFC00)

/******************************************************************************
   OCF opcode defines - Link Control Commands
******************************************************************************/
#define HCI_INQUIRY                             ((HCI_OPCODE)HCI_LINK | 0x0001)
#define HCI_INQUIRY_CANCEL                      ((HCI_OPCODE)HCI_LINK | 0x0002)
#define HCI_PERIODIC_INQUIRY_MODE               ((HCI_OPCODE)HCI_LINK | 0x0003)
#define HCI_EXIT_PERIODIC_INQUIRY_MODE          ((HCI_OPCODE)HCI_LINK | 0x0004)
#define HCI_CREATE_CONNECTION                   ((HCI_OPCODE)HCI_LINK | 0x0005)
#define HCI_DISCONNECT                          ((HCI_OPCODE)HCI_LINK | 0x0006)
#define HCI_ADD_SCO_CONNECTION                  ((HCI_OPCODE)HCI_LINK | 0x0007)

#define HCI_CREATE_CONNECTION_CANCEL            ((HCI_OPCODE)HCI_LINK | 0x0008)
#define HCI_ACCEPT_CONNECTION_REQUEST           ((HCI_OPCODE)HCI_LINK | 0x0009)
#define HCI_REJECT_CONNECTION_REQUEST           ((HCI_OPCODE)HCI_LINK | 0x000A)
#define HCI_LINK_KEY_REQUEST_REPLY              ((HCI_OPCODE)HCI_LINK | 0x000B)
#define HCI_LINK_KEY_REQUEST_NEGATIVE_REPLY     ((HCI_OPCODE)HCI_LINK | 0x000C)
#define HCI_PIN_CODE_REQUEST_REPLY              ((HCI_OPCODE)HCI_LINK | 0x000D)
#define HCI_PIN_CODE_REQUEST_NEGATIVE_REPLY     ((HCI_OPCODE)HCI_LINK | 0x000E)
#define HCI_CHANGE_CONNECTION_PACKET_TYPE       ((HCI_OPCODE)HCI_LINK | 0x000F)
#define HCI_AUTHENTICATION_REQUEST              ((HCI_OPCODE)HCI_LINK | 0x0011)
#define HCI_SET_CONNECTION_ENCRYPTION           ((HCI_OPCODE)HCI_LINK | 0x0013)
#define HCI_CHANGE_CONNECTION_LINK_KEY          ((HCI_OPCODE)HCI_LINK | 0x0015)
#define HCI_MASTER_LINK_KEY                     ((HCI_OPCODE)HCI_LINK | 0x0017)
#define HCI_REMOTE_NAME_REQUEST                 ((HCI_OPCODE)HCI_LINK | 0x0019)

#define HCI_REMOTE_NAME_REQUEST_CANCEL          ((HCI_OPCODE)HCI_LINK | 0x001A)

#define HCI_READ_REMOTE_SUPPORTED_FEATURES      ((HCI_OPCODE)HCI_LINK | 0x001B)
#define HCI_READ_REMOTE_VERSION_INFORMATION     ((HCI_OPCODE)HCI_LINK | 0x001D)
#define HCI_READ_CLOCK_OFFSET                   ((HCI_OPCODE)HCI_LINK | 0x001F)
#define HCI_READ_LMP_HANDLE                     ((HCI_OPCODE)HCI_LINK | 0x0020)


/******************************************************************************
   OP CODE defines - Link Policy Commands
 *****************************************************************************/
#define HCI_HOLD_MODE                           ((HCI_OPCODE)HCI_POLICY | 0x0001)
#define HCI_SNIFF_MODE                          ((HCI_OPCODE)HCI_POLICY | 0x0003)
#define HCI_EXIT_SNIFF_MODE                     ((HCI_OPCODE)HCI_POLICY | 0x0004)
#define HCI_PARK_MODE                           ((HCI_OPCODE)HCI_POLICY | 0x0005)
#define HCI_EXIT_PARK_MODE                      ((HCI_OPCODE)HCI_POLICY | 0x0006)
#define HCI_QOS_SETUP                           ((HCI_OPCODE)HCI_POLICY | 0x0007)
#define HCI_ROLE_DISCOVERY                      ((HCI_OPCODE)HCI_POLICY | 0x0009)
#define HCI_SWITCH_ROLE                         ((HCI_OPCODE)HCI_POLICY | 0x000B)
#define HCI_READ_LINK_POLICY_SETTINGS           ((HCI_OPCODE)HCI_POLICY | 0x000C)
#define HCI_WRITE_LINK_POLICY_SETTINGS          ((HCI_OPCODE)HCI_POLICY | 0x000D)
#define HCI_READ_DEFAULT_LINK_POLICY_SETTINGS   ((HCI_OPCODE)HCI_POLICY | 0x000E)

#define HCI_WRITE_DEFAULT_LINK_POLICY_SETTINGS  ((HCI_OPCODE)HCI_POLICY | 0x000F)


/******************************************************************************
   OP CODE defines - Host Controller and Baseband Commands
 *****************************************************************************/
#define HCI_SET_EVENT_MASK                      ((HCI_OPCODE)HCI_HOST_BB | 0x0001)
#define HCI_RESET                               ((HCI_OPCODE)HCI_HOST_BB | 0x0003)
#define HCI_SET_EVENT_FILTER                    ((HCI_OPCODE)HCI_HOST_BB | 0x0005)
#define HCI_FLUSH                               ((HCI_OPCODE)HCI_HOST_BB | 0x0008)
#define HCI_READ_PIN_TYPE                       ((HCI_OPCODE)HCI_HOST_BB | 0x0009)
#define HCI_WRITE_PIN_TYPE                      ((HCI_OPCODE)HCI_HOST_BB | 0x000A)
#define HCI_CREATE_NEW_UNIT_KEY                 ((HCI_OPCODE)HCI_HOST_BB | 0x000B)
#define HCI_READ_STORED_LINK_KEY                ((HCI_OPCODE)HCI_HOST_BB | 0x000D)
#define HCI_WRITE_STORED_LINK_KEY               ((HCI_OPCODE)HCI_HOST_BB | 0x0011)
#define HCI_DELETE_STORED_LINK_KEY              ((HCI_OPCODE)HCI_HOST_BB | 0x0012)
#define HCI_CHANGE_LOCAL_NAME                   ((HCI_OPCODE)HCI_HOST_BB | 0x0013)
#define HCI_READ_LOCAL_NAME                     ((HCI_OPCODE)HCI_HOST_BB | 0x0014)
#define HCI_READ_CONNECTION_ACCEPT_TIMEOUT      ((HCI_OPCODE)HCI_HOST_BB | 0x0015)
#define HCI_WRITE_CONNECTION_ACCEPT_TIMEOUT     ((HCI_OPCODE)HCI_HOST_BB | 0x0016)
#define HCI_READ_PAGE_TIMEOUT                   ((HCI_OPCODE)HCI_HOST_BB | 0x0017)
#define HCI_WRITE_PAGE_TIMEOUT                  ((HCI_OPCODE)HCI_HOST_BB | 0x0018)
#define HCI_READ_SCAN_ENABLE                    ((HCI_OPCODE)HCI_HOST_BB | 0x0019)
#define HCI_WRITE_SCAN_ENABLE                   ((HCI_OPCODE)HCI_HOST_BB | 0x001A)
#define HCI_READ_PAGESCAN_ACTIVITY              ((HCI_OPCODE)HCI_HOST_BB | 0x001B)
#define HCI_WRITE_PAGESCAN_ACTIVITY             ((HCI_OPCODE)HCI_HOST_BB | 0x001C)
#define HCI_READ_INQUIRYSCAN_ACTIVITY           ((HCI_OPCODE)HCI_HOST_BB | 0x001D)
#define HCI_WRITE_INQUIRYSCAN_ACTIVITY          ((HCI_OPCODE)HCI_HOST_BB | 0x001E)
#define HCI_READ_AUTHENTICATION_ENABLE          ((HCI_OPCODE)HCI_HOST_BB | 0x001F)
#define HCI_WRITE_AUTHENTICATION_ENABLE         ((HCI_OPCODE)HCI_HOST_BB | 0x0020)
#define HCI_READ_ENCRYPTION_MODE                ((HCI_OPCODE)HCI_HOST_BB | 0x0021)
#define HCI_WRITE_ENCRYPTION_MODE               ((HCI_OPCODE)HCI_HOST_BB | 0x0022)
#define HCI_READ_CLASS_OF_DEVICE                ((HCI_OPCODE)HCI_HOST_BB | 0x0023)
#define HCI_WRITE_CLASS_OF_DEVICE               ((HCI_OPCODE)HCI_HOST_BB | 0x0024)
#define HCI_READ_VOICE_SETTING                  ((HCI_OPCODE)HCI_HOST_BB | 0x0025)
#define HCI_WRITE_VOICE_SETTING                 ((HCI_OPCODE)HCI_HOST_BB | 0x0026)
#define HCI_READ_AUTO_FLUSH_TIMEOUT             ((HCI_OPCODE)HCI_HOST_BB | 0x0027)
#define HCI_WRITE_AUTO_FLUSH_TIMEOUT            ((HCI_OPCODE)HCI_HOST_BB | 0x0028)
#define HCI_READ_NUM_BROADCAST_RETRANSMISSIONS  ((HCI_OPCODE)HCI_HOST_BB | 0x0029)
#define HCI_WRITE_NUM_BROADCAST_RETRANSMISSIONS ((HCI_OPCODE)HCI_HOST_BB | 0x002A)
#define HCI_READ_HOLD_MODE_ACTIVITY             ((HCI_OPCODE)HCI_HOST_BB | 0x002B)
#define HCI_WRITE_HOLD_MODE_ACTIVITY            ((HCI_OPCODE)HCI_HOST_BB | 0x002C)
#define HCI_READ_TRANSMIT_POWER_LEVEL           ((HCI_OPCODE)HCI_HOST_BB | 0x002D)
#define HCI_READ_SCO_FLOW_CONTROL_ENABLE        ((HCI_OPCODE)HCI_HOST_BB | 0x002E)
#define HCI_WRITE_SCO_FLOW_CONTROL_ENABLE       ((HCI_OPCODE)HCI_HOST_BB | 0x002F)
#define HCI_SET_HC_TO_H_FLOW_CONTROL            ((HCI_OPCODE)HCI_HOST_BB | 0x0031)
#define HCI_HOST_BUFFER_SIZE                    ((HCI_OPCODE)HCI_HOST_BB | 0x0033)
#define HCI_HOST_NUM_COMPLETED_PACKETS          ((HCI_OPCODE)HCI_HOST_BB | 0x0035)
#define HCI_READ_LINK_SUPERVISION_TIMEOUT       ((HCI_OPCODE)HCI_HOST_BB | 0x0036)
#define HCI_WRITE_LINK_SUPERVISION_TIMEOUT      ((HCI_OPCODE)HCI_HOST_BB | 0x0037)
#define HCI_READ_NUMBER_OF_SUPPORTED_IAC        ((HCI_OPCODE)HCI_HOST_BB | 0x0038)
#define HCI_READ_CURRENT_IAC_LAP                ((HCI_OPCODE)HCI_HOST_BB | 0x0039)
#define HCI_WRITE_CURRENT_IAC_LAP               ((HCI_OPCODE)HCI_HOST_BB | 0x003A)
#define HCI_READ_PAGESCAN_PERIOD_MODE           ((HCI_OPCODE)HCI_HOST_BB | 0x003B)
#define HCI_WRITE_PAGESCAN_PERIOD_MODE          ((HCI_OPCODE)HCI_HOST_BB | 0x003C)
#define HCI_READ_PAGESCAN_MODE                  ((HCI_OPCODE)HCI_HOST_BB | 0x003D)
#define HCI_WRITE_PAGESCAN_MODE                 ((HCI_OPCODE)HCI_HOST_BB | 0x003E)
#define HCI_SET_AFH_CHANNEL_CLASSIFICATION      ((HCI_OPCODE)HCI_HOST_BB | 0x003F)

#define HCI_READ_INQUIRY_SCAN_TYPE              ((HCI_OPCODE)HCI_HOST_BB | 0x0042)

#define HCI_WRITE_INQUIRY_SCAN_TYPE             ((HCI_OPCODE)HCI_HOST_BB | 0x0043)

#define HCI_READ_INQUIRY_MODE                   ((HCI_OPCODE)HCI_HOST_BB | 0x0044)

#define HCI_WRITE_INQUIRY_MODE                  ((HCI_OPCODE)HCI_HOST_BB | 0x0045)

#define HCI_READ_PAGE_SCAN_TYPE                 ((HCI_OPCODE)HCI_HOST_BB | 0x0046)
#define HCI_WRITE_PAGE_SCAN_TYPE                ((HCI_OPCODE)HCI_HOST_BB | 0x0047)

#define HCI_READ_AFH_CHANNEL_CLASSIFICATION_MODE ((HCI_OPCODE)HCI_HOST_BB | 0x0048)
#define HCI_WRITE_AFH_CHANNEL_CLASSIFICATION_MODE ((HCI_OPCODE)HCI_HOST_BB | 0x0049)

/******************************************************************************
   OP CODE defines - Informational Parameters
 *****************************************************************************/
#define HCI_READ_LOCAL_VERSION_INFORMATION      ((HCI_OPCODE)HCI_INFO | 0x0001)
#define HCI_READ_LOCAL_SUPPORTED_FEATURES       ((HCI_OPCODE)HCI_INFO | 0x0003)
#define HCI_READ_BUFFER_SIZE                    ((HCI_OPCODE)HCI_INFO | 0x0005)
#define HCI_READ_COUNTRY_CODE                   ((HCI_OPCODE)HCI_INFO | 0x0007)
#define HCI_READ_BD_ADDR                        ((HCI_OPCODE)HCI_INFO | 0x0009)

/******************************************************************************
   OP CODE defines - Status Parameters
 *****************************************************************************/
#define HCI_READ_FAILED_CONTACT_COUNTER         ((HCI_OPCODE)HCI_STATUS | 0x0001)
#define HCI_RESET_FAILED_CONTACT_COUNTER        ((HCI_OPCODE)HCI_STATUS | 0x0002)
#define HCI_GET_LINK_QUALITY                    ((HCI_OPCODE)HCI_STATUS | 0x0003)
#define HCI_READ_RSSI                           ((HCI_OPCODE)HCI_STATUS | 0x0005)
#define HCI_READ_AFH_CHANNEL_MAP                ((HCI_OPCODE)HCI_STATUS | 0x0006)
#define HCI_READ_CLOCK                          ((HCI_OPCODE)HCI_STATUS | 0x0007)


/******************************************************************************
   OP CODE defines - Testing Parameters
 *****************************************************************************/
#define HCI_READ_LOOPBACK_MODE                  ((HCI_OPCODE)HCI_TEST | 0x0001)
#define HCI_WRITE_LOOPBACK_MODE                 ((HCI_OPCODE)HCI_TEST | 0x0002)
#define HCI_ENABLE_DEVICE_UNDER_TEST_MODE       ((HCI_OPCODE)HCI_TEST | 0x0003)

/******************************************************************************
        Event defines - these are events as per the HCI definition
 *****************************************************************************/
#define HCI_EV_INQUIRY_COMPLETE                          ((HCI_EVTCODE)0x01)
#define HCI_EV_INQUIRY_RESULT                            ((HCI_EVTCODE)0x02)
#define HCI_EV_CONNECTION_COMPLETE                       ((HCI_EVTCODE)0x03)
#define HCI_EV_CONNECTION_REQUEST                        ((HCI_EVTCODE)0x04)
#define HCI_EV_DISCONNECTION_COMPLETE                    ((HCI_EVTCODE)0x05)
#define HCI_EV_AUTHENTICATION_COMPLETE                   ((HCI_EVTCODE)0x06)
#define HCI_EV_REMOTE_NAME_REQUEST_COMPLETE              ((HCI_EVTCODE)0x07)
#define HCI_EV_ENCRYPTION_CHANGE                         ((HCI_EVTCODE)0x08)
#define HCI_EV_CHANGE_CONNECTION_LINK_KEY_COMPLETE       ((HCI_EVTCODE)0x09)
#define HCI_EV_MASTER_LINK_KEY_COMPLETE                  ((HCI_EVTCODE)0x0A)
#define HCI_EV_READ_REMOTE_SUPPORTED_FEATURES_COMPLETE   ((HCI_EVTCODE)0x0B)
#define HCI_EV_READ_REMOTE_VERSION_INFORMATION_COMPLETE  ((HCI_EVTCODE)0x0C)
#define HCI_EV_QOS_SETUP_COMPLETE                        ((HCI_EVTCODE)0x0D)
#define HCI_EV_COMMAND_COMPLETE                          ((HCI_EVTCODE)0x0E)
#define HCI_EV_COMMAND_STATUS                            ((HCI_EVTCODE)0x0F)
#define HCI_EV_HARDWARE_ERROR                            ((HCI_EVTCODE)0x10)
#define HCI_EV_FLUSH_OCCURRED                            ((HCI_EVTCODE)0x11)
#define HCI_EV_ROLE_CHANGE                               ((HCI_EVTCODE)0x12)
#define HCI_EV_NUMBER_COMPLETED_PACKETS                  ((HCI_EVTCODE)0x13)
#define HCI_EV_MODE_CHANGE                               ((HCI_EVTCODE)0x14)
#define HCI_EV_RETURN_LINK_KEYS                          ((HCI_EVTCODE)0x15)
#define HCI_EV_PIN_CODE_REQUEST                          ((HCI_EVTCODE)0x16)
#define HCI_EV_LINK_KEY_REQUEST                          ((HCI_EVTCODE)0x17)
#define HCI_EV_LINK_KEY_NOTIFICATION                     ((HCI_EVTCODE)0x18)
#define HCI_EV_LOOPBACK_COMMAND                          ((HCI_EVTCODE)0x19)
#define HCI_EV_DATA_BUFFER_OVERFLOW                      ((HCI_EVTCODE)0x1A)
#define HCI_EV_MAX_SLOTS_CHANGE                          ((HCI_EVTCODE)0x1B)
#define HCI_EV_READ_CLOCK_OFFSET_COMPLETE                ((HCI_EVTCODE)0x1C)
#define HCI_EV_CONNECTION_PACKET_TYPE_CHANGED            ((HCI_EVTCODE)0x1D)
#define HCI_EV_QOS_VIOLATION                             ((HCI_EVTCODE)0x1E)
#define HCI_EV_PAGE_SCAN_MODE_CHANGE                     ((HCI_EVTCODE)0x1F)
#define HCI_EV_PAGE_SCAN_REPETITION_MODE_CHANGE          ((HCI_EVTCODE)0x20)
//1.2 Starts Here

#define HCI_EV_FLOW_SPECIFICATION_COMPLETE               ((HCI_EVTCODE)0x21)

#define HCI_EV_INQUIRY_RESULT_WITH_RSSI                  ((HCI_EVTCODE)0x22)


#define HCI_EV_MANUFACTURER_EXTENSION                    ((HCI_EVTCODE)0xFF)

/******************************************************************************
    HCI Successful return value 
 *****************************************************************************/
#define HCI_SUCCESS                         (0x00)
#define HCI_COMMAND_CURRENTLY_PENDING       (0x00)

/******************************************************************************
    HCI Error codes 
 *****************************************************************************/
#define HCI_ERROR_ILLEGAL_COMMAND           (0x01)
#define HCI_ERROR_NO_CONNECTION             (0x02)
#define HCI_ERROR_HARDWARE_FAIL             (0x03)
#define HCI_ERROR_PAGE_TIMEOUT              (0x04)
#define HCI_ERROR_AUTH_FAIL                 (0x05)
#define HCI_ERROR_KEY_MISSING               (0x06)
#define HCI_ERROR_MEMORY_FULL               (0x07)
#define HCI_ERROR_CONN_TIMEOUT              (0x08)
#define HCI_ERROR_MAX_NR_OF_CONNS           (0x09)
#define HCI_ERROR_MAX_NR_OF_SCO             (0x0A)
#define HCI_ERROR_MAX_NR_OF_ACL             (0x0B)
#define HCI_ERROR_COMMAND_DISALLOWED        (0x0C)
#define HCI_ERROR_REJ_BY_REMOTE_NO_RES      (0x0D)  /* no resource */
#define HCI_ERROR_REJ_BY_REMOTE_SEC         (0x0E)  /* security violation */
#define HCI_ERROR_REJ_BY_REMOTE_PERS        (0x0F)  /* personal device */
#define HCI_ERROR_HOST_TIMEOUT              (0x10) 
#define HCI_ERROR_UNSUPPORTED_FEATURE       (0x11)  /* or incorrect param value */
#define HCI_ERROR_ILLEGAL_FORMAT            (0x12) 
#define HCI_ERROR_OETC_USER                 (0x13)  /* other end terminated */
#define HCI_ERROR_OETC_LOW_RESOURCE         (0x14)  /* other end terminated */
#define HCI_ERROR_OETC_POWERING_OFF         (0x15)  /* other end terminated */
#define HCI_ERROR_CONN_TERM_LOCAL_HOST      (0x16)  /* local host terminated */
#define HCI_ERROR_AUTH_REPEATED             (0x17)  
#define HCI_ERROR_PAIRING_NOT_ALLOWED       (0x18) 
#define HCI_ERROR_UNKNOWN_LMP_PDU           (0x19) 
#define HCI_ERROR_UNSUPPORTED_REM_FEATURE   (0x1A) 
#define HCI_ERROR_SCO_OFFSET_REJECTED       (0x1B) 
#define HCI_ERROR_SCO_INTERVAL_REJECTED     (0x1C) 
#define HCI_ERROR_SCO_AIR_MODE_REJECTED     (0x1D) 
#define HCI_ERROR_INVALID_LMP_PARAMETERS    (0x1E) 
#define HCI_ERROR_UNSPECIFIED               (0x1F) 
#define HCI_ERROR_UNSUPP_LMP_PARAM          (0x20) 
#define HCI_ERROR_ROLE_CHANGE_NOT_ALLOWED   (0x21) 
#define HCI_ERROR_LMP_RESPONSE_TIMEOUT      (0x22)
#define HCI_ERROR_LMP_TRANSACTION_COLLISION (0x23)
#define HCI_ERROR_LMP_PDU_NOT_ALLOWED       (0x24) 

/******************************************************************************
**
**  HCI packet types
**
******************************************************************************/
#define HCI_PKT_DM1       (0x0008)    /* SCO and ACL */
#define HCI_PKT_DH1       (0x0010)    /* ACL only */
#define HCI_PKT_HV1       (0x0020)    /* SCO only */
#define HCI_PKT_HV2       (0x0040)    /* SCO only */
#define HCI_PKT_HV3       (0x0080)    /* SCO only */
#define HCI_PKT_DV        (0x0100)    /* SCO only */
#define HCI_PKT_AUX1      (0x0200)    /* ACL only NOT L2CAP */
#define HCI_PKT_DM3       (0x0400)    /* ACL only */
#define HCI_PKT_DH3       (0x0800)    /* ACL only */
#define HCI_PKT_DM5       (0x4000)    /* ACL only */
#define HCI_PKT_DH5       (0x8000)    /* ACL only */


#endif /* _HCI_DEF_H */
