/****************************************************************************
**
**  File: hciTables.h
**
**            (C) CSR 2000
**
**  Purpose:  Define the protocol for and the data in the tables necessary
**            to run the hci packet generation and analysis.
**
**  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/Packer/HCIPacker/hci_tables.h#22 $
**
****************************************************************************/

/****************************************************************************
**  Modification history
***************************************************************************** 
**  1.1   8:may:00	ajh	Created.
**  1.2   4:sep:00      pws     Finish command complete event table.
*/

#ifndef	__HCITABLES_H__
#define	__HCITABLES_H__

/*
**  Define the formatting for reading and writing parameters to
**  buffers.  This is in the "format" element of the tables.
**
**  0 = null filler for ends.
**  1 = byte
**  2 = uint16
**  3 = uint24 (which we store in 4 bytes)
**  4 = uint32
**  5 = uint32 in bluecore friendly format.  used in bccmd stuff only.
**  6 = six byte BD_ADDR
**  7 = 16 byte link keys and PIN codes
**  8 = 8 byte event mask.
**  A = a byte buffer, either a string or a block of binary data.
**      In analysis, considered to end the packet, all the rest
**      of the data will be treated as a buffer.
**  B = Use the last number as the length of an array of the
**      data structure following the B (zero ended).
**  C = command parameters follow, use the last parameter as the
**      command op code to process command complete return parameters.
**  D = Event Filter type.  This is something of a git.  See the Spec for details.
*/

/*
**  Populate the event table with a complete set of HCI events,  BCS v1.0B.
*/

static PACKER_PDU_TBL evt_table[] =
{
    /* {0x00,0x00000000}, */

    {HCI_EV_INQUIRY_COMPLETE,		        0x00000011},
    {HCI_EV_INQUIRY_RESULT,		            0x231116B1},
    {HCI_EV_INQUIRY_RESULT_WITH_RSSI,       0x123116B1},
    {HCI_EV_CONNECTION_COMPLETE,	        0x00011621},
    {HCI_EV_CONNECTION_REQUEST,		        0x00000136},
    {HCI_EV_DISCONNECTION_COMPLETE,	        0x00000121},

    {HCI_EV_AUTHENTICATION_COMPLETE,	        0x00000021},
    {HCI_EV_REMOTE_NAME_REQUEST_COMPLETE,       0x00000A61},
    {HCI_EV_ENCRYPTION_CHANGE,		        0x00000121},
    {HCI_EV_CHANGE_CONNECTION_LINK_KEY_COMPLETE,0x00000021},
    {HCI_EV_MASTER_LINK_KEY_COMPLETE,	        0x00000121},

    {HCI_EV_READ_REMOTE_SUPPORTED_FEATURES_COMPLETE, 0x00000821},
    {HCI_EV_READ_REMOTE_VERSION_INFORMATION_COMPLETE,  0x00022121},
    {HCI_EV_QOS_SETUP_COMPLETE,		        0x44441121},
    {HCI_EV_COMMAND_COMPLETE,		        0x00000C21},
    {HCI_EV_COMMAND_STATUS,		        0x00000211},

    {HCI_EV_HARDWARE_ERROR,		        0x00000001},
    {HCI_EV_FLUSH_OCCURRED,		        0x00000002},
    {HCI_EV_ROLE_CHANGE,		        0x00000161},
    {HCI_EV_NUMBER_COMPLETED_PACKETS,	        0x000022B1},
    {HCI_EV_MODE_CHANGE,		        0x00002121},

    {HCI_EV_RETURN_LINK_KEYS,		        0x000076B1},
    {HCI_EV_PIN_CODE_REQUEST,		        0x00000006},
    {HCI_EV_LINK_KEY_REQUEST,		        0x00000006},
    {HCI_EV_LINK_KEY_NOTIFICATION,	        0x00000076},
    {HCI_EV_LOOPBACK_COMMAND,		        0x0000000A},

    {HCI_EV_DATA_BUFFER_OVERFLOW,	        0x00000001},
    {HCI_EV_MAX_SLOTS_CHANGE,		        0x00000012},
    {HCI_EV_READ_CLOCK_OFFSET_COMPLETE,	        0x00000221},
    {HCI_EV_CONNECTION_PACKET_TYPE_CHANGED,     0x00000221},
    {HCI_EV_QOS_VIOLATION,		        0x00000002},

    {HCI_EV_PAGE_SCAN_MODE_CHANGE,	        0x00000016},
    {HCI_EV_PAGE_SCAN_REPETITION_MODE_CHANGE,   0x00000016},

    {0x21 /* HCI_EV_FLOW_SPECIFICATION_COMPLETE */, 0x44411121}, /* and then 4 */
    {0x23 /* HCI_EV_READ_REMOTE_EXTENDED_FEATURES_COMPLETE */, 0x00081121},
    {0x2c /* HCI_EV_SYNCHRONOUS_CONNECTION_COMPLETE */, 0x22111621}, /* and then 1 */
    {0x2d /* HCI_EV_SYNCHRONOUS_CONNECTION_CHANGED */,  0x00221121},


    {HCI_EV_MANUFACTURER_EXTENSION,	        0x0000000A}
};

#define EVT_TABLE_SIZE (sizeof (evt_table) / sizeof (evt_table[0]))

/*
**  Populate the Command Complete table with a minimal set of Command return parameters:
*/
static PACKER_PDU_TBL comcom_table[] =
{
    /* {0x00,0x00000000}, */
    {HCI_INQUIRY,			        0x00000000},
    {HCI_INQUIRY_CANCEL,		        0x00000001},
    {HCI_PERIODIC_INQUIRY_MODE,			0x00000001},
    {HCI_EXIT_PERIODIC_INQUIRY_MODE,		0x00000001},
    {HCI_CREATE_CONNECTION,		        0x00000000},
    {HCI_CREATE_CONNECTION_CANCEL,		0x00000061},
    {HCI_DISCONNECT,			        0x00000000},
    {HCI_ADD_SCO_CONNECTION,		        0x00000000},
    {HCI_ACCEPT_CONNECTION_REQUEST,             0x00000000},
    {HCI_REJECT_CONNECTION_REQUEST, 	        0x00000000},
    {HCI_LINK_KEY_REQUEST_REPLY,		0x00000061},
    {HCI_LINK_KEY_REQUEST_NEGATIVE_REPLY,	0x00000061},
    {HCI_PIN_CODE_REQUEST_REPLY,		0x00000061},
    {HCI_PIN_CODE_REQUEST_NEGATIVE_REPLY,	0x00000061},
    {HCI_CHANGE_CONNECTION_PACKET_TYPE,		0x00000000},
    {HCI_AUTHENTICATION_REQUEST,		0x00000000},
    {HCI_SET_CONNECTION_ENCRYPTION,		0x00000000},
    {HCI_CHANGE_CONNECTION_LINK_KEY,		0x00000000},
    {HCI_MASTER_LINK_KEY,			0x00000000},
    {HCI_REMOTE_NAME_REQUEST,			0x00000000},
    {HCI_REMOTE_NAME_REQUEST_CANCEL,		0x00000061},
    {HCI_READ_REMOTE_SUPPORTED_FEATURES,	0x00000000},
    {HCI_READ_REMOTE_VERSION_INFORMATION,	0x00000000},
    {HCI_READ_CLOCK_OFFSET,			0x00000000},
    {HCI_READ_LMP_HANDLE,			0x00004121},
    {HCI_HOLD_MODE,				0x00000000},
    {HCI_SNIFF_MODE,				0x00000000},
    {HCI_EXIT_SNIFF_MODE,			0x00000000},
    {HCI_PARK_MODE,				0x00000000},
    {HCI_EXIT_PARK_MODE,			0x00000000},
    {HCI_QOS_SETUP,				0x00000000},
    {HCI_ROLE_DISCOVERY,			0x00000121},
    {HCI_READ_LINK_POLICY_SETTINGS,		0x00000221},
    {HCI_WRITE_LINK_POLICY_SETTINGS,		0x00000021},
    {HCI_READ_DEFAULT_LINK_POLICY_SETTINGS,		0x00000021},
    {HCI_WRITE_DEFAULT_LINK_POLICY_SETTINGS,		0x00000001},
    {HCI_SET_EVENT_MASK,		        0x00000001},
    {HCI_RESET,				        0x00000001},
    {HCI_SET_EVENT_FILTER,		        0x00000001},
    {HCI_FLUSH,					0x00000021},
    {HCI_READ_PIN_TYPE,				0x00000011},
    {HCI_WRITE_PIN_TYPE,			0x00000001},
    {HCI_CREATE_NEW_UNIT_KEY,			0x00000001},
    {HCI_READ_STORED_LINK_KEY,			0x00000221},
    {HCI_WRITE_STORED_LINK_KEY,			0x00000011},
    {HCI_DELETE_STORED_LINK_KEY,		0x00000021},
    {HCI_CHANGE_LOCAL_NAME,			0x00000001},
    {HCI_READ_LOCAL_NAME,			0x000000A1},
    {HCI_READ_CONNECTION_ACCEPT_TIMEOUT,	0x00000021},
    {HCI_WRITE_CONNECTION_ACCEPT_TIMEOUT,	0x00000001},
    {HCI_READ_PAGE_TIMEOUT,			0x00000021},
    {HCI_WRITE_PAGE_TIMEOUT,			0x00000001},
    {HCI_READ_SCAN_ENABLE,			0x00000011},
    {HCI_WRITE_SCAN_ENABLE,		        0x00000001},
    {HCI_READ_PAGESCAN_ACTIVITY,		0x00000221},
    {HCI_WRITE_PAGESCAN_ACTIVITY,	        0x00000001},
    {HCI_READ_INQUIRYSCAN_ACTIVITY,		0x00000221},
    {HCI_WRITE_INQUIRYSCAN_ACTIVITY,	        0x00000001},
    {HCI_READ_AUTHENTICATION_ENABLE,		0x00000011},
    {HCI_WRITE_AUTHENTICATION_ENABLE,		0x00000001},
    {HCI_READ_ENCRYPTION_MODE,			0x00000011},
    {HCI_WRITE_ENCRYPTION_MODE,			0x00000001},
    {HCI_READ_CLASS_OF_DEVICE,			0x00000031},
    {HCI_WRITE_CLASS_OF_DEVICE,			0x00000001},
    {HCI_READ_VOICE_SETTING,			0x00000021},
    {HCI_WRITE_VOICE_SETTING,			0x00000001},
    {HCI_READ_AUTO_FLUSH_TIMEOUT,		0x00000221},
    {HCI_WRITE_AUTO_FLUSH_TIMEOUT,		0x00000021},
    {HCI_READ_NUM_BROADCAST_RETRANSMISSIONS,	0x00000011},
    {HCI_WRITE_NUM_BROADCAST_RETRANSMISSIONS,	0x00000001},
    {HCI_READ_HOLD_MODE_ACTIVITY,		0x00000011},
    {HCI_WRITE_HOLD_MODE_ACTIVITY,		0x00000001},
    {HCI_READ_TRANSMIT_POWER_LEVEL,		0x00000121},
    {HCI_READ_SCO_FLOW_CONTROL_ENABLE,		0x00000011},
    {HCI_WRITE_SCO_FLOW_CONTROL_ENABLE,		0x00000001},
    {HCI_SET_HC_TO_H_FLOW_CONTROL,		0x00000001},
    {HCI_HOST_BUFFER_SIZE,			0x00000001},
    {HCI_HOST_NUM_COMPLETED_PACKETS,		0x00000001},
    {HCI_READ_LINK_SUPERVISION_TIMEOUT,		0x00000221},
    {HCI_WRITE_LINK_SUPERVISION_TIMEOUT,	0x00000021},
    {HCI_READ_NUMBER_OF_SUPPORTED_IAC,		0x00000011},
    {HCI_READ_CURRENT_IAC_LAP,			0x00003B11},
    {HCI_WRITE_CURRENT_IAC_LAP,			0x00000001},
    {HCI_READ_PAGESCAN_PERIOD_MODE,		0x00000011},
    {HCI_WRITE_PAGESCAN_PERIOD_MODE,		0x00000001},
    {HCI_READ_PAGESCAN_MODE,			0x00000011},
    {HCI_WRITE_PAGESCAN_MODE,			0x00000001},
    {HCI_SET_AFH_CHANNEL_CLASSIFICATION,	0x00000001},
    {HCI_READ_INQUIRY_SCAN_TYPE,           0x00000011},
    {HCI_WRITE_INQUIRY_SCAN_TYPE,          0x00000001},
    {HCI_READ_INQUIRY_MODE,           0x00000011},
    {HCI_WRITE_INQUIRY_MODE,          0x00000001},
    {HCI_READ_PAGE_SCAN_TYPE,           0x00000011},
    {HCI_WRITE_PAGE_SCAN_TYPE,          0x00000001},
    {HCI_READ_AFH_CHANNEL_CLASSIFICATION_MODE,	0x00000011},
    {HCI_WRITE_AFH_CHANNEL_CLASSIFICATION_MODE,	0x00000001},
    {HCI_READ_LOCAL_VERSION_INFORMATION,	0x00221211},
    {HCI_READ_LOCAL_SUPPORTED_FEATURES,		0x00000081},
    {0x1004 /* HCI_READ_LOCAL_EXTENDED_FEATURES */, 0x8111},
    {HCI_READ_BUFFER_SIZE,		        0x00022121},
    {HCI_READ_COUNTRY_CODE,			0x00000011},
    {HCI_READ_BD_ADDR,			        0x00000061},
    {HCI_READ_FAILED_CONTACT_COUNTER,		0x00000221},
    {HCI_RESET_FAILED_CONTACT_COUNTER,		0x00000021},
    {HCI_GET_LINK_QUALITY,			0x00000121},
    {HCI_READ_RSSI,				0x00000121},
    {HCI_READ_AFH_CHANNEL_MAP,	0x0000A121}, /* HERE need new 10-octet type */
    {HCI_READ_CLOCK,	        0x00002421}, 
    {HCI_READ_LOOPBACK_MODE,			0x00000011},
    {HCI_WRITE_LOOPBACK_MODE,			0x00000001},
    {HCI_ENABLE_DEVICE_UNDER_TEST_MODE,		0x00000001},
    {HCI_DEBUG,					0x00000000},
    {0x00,0x00000000}
};

/*
**  Populate the command generation table.  Complete BCS v1.0B.
*/
static PACKER_PDU_TBL cmd_table[] =
{
    /******************************************************************************
    **  Link Control Commands
    ******************************************************************************/
    {HCI_INQUIRY,				0x00000113},
    {HCI_INQUIRY_CANCEL,			0x00000000},
    {HCI_PERIODIC_INQUIRY_MODE,			0x00011322},
    {HCI_EXIT_PERIODIC_INQUIRY_MODE,		0x00000000},
    {HCI_CREATE_CONNECTION,			0x00121126},
    {HCI_CREATE_CONNECTION_CANCEL,			0x00000006},
    {HCI_DISCONNECT,				0x00000012},
    {HCI_ADD_SCO_CONNECTION,			0x00000022},
    {HCI_ACCEPT_CONNECTION_REQUEST,		0x00000016},
    {HCI_REJECT_CONNECTION_REQUEST,		0x00000016},
    {HCI_LINK_KEY_REQUEST_REPLY,		0x00000076},
    {HCI_LINK_KEY_REQUEST_NEGATIVE_REPLY,	0x00000006},
    {HCI_PIN_CODE_REQUEST_REPLY,		0x00000716},
    {HCI_PIN_CODE_REQUEST_NEGATIVE_REPLY,	0x00000006},
    {HCI_CHANGE_CONNECTION_PACKET_TYPE,		0x00000022},
    {HCI_AUTHENTICATION_REQUEST,		0x00000002},
    {HCI_SET_CONNECTION_ENCRYPTION,		0x00000012},
    {HCI_CHANGE_CONNECTION_LINK_KEY,		0x00000002},
    {HCI_MASTER_LINK_KEY,			0x00000001},
    {HCI_REMOTE_NAME_REQUEST,			0x00002116},
    {HCI_REMOTE_NAME_REQUEST_CANCEL,		0x00000006},
    {HCI_READ_REMOTE_SUPPORTED_FEATURES,	0x00000002},
    {0x041c /* HCI_READ_REMOTE_EXTENDED_FEATURES */, 0x012},
    {HCI_READ_REMOTE_VERSION_INFORMATION,	0x00000002},
    {HCI_READ_CLOCK_OFFSET,			0x00000002},
    {HCI_READ_LMP_HANDLE,			0x00000002},
    {0x0428 /* HCI_SETUP_SYNCHRONOUS_CONNECTION */, 0x2122442},
    {0x0429 /* HCI_ACCEPT_CONNECTION_REQUEST */, 0x2122446},
    {0x042a /* HCI_REJECT_CONNECTION_REQUEST */, 0x0000016},

    /******************************************************************************
    **  Link Policy Commands
     *****************************************************************************/
    {HCI_HOLD_MODE,				0x00000222},
    {HCI_SNIFF_MODE,				0x00022222},
    {HCI_EXIT_SNIFF_MODE,			0x00000002},
    {HCI_PARK_MODE,				0x00000222},
    {HCI_EXIT_PARK_MODE,			0x00000002},
    {HCI_QOS_SETUP,				0x04444112},
    {HCI_ROLE_DISCOVERY,			0x00000002},
    {HCI_SWITCH_ROLE,				0x00000016},
    {HCI_READ_LINK_POLICY_SETTINGS,		        0x00000002},
    {HCI_WRITE_LINK_POLICY_SETTINGS,		    0x00000022},
    {HCI_READ_DEFAULT_LINK_POLICY_SETTINGS,		0x00000000},
    {HCI_WRITE_DEFAULT_LINK_POLICY_SETTINGS,	0x00000002},
    {0x0810 /* HCI_FLOW_SPECIFICATION */,	0x44441112},

    /******************************************************************************
    **  Host Controller and Baseband Commands
     *****************************************************************************/
    {HCI_SET_EVENT_MASK,			0x00000008},
    {HCI_RESET,					0x00000000},
    {HCI_SET_EVENT_FILTER,			0x0000000D},
    {HCI_FLUSH,					0x00000002},
    {HCI_READ_PIN_TYPE,				0x00000000},
    {HCI_WRITE_PIN_TYPE,			0x00000001},
    {HCI_CREATE_NEW_UNIT_KEY,			0x00000000},
    {HCI_READ_STORED_LINK_KEY,			0x00000016},
    {HCI_WRITE_STORED_LINK_KEY,			0x000076B1},
    {HCI_DELETE_STORED_LINK_KEY,		0x00000016},
    {HCI_CHANGE_LOCAL_NAME,			0x0000000A},
    {HCI_READ_LOCAL_NAME,			0x00000000},
    {HCI_READ_CONNECTION_ACCEPT_TIMEOUT,	0x00000000},
    {HCI_WRITE_CONNECTION_ACCEPT_TIMEOUT,	0x00000002},
    {HCI_READ_PAGE_TIMEOUT,			0x00000000},
    {HCI_WRITE_PAGE_TIMEOUT,			0x00000002},
    {HCI_READ_SCAN_ENABLE,			0x00000000},
    {HCI_WRITE_SCAN_ENABLE,			0x00000001},
    {HCI_READ_PAGESCAN_ACTIVITY,		0x00000000},
    {HCI_WRITE_PAGESCAN_ACTIVITY,		0x00000022},
    {HCI_READ_INQUIRYSCAN_ACTIVITY,		0x00000000},
    {HCI_WRITE_INQUIRYSCAN_ACTIVITY,		0x00000022},
    {HCI_READ_AUTHENTICATION_ENABLE,		0x00000000},
    {HCI_WRITE_AUTHENTICATION_ENABLE,		0x00000001},
    {HCI_READ_ENCRYPTION_MODE,			0x00000000},
    {HCI_WRITE_ENCRYPTION_MODE,			0x00000001},
    {HCI_READ_CLASS_OF_DEVICE,			0x00000000},
    {HCI_WRITE_CLASS_OF_DEVICE,			0x00000003},
    {HCI_READ_VOICE_SETTING,			0x00000000},
    {HCI_WRITE_VOICE_SETTING,			0x00000002},
    {HCI_READ_AUTO_FLUSH_TIMEOUT,		0x00000002},
    {HCI_WRITE_AUTO_FLUSH_TIMEOUT,		0x00000022},
    {HCI_READ_NUM_BROADCAST_RETRANSMISSIONS,	0x00000000},
    {HCI_WRITE_NUM_BROADCAST_RETRANSMISSIONS,	0x00000001},
    {HCI_READ_HOLD_MODE_ACTIVITY,		0x00000000},
    {HCI_WRITE_HOLD_MODE_ACTIVITY,		0x00000001},
    {HCI_READ_TRANSMIT_POWER_LEVEL,		0x00000012},
    {HCI_READ_SCO_FLOW_CONTROL_ENABLE,		0x00000000},
    {HCI_WRITE_SCO_FLOW_CONTROL_ENABLE,		0x00000001},
    {HCI_SET_HC_TO_H_FLOW_CONTROL,		0x00000001},
    {HCI_HOST_BUFFER_SIZE,			0x00002212},
    {HCI_HOST_NUM_COMPLETED_PACKETS,		0x000022B1},
    {HCI_READ_LINK_SUPERVISION_TIMEOUT,		0x00000002},
    {HCI_WRITE_LINK_SUPERVISION_TIMEOUT,	0x00000022},
    {HCI_READ_NUMBER_OF_SUPPORTED_IAC,		0x00000000},
    {HCI_READ_CURRENT_IAC_LAP,			0x00000000},
    {HCI_WRITE_CURRENT_IAC_LAP,			0x000003B1},
    {HCI_READ_PAGESCAN_PERIOD_MODE,		0x00000000},
    {HCI_WRITE_PAGESCAN_PERIOD_MODE,		0x00000001},
    {HCI_READ_PAGESCAN_MODE,			0x00000000},
    {HCI_WRITE_PAGESCAN_MODE,			0x00000001},
    {HCI_SET_AFH_CHANNEL_CLASSIFICATION,	0x0000000A}, /* HERE need new 10-octet type */
    {HCI_READ_INQUIRY_SCAN_TYPE,		    0x00000000},
    {HCI_WRITE_INQUIRY_SCAN_TYPE,		    0x00000001},
    {HCI_READ_INQUIRY_MODE,	    	    0x00000000},
    {HCI_WRITE_INQUIRY_MODE,		    0x00000001},
    {HCI_READ_PAGE_SCAN_TYPE,		    0x00000000},
    {HCI_WRITE_PAGE_SCAN_TYPE,		    0x00000001},
    {HCI_READ_AFH_CHANNEL_CLASSIFICATION_MODE,	0x00000000},
    {HCI_WRITE_AFH_CHANNEL_CLASSIFICATION_MODE,	0x00000001},

    /******************************************************************************
    **  Informational Parameters
     *****************************************************************************/
    {HCI_READ_LOCAL_VERSION_INFORMATION,	0x00000000},
    {HCI_READ_LOCAL_SUPPORTED_FEATURES,		0x00000000},
    {0x1004 /* HCI_READ_LOCAL_EXTENDED_FEATURES */, 0x0001},
    {HCI_READ_BUFFER_SIZE,			0x00000000},
    {HCI_READ_COUNTRY_CODE,			0x00000000},
    {HCI_READ_BD_ADDR,				0x00000000},

    /******************************************************************************
    **  Status Parameters
     *****************************************************************************/
    {HCI_READ_FAILED_CONTACT_COUNTER,		0x00000002},
    {HCI_RESET_FAILED_CONTACT_COUNTER,		0x00000002},
    {HCI_GET_LINK_QUALITY,	        		0x00000002},
    {HCI_READ_RSSI,				            0x00000002},
    {HCI_READ_AFH_CHANNEL_MAP,          	0x00000002},
    {HCI_READ_CLOCK,        	            0x00000012},

    /******************************************************************************
    **  Testing Parameters
     *****************************************************************************/
    {HCI_READ_LOOPBACK_MODE,			0x00000000},
    {HCI_WRITE_LOOPBACK_MODE,			0x00000001},
    {HCI_ENABLE_DEVICE_UNDER_TEST_MODE,			0x00000000},
    /******************************************************************************
    **  Manufacturer specific
     *****************************************************************************/
    {HCI_TUNNEL_COMMAND,                        0x0000000A }
};

#define CMD_TABLE_SIZE (sizeof (cmd_table) / sizeof (cmd_table[0]))

#endif /* __HCITABLES_H__ */
