#ifndef _PRIM_CONVERTER_H
#define _PRIM_CONVERTER_H


#include "common/linkage.h"
#include <stdio.h>
#include <string.h>
#include "malloc.h"

#include "app/bluestack/types.h"
#include "app/bluestack/bluetooth.h"
#include "app/bluestack/hci.h"
#include "app/bluestack/dm_prim.h"
#include "app/bluestack/rfcomm_prim.h"
#include "app/bluestack/sdc_prim.h"
#include "app/bluestack/sds_prim.h"
#include "app/bluestack/l2cap_prim.h"
#include "app/pan/bnep_hacked.h"
#include "app/pan/tcp_hacked.h"
#include "app/pan/udp_hacked.h"

enum {fromXAP, toXAP};


class DLL_API PrimConverter {
public:
	PrimConverter();
	~PrimConverter();

	// hand made functions
	uint32_t getXAPoffset();
	void    updateXAPprim(uint8_t *new_prim, uint32_t length);

	// generated functions
	void * conv_DM_UPRIM_T(uint8_t dir, uint16_t type, void * prim);
	void * conv_RFCOMM_UPRIM_T(uint8_t dir, uint16_t type, void * prim);
	void * conv_SDC_UPRIM_T(uint8_t dir, uint16_t type, void * prim);
	void * conv_L2CA_UPRIM_T(uint8_t dir, uint16_t type, void * prim);

private:
	// variables
	uint32_t xapOffset;
	uint8_t	*xapPrim;
	uint8_t convertDir;
	uint8_t ptrMode;
	uint32_t SIZE_OF_PRIM_BUF;
	uint32_t lengthXAPprim;

	// hand made functions
	void conv_uint8_t(uint8_t * num) ;
	void conv_uint16_t(uint16_t *num);
	void conv_uint24_t(uint24_t * num);
	void conv_uint32_t(uint32_t * num);
	void conv_bool_t(bool_t * num);
	void conv_enum(void * num);
	void conv_ptr(void * ptr);
	void startPointer(void**ptrptr,uint32_t size);
	void prepareConvert(uint8_t * aXapPrim, uint8_t toXAP);
	void dumpStr(int indent,char * str) ;

	uint8_t getXAP8(void);
	uint16_t getXAP16(void);
	uint32_t getXAP32(void);
	void putXAP8(uint8_t data);
	void putXAP16(uint16_t data);
	void putXAP32(uint32_t data);

	uint16_t namelen(const uint8_t *s, uint16_t n);
	void unsetPtrMode();
	void conv_int8_t(int8_t *num);
	void conv_int32_t(int32_t *num);
	void conv_IP_ADDR (IP_ADDR * prim); 

	void conv_customHCI(void *prim);
	void conv_customDM_HCI_REMOTE_NAME_COMPLETE(void *prim);
	void conv_customDM_HCI_READ_LOCAL_NAME_COMPLETE(void *prim);
	void conv_customWriteIacLap(void *prim);

	void conv_customCondition(void *prim); 
	void conv_customWriteLPSettings(void *prim);
	void conv_PARK_SETTINGS(U_SNIFF_PARK_T u);
	void conv_SNIFF_SETTINGS(U_SNIFF_PARK_T u);

	// generated functions
	void conv_BD_ADDR_T (BD_ADDR_T * prim) 
 ; 
void conv_DM_HCI_STANDARD_COMMAND_COMPLETE_T (DM_HCI_STANDARD_COMMAND_COMPLETE_T * prim) 
 ; 
void conv_DM_AM_REGISTER_REQ_T (DM_AM_REGISTER_REQ_T * prim) 
 ; 
void conv_DM_AM_REGISTER_CFM_T (DM_AM_REGISTER_CFM_T * prim) 
 ; 
void conv_DM_ACL_REGISTER_REQ_T (DM_ACL_REGISTER_REQ_T * prim) 
 ; 
void conv_DM_ACL_REGISTER_CFM_T (DM_ACL_REGISTER_CFM_T * prim) 
 ; 
void conv_DM_ACL_CONNECT_REQ_T (DM_ACL_CONNECT_REQ_T * prim) 
 ; 
void conv_DM_ACL_CONNECT_CFM_T (DM_ACL_CONNECT_CFM_T * prim) 
 ; 
void conv_DM_ACL_CONNECT_IND_T (DM_ACL_CONNECT_IND_T * prim) 
 ; 
void conv_DM_ACL_DISCONNECT_REQ_T (DM_ACL_DISCONNECT_REQ_T * prim) 
 ; 
void conv_DM_ACL_DISCONNECT_IND_T (DM_ACL_DISCONNECT_IND_T * prim) 
 ; 
void conv_DM_ACL_BUFFER_SIZE_IND_T (DM_ACL_BUFFER_SIZE_IND_T * prim) 
 ; 
void conv_DM_ACL_DATA_SENT_IND_T (DM_ACL_DATA_SENT_IND_T * prim) 
 ; 
void conv_DM_CONNECTIONLESS_CH_REG_REQ_T (DM_CONNECTIONLESS_CH_REG_REQ_T * prim) 
 ; 
void conv_DM_SCO_INCOMING_REGISTER_REQ_T (DM_SCO_INCOMING_REGISTER_REQ_T * prim) 
 ; 
void conv_DM_SCO_INCOMING_UNREGISTER_REQ_T (DM_SCO_INCOMING_UNREGISTER_REQ_T * prim) 
 ; 
void conv_DM_SCO_CONNECT_REQ_T (DM_SCO_CONNECT_REQ_T * prim) 
 ; 
void conv_DM_SCO_CONNECT_CFM_T (DM_SCO_CONNECT_CFM_T * prim) 
 ; 
void conv_DM_SCO_CONNECT_IND_T (DM_SCO_CONNECT_IND_T * prim) 
 ; 
void conv_DM_SCO_CONNECT_RES_T (DM_SCO_CONNECT_RES_T * prim) 
 ; 
void conv_DM_SCO_DISCONNECT_REQ_T (DM_SCO_DISCONNECT_REQ_T * prim) 
 ; 
void conv_DM_SCO_DISCONNECT_IND_T (DM_SCO_DISCONNECT_IND_T * prim) 
 ; 
void conv_DM_SCO_BUFFER_SIZE_IND_T (DM_SCO_BUFFER_SIZE_IND_T * prim) 
 ; 
void conv_DM_SCO_DATA_SENT_IND_T (DM_SCO_DATA_SENT_IND_T * prim) 
 ; 
void conv_DM_SM_SET_DEFAULT_SECURITY_REQ_T (DM_SM_SET_DEFAULT_SECURITY_REQ_T * prim) 
 ; 
void conv_DM_SM_REGISTER_REQ_T (DM_SM_REGISTER_REQ_T * prim) 
 ; 
void conv_DM_SM_REGISTER_OUTGOING_REQ_T (DM_SM_REGISTER_OUTGOING_REQ_T * prim) 
 ; 
void conv_DM_SM_UNREGISTER_REQ_T (DM_SM_UNREGISTER_REQ_T * prim) 
 ; 
void conv_DM_SM_UNREGISTER_OUTGOING_REQ_T (DM_SM_UNREGISTER_OUTGOING_REQ_T * prim) 
 ; 
void conv_DM_SM_ACCESS_REQ_T (DM_SM_ACCESS_REQ_T * prim) 
 ; 
void conv_DM_SM_ACCESS_CFM_T (DM_SM_ACCESS_CFM_T * prim) 
 ; 
void conv_DM_SM_SET_SEC_MODE_REQ_T (DM_SM_SET_SEC_MODE_REQ_T * prim) 
 ; 
void conv_DM_SM_SET_SEC_MODE_CFM_T (DM_SM_SET_SEC_MODE_CFM_T * prim) 
 ; 
void conv_DM_SM_ADD_DEVICE_REQ_T (DM_SM_ADD_DEVICE_REQ_T * prim) 
 ; 
void conv_DM_SM_ADD_DEVICE_CFM_T (DM_SM_ADD_DEVICE_CFM_T * prim) 
 ; 
void conv_DM_SM_REMOVE_DEVICE_REQ_T (DM_SM_REMOVE_DEVICE_REQ_T * prim) 
 ; 
void conv_DM_SM_REMOVE_DEVICE_CFM_T (DM_SM_REMOVE_DEVICE_CFM_T * prim) 
 ; 
void conv_DM_SM_LINK_KEY_REQUEST_IND_T (DM_SM_LINK_KEY_REQUEST_IND_T * prim) 
 ; 
void conv_DM_SM_LINK_KEY_REQUEST_RES_T (DM_SM_LINK_KEY_REQUEST_RES_T * prim) 
 ; 
void conv_DM_SM_PIN_REQUEST_IND_T (DM_SM_PIN_REQUEST_IND_T * prim) 
 ; 
void conv_DM_SM_PIN_REQUEST_RES_T (DM_SM_PIN_REQUEST_RES_T * prim) 
 ; 
void conv_DM_SM_LINK_KEY_IND_T (DM_SM_LINK_KEY_IND_T * prim) 
 ; 
void conv_DM_SM_AUTHORISE_IND_T (DM_SM_AUTHORISE_IND_T * prim) 
 ; 
void conv_DM_SM_AUTHORISE_RES_T (DM_SM_AUTHORISE_RES_T * prim) 
 ; 
void conv_DM_SM_AUTHENTICATE_REQ_T (DM_SM_AUTHENTICATE_REQ_T * prim) 
 ; 
void conv_DM_SM_AUTHENTICATE_CFM_T (DM_SM_AUTHENTICATE_CFM_T * prim) 
 ; 
void conv_DM_SM_ENCRYPT_REQ_T (DM_SM_ENCRYPT_REQ_T * prim) 
 ; 
void conv_DM_SM_ENCRYPT_CFM_T (DM_SM_ENCRYPT_CFM_T * prim) 
 ; 
void conv_DM_SM_ENCRYPTION_CHANGE_T (DM_SM_ENCRYPTION_CHANGE_T * prim) 
 ; 
void conv_DM_SM_L2CAP_REGISTER_REQ_T (DM_SM_L2CAP_REGISTER_REQ_T * prim) 
 ; 
void conv_DM_SM_L2CAP_CLRX_ENABLE_IND_T (DM_SM_L2CAP_CLRX_ENABLE_IND_T * prim) 
 ; 
void conv_DM_WRITE_CACHED_PAGE_MODE_REQ_T (DM_WRITE_CACHED_PAGE_MODE_REQ_T * prim) 
 ; 
void conv_DM_WRITE_CACHED_CLOCK_OFFSET_REQ_T (DM_WRITE_CACHED_CLOCK_OFFSET_REQ_T * prim) 
 ; 
void conv_DM_CLEAR_PARAM_CACHE_REQ_T (DM_CLEAR_PARAM_CACHE_REQ_T * prim) 
 ; 
void conv_DM_HC_TO_HOST_FLOW_CONTROL_T (DM_HC_TO_HOST_FLOW_CONTROL_T * prim) 
 ; 
void conv_DM_ACL_OPEN_REQ_T (DM_ACL_OPEN_REQ_T * prim) 
 ; 
void conv_DM_ACL_OPEN_CFM_T (DM_ACL_OPEN_CFM_T * prim) 
 ; 
void conv_DM_ACL_CLOSE_REQ_T (DM_ACL_CLOSE_REQ_T * prim) 
 ; 
void conv_DM_ACL_OPENED_IND_T (DM_ACL_OPENED_IND_T * prim) 
 ; 
void conv_DM_ACL_CLOSED_IND_T (DM_ACL_CLOSED_IND_T * prim) 
 ; 
void conv_DM_SET_DEFAULT_LINK_POLICY_T (DM_SET_DEFAULT_LINK_POLICY_T * prim) 
 ; 
void conv_DM_HCI_INQUIRY_RESULT_T (DM_HCI_INQUIRY_RESULT_T * prim) 
 ; 
void conv_DM_HCI_INQUIRY_COMPLETE_T (DM_HCI_INQUIRY_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_LINK_KEY_CHANGE_COMPLETE_T (DM_HCI_LINK_KEY_CHANGE_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_RETURN_LINK_KEYS_T (DM_HCI_RETURN_LINK_KEYS_T * prim) 
 ; 
void conv_DM_HCI_MASTER_LINK_KEY_COMPLETE_T (DM_HCI_MASTER_LINK_KEY_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_REMOTE_NAME_COMPLETE_T (DM_HCI_REMOTE_NAME_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_READ_REMOTE_FEATURES_COMPLETE_T (DM_HCI_READ_REMOTE_FEATURES_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_READ_REMOTE_VERSION_COMPLETE_T (DM_HCI_READ_REMOTE_VERSION_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_READ_CLOCK_OFFSET_COMPLETE_T (DM_HCI_READ_CLOCK_OFFSET_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_MODE_CHANGE_EVENT_T (DM_HCI_MODE_CHANGE_EVENT_T * prim) 
 ; 
void conv_DM_HCI_QOS_SETUP_CFM_T (DM_HCI_QOS_SETUP_CFM_T * prim) 
 ; 
void conv_DM_HCI_QOS_VIOLATION_IND_T (DM_HCI_QOS_VIOLATION_IND_T * prim) 
 ; 
void conv_DM_HCI_ROLE_DISCOVERY_COMPLETE_T (DM_HCI_ROLE_DISCOVERY_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_SWITCH_ROLE_COMPLETE_T (DM_HCI_SWITCH_ROLE_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_READ_LP_SETTINGS_COMPLETE_T (DM_HCI_READ_LP_SETTINGS_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_WRITE_LP_SETTINGS_T (DM_HCI_WRITE_LP_SETTINGS_T * prim) 
 ; 
void conv_DM_HCI_WRITE_LP_SETTINGS_COMPLETE_T (DM_HCI_WRITE_LP_SETTINGS_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_SET_EVENT_MASK_COMPLETE_T (DM_HCI_SET_EVENT_MASK_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_FLUSH_COMPLETE_T (DM_HCI_FLUSH_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_READ_PIN_TYPE_COMPLETE_T (DM_HCI_READ_PIN_TYPE_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_READ_STORED_LINK_KEY_COMPLETE_T (DM_HCI_READ_STORED_LINK_KEY_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_WRITE_STORED_LINK_KEY_COMPLETE_T (DM_HCI_WRITE_STORED_LINK_KEY_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_DELETE_STORED_LINK_KEY_COMPLETE_T (DM_HCI_DELETE_STORED_LINK_KEY_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_READ_LOCAL_NAME_COMPLETE_T (DM_HCI_READ_LOCAL_NAME_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_READ_CONN_ACCEPT_TO_COMPLETE_T (DM_HCI_READ_CONN_ACCEPT_TO_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_READ_PAGE_TO_COMPLETE_T (DM_HCI_READ_PAGE_TO_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_READ_SCAN_ENABLE_COMPLETE_T (DM_HCI_READ_SCAN_ENABLE_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_READ_PAGESCAN_ACTIVITY_COMPLETE_T (DM_HCI_READ_PAGESCAN_ACTIVITY_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_READ_INQUIRYSCAN_ACTIVITY_COMPLETE_T (DM_HCI_READ_INQUIRYSCAN_ACTIVITY_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_READ_AUTH_ENABLE_COMPLETE_T (DM_HCI_READ_AUTH_ENABLE_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_READ_ENCRYPTION_MODE_COMPLETE_T (DM_HCI_READ_ENCRYPTION_MODE_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_READ_CLASS_OF_DEVICE_COMPLETE_T (DM_HCI_READ_CLASS_OF_DEVICE_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_READ_VOICE_SETTING_COMPLETE_T (DM_HCI_READ_VOICE_SETTING_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_READ_AUTO_FLUSH_COMPLETE_T (DM_HCI_READ_AUTO_FLUSH_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_WRITE_AUTO_FLUSH_COMPLETE_T (DM_HCI_WRITE_AUTO_FLUSH_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_READ_NUM_BCAST_TXS_COMPLETE_T (DM_HCI_READ_NUM_BCAST_TXS_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_READ_HOLD_MODE_ACTIVITY_COMPLETE_T (DM_HCI_READ_HOLD_MODE_ACTIVITY_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_READ_TX_POWER_LEVEL_COMPLETE_T (DM_HCI_READ_TX_POWER_LEVEL_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_READ_LINK_SUPERV_TIMEOUT_COMPLETE_T (DM_HCI_READ_LINK_SUPERV_TIMEOUT_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_WRITE_LINK_SUPERV_TIMEOUT_COMPLETE_T (DM_HCI_WRITE_LINK_SUPERV_TIMEOUT_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_READ_NUM_SUPPORTED_IAC_COMPLETE_T (DM_HCI_READ_NUM_SUPPORTED_IAC_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_READ_CURRENT_IAC_LAP_COMPLETE_T (DM_HCI_READ_CURRENT_IAC_LAP_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_READ_PAGESCAN_PERIOD_MODE_COMPLETE_T (DM_HCI_READ_PAGESCAN_PERIOD_MODE_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_READ_PAGESCAN_MODE_COMPLETE_T (DM_HCI_READ_PAGESCAN_MODE_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_HARDWARE_ERROR_T (DM_HCI_HARDWARE_ERROR_T * prim) 
 ; 
void conv_DM_HCI_DATA_BUFFER_OVERFLOW_T (DM_HCI_DATA_BUFFER_OVERFLOW_T * prim) 
 ; 
void conv_DM_HCI_MAX_SLOTS_CHANGE_T (DM_HCI_MAX_SLOTS_CHANGE_T * prim) 
 ; 
void conv_DM_HCI_READ_LOCAL_VERSION_COMPLETE_T (DM_HCI_READ_LOCAL_VERSION_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_READ_LOCAL_FEATURES_COMPLETE_T (DM_HCI_READ_LOCAL_FEATURES_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_READ_COUNTRY_CODE_COMPLETE_T (DM_HCI_READ_COUNTRY_CODE_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_READ_BD_ADDR_COMPLETE_T (DM_HCI_READ_BD_ADDR_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_FAILED_CONTACT_COUNTER_COMPLETE_T (DM_HCI_FAILED_CONTACT_COUNTER_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_RESET_CONTACT_COUNTER_COMPLETE_T (DM_HCI_RESET_CONTACT_COUNTER_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_GET_LINK_QUALITY_COMPLETE_T (DM_HCI_GET_LINK_QUALITY_COMPLETE_T * prim) 
 ; 
void conv_DM_HCI_READ_RSSI_COMPLETE_T (DM_HCI_READ_RSSI_COMPLETE_T * prim) 
 ; 
void conv_HCI_COMMAND_COMMON_T (HCI_COMMAND_COMMON_T * prim) 
 ; 
void conv_HANDLE_COMPLETE_T (HANDLE_COMPLETE_T * prim) 
 ; 
void conv_LINK_KEY_BD_ADDR_T (LINK_KEY_BD_ADDR_T * prim) 
 ; 
void conv_HCI_INQUIRY_T (HCI_INQUIRY_T * prim) 
 ; 
void conv_HCI_INQUIRY_CANCEL_T (HCI_INQUIRY_CANCEL_T * prim) 
 ; 
void conv_HCI_PERIODIC_INQUIRY_MODE_T (HCI_PERIODIC_INQUIRY_MODE_T * prim) 
 ; 
void conv_HCI_EXIT_PERIODIC_INQUIRY_MODE_T (HCI_EXIT_PERIODIC_INQUIRY_MODE_T * prim) 
 ; 
void conv_HCI_CHANGE_CONN_PKT_TYPE_T (HCI_CHANGE_CONN_PKT_TYPE_T * prim) 
 ; 
void conv_HCI_CHANGE_CONN_LINK_KEY_T (HCI_CHANGE_CONN_LINK_KEY_T * prim) 
 ; 
void conv_HCI_MASTER_LINK_KEY_T (HCI_MASTER_LINK_KEY_T * prim) 
 ; 
void conv_HCI_REMOTE_NAME_REQ_T (HCI_REMOTE_NAME_REQ_T * prim) 
 ; 
void conv_HCI_READ_REMOTE_SUPP_FEATURES_T (HCI_READ_REMOTE_SUPP_FEATURES_T * prim) 
 ; 
void conv_HCI_READ_REMOTE_VER_INFO_T (HCI_READ_REMOTE_VER_INFO_T * prim) 
 ; 
void conv_HCI_READ_CLOCK_OFFSET_T (HCI_READ_CLOCK_OFFSET_T * prim) 
 ; 
void conv_HCI_HOLD_MODE_T (HCI_HOLD_MODE_T * prim) 
 ; 
void conv_HCI_SNIFF_MODE_T (HCI_SNIFF_MODE_T * prim) 
 ; 
void conv_HCI_EXIT_SNIFF_MODE_T (HCI_EXIT_SNIFF_MODE_T * prim) 
 ; 
void conv_HCI_PARK_MODE_T (HCI_PARK_MODE_T * prim) 
 ; 
void conv_HCI_EXIT_PARK_MODE_T (HCI_EXIT_PARK_MODE_T * prim) 
 ; 
void conv_HCI_QOS_SETUP_T (HCI_QOS_SETUP_T * prim) 
 ; 
void conv_HCI_ROLE_DISCOVERY_T (HCI_ROLE_DISCOVERY_T * prim) 
 ; 
void conv_HCI_SWITCH_ROLE_T (HCI_SWITCH_ROLE_T * prim) 
 ; 
void conv_HCI_READ_LINK_POLICY_SETTINGS_T (HCI_READ_LINK_POLICY_SETTINGS_T * prim) 
 ; 
void conv_HCI_SET_EVENT_MASK_T (HCI_SET_EVENT_MASK_T * prim) 
 ; 
void conv_HCI_RESET_T (HCI_RESET_T * prim) 
 ; 
void conv_HCI_SET_EVENT_FILTER_T (HCI_SET_EVENT_FILTER_T * prim) 
 ; 
void conv_HCI_FLUSH_T (HCI_FLUSH_T * prim) 
 ; 
void conv_HCI_READ_PIN_TYPE_T (HCI_READ_PIN_TYPE_T * prim) 
 ; 
void conv_HCI_WRITE_PIN_TYPE_T (HCI_WRITE_PIN_TYPE_T * prim) 
 ; 
void conv_HCI_CREATE_NEW_UNIT_KEY_T (HCI_CREATE_NEW_UNIT_KEY_T * prim) 
 ; 
void conv_HCI_READ_STORED_LINK_KEY_T (HCI_READ_STORED_LINK_KEY_T * prim) 
 ; 
void conv_HCI_WRITE_STORED_LINK_KEY_T (HCI_WRITE_STORED_LINK_KEY_T * prim) 
 ; 
void conv_HCI_DELETE_STORED_LINK_KEY_T (HCI_DELETE_STORED_LINK_KEY_T * prim) 
 ; 
void conv_HCI_CHANGE_LOCAL_NAME_T (HCI_CHANGE_LOCAL_NAME_T * prim) 
 ; 
void conv_HCI_READ_LOCAL_NAME_T (HCI_READ_LOCAL_NAME_T * prim) 
 ; 
void conv_HCI_READ_CONN_ACCEPT_TIMEOUT_T (HCI_READ_CONN_ACCEPT_TIMEOUT_T * prim) 
 ; 
void conv_HCI_WRITE_CONN_ACCEPT_TIMEOUT_T (HCI_WRITE_CONN_ACCEPT_TIMEOUT_T * prim) 
 ; 
void conv_HCI_READ_PAGE_TIMEOUT_T (HCI_READ_PAGE_TIMEOUT_T * prim) 
 ; 
void conv_HCI_WRITE_PAGE_TIMEOUT_T (HCI_WRITE_PAGE_TIMEOUT_T * prim) 
 ; 
void conv_HCI_READ_SCAN_ENABLE_T (HCI_READ_SCAN_ENABLE_T * prim) 
 ; 
void conv_HCI_WRITE_SCAN_ENABLE_T (HCI_WRITE_SCAN_ENABLE_T * prim) 
 ; 
void conv_HCI_READ_PAGESCAN_ACTIVITY_T (HCI_READ_PAGESCAN_ACTIVITY_T * prim) 
 ; 
void conv_HCI_WRITE_PAGESCAN_ACTIVITY_T (HCI_WRITE_PAGESCAN_ACTIVITY_T * prim) 
 ; 
void conv_HCI_READ_INQUIRYSCAN_ACTIVITY_T (HCI_READ_INQUIRYSCAN_ACTIVITY_T * prim) 
 ; 
void conv_HCI_WRITE_INQUIRYSCAN_ACTIVITY_T (HCI_WRITE_INQUIRYSCAN_ACTIVITY_T * prim) 
 ; 
void conv_HCI_READ_AUTH_ENABLE_T (HCI_READ_AUTH_ENABLE_T * prim) 
 ; 
void conv_HCI_READ_ENC_MODE_T (HCI_READ_ENC_MODE_T * prim) 
 ; 
void conv_HCI_READ_CLASS_OF_DEVICE_T (HCI_READ_CLASS_OF_DEVICE_T * prim) 
 ; 
void conv_HCI_WRITE_CLASS_OF_DEVICE_T (HCI_WRITE_CLASS_OF_DEVICE_T * prim) 
 ; 
void conv_HCI_READ_VOICE_SETTING_T (HCI_READ_VOICE_SETTING_T * prim) 
 ; 
void conv_HCI_WRITE_VOICE_SETTING_T (HCI_WRITE_VOICE_SETTING_T * prim) 
 ; 
void conv_HCI_READ_AUTO_FLUSH_TIMEOUT_T (HCI_READ_AUTO_FLUSH_TIMEOUT_T * prim) 
 ; 
void conv_HCI_WRITE_AUTO_FLUSH_TIMEOUT_T (HCI_WRITE_AUTO_FLUSH_TIMEOUT_T * prim) 
 ; 
void conv_HCI_READ_NUM_BCAST_RETXS_T (HCI_READ_NUM_BCAST_RETXS_T * prim) 
 ; 
void conv_HCI_WRITE_NUM_BCAST_RETXS_T (HCI_WRITE_NUM_BCAST_RETXS_T * prim) 
 ; 
void conv_HCI_READ_HOLD_MODE_ACTIVITY_T (HCI_READ_HOLD_MODE_ACTIVITY_T * prim) 
 ; 
void conv_HCI_WRITE_HOLD_MODE_ACTIVITY_T (HCI_WRITE_HOLD_MODE_ACTIVITY_T * prim) 
 ; 
void conv_HCI_READ_TX_POWER_LEVEL_T (HCI_READ_TX_POWER_LEVEL_T * prim) 
 ; 
void conv_HCI_READ_SCO_FLOW_CON_ENABLE_T (HCI_READ_SCO_FLOW_CON_ENABLE_T * prim) 
 ; 
void conv_HCI_WRITE_SCO_FLOW_CON_ENABLE_T (HCI_WRITE_SCO_FLOW_CON_ENABLE_T * prim) 
 ; 
void conv_HCI_SET_HCTOHOST_FLOW_CONTROL_T (HCI_SET_HCTOHOST_FLOW_CONTROL_T * prim) 
 ; 
void conv_HCI_HOST_NUM_COMPLETED_PACKETS_T (HCI_HOST_NUM_COMPLETED_PACKETS_T * prim) 
 ; 
void conv_HCI_READ_LINK_SUPERV_TIMEOUT_T (HCI_READ_LINK_SUPERV_TIMEOUT_T * prim) 
 ; 
void conv_HCI_WRITE_LINK_SUPERV_TIMEOUT_T (HCI_WRITE_LINK_SUPERV_TIMEOUT_T * prim) 
 ; 
void conv_HCI_READ_NUM_SUPPORTED_IAC_T (HCI_READ_NUM_SUPPORTED_IAC_T * prim) 
 ; 
void conv_HCI_READ_CURRENT_IAC_LAP_T (HCI_READ_CURRENT_IAC_LAP_T * prim) 
 ; 
void conv_HCI_WRITE_CURRENT_IAC_LAP_T (HCI_WRITE_CURRENT_IAC_LAP_T * prim) 
 ; 
void conv_HCI_READ_PAGESCAN_PERIOD_MODE_T (HCI_READ_PAGESCAN_PERIOD_MODE_T * prim) 
 ; 
void conv_HCI_WRITE_PAGESCAN_PERIOD_MODE_T (HCI_WRITE_PAGESCAN_PERIOD_MODE_T * prim) 
 ; 
void conv_HCI_READ_PAGESCAN_MODE_T (HCI_READ_PAGESCAN_MODE_T * prim) 
 ; 
void conv_HCI_WRITE_PAGESCAN_MODE_T (HCI_WRITE_PAGESCAN_MODE_T * prim) 
 ; 
void conv_HCI_READ_LOCAL_VER_INFO_T (HCI_READ_LOCAL_VER_INFO_T * prim) 
 ; 
void conv_HCI_READ_LOCAL_SUPP_FEATURES_T (HCI_READ_LOCAL_SUPP_FEATURES_T * prim) 
 ; 
void conv_HCI_READ_COUNTRY_CODE_T (HCI_READ_COUNTRY_CODE_T * prim) 
 ; 
void conv_HCI_READ_BD_ADDR_T (HCI_READ_BD_ADDR_T * prim) 
 ; 
void conv_HCI_READ_FAILED_CONTACT_COUNT_T (HCI_READ_FAILED_CONTACT_COUNT_T * prim) 
 ; 
void conv_HCI_RESET_FAILED_CONTACT_COUNT_T (HCI_RESET_FAILED_CONTACT_COUNT_T * prim) 
 ; 
void conv_HCI_GET_LINK_QUALITY_T (HCI_GET_LINK_QUALITY_T * prim) 
 ; 
void conv_HCI_READ_RSSI_T (HCI_READ_RSSI_T * prim) 
 ; 
void conv_HCI_ENABLE_DUT_MODE_T (HCI_ENABLE_DUT_MODE_T * prim) 
 ; 
void conv_HCI_INQ_RESULT_T (HCI_INQ_RESULT_T * prim) 
 ; 
void conv_QOS_FLOW_T (QOS_FLOW_T * prim) 
 ; 
void conv_L2CA_REGISTER_REQ_T (L2CA_REGISTER_REQ_T * prim) 
 ; 
void conv_L2CA_REGISTER_CFM_T (L2CA_REGISTER_CFM_T * prim) 
 ; 
void conv_L2CA_CONNECT_REQ_T (L2CA_CONNECT_REQ_T * prim) 
 ; 
void conv_L2CA_CONNECT_RSP_T (L2CA_CONNECT_RSP_T * prim) 
 ; 
void conv_L2CA_CONNECT_IND_T (L2CA_CONNECT_IND_T * prim) 
 ; 
void conv_L2CA_CONNECT_CFM_T (L2CA_CONNECT_CFM_T * prim) 
 ; 
void conv_L2CA_CONFIG_REQ_T (L2CA_CONFIG_REQ_T * prim) 
 ; 
void conv_L2CA_CONFIG_RSP_T (L2CA_CONFIG_RSP_T * prim) 
 ; 
void conv_L2CA_CONFIG_IND_T (L2CA_CONFIG_IND_T * prim) 
 ; 
void conv_L2CA_CONFIG_CFM_T (L2CA_CONFIG_CFM_T * prim) 
 ; 
void conv_L2CA_DATAWRITE_REQ_T (L2CA_DATAWRITE_REQ_T * prim) 
 ; 
void conv_L2CA_DATAWRITE_CFM_T (L2CA_DATAWRITE_CFM_T * prim) 
 ; 
void conv_L2CA_DATAREAD_IND_T (L2CA_DATAREAD_IND_T * prim) 
 ; 
void conv_L2CA_DISCONNECT_REQ_T (L2CA_DISCONNECT_REQ_T * prim) 
 ; 
void conv_L2CA_DISCONNECT_RSP_T (L2CA_DISCONNECT_RSP_T * prim) 
 ; 
void conv_L2CA_DISCONNECT_IND_T (L2CA_DISCONNECT_IND_T * prim) 
 ; 
void conv_L2CA_DISCONNECT_CFM_T (L2CA_DISCONNECT_CFM_T * prim) 
 ; 
void conv_L2CA_TIMEOUT_IND_T (L2CA_TIMEOUT_IND_T * prim) 
 ; 
void conv_L2CA_QOSVIOLATION_IND_T (L2CA_QOSVIOLATION_IND_T * prim) 
 ; 
void conv_L2CA_PING_REQ_T (L2CA_PING_REQ_T * prim) 
 ; 
void conv_L2CA_PING_CFM_T (L2CA_PING_CFM_T * prim) 
 ; 
void conv_L2CA_GETINFO_REQ_T (L2CA_GETINFO_REQ_T * prim) 
 ; 
void conv_L2CA_GETINFO_CFM_T (L2CA_GETINFO_CFM_T * prim) 
 ; 
void conv_L2CA_GROUP_CREATE_REQ_T (L2CA_GROUP_CREATE_REQ_T * prim) 
 ; 
void conv_L2CA_GROUP_CREATE_CFM_T (L2CA_GROUP_CREATE_CFM_T * prim) 
 ; 
void conv_L2CA_GROUP_CLOSE_REQ_T (L2CA_GROUP_CLOSE_REQ_T * prim) 
 ; 
void conv_L2CA_GROUP_CLOSE_CFM_T (L2CA_GROUP_CLOSE_CFM_T * prim) 
 ; 
void conv_L2CA_ENABLE_CONNECTIONLESS_REQ_T (L2CA_ENABLE_CONNECTIONLESS_REQ_T * prim) 
 ; 
void conv_L2CA_ENABLE_CONNECTIONLESS_CFM_T (L2CA_ENABLE_CONNECTIONLESS_CFM_T * prim) 
 ; 
void conv_L2CA_DISABLE_CONNECTIONLESS_REQ_T (L2CA_DISABLE_CONNECTIONLESS_REQ_T * prim) 
 ; 
void conv_L2CA_DISABLE_CONNECTIONLESS_CFM_T (L2CA_DISABLE_CONNECTIONLESS_CFM_T * prim) 
 ; 
void conv_SYS_PAR_T (SYS_PAR_T * prim) 
 ; 
void conv_DLC_PAR_T (DLC_PAR_T * prim) 
 ; 
void conv_CONTROL_PAR_T (CONTROL_PAR_T * prim) 
 ; 
void conv_PORT_PAR_T (PORT_PAR_T * prim) 
 ; 
void conv_RFC_INIT_REQ_T (RFC_INIT_REQ_T * prim) 
 ; 
void conv_RFC_INIT_CFM_T (RFC_INIT_CFM_T * prim) 
 ; 
void conv_RFC_REGISTER_REQ_T (RFC_REGISTER_REQ_T * prim) 
 ; 
void conv_RFC_REGISTER_CFM_T (RFC_REGISTER_CFM_T * prim) 
 ; 
void conv_RFC_START_REQ_T (RFC_START_REQ_T * prim) 
 ; 
void conv_RFC_START_CFM_T (RFC_START_CFM_T * prim) 
 ; 
void conv_RFC_START_IND_T (RFC_START_IND_T * prim) 
 ; 
void conv_RFC_START_RES_T (RFC_START_RES_T * prim) 
 ; 
void conv_RFC_STARTCMP_IND_T (RFC_STARTCMP_IND_T * prim) 
 ; 
void conv_RFC_CLOSE_REQ_T (RFC_CLOSE_REQ_T * prim) 
 ; 
void conv_RFC_CLOSE_IND_T (RFC_CLOSE_IND_T * prim) 
 ; 
void conv_RFC_PARNEG_REQ_T (RFC_PARNEG_REQ_T * prim) 
 ; 
void conv_RFC_PARNEG_CFM_T (RFC_PARNEG_CFM_T * prim) 
 ; 
void conv_RFC_PARNEG_IND_T (RFC_PARNEG_IND_T * prim) 
 ; 
void conv_RFC_PARNEG_RES_T (RFC_PARNEG_RES_T * prim) 
 ; 
void conv_RFC_ESTABLISH_REQ_T (RFC_ESTABLISH_REQ_T * prim) 
 ; 
void conv_RFC_ESTABLISH_CFM_T (RFC_ESTABLISH_CFM_T * prim) 
 ; 
void conv_RFC_ESTABLISH_IND_T (RFC_ESTABLISH_IND_T * prim) 
 ; 
void conv_RFC_ESTABLISH_RES_T (RFC_ESTABLISH_RES_T * prim) 
 ; 
void conv_RFC_RELEASE_REQ_T (RFC_RELEASE_REQ_T * prim) 
 ; 
void conv_RFC_RELEASE_IND_T (RFC_RELEASE_IND_T * prim) 
 ; 
void conv_RFC_CONTROL_REQ_T (RFC_CONTROL_REQ_T * prim) 
 ; 
void conv_RFC_CONTROL_IND_T (RFC_CONTROL_IND_T * prim) 
 ; 
void conv_RFC_PORTNEG_REQ_T (RFC_PORTNEG_REQ_T * prim) 
 ; 
void conv_RFC_PORTNEG_CFM_T (RFC_PORTNEG_CFM_T * prim) 
 ; 
void conv_RFC_PORTNEG_IND_T (RFC_PORTNEG_IND_T * prim) 
 ; 
void conv_RFC_PORTNEG_RES_T (RFC_PORTNEG_RES_T * prim) 
 ; 
void conv_RFC_FCOFF_REQ_T (RFC_FCOFF_REQ_T * prim) 
 ; 
void conv_RFC_FCOFF_CFM_T (RFC_FCOFF_CFM_T * prim) 
 ; 
void conv_RFC_FCON_REQ_T (RFC_FCON_REQ_T * prim) 
 ; 
void conv_RFC_FCON_CFM_T (RFC_FCON_CFM_T * prim) 
 ; 
void conv_RFC_FCOFF_IND_T (RFC_FCOFF_IND_T * prim) 
 ; 
void conv_RFC_FCON_IND_T (RFC_FCON_IND_T * prim) 
 ; 
void conv_RFC_DATA_REQ_T (RFC_DATA_REQ_T * prim) 
 ; 
void conv_RFC_DATAWRITE_REQ_T (RFC_DATAWRITE_REQ_T * prim) 
 ; 
void conv_RFC_DATAWRITE_CFM_T (RFC_DATAWRITE_CFM_T * prim) 
 ; 
void conv_RFC_DATA_IND_T (RFC_DATA_IND_T * prim) 
 ; 
void conv_RFC_LINESTATUS_REQ_T (RFC_LINESTATUS_REQ_T * prim) 
 ; 
void conv_RFC_LINESTATUS_IND_T (RFC_LINESTATUS_IND_T * prim) 
 ; 
void conv_RFC_TEST_REQ_T (RFC_TEST_REQ_T * prim) 
 ; 
void conv_RFC_TEST_CFM_T (RFC_TEST_CFM_T * prim) 
 ; 
void conv_RFC_NSC_IND_T (RFC_NSC_IND_T * prim) 
 ; 
void conv_SDC_SERVICE_SEARCH_REQ_T (SDC_SERVICE_SEARCH_REQ_T * prim) 
 ; 
void conv_SDC_SERVICE_SEARCH_CFM_T (SDC_SERVICE_SEARCH_CFM_T * prim) 
 ; 
void conv_SDC_SERVICE_ATTRIBUTE_REQ_T (SDC_SERVICE_ATTRIBUTE_REQ_T * prim) 
 ; 
void conv_SDC_SERVICE_ATTRIBUTE_CFM_T (SDC_SERVICE_ATTRIBUTE_CFM_T * prim) 
 ; 
void conv_SDC_SERVICE_SEARCH_ATTRIBUTE_REQ_T (SDC_SERVICE_SEARCH_ATTRIBUTE_REQ_T * prim) 
 ; 
void conv_SDC_SERVICE_SEARCH_ATTRIBUTE_CFM_T (SDC_SERVICE_SEARCH_ATTRIBUTE_CFM_T * prim) 
 ; 
void conv_SDC_TERMINATE_PRIMITIVE_REQ_T (SDC_TERMINATE_PRIMITIVE_REQ_T * prim) 
 ; 
void conv_SDC_OPEN_SEARCH_REQ_T (SDC_OPEN_SEARCH_REQ_T * prim) 
 ; 
void conv_SDC_OPEN_SEARCH_CFM_T (SDC_OPEN_SEARCH_CFM_T * prim) 
 ; 
void conv_SDC_CLOSE_SEARCH_REQ_T (SDC_CLOSE_SEARCH_REQ_T * prim) 
 ; 
void conv_SDC_CONFIG_REQ_T (SDC_CONFIG_REQ_T * prim) 
 ; 
void conv_SDC_CLOSE_SEARCH_IND_T (SDC_CLOSE_SEARCH_IND_T * prim) 
 ; 
void conv_SDS_REGISTER_REQ_T (SDS_REGISTER_REQ_T * prim) 
 ; 
void conv_SDS_REGISTER_CFM_T (SDS_REGISTER_CFM_T * prim) 
 ; 
void conv_SDS_UNREGISTER_REQ_T (SDS_UNREGISTER_REQ_T * prim) 
 ; 
void conv_SDS_UNREGISTER_CFM_T (SDS_UNREGISTER_CFM_T * prim) 
 ; 
void conv_SDS_CONFIG_REQ_T (SDS_CONFIG_REQ_T * prim) 
 ; 
void conv_TCP_OPEN_REQ_T (TCP_OPEN_REQ_T * prim) 
 ; 
void conv_TCP_OPEN_CFM_T (TCP_OPEN_CFM_T * prim) 
 ; 
void conv_TCP_OPEN_IND_T (TCP_OPEN_IND_T * prim) 
 ; 
void conv_TCP_OPEN_RSP_T (TCP_OPEN_RSP_T * prim) 
 ; 
void conv_TCP_DATA_REQ_T (TCP_DATA_REQ_T * prim) 
 ; 
void conv_TCP_DATA_CFM_T (TCP_DATA_CFM_T * prim) 
 ; 
void conv_TCP_DATA_IND_T (TCP_DATA_IND_T * prim) 
 ; 
void conv_TCP_DATA_RSP_T (TCP_DATA_RSP_T * prim) 
 ; 
void conv_TCP_CLOSE_REQ_T (TCP_CLOSE_REQ_T * prim) 
 ; 
void conv_TCP_CLOSE_IND_T (TCP_CLOSE_IND_T * prim) 
 ; 
void conv_TCP_CLOSED_IND_T (TCP_CLOSED_IND_T * prim) 
 ; 
void conv_TCP_CLOSED_RSP_T (TCP_CLOSED_RSP_T * prim) 
 ; 
void conv_TCP_ABORT_REQ_T (TCP_ABORT_REQ_T * prim) 
 ; 
void conv_TCP_ABORT_IND_T (TCP_ABORT_IND_T * prim) 
 ; 
void conv_TCP_ABORT_RSP_T (TCP_ABORT_RSP_T * prim) 
 ; 
void conv_UDP_OPEN_REQ_T (UDP_OPEN_REQ_T * prim) 
 ; 
void conv_UDP_OPEN_CFM_T (UDP_OPEN_CFM_T * prim) 
 ; 
void conv_UDP_LISTEN_REQ_T (UDP_LISTEN_REQ_T * prim) 
 ; 
void conv_UDP_IGNORE_REQ_T (UDP_IGNORE_REQ_T * prim) 
 ; 
void conv_UDP_DATA_REQ_T (UDP_DATA_REQ_T * prim) 
 ; 
void conv_UDP_EXTENDED_DATA_REQ_T (UDP_EXTENDED_DATA_REQ_T * prim) 
 ; 
void conv_UDP_DATA_CFM_T (UDP_DATA_CFM_T * prim) 
 ; 
void conv_UDP_DATA_IND_T (UDP_DATA_IND_T * prim) 
 ; 
void conv_UDP_EXTENDED_DATA_IND_T (UDP_EXTENDED_DATA_IND_T * prim) 
 ; 
void conv_UDP_CLOSE_REQ_T (UDP_CLOSE_REQ_T * prim) 
 ; 
void conv_UDP_CLOSE_CFM_T (UDP_CLOSE_CFM_T * prim) 
 ; 
void conv_ETHER_ADDR (ETHER_ADDR * prim) 
 ; 
void conv_BNEP_REGISTER_REQ_T (BNEP_REGISTER_REQ_T * prim) 
 ; 
void conv_BNEP_CONNECT_REQ_T (BNEP_CONNECT_REQ_T * prim) 
 ; 
void conv_BNEP_CONNECT_CFM_T (BNEP_CONNECT_CFM_T * prim) 
 ; 
void conv_BNEP_CONNECT_IND_T (BNEP_CONNECT_IND_T * prim) 
 ; 
void conv_BNEP_LISTEN_REQ_T (BNEP_LISTEN_REQ_T * prim) 
 ; 
void conv_BNEP_IGNORE_REQ_T (BNEP_IGNORE_REQ_T * prim) 
 ; 
void conv_BNEP_DATA_REQ_T (BNEP_DATA_REQ_T * prim) 
 ; 
void conv_BNEP_EXTENDED_DATA_REQ_T (BNEP_EXTENDED_DATA_REQ_T * prim) 
 ; 
void conv_BNEP_EXTENDED_MULTICAST_DATA_REQ_T (BNEP_EXTENDED_MULTICAST_DATA_REQ_T * prim) 
 ; 
void conv_BNEP_DATA_IND_T (BNEP_DATA_IND_T * prim) 
 ; 
void conv_BNEP_EXTENDED_DATA_IND_T (BNEP_EXTENDED_DATA_IND_T * prim) 
 ; 
void conv_BNEP_MULTICAST_DATA_IND_T (BNEP_MULTICAST_DATA_IND_T * prim) 
 ; 
void conv_BNEP_LOOPBACK_DATA_IND_T (BNEP_LOOPBACK_DATA_IND_T * prim) 
 ; 
void conv_BNEP_UP_IND_T (BNEP_UP_IND_T * prim) 
 ; 
void conv_BNEP_DOWN_IND_T (BNEP_DOWN_IND_T * prim) 
 ; 
void conv_BNEP_DISCONNECT_REQ_T (BNEP_DISCONNECT_REQ_T * prim) 
 ; 
void conv_BNEP_DISCONNECT_IND_T (BNEP_DISCONNECT_IND_T * prim) 
 ; 
void conv_BNEP_DEBUG_REQ_T (BNEP_DEBUG_REQ_T * prim) 
 ; 
void conv_BNEP_DEBUG_IND_T (BNEP_DEBUG_IND_T * prim) 
 ; 
void conv_BNEP_FLOW_REQ_T (BNEP_FLOW_REQ_T * prim) 
 ; 
void conv_BNEP_FLOW_IND_T (BNEP_FLOW_IND_T * prim) 
 ; 
void conv_BNEP_HCI_SWITCH_ROLE_RSP_T (BNEP_HCI_SWITCH_ROLE_RSP_T * prim) 
 ; 

};


#endif _PRIM_CONVERTER_H
