#include "inc.h"


//The rules for converting to XAP are ...
//uint8 == uint16 (ie it takes two bytes
//uint32 = uint16 MSW uint16 LSW

PrimConverter::PrimConverter()
{
	xapOffset = 0 ;
	xapPrim = 0;
	convertDir = toXAP;
	ptrMode = 0;
	SIZE_OF_PRIM_BUF = 200;
	lengthXAPprim = 0;
}

PrimConverter::~PrimConverter()
{

}

uint32_t PrimConverter::getXAPoffset()
{
	return xapOffset;
}

void PrimConverter::updateXAPprim(uint8_t *rx_prim, uint32_t len)
{
	xapPrim = rx_prim;
	lengthXAPprim = len;
	// xapOffset = 0 ;
}

void PrimConverter::prepareConvert(uint8_t * aXapPrim, uint8_t to_or_from)
{
    xapOffset = 0 ;
	if (to_or_from == toXAP)
		xapPrim = new uint8_t [SIZE_OF_PRIM_BUF];
	convertDir = to_or_from;
	ptrMode = 0;
}

uint8_t PrimConverter::getXAP8(void)
{
	if (xapOffset < lengthXAPprim)
	{
		uint8_t retval = xapPrim[xapOffset] ;
		if (ptrMode) 
			xapOffset++ ;
		else 
			xapOffset +=2 ;
		
		return retval ;
	}
	else
		return 0;
}

uint16_t PrimConverter::getXAP16(void)
{
	if (xapOffset < lengthXAPprim)
	{
		uint16_t retval;
		if (xapOffset & 1) 
	 		xapOffset ++ ;

		//iprintf(II("xap offset %d value %02x \n"), xapOffset, xapPrim[xapOffset]);
		retval = xapPrim[xapOffset] + (xapPrim[xapOffset+1] << 8) ;
		xapOffset += 2 ;
		return retval ;
	}
	else
		return 0;
}

uint32_t PrimConverter::getXAP32(void)
{
	if (xapOffset < lengthXAPprim)
	{
		uint32_t retval ;
		retval = getXAP16() ; //yes this is correct - msw first
		retval <<=16 ;
		retval += getXAP16() ;
		return retval ;
	}
	else
		return 0;
}


void PrimConverter::putXAP8(uint8_t data)
{
	int i=0;

    xapPrim[xapOffset]   = data ;
    xapPrim[xapOffset+1] = 0 ;

	if (ptrMode) 
		xapOffset++;
	else 
		xapOffset += 2 ;	
}

void PrimConverter::putXAP16(uint16_t data)
{
	if (xapOffset & 1) 
		xapOffset ++ ;

    xapPrim[xapOffset] = data & 0xff ;
    xapPrim[xapOffset+1] = (data >> 8) ;
    xapOffset += 2 ;
}

void PrimConverter::putXAP32(uint32_t data)
{
    putXAP16((uint16_t)(data >>16)) ; //yes this is correct - msw first
    putXAP16((uint16_t)(data & 0xffff)) ;
}

void PrimConverter::conv_uint8_t(uint8_t * num)
{
    if (convertDir==toXAP) 
		putXAP8(*num) ;
    else 
		*num = getXAP8();
}


void PrimConverter::conv_int8_t(int8_t *num)
{
	conv_uint8_t((uint8_t *)num) ;
}

void PrimConverter::conv_int32_t(int32_t *num)
{
	conv_uint32_t((uint32_t *)num) ;
}

void PrimConverter::conv_uint16_t(uint16_t *num)
{
    if (convertDir==toXAP) 
		putXAP16(*num) ;
    else 
		*num = getXAP16() ;
}

void PrimConverter::conv_uint24_t(uint24_t * num)
{
    conv_uint32_t(num) ;
}

void PrimConverter::conv_uint32_t(uint32_t * num)
{
    if (convertDir==toXAP) 
		putXAP32(*num) ;
    else 
		*num = getXAP32() ;
}

void PrimConverter::conv_ptr(void * ptr)
{
    if (convertDir==toXAP) 
		putXAP16(0) ;
    else 
		getXAP16() ; //the pointer will be filled in later
}

void PrimConverter::conv_bool_t(bool_t * num)
{
    conv_uint8_t(num) ;
}

void PrimConverter::conv_enum(void * space)
{
    // Assume enum's are the sizeof unsigned ints, both here and on Xap
    unsigned int *real = (unsigned int *) space;
    if (convertDir==toXAP) 
		putXAP16(*real) ;
    else 
		*real = getXAP16() ;
}

void PrimConverter::startPointer(void**ptrptr,uint32_t size) 
{
	//ptrptr is a pointer to the pointer we need to set to the memory we're going to alloc
	ptrMode = 1 ;
	if (convertDir==fromXAP)
	{
		//iprintf(II("Allocating %d bytes \n"), size);
		if (size) 		*ptrptr = (void*) new uint8_t[size];
		else *ptrptr = 0 ;
	}
	else
	{
		if ((size+xapOffset) >= SIZE_OF_PRIM_BUF)
		{
			// need to alloc a bigger buf and copy converted stuff into it
			uint8_t *temp_xapPrim;
			temp_xapPrim = new uint8_t [(xapOffset + (size*2))];
			memcpy(temp_xapPrim, xapPrim, xapOffset);
			delete [] xapPrim;
			xapPrim = temp_xapPrim;
		}
	}
}

void PrimConverter::unsetPtrMode()
{
	ptrMode = 0;
}

void PrimConverter::conv_IP_ADDR (IP_ADDR * prim) 
 
{
	if(1)
	{
		// array template
		uint16_t m ;
		for (m=0 ;m <8 ; m++)
			conv_uint16_t(&prim->w [m]) ;
	}
}

