#include "inc.h"


uint16_t PrimConverter::namelen(const uint8_t *s, uint16_t n)
{
    if (s)
    {
        for(uint16_t i = 0; i < n && s[i]; ++i) ;
        return i;
    }
    return 0;
}

void PrimConverter::conv_SNIFF_SETTINGS(U_SNIFF_PARK_T u)
{
    conv_uint16_t(&u.sniff_settings.max_interval);    
	conv_uint16_t(&u.sniff_settings.min_interval);
	conv_uint16_t(&u.sniff_settings.attempt);
	conv_uint16_t(&u.sniff_settings.timeout);            
}


void PrimConverter::conv_PARK_SETTINGS(U_SNIFF_PARK_T u)
{
    conv_uint16_t(&u.park_settings.max_interval);
	conv_uint16_t(&u.park_settings.min_interval);         
	conv_uint32_t(&u.park_settings.park_idle_time);       
}


void PrimConverter::conv_customWriteLPSettings(void *prim) 
{
    DM_HCI_WRITE_LP_SETTINGS_T *lp_set = (DM_HCI_WRITE_LP_SETTINGS_T *) prim;
	
    if (lp_set->dm_policy == DM_LINK_POLICY_KEEP_SNIFF)
        conv_SNIFF_SETTINGS(lp_set->u);
    else if (lp_set->dm_policy == DM_LINK_POLICY_KEEP_PARKED)
        conv_PARK_SETTINGS(lp_set->u);
	else
		// we need to send the largest member of the union down
		// the wire, otherwise the sizeof() check fails and command is discarded
		conv_SNIFF_SETTINGS(lp_set->u);
}

void PrimConverter::conv_customCondition(void *prim) 
{
	DM_HCI_SET_EVENT_FILTER_T *con = (DM_HCI_SET_EVENT_FILTER_T *) prim;

	unsigned int type = (con->filter_type << 8) | con->filter_condition_type;
	unsigned short temp = 0;

	// bit of a hack - not only do we need to convert, but also make sure it's
    // the size of the largest member of the CONDITION union
	switch (type)
	{
		case 0x0101:
			conv_uint24_t(&con->condition.class_mask.class_of_device);
			conv_uint24_t(&con->condition.class_mask.mask);
			conv_uint16_t(&temp);
			break;
		case 0x0102:
			conv_BD_ADDR_T(&con->condition.bd_addr);
			conv_uint8_t((unsigned char *)&temp);
			break;
		case 0x0200:
			conv_uint8_t(&con->condition.auto_accept);
			conv_uint16_t(&temp);
			conv_uint16_t(&temp);
			conv_uint16_t(&temp);
			conv_uint16_t(&temp);
			break;
		case 0x0201:
			conv_uint24_t(&con->condition.cma.class_mask.class_of_device);
			conv_uint24_t(&con->condition.cma.class_mask.mask);
			conv_uint8_t(&con->condition.cma.auto_accept);
			conv_uint16_t(&temp);
			break;
		case 0x0202:
			conv_BD_ADDR_T(&con->condition.addr_auto.bd_addr);
			conv_uint8_t(&con->condition.addr_auto.auto_accept);
			break;
	}
}

void PrimConverter::conv_customHCI(void *prim)
{
	//called to handle primitives that are too complicated for 
	//autogenerated code.
	//At the moment, this means the name-related stuff

	// handles HCI_CHANGE_LOCAL_NAME_T
	HCI_CHANGE_LOCAL_NAME_T *spec_prim = (HCI_CHANGE_LOCAL_NAME_T *) prim;

	if (convertDir==toXAP)
	{        		
        // we can't rely on the prim length field to give us the correct len of
        // the name so allocate the max num of allowed bytes just in case
        uint8_t* temp_xapPrim;
        temp_xapPrim = new uint8_t [xapOffset + HCI_CHANGE_LOCAL_NAME_PARAM_LEN*sizeof(uint8_t) + 1];
        memmove(temp_xapPrim, xapPrim, xapOffset);
        delete [] xapPrim;
        xapPrim = temp_xapPrim;
        
        int arr_index=0;
        int chars_to_go = 0;
        uint16_t chars_copied = 0;
                
        chars_to_go = namelen(spec_prim->name_part[arr_index], HCI_LOCAL_NAME_BYTES_PER_PTR);
        
        while (chars_to_go)
        {
            // check we have not exceeded the char limit (currently 248) and
            // if we have truncate
            if ((chars_copied + chars_to_go) >= HCI_CHANGE_LOCAL_NAME_PARAM_LEN)
            {
                chars_to_go = HCI_CHANGE_LOCAL_NAME_PARAM_LEN - chars_copied;
            }
            chars_copied += chars_to_go;

            // copy the right number of bytes
            if (chars_to_go == HCI_LOCAL_NAME_BYTES_PER_PTR)
            {
                memcpy(xapPrim+xapOffset, 
                       spec_prim->name_part[arr_index], 
                       HCI_LOCAL_NAME_BYTES_PER_PTR);                
                xapOffset += HCI_LOCAL_NAME_BYTES_PER_PTR;
                arr_index++;				
            }
            else
            {
                memcpy(xapPrim+xapOffset, spec_prim->name_part[arr_index], chars_to_go);
                xapOffset += chars_to_go;                
            }

            // get the length of the next array of bytes only if we think there is one
            if (chars_to_go==HCI_LOCAL_NAME_BYTES_PER_PTR)
            {
                // the last array will not have the full HCI_LOCAL_NAME_BYTES_PER_PTR
                // bytes and may not be null terminated so we need to handle this
                if (arr_index == (HCI_LOCAL_NAME_BYTE_PACKET_PTRS-1))
                {
                    // last array is not full 
                    int to_read = HCI_CHANGE_LOCAL_NAME_PARAM_LEN - ((HCI_LOCAL_NAME_BYTE_PACKET_PTRS-1)*HCI_LOCAL_NAME_BYTES_PER_PTR); 
                    chars_to_go = namelen(spec_prim->name_part[arr_index], to_read);                    
                }
                else
                    chars_to_go = namelen(spec_prim->name_part[arr_index], HCI_LOCAL_NAME_BYTES_PER_PTR);
            }
            else
                break;
        }

        if (chars_copied<HCI_CHANGE_LOCAL_NAME_PARAM_LEN)
        {
            // null terminate the string
            xapPrim[xapOffset] = '\0';
            xapOffset++;
        }
	}
	else
	{
		//from XAP
        uint16_t bytes_left = 0;
        uint16_t chars_copied = 0;
        uint32_t prim_ix = xapOffset;
        while (xapPrim[prim_ix])
        {
            bytes_left++;
            prim_ix++;
            if (bytes_left >= HCI_CHANGE_LOCAL_NAME_PARAM_LEN)
                break;
        }           
		int array_idx=0;
		int pos_idx=0;

        if (bytes_left)
        {
            spec_prim->name_part[0] = new uint8_t[HCI_LOCAL_NAME_BYTES_PER_PTR];
            while (bytes_left)
            {			
                spec_prim->name_part[array_idx][pos_idx] = xapPrim[xapOffset]; 
                xapOffset++;
                pos_idx++;
                bytes_left--;

                if (pos_idx == HCI_LOCAL_NAME_BYTES_PER_PTR)
                {
                    array_idx++;
                    spec_prim->name_part[array_idx] = new uint8_t[HCI_LOCAL_NAME_BYTES_PER_PTR];
                    pos_idx = 0;
                }
            }
            if (pos_idx == 0 && array_idx < HCI_LOCAL_NAME_BYTE_PACKET_PTRS)
            {
                spec_prim->name_part[array_idx] = new uint8_t[HCI_LOCAL_NAME_BYTES_PER_PTR];
                spec_prim->name_part[array_idx][pos_idx] = '\0';
            }
        }
        else
        {
            if (array_idx < HCI_LOCAL_NAME_BYTE_PACKET_PTRS)
                spec_prim->name_part[0] = 0;
        }
    }
}


void PrimConverter::conv_customDM_HCI_REMOTE_NAME_COMPLETE(void *prim)
{
	//called to handle primitives that are too complicated for 
	//autogenerated code.
	//At the moment, this means the name-related stuff

	// handles DM_HCI_REMOTE_NAME_COMPLETE_T 
	DM_HCI_REMOTE_NAME_COMPLETE_T *spec_prim = (DM_HCI_REMOTE_NAME_COMPLETE_T *)prim;

    // get the number of bytes we need to copy
    uint16_t bytes_left = 0;
    uint16_t chars_copied = 0;
    uint32_t prim_ix = xapOffset;
    while (xapPrim[prim_ix])
    {
        bytes_left++;
        prim_ix++;
        if (bytes_left >= HCI_CHANGE_LOCAL_NAME_PARAM_LEN)
            break;
    }           
    int array_ix = 0;    
    uint16_t end_ix=0;
    
    while (bytes_left)
    {
        spec_prim->name_part[array_ix] = new unsigned char[HCI_LOCAL_NAME_BYTES_PER_PTR];                    
        chars_copied = 0;
        
        if (bytes_left >= HCI_LOCAL_NAME_BYTES_PER_PTR)
            chars_copied = HCI_LOCAL_NAME_BYTES_PER_PTR;                    
        else        
            chars_copied = bytes_left;                                                                        
        
        memcpy(spec_prim->name_part[array_ix], xapPrim+xapOffset, chars_copied);
        xapOffset += chars_copied;
        bytes_left -= chars_copied;
        array_ix++;                
    }            

    // null terminate the string
    if (chars_copied==HCI_LOCAL_NAME_BYTES_PER_PTR && array_ix < HCI_LOCAL_NAME_BYTE_PACKET_PTRS)
    {
        spec_prim->name_part[array_ix] = new unsigned char[HCI_LOCAL_NAME_BYTES_PER_PTR];
        spec_prim->name_part[array_ix][0] = '\0';        
    }    
    else
    {
        // last thing while loop does is incr the array count so we want to go back one
        spec_prim->name_part[(array_ix-1)][chars_copied] = '\0';        
    }   
    xapOffset++;
}


void PrimConverter::conv_customDM_HCI_READ_LOCAL_NAME_COMPLETE(void *prim)
{
	// handles DM_HCI_READ_LOCAL_NAME_COMPLETE_T
	DM_HCI_READ_LOCAL_NAME_COMPLETE_T *spec_prim = (DM_HCI_READ_LOCAL_NAME_COMPLETE_T *)prim;
    
    uint32_t bytes_left = lengthXAPprim - xapOffset;
    uint32_t chars_copied = 0;
    int array_ix = 0;

    while (bytes_left)
    {
        spec_prim->name_part[array_ix] = new unsigned char[HCI_LOCAL_NAME_BYTES_PER_PTR];
        
        if (bytes_left >= HCI_LOCAL_NAME_BYTES_PER_PTR)
            chars_copied = HCI_LOCAL_NAME_BYTES_PER_PTR;
        else
            chars_copied = bytes_left;
                    
        memcpy(spec_prim->name_part[array_ix], xapPrim+xapOffset, chars_copied);
        xapOffset += chars_copied;
        bytes_left -= chars_copied;        
        array_ix++;
    }
    spec_prim->name_part[array_ix] = 0;
}


void PrimConverter::conv_customWriteIacLap(void *prim)
{
    HCI_WRITE_CURRENT_IAC_LAP_T *spec_prim = (HCI_WRITE_CURRENT_IAC_LAP_T *)prim;
    
    if (convertDir==toXAP)
    {
        if ((unsigned int)((spec_prim->num_current_iac*sizeof(uint24_t))+xapOffset) > SIZE_OF_PRIM_BUF)
		{
			// need to alloc a bigger buf and copy converted stuff into it
			uint8_t* temp_xapPrim;
			temp_xapPrim = new uint8_t [xapOffset + (spec_prim->num_current_iac*sizeof(uint24_t))];
			memmove(temp_xapPrim, xapPrim, xapOffset);
			delete [] xapPrim;
			xapPrim = temp_xapPrim;
		}

        int arr_index=0;
		int posn_index=0;
		
		for (int d=0; d <= spec_prim->num_current_iac; d++)
		{
			if (posn_index >= HCI_IAC_LAP_PER_PTR)
			{
				arr_index++;
				posn_index = 0;
			}
            conv_uint24_t(&spec_prim->iac_lap[arr_index][posn_index]);			
			posn_index++;
		}		
	}
	else
	{
		//from XAP
		int array_idx=0;
		int pos_idx=0;
        spec_prim->iac_lap[0] = new uint24_t[HCI_IAC_LAP_PER_PTR];

		for (int d=0; d<spec_prim->num_current_iac; d++)
		{
			if (pos_idx >= HCI_IAC_LAP_PER_PTR)
			{
				array_idx++;
				spec_prim->iac_lap[array_idx] = new uint24_t[HCI_IAC_LAP_PER_PTR];
				pos_idx = 0;
			}
            conv_uint24_t(&spec_prim->iac_lap[array_idx][pos_idx]);
			pos_idx++;
		}
	}
}