/******************************************************************************
**
**   File:	packer_parameter.h
**
**            (C) CSR 2000
**
**   Purpose:	Define the interface for passing parameters to the
**              HCI pdu packing routines
**
**  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/Packer/packer_parameters.h#15 $
**
******************************************************************************/

/****************************************************************************
**  Modification history
***************************************************************************** 
**  1.1  22:june:00	ajh	Created.
**  1.2   9:aug:00      ajh     Carl and Neil have agreed a data parameter
**                              format.  Time for a rewrite.
**  1.3  22:aug:00      ajh     Added #defines in HCIPacker_Interpretation.h
**                              which make interpretation of the packed info
**                              easier to understand.  Maybe.
*/


/****************************************************************************
DATA STRUCTURE
        PARAMETER_ARRAY

        An array of uint32s.
        The first contains length data (ie the size of the array, one
        more than the number of data elements.).
        All data is packed as uint8s, uint16s, uint24s or uint32s
        into uint32s.
        Bluetooth device addresses are treated as Lap, Uap, Nap.
        Link keys, event masks, pin codes, local names are wastefully
        kept as arrays of uint8s spread into the array of uint32s.
****************************************************************************/

#ifndef __PACKER_PARAMETERS_H__
#define __PACKER_PARAMETERS_H__

#include "common/types.h"
#include "HCI/HCILinkage.h"
#include "HCIPacker_Interpretation.h"
#include "common/bool.h"

typedef uint32 PARAMETER_VALUE;

typedef PARAMETER_VALUE* PARAMETER_ARRAY;

/****************************************************************************
SUPPORTING FUNCTIONS
        encodeBytes
        decodeBytes

        To make it easy (or at least unnecessary) to worry about packing
        uint8 arrays into the array, there are these two functions
        to do it.

        Pass a pointer and a size to encodeBytes, and it'll put them into
        your array, checking first to make sure there is enough space.

        Pass a pointer to an address of a buffer, and the size of that buffer
        to decodeBytes and it will read as much data as it can (limited by
        the size of data in the array and the size of the buffer it is given.
        If you pass it a pointer to a NULL, and/or a tell it the length is
        zero, then it changes the null to a pointer to a "new uint8[]" bit
        of memory, and puts the whole of the rest of the array into the buffer.
****************************************************************************/

DEVICECONTROLLER_API bool   encodeBytes ( uint8 *aDataStart ,
                             uint32 aDataLength ,
                             PARAMETER_ARRAY aParameterArray ,
                             PARAMETER_ARRAY *aCurrentIndex );

DEVICECONTROLLER_API uint32 decodeBytes ( uint8 **aPointerToBuffer ,
                             uint32  aLengthOfReceivingBuffer ,
                             PARAMETER_ARRAY aParameterArray ,
                             PARAMETER_ARRAY *aCurrentOutdex );

/*
**   Bad pdu types (keep in sync with devHost/engine/hcitransport/pduapi.h)
*/

typedef enum BadPDUReason_TAG
{
    BP_HCI_WRONG_SIZE ,
    BP_PC_TOO_SMALL ,
    BP_NO_SUCH_VARID ,
    BP_NO_SUCH_EVENT_CODE ,
    BP_NO_SUCH_COMMAND ,
    BP_OUT_OF_DATA ,
    BP_WARNING_UNKNOWN_FORMAT,
    BP_HCI_DATA_RESERVED_PBF,
    BP_HCI_DATA_WRONG_SIZE,
    BP_HCI_DATA_RESERVED_BCF,
    BP_WRONG_TYPE,
    BP_UNHANDLED_CHANNEL
}
BadPDUReason;

#endif /* __PACKER_PARAMETERS_H__ */
