///////////////////////////////////////////////////////////////////////
//
//  FILE   :  HCITransport.cpp
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  HCITransport
//
//  PURPOSE:  Win32 plain vanilla HCI transport interface to BCSP.
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/TransportLayer/HCITransport.cpp#14 $
//
///////////////////////////////////////////////////////////////////////

#include "HCITransport.h"
#include "callBack.h"

#include <string.h>

HCITransport::HCITransport()
{
    mComCallBackObject = 0 ;
    mComCallBackFunction = 0 ;
    mACLCallBackObject = 0 ;
    mACLCallBackFunction = 0 ;
    mSCOCallBackObject = 0 ;
    mSCOCallBackFunction = 0 ;
    mName = 0;
}

HCITransport::~HCITransport()
{
}

ichar * HCITransport::getName()
{
    //  Get a copy of the name
    int length = istrlen (mName);
    if ( length != 0 )
    {
        ichar *copyOfName = new ichar[length];
        istrcpy (copyOfName,mName);
        return copyOfName;
    }
    return 0;
}

//  Allow zero objects or callbacks.
//  This is so we can disable callbacks easily.
void HCITransport::setupRecvCommand ( void * aObject,
                        void (*aRecvCallBack) ( uint8 * , uint32 , void * ) )
{
    mComCallBackObject = aObject ;
    mComCallBackFunction = aRecvCallBack ;
}

void HCITransport::setupRecvACLData ( void * aObject, 
                        void (*aRecvCallBack) ( uint8 * , uint32 , void * ) )
{
    mACLCallBackObject = aObject ;
    mACLCallBackFunction = aRecvCallBack ;
}

void HCITransport::setupRecvSCOData ( void * aObject,
                        void (*aRecvCallBack) ( uint8 * , uint32 , void * ) )
{
    mSCOCallBackObject = aObject ;
    mSCOCallBackFunction = aRecvCallBack ;
}

void HCITransport::recvCommand ( uint8 *apdu, uint32 aLength )
{
    if ( mComCallBackFunction != 0 )
        mComCallBackFunction ( apdu , aLength , mComCallBackObject ) ;
    else
        delete apdu;
}

void HCITransport::recvACLData ( uint8 *apdu, uint32 aLength )
{
    if ( mACLCallBackFunction != 0 )
        mACLCallBackFunction ( apdu , aLength , mACLCallBackObject ) ;
    else
        delete apdu;
}

void HCITransport::recvSCOData ( uint8 *apdu, uint32 aLength )
{
    if ( mSCOCallBackFunction != 0 )
        mSCOCallBackFunction ( apdu , aLength , mSCOCallBackObject ) ;
    else
        delete apdu;
}
