///////////////////////////////////////////////////////////////////////
//
//  FILE   :  HCITransport.h
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  HCITransport
//
//  PURPOSE:  Provide standard API to transport layer.
//            Uses the "one-time" CallBack objects to return transmission
//            info, and needs the creating application to set up permanent
//            callbacks for events and data.
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/TransportLayer/HCITransport.h#19 $
//
///////////////////////////////////////////////////////////////////////

#ifndef __HCITRANSPORT_H__
#define __HCITRANSPORT_H__

#include "HCI/HCILinkage.h"
#include "common/types.h"
#include "callBack.h"
#include "unicode/ichar.h"

enum startTransportReturnValue { OK , serialProblem , otherProblem };

class DEVICECONTROLLER_API_CLASS HCITransport
{
public:
    HCITransport();
    virtual ~HCITransport();

    virtual startTransportReturnValue start ( void ) = 0;
    virtual bool ready ( uint32 timeOut ) = 0;
    virtual void stop ( void ) = 0;

    virtual void sendCommand ( uint8 *apdu, uint16 aLength , CallBack *aCallBack ) = 0;
    virtual void sendACLData ( uint8 *apdu, uint32 aLength , CallBack *aCallBack ) = 0;
    virtual void sendSCOData ( uint8 *apdu, uint16 aLength , CallBack *aCallBack ) = 0;

    void setupRecvCommand ( void * aObject,
                            void (*aRecvCallBack) ( uint8 * , uint32 , void * ) ); // HCI
    void setupRecvACLData ( void * aObject, 
                            void (*aRecvCallBack) ( uint8 * , uint32 , void * ) ); // ACL
    void setupRecvSCOData ( void * aObject,
                            void (*aRecvCallBack) ( uint8 * , uint32 , void * ) ); // SCO

	virtual void set_sco_bandwidth ( int bytes_per_second, void *additional_info, int additional_info_size ) = 0;
    // get a copy of the name.
    ichar * getName();

protected:
    void recvCommand ( uint8 *apdu, uint32 aLength );
    void recvACLData ( uint8 *apdu, uint32 aLength );
    void recvSCOData ( uint8 *apdu, uint32 aLength );

    //  name the type of transport layer in use.
    ichar * mName ;

private:
    //  Pointers to objects and callbacks.
    //  The callbacks must be declared static in the object,
    void *mComCallBackObject;
    void (*mComCallBackFunction) ( uint8 *, uint32 , void * );
    void *mACLCallBackObject;
    void (*mACLCallBackFunction) ( uint8 *, uint32 , void * );
    void *mSCOCallBackObject;
    void (*mSCOCallBackFunction) ( uint8 *, uint32 , void * );

#ifdef _INDEBUG
    //  release debugging!
public:
    indebug * mDebug;
#else
    //  keep the size of the object the same.
    void * dummy;
#endif
};

#endif // __HCITRANSPORT_H__
