///////////////////////////////////////////////////////////////////////
//
//  FILE   :  localTypes.cpp
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  PURPOSE:  Define the generic types for the HCI device Driver.
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/localTypes.cpp#16 $
//
///////////////////////////////////////////////////////////////////////

#include "localTypes.h"
#include "memfuncs.h"

#include <stdlib.h>

///////////////////////////////////////////////////////////////////////
//
//  CLASS:    Genericpdu
//
//  PURPOSE:  to hold a pdu and its length safely.
//
///////////////////////////////////////////////////////////////////////

Genericpdu::Genericpdu()
{
    mLength = 0;
    flowControlled = true;
}

Genericpdu::~Genericpdu()
{
    if (mLength != 0)
    {
        freepdu ( mBuffer );
        mBuffer = NULL;
        mLength = 0;
    }
}

void Genericpdu::disownData ( void )
{
    mLength = 0;
}

///////////////////////////////////////////////////////////////////////
//
//  CLASS:    Datapdu
//
//  PURPOSE:  to hold a data pdu, with its channel safely.
//
///////////////////////////////////////////////////////////////////////

Datapdu::Datapdu()
{
}

Datapdu::~Datapdu()
{
}

void Datapdu::size(uint32 aLength)
{
    mBuffer = ( uint8* ) getpdu ( aLength );
    mLength = aLength;
}

///////////////////////////////////////////////////////////////////////
//
//  CLASS:    Commandpdu
//
//  PURPOSE:  to hold a command pdu and whether it was internally
//            or externally generated.
//
///////////////////////////////////////////////////////////////////////

Commandpdu::Commandpdu()
{
}

Commandpdu::Commandpdu( Eventpdu * hq )
{
   // Hand the pdu across
    mLength = hq->mLength;
    mBuffer = hq->mBuffer;
    internallyGeneratedCommand = true;
    hq->disownData();
}

Commandpdu::~Commandpdu()
{
}

///////////////////////////////////////////////////////////////////////
//
//  CLASS:    Eventpdu
//
//  PURPOSE:  to hold an event pdu 
//
///////////////////////////////////////////////////////////////////////

Eventpdu::Eventpdu()
{
}

Eventpdu::~Eventpdu()
{
}

///////////////////////////////////////////////////////////////////////
//
//  CLASS:    Badpdu
//
//  PURPOSE:  to hold a bad pdu 
//
///////////////////////////////////////////////////////////////////////

Badpdu::Badpdu()
{
}

Badpdu::~Badpdu()
{
}


///////////////////////////////////////////////////////////////////////
//
//  CLASS:    DMpdu
//
//  PURPOSE:  to hold a DM pdu 
//
///////////////////////////////////////////////////////////////////////

DMpdu::DMpdu()
{
	flowControlled = false;
}

DMpdu::~DMpdu()
{
}


///////////////////////////////////////////////////////////////////////
//
//  CLASS:    RFCOMMpdu
//
//  PURPOSE:  to hold a RFCOMM pdu 
//
///////////////////////////////////////////////////////////////////////

RFCOMMpdu::RFCOMMpdu()
{
	flowControlled = false;
}

RFCOMMpdu::~RFCOMMpdu()
{
}


///////////////////////////////////////////////////////////////////////
//
//  CLASS:    SDPpdu
//
//  PURPOSE:  to hold a SDP pdu 
//
///////////////////////////////////////////////////////////////////////

SDPpdu::SDPpdu()
{
	flowControlled = false;
}

SDPpdu::~SDPpdu()
{
}


///////////////////////////////////////////////////////////////////////
//
//  CLASS:    L2CAPpdu
//
//  PURPOSE:  to hold a L2CAP pdu 
//
///////////////////////////////////////////////////////////////////////

L2CAPpdu::L2CAPpdu()
{
	flowControlled = false;
}

L2CAPpdu::~L2CAPpdu()
{
}
