///////////////////////////////////////////////////////////////////////
//
//  FILE   :  PosixH4
//
//  AUTHOR :  Mark Marshall
//
//  CLASS  :  PosixH4
//
//  PURPOSE:  Posix-specific portion of H4 implementation of HCITransport
//
//  NOTES  :  I have used Pointer-To-Implimentation because I like it,
//            so there!
//            The virtual keyword is unneeded in this file, but I like
//            to include as much extra text as possible.  We now need
//            a constructor, which should be ok.
//
//            We will only return an RX'd event if we have read the
//            number of bytes that was asked for.  This might mean
//            that we have to make a number of reads.
//
//            There are now TWO classes in this file.  This is for a
//            very good reason.  It turns out that it is very easy to
//            frig the BlueZ USB driver.  It then presents packets
//            that are the same as those obtained by H4.  We can
//            therefore reuse all of the code from the H4 Layer for a
//            Linux/BlueZ layer.  The only code that is different is
//            the init / final code, which is the only real code in
//            the PosixH4 class.
//
///////////////////////////////////////////////////////////////////////

#ifndef PosixH4_H__
#define PosixH4_H__

#include "H4Stack.h"

class H4_API_CLASS PosixH4Base : public H4Stack
{
 public:
    PosixH4Base();
    virtual ~PosixH4Base();

 private:
    virtual bool initEvents();
    virtual H4_EVENT waitForEvent();
    virtual void finalEvents();

    virtual bool txData (uint32 len, const uint8 *buf);
    virtual bool rxData (uint32 len, uint8 *buf);

    virtual bool initSerialDaemon();
    virtual void notifySerialDaemon(H4_EVENT event);
    virtual void finalSerialDaemon();

    virtual bool initCriticalSection();
    virtual void enterCriticalSection();
    virtual void exitCriticalSection();
    virtual void finalCriticalSection();

    /* This function is only called by one of the two derived classess
       PosixH4 or LinuxUSB. */
    virtual void flushHandle(int handle) = 0;

 protected:
    /* Set handle - This function is called with the handle that we
       will use for the read and write operations. */
    void setHandle(int h);
    int getHandle();

 private:
    class Impl;
    friend class Impl;

    Impl &impl_;
};

class H4_API_CLASS PosixH4 : public PosixH4Base
{
 private:
    virtual startReturnValue initSerial(const ichar *portName,
					uint32 datarate);
    virtual startReturnValue initSerial(void *portHandle);

    virtual void finalSerial();

    virtual void flushHandle(int handle);
};

#endif // PosixH4_H__
