#include "inc.h"


uint16_t WinPrimDebug::namelen(const uint8_t *s, uint16_t n)
{
    if (s)
    {
        for(uint16_t i = 0; i < n && s[i]; ++i) ;
        return i;
    }
    return 0;
}


void WinPrimDebug::debug_customWriteLPSettings(void *prim, int indent, std::string& monster_buffer) 
{
    DM_HCI_WRITE_LP_SETTINGS_T *lp_set = (DM_HCI_WRITE_LP_SETTINGS_T *) prim;
	
    if (lp_set->dm_policy == DM_LINK_POLICY_KEEP_SNIFF)
    {
        debug_uint16_t(indent+1, &lp_set->u.sniff_settings.max_interval, monster_buffer, "sniff_settings.max_interval = ");    
        debug_uint16_t(indent+1, &lp_set->u.sniff_settings.min_interval, monster_buffer, "sniff_settings.min_interval = ");
        debug_uint16_t(indent+1, &lp_set->u.sniff_settings.attempt, monster_buffer, "sniff_settings.attempt = ");
        debug_uint16_t(indent+1, &lp_set->u.sniff_settings.timeout, monster_buffer, "sniff_settings.timeout = ");                    
    }
    else if (lp_set->dm_policy == DM_LINK_POLICY_KEEP_PARKED)
    {
        debug_uint16_t(indent+1, &lp_set->u.park_settings.max_interval, monster_buffer, "park_settings.max_interval = ");
	    debug_uint16_t(indent+1, &lp_set->u.park_settings.min_interval, monster_buffer, "park_settings.min_interval = ");         
	    debug_uint32_t(indent+1, &lp_set->u.park_settings.park_idle_time, monster_buffer, "park_settings.park_idle_time = ");       
    }       
}

void WinPrimDebug::debug_printName(uint8_t **name_part, std::string& monster_buffer)
{
    int len = 0;
    int array_ix = 0;            
    add_str_to_monster_buf(1, "name_part = ", monster_buffer);
        
    uint8_t *tmp_buf = *(name_part+array_ix);

    // get the length of the first buffer
    if (tmp_buf)
        len = namelen(tmp_buf, HCI_LOCAL_NAME_BYTES_PER_PTR);
    else
        len = 0;
    
    // while we have allocated buffers copy their data to the debug string
    while (len)
    {           
        monster_buffer.append((char *) tmp_buf, len);                
        monster_buffer.append((char)'\0', 1);
        array_ix++;
        
        if (array_ix < HCI_LOCAL_NAME_BYTE_PACKET_PTRS)
            tmp_buf = *(name_part+array_ix);
        else
            break;

        // get the len of the next buffer if it exists        
        if (tmp_buf && len == HCI_LOCAL_NAME_BYTES_PER_PTR)
        {
            if (array_ix == (HCI_LOCAL_NAME_BYTE_PACKET_PTRS-1))
            {
                // last array is not full 
                int to_read = HCI_CHANGE_LOCAL_NAME_PARAM_LEN - ((HCI_LOCAL_NAME_BYTE_PACKET_PTRS-1)*HCI_LOCAL_NAME_BYTES_PER_PTR); 
                len = namelen(tmp_buf, to_read);
            }
            else
                len = namelen(tmp_buf, HCI_LOCAL_NAME_BYTES_PER_PTR);
        }
        else
            len = 0;
    } 		    
}


void WinPrimDebug::debug_customDebug(void *prim, std::string& monster_buffer)
{
    uint16_t *type = (uint16_t *)prim;
    
    switch (*type)
	{
	case DM_HCI_CHANGE_LOCAL_NAME:
		{
			// handles HCI_CHANGE_LOCAL_NAME_T
			HCI_CHANGE_LOCAL_NAME_T *spec_prim = (HCI_CHANGE_LOCAL_NAME_T *) prim;                        
            debug_printName((uint8_t **)&(spec_prim->name_part), monster_buffer);
		}
		break;

	case DM_HCI_REMOTE_NAME_COMPLETE:
		{
			// handles DM_HCI_REMOTE_NAME_COMPLETE_T 
			DM_HCI_REMOTE_NAME_COMPLETE_T *spec_prim = (DM_HCI_REMOTE_NAME_COMPLETE_T *)prim;
	        debug_printName((uint8_t **)&(spec_prim->name_part), monster_buffer);		            
		}
		break;

	case DM_HCI_READ_LOCAL_NAME_COMPLETE:
		{
			// handles DM_HCI_READ_LOCAL_NAME_COMPLETE_T
			DM_HCI_READ_LOCAL_NAME_COMPLETE_T *spec_prim = (DM_HCI_READ_LOCAL_NAME_COMPLETE_T *)prim;
            debug_printName((uint8_t **)&(spec_prim->name_part), monster_buffer);             
		}
		break;

    case HCI_WRITE_CURRENT_IAC_LAP:
        {
            // handles HCI_WRITE_CURRENT_IAC_LAP_T
            HCI_WRITE_CURRENT_IAC_LAP_T *spec_prim = (HCI_WRITE_CURRENT_IAC_LAP_T *) prim;
            int num_arrays = spec_prim->num_current_iac/HCI_IAC_LAP_PER_PTR;
            unsigned int num_recs = spec_prim->num_current_iac;
            
            for (int d=0; d <= num_arrays; d++)
            {                
                for (int t=0; t < HCI_IAC_LAP_PER_PTR; t++)
                {
                    if (num_recs)
                    {                        
                        debug_uint24_t(1, &spec_prim->iac_lap[d][t], monster_buffer, "iac_lap = ");                   
                        num_recs--;
                    }
                    else
                        break;
                }
            }
        }
        break;
    
    default:
        monster_buffer += "Unknown customDebug Primitive type\n";
    } // end switch
    monster_buffer += "\n";
}
    