// Implementation of a condition variable using boost

#include "thread_impl.h"

#include "boost/thread/xtime.hpp"


void Condition::Impl::wait(Mutex::Lock::Impl &x)
{
    real_condition.wait(x.real_lock);
}


bool Condition::Impl::wait(Mutex::Lock::Impl &x, uint32 udelay)
{
    boost::xtime end;
    boost::xtime_get(&end, boost::TIME_UTC);

    const uint32 m = 1000000ul;
    const uint32 n = m * 1000;
    uint32 t = end.nsec + (udelay % m) * 1000;
    end.sec  += t / n + udelay / m;
    end.nsec  = t % n;

    return real_condition.timed_wait(x.real_lock, end);
}

void Condition::Impl::notifyOne()
{
    real_condition.notify_one();
}

void Condition::Impl::notifyAll()
{
    real_condition.notify_all();
}

Condition::Condition()
    : impl(new Impl)
{ }

Condition::~Condition()
{ delete impl; }

void Condition::wait(Mutex::Lock &x)
{ impl->wait(*x.impl); }

bool Condition::wait(Mutex::Lock &x, uint32 delay)
{ return impl->wait(*x.impl, delay); }

void Condition::notifyOne()
{ impl->notifyOne(); }

void Condition::notifyAll()
{ impl->notifyAll(); }
