///////////////////////////////////////////////////////////////////////
//
//  FILE   :  critical_section.h
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  PURPOSE:  the header file for all the machine dependent
//            definitions of Critical section control functions.
//            this bit should be the same for all of them.
//
//  $Id: //depot/bc/main/devHost/util/thread/critical_section.h#7 $
//
///////////////////////////////////////////////////////////////////////

#ifndef CRITICAL_SECTION_H
#define CRITICAL_SECTION_H

#include "common/nocopy.h"

class CriticalSection : private NoCopy
{
public:
    CriticalSection()
        : cs(InitialiseCS())
    { }

    ~CriticalSection()
    { DestroyCS(cs); }

    class Lock;
    friend class Lock;

    class Lock : private NoCopy
    {
    public:
        Lock(CriticalSection &x)
            : locked(x)
        { EnterCS(locked.cs); }

        Lock(CriticalSection *x)
            : locked(*x)
        { EnterCS(locked.cs); }

        ~Lock()
        { LeaveCS(locked.cs); }

    private:
        CriticalSection &locked;
    };

    class AntiLock;
    friend class AntiLock;

    class AntiLock : private NoCopy
    {
    public:
        AntiLock(CriticalSection &x)
            : locked(x)
        { LeaveCS(locked.cs); }

        AntiLock(CriticalSection *x)
            : locked(*x)
        { LeaveCS(locked.cs); }

        ~AntiLock()
        { EnterCS(locked.cs); }

    private:
        CriticalSection &locked;
    };
private:
    void *cs;

    // Platform dependent code
    static void *InitialiseCS();
    static void EnterCS(void *section);
    static void LeaveCS(void *section);
    static void DestroyCS(void *section);
};

#endif /* ndef CRITICAL_SECTION_H */
