// Win32 implememtation of the Mutex thread.  Uses the CreateMutex function
// 
// Modification History
// $Log: system_wide_mutex.cpp,v $
// Revision 1.1  2001/10/17 17:18:26  mm01
// First go at this.  not tested!
//
// Revision 1.1  2001/10/11 16:44:56  bs01
// Rename from mutex to system_wide_mutex (the shorter name could have cause people to use it inappropriately and get unintended interactions between separate processes.)
//
// Revision 1.5  2001/10/11 10:13:52  cl01
// Added code to prefix any mutex name.
//
// Revision 1.4  2001/10/10 17:26:17  cl01
// Added code to prefix any mutex name.
//
// Revision 1.3  2001/10/10 11:53:47  cl01
// Uses Semaphore as underlying sync obj.  Mutex had problems when moved between threads.
//
// Revision 1.2  2001/10/09 18:03:47  cl01
// Added Exclusive Access for Widcomm USB driver.
//
//
// $Id: //depot/bc/main/devHost/util/thread/posix/system_wide_mutex.cpp#3 $


#include "thread/system_wide_mutex.h"

#include <sys/ioctl.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>

// m_name is the name of the temp file that we are pretending is a mutex
// m_acquired is the golden version of this bool.  The bool in the generic
// wrapper mirrors this one (they should both be the same).  The destructor
// of the MutexData needs to know whether we own the mutex or not.
class SystemWideMutex::MutexData
{
public:
    MutexData(const char * mutex_name);
    ~MutexData();

    char* m_name;
    bool m_acquired;
};


SystemWideMutex::MutexData::MutexData(const char *mutex_name)
{
    static const char prefix[] = "/tmp/csr_mutex_";
    m_name = (char *) malloc(strlen(mutex_name) + strlen(prefix) + 1);
    char *sp = m_name;
    strcpy(m_name, prefix);
    strcat(m_name, mutex_name);
    while (*sp)
    {
        if (strchr(".\\/\"*", *sp))
	    *sp = '-';
	sp++;
    }
    m_acquired = false;
}
   

SystemWideMutex::MutexData::~MutexData()
{
    if(m_acquired)
    {
        unlink(m_name);
    }
    free(m_name);
}


SystemWideMutex::SystemWideMutex(const char *mutex_name, bool AcquireImmed)
    : data_(new MutexData(mutex_name))
{
    if (AcquireImmed) Acquire(0);
}


SystemWideMutex::~SystemWideMutex()
{
    Release();
    delete data_;
}

// Not sure what to do about the time ?
bool SystemWideMutex::Acquire(uint32 dwMilliseconds)
{
    char buf[80];
    int fd;

    if(data_->m_acquired)
        return true;

    while(1)
    {
        if ((fd = open(data_->m_name, O_CREAT | O_WRONLY | O_TRUNC | O_EXCL, 0666)) > 0)
        {
	    sprintf(buf, "%d\n", getpid());
	    write(fd, buf, strlen(buf));
	    close(fd);

	    data_->m_acquired = true;

	    return data_->m_acquired;
	}
	else
	{
	    if (errno != EEXIST)
	    {
	        printf("Could not create pid file for SystemWideMutex.\n");
		exit(0);
	    }
	}

	// If the time is zero then we have failed to acquire the mutex
	// Return false
	if(dwMilliseconds == 0)
	    return false;

	// We are going to sleep in secound intervals (at the moment)
	if(dwMilliseconds >= 1000)
	    dwMilliseconds -= 1000;
	else
	    dwMilliseconds = 0;

	// Pause for a secound
	sleep(1);
    }
}

void SystemWideMutex::Release()
{
    if (data_->m_acquired)
    {
        unlink(data_->m_name);
    }
    data_->m_acquired = false;
}

bool SystemWideMutex::IsAcquired() const
{
    return data_->m_acquired;
}
