///////////////////////////////////////////////////////////////////////////////
//
//  FILE    :    dichar_ptr.cpp
//
//  Copyright CSR 2001
//
//  CLASS   :    dichar_ptr
//
//  PURPOSE :    to implement an "ichar *" type which is dually char and 
//               wchar_t, both at the same time in terms of typing, but not
//               data.  The class contains two strings one of which is always
//               empty.
//
///////////////////////////////////////////////////////////////////////////////

#include "dichar_ptr.h"
#include <string.h>
#include <wchar.h>
#include <assert.h>

static char * clone ( const char * aString )
{
    if ( aString )
        return strcpy ( new char   [strlen (aString) + 1] , aString );
    else
        return 0;
}

static wchar_t * clone ( const wchar_t * aString )
{
    if ( aString )
        return wcscpy ( new wchar_t[wcslen (aString) + 1] , aString );
    else
        return 0;
}

dichar_ptr::dichar_ptr ( const char * const aString )
: mStringA ( clone (aString) ) ,
  mStringW ( 0 )
{
    assert ( aString );
}

dichar_ptr::dichar_ptr ( const wchar_t * const aString )
: mStringA ( 0 ) ,
  mStringW ( clone (aString) )
{
    assert ( aString );
}

dichar_ptr::~dichar_ptr ()
{
    delete[] mStringA;
    delete[] mStringW;
}

dichar_ptr::dichar_ptr ( const dichar_ptr & aFrom )
: mStringA ( clone (aFrom.mStringA) ) ,
  mStringW ( clone (aFrom.mStringW) )
{
}

const char * dichar_ptr::getA () const
{
    assert ( mStringA );
    return mStringA;
}

const wchar_t * dichar_ptr::getW () const
{
    assert ( mStringW );
    return mStringW;
}

void dichar_ptr::get ( const char *& aString ) const
{
    aString = getA();
}

void dichar_ptr::get ( const wchar_t *& aString ) const
{
    aString = getW();
}

/*
distring & distring::operator= ( const distring & aFrom )
{
    abort();
    return *this;
}
*/
