##############################################################################
#
# Slurp up a header file and generate a list of id to type mappings
#
# Relies on the fact that CCL headers have #define ID blah in them
# and that the primitive for id X is called  X_T
#
# JBS, 3 Sep 2002. Extend to cope with enum's for the message ids.
#
##############################################################################

use Getopt::Std ;

getopts('p:f:') ;

%ids ;
%typedefs ;
$prefix      = $opt_p ;
$inside_enum = 0;

open input,$opt_f or die "ERROR: couldn't open file '$opt_f'" ;

while (<input>)
{
    if(/^\s* enum \s+ $prefix MSG_TYPE \s*$/x)
    {
        $inside_enum = 1;
    }
    elsif($inside_enum)
    {
        if(/^\s*\}\s*;\s*/x)
        {
            $inside_enum = 0;
        }
        elsif(/^\s* (ENUM_)? ($prefix \w+)/x)
        {
            $ids{$2} = "-" ;
        }
    }
    elsif (/\#define \s+ ($prefix \w+) \s+ .+/x)
    {
        #add this type to those we know about
        $ids{$1} = "-" ;
    }
    elsif (/typedef \s+ (\w+) \s+ (\w+) \s* ;/x)
    {
        #this is probably only going to happen for dm primitives which
        #are typedef'd across to HCI primitives
        $typedefs{$2} = $1 ;
    }
}
close input ;

#now lets try and produce a mapping of ids to types:

foreach $id (keys %ids)
{
    $type = "$id" . "_T" ;
    while($typedefs{$type})
    {	
        $type = $typedefs{$type} ;
    }
    print "$id $type\n" ;
}
