
############################################################
#
#  reduces typedef'd types to their simplest base type
#
#
############################################################

use Getopt::Std ;

        # the ccl coding convention is rather
	#conveniently that structs are defined like
	#typedef struct
	#{
	#} type ;
	# so we take advantage of this



%baseTypes ;
$basicTypes ="unknown|uint8_t|uint16_t|uint32_t|uint24_t|bool_t";


#turn the entire file into a single string


sub slurpFile
{
   my $concat ="" ;
   open prims,@_[0] ;
   while (<prims>)
   {
        $concat .= $_ ;
   }
   close prims ;

   $concat =~ s/\/\*.*?\*\///gs ; #strip out C comments

   my $in = $concat ;

   while ($in =~ /typedef\s+(\w+)\s+(\w+)\s*;/gs)
   {
     my $baseType = $1 ;
     my $type = $2 ;
     $baseTypes{$type} = $baseType ;
   }
}


#now recursively try and nail down basictypes


sub track
{
  my $type = $_[0] ;
  my $depth = $_[1] ;
  my $indent = "." x $depth ;
  #print "$indent ($depth) $type\n" ;
  die "nesting ($depth) too deep tracking $type" if ($depth > 10) ;
  return "unknown" if ($type eq "");
  my $base = $baseTypes{$type} ;
  return $base if ($base =~ /$basicTypes/) ;
  $baseTypes{$type} = track($base,$depth+1) ;
  return $baseTypes{$type} ;
}


sub slurpList
{
   open listfile,@_[0] ;
   while (<listfile>)
   {
        chomp ;
        slurpFile($1) if (/(\S+)/) ;
   }
   close listfile ;
}

sub main
{
   getopts('uUl:') ;
   slurpList($opt_l) if ($opt_l) ;

   $baseTypes{"uint8"}  = "uint8_t";
   $baseTypes{"uint16"} = "uint16_t";
   $baseTypes{"uint24"} = "uint24_t";
   $baseTypes{"uint32"} = "uint32_t";
   $baseTypes{"bool"}   = "bool_t";

   if (!$opt_U)
   {
       print "uint8_t uint8_t\n" ;
       print "uint16_t uint16_t\n" ;
       print "uint24_t uint24_t\n" ;
       print "uint32_t uint32_t\n" ;
       print "bool_t bool_t\n" ;
   }

   foreach $type (keys %baseTypes)
   {
       $base = track($type,1) ;
       if ($base eq "unknown")
       {
           print "$type has unknown base type\n" if ($opt_u or $opt_U) ;
       }
       else
       {
           print "$type $base\n" if (!$opt_U);
       }
   }
}

main() ;
