/******************************************************************************

    (C) COPYRIGHT Cambridge Consultants Ltd 1998

    CCL Project Reference C6066 - Lindisfarne

FILE
        $Workfile: sdc_prim.h $
        $Revision: #6 $
        $Author: bcsw $
    

ORIGINAL AUTHOR
        Rob Davis

CONTAINS
        SDA/SDC Interface Primitives.

DESCRIPTION
    The interface between the Service Discovery Application (SDA) and the 
        Service Discovery Client (SDC) sub-system.
        
NOTES/LIMITATIONS
    Based on Draft Version 1.0b of the Bluetooth specification


REFERENCES
    c6066-s-001 Bluetooth Software Architecture
    c6066-s-003 L2CAP Subsystem design
    c6066-s-008 Service Discovery Subsystem design
  

******************************************************************************/
#ifndef BLUESTACK__SDC_PRIM_H     /* Once is enough */
#define BLUESTACK__SDC_PRIM_H

#ifdef RCS_STRINGS
static const char* const sdc_prim_h_id = 
"$Header: //depot/bc/main/interface/app/bluestack/sdc_prim.h#6 $";
#endif

/*============================================================================*
                    Header files on which this one depends
 *============================================================================*/

/*============================================================================*
                       Interface #defined parameters
 *============================================================================*/

/*------------------------------------------------
     SDC open search result codes
--------------------------------------------------*/
#define SDC_OPEN_SEARCH_OK         0x00  /* Open search succeeded */
#define SDC_OPEN_SEARCH_BUSY       0x01  /* Currently performing search */
#define SDC_OPEN_SEARCH_FAILED     0x02  /* Remote dev refused connection */
#define SDC_OPEN_SEARCH_OPEN       0x03	 /* SDC already has connection open */
#define SDC_OPEN_DISCONNECTED      0x04	 /* Remote server device disconnected */

/*------------------------------------------------
     SDC_*_CFM.response Codes

	 These codes indicate if a search succeeded (0)
	 and if not, why not.
--------------------------------------------------*/

#define SDC_RESPONSE_SUCCESS          0x0000 /* Search succeeded */
#define SDC_ERROR_RESPONSE_PDU        0x0010 /* SDP_ErrorResponse PDU */
                                             /* received */
#define SDC_NO_RESPONSE_DATA          0x0011 /* Empty response - no results */
#define SDC_CON_DISCONNECTED          0x0012 /* Remote device disconnected */
#define SDC_CONNECTION_ERROR          0x0013 /* Remote device refused */
                                             /* connection */
#define SDC_CONFIGURE_ERROR           0x0014 /* L2CAP config failed */
#define SDC_SEARCH_DATA_ERROR         0x0015 /* Search data is invalid */
#define SDC_DATA_CFM_ERROR            0x0016 /* Failed to transmit PDU */
#define SDC_SEARCH_BUSY               0x0017 /* Search is busy */
#define SDC_RESPONSE_PDU_HEADER_ERROR 0x0018 /* the response from sds had a */
                                             /* header error */
#define SDC_RESPONSE_PDU_SIZE_ERROR   0x0019 /* the response from sds had a */
                                             /* size error */
#define SDC_RESPONSE_TIMEOUT_ERROR    0x001a /* the response from sds has */
                                             /* timed out */
#define SDC_SEARCH_SIZE_TO_BIG        0x001b /* the size of the search will */
                                             /* not fit into the L2CAP packet */
#define SDC_RESPONSE_OUT_OF_MEMORY    0x001c /* The response was too big to fit
                                              * in memory */
#define SDC_RESPONSE_TERMINATED       0x001d /* The search was terminated */

/*------------------------------------------------
     SDC_*_CFM.err_code Codes

	 These codes are those returned as the ErrorCode
	 parameter of the SDP_ErrorResponse PDU. See
	 the Bluetooth spec, section 4.4.1 for details.
--------------------------------------------------*/
#define PDU_ERROR_VERSION        0x0001
#define PDU_ERROR_SR_HANDLE      0x0002
#define PDU_ERROR_SYNTAX         0x0003
#define PDU_ERROR_PDU_SIZE       0x0004
#define PDU_ERROR_CONT_STATE     0x0005
#define PDU_ERROR_INSUFF_RES     0x0006

/*------------------------------------------------
     SDC_CLOSE_SEARCH_IND. result Codes
--------------------------------------------------*/
#define SDC_SDS_DISCONNECTED     0x00 /* the connection was dicsonnected */
                                      /* by the SDS server */
#define SDC_SDC_DISCONNECTED     0x01 /* the connection was dicsonnected */
                                      /* by the SDC client */

/*------------------------------------------------
     Protocol Data Unit PDU ID's 
	 (Not to be used within BlueStack SDP primitives)
--------------------------------------------------*/
#define SDP_ERROR_RESPONSE                    0x01
#define SDP_SERVICE_SEARCH_REQUEST            0x02
#define SDP_SERVICE_SEARCH_RESPONSE           0x03
#define SDP_SERVICE_ATTRIBUTE_REQUEST         0x04
#define SDP_SERVICE_ATTRIBUTE_RESPONSE        0x05
#define SDP_SERVICE_SEARCH_ATTRIBUTE_REQUEST  0x06
#define SDP_SERVICE_SEARCH_ATTRIBUTE_RESPONSE 0x07

/*-------------------------------------------------------------
 Start of SDC primitives 
---------------------------------------------------------------*/

#define SDC_PRIM_BASE    0x0000  

/*-------------------------------------------------------------
SDC Primitives
---------------------------------------------------------------*/

typedef enum sdc_prim_tag
{
    ENUM_SEP_SDC_FIRST_PRIM = SDC_PRIM_BASE,
    
    ENUM_SDC_SERVICE_SEARCH_REQ,
    ENUM_SDC_SERVICE_SEARCH_CFM,
    ENUM_SDC_SERVICE_ATTRIBUTE_REQ,
    ENUM_SDC_SERVICE_ATTRIBUTE_CFM,
    ENUM_SDC_SERVICE_SEARCH_ATTRIBUTE_REQ,
    ENUM_SDC_SERVICE_SEARCH_ATTRIBUTE_CFM,
    ENUM_SDC_TERMINATE_PRIMITIVE_REQ,
	ENUM_SDC_OPEN_SEARCH_REQ,
	ENUM_SDC_OPEN_SEARCH_CFM,
	ENUM_SDC_CLOSE_SEARCH_REQ,
	ENUM_SDC_CONFIG_REQ,
    ENUM_SDC_CLOSE_SEARCH_IND,

    ENUM_SEP_SDC_LAST_PRIM
} SDC_PRIM_T;

#define SDC_SERVICE_SEARCH_REQ           ((sdc_prim_t)(ENUM_SDC_SERVICE_SEARCH_REQ))
#define SDC_SERVICE_SEARCH_CFM           ((sdc_prim_t)(ENUM_SDC_SERVICE_SEARCH_CFM))
#define SDC_SERVICE_ATTRIBUTE_REQ        ((sdc_prim_t)(ENUM_SDC_SERVICE_ATTRIBUTE_REQ))
#define SDC_SERVICE_ATTRIBUTE_CFM        ((sdc_prim_t)(ENUM_SDC_SERVICE_ATTRIBUTE_CFM))
#define SDC_SERVICE_SEARCH_ATTRIBUTE_REQ ((sdc_prim_t)(ENUM_SDC_SERVICE_SEARCH_ATTRIBUTE_REQ))
#define SDC_SERVICE_SEARCH_ATTRIBUTE_CFM ((sdc_prim_t)(ENUM_SDC_SERVICE_SEARCH_ATTRIBUTE_CFM))
#define SDC_TERMINATE_PRIMITIVE_REQ      ((sdc_prim_t)(ENUM_SDC_TERMINATE_PRIMITIVE_REQ))
#define SDC_OPEN_SEARCH_REQ              ((sdc_prim_t)(ENUM_SDC_OPEN_SEARCH_REQ))
#define SDC_OPEN_SEARCH_CFM              ((sdc_prim_t)(ENUM_SDC_OPEN_SEARCH_CFM))
#define SDC_CLOSE_SEARCH_REQ             ((sdc_prim_t)(ENUM_SDC_CLOSE_SEARCH_REQ))
#define SDC_CONFIG_REQ					 ((sdc_prim_t)(ENUM_SDC_CONFIG_REQ))
#define SDC_CLOSE_SEARCH_IND             ((sdc_prim_t)(ENUM_SDC_CLOSE_SEARCH_IND))

#define SDC_BASE	                     (ENUM_SEP_SDC_FIRST_PRIM + 1)
#define SDC_MAX                          (ENUM_SEP_SDC_LAST_PRIM - 1)

/*---------------------------------- 
 end of #define of primitives 
------------------------------------*/

/*----------------------------------
 primitive type for sdp 
------------------------------------*/
typedef uint16_t        sdc_prim_t;             /* same as l2cap_prim.h */

/*----------------------------------
 other parameter typedefs 
------------------------------------*/

/*----------------------------------------------------------------------------*
 * PURPOSE
 *       Submits a search request to the SDC sub-system.  The device
 *       specified by bd_addr is searched using the search pattern 
 *       specified by *srch_pttrn.  
 *
 *---------------------------------------------------------------------------*/
typedef struct
{
        sdc_prim_t          type;            /* Always SDC_SERVICE_SEARCH_REQ*/
        phandle_t           phandle;          /* routing handle */
        BD_ADDR_T           bd_addr;         /* Address of Bluetooth device */
		                                     /* to be searched */
        uint16_t            size_srch_pttrn; /* size of the search pattern */
        uint8_t             *srch_pttrn;     /* Pointer to search pattern */
		                                     /* described as data element */
		                                     /* sequence */
        uint16_t            max_num_recs;    /* Maximum number of service */
		                                     /* records to be returned */
} SDC_SERVICE_SEARCH_REQ_T;


/*----------------------------------------------------------------------------*
 * PURPOSE
 *     Returns the results of a search request.  The number of 
 *     service records is returned in num_recs_ret, the size of the service 
 *     records is returned in size_rec_list and the service record handles are 
 *     returned via the *rec_list pointer.  
 *     It also indicates whether or not a search is successful, 
 *     using response, and if an error has occurred it indicates the error code
 *     in err_code.  The *err_info is used as a place-holder for error-specific
 *     information that may be used in future versions of the specification.
 *
 *---------------------------------------------------------------------------*/
typedef struct
{
        sdc_prim_t          type;         /* Always SDC_SERVICE_SEARCH_CFM */
        phandle_t           phandle;      /* as supplied in REQ */
        uint16_t            num_recs_ret; /* Number of service records */
		                                  /* returned */
        uint16_t            size_rec_list;/* size of the rec_list in bytes */
        uint8_t             *rec_list;    /* Pointer to list of */
		                                  /* ServiceRecordHandles */
        uint16_t            response;     /* Indicates whether or not the */
		                                  /* search was successful */
        uint16_t            err_code;     /* Error code if one has occurred */
        uint16_t            size_err_info;/* size of the err_info */
        uint8_t             *err_info;    /* Error-specific information - not*/
		                                  /* used in current version of spec */
} SDC_SERVICE_SEARCH_CFM_T;


/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Submits an attribute request to the SDC sub-system.  The attributes
 *      specified by *attr_list, in the device specified by bd_addr, are
 *      obtained from the service record with a ServiceRecordsHandle of 
 *      svc_rec_hndl.
 *      The results of the search are the attribute values of the attributes 
 *      specified in *attr_list.
 *
 *---------------------------------------------------------------------------*/
typedef struct
{
        sdc_prim_t          type;           /* Always */
		                                    /* SDC_SERVICE_ATTRIBUTE_REQ */
        phandle_t           phandle;        /* routing handle */
        BD_ADDR_T           bd_addr;        /* Address of Bluetooth device */
		                                    /* to be searched */
        uint32_t            svc_rec_hndl;   /* ServiceRecordHandle of */
		                                    /* service being queried */
        uint16_t            size_attr_list; /* size of the attr_list in bytes */
        uint8_t             *attr_list;     /* Pointer to attribute list */
		                                    /* described as data element */
		                                    /* sequence */

        /* The maximum number of attribute bytes to be returned in each response
         * packet. If multiple responses are required, they may each contain
         * up to this amount of attribute data.
         */
        uint16_t            max_num_attr;
} SDC_SERVICE_ATTRIBUTE_REQ_T;


/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Returns the results of a service attribute request. The attribute list
 *      is contained in 'attr_list' and its size in 'size_attr_list'. The event
 *      also indicates whether or not the request is successful, using
 *      response, and if an error has occurred it indicates the error code in
 *      err_code. The *err_info is used as a place-holder for error-specific
 *      information that may be used in future versions of the specification.
 *
 *      NOTE - The initial Data Element Size descriptor, which contains the
 *          overall size of the service record, is stripped off by SDP and is
 *          not included in the 'attr_list' data.
 *
 *---------------------------------------------------------------------------*/
typedef struct
{
        sdc_prim_t          type;           /* Always */
		                                    /* SDC_SERVICE_ATTRIBUTE_CFM */
        phandle_t           phandle;        /* as supplied in REQ */
        uint16_t            size_attr_list; /* size of the attr_list in bytes */

        /* The attribute list, described as a data element sequence */
        uint8_t             *attr_list;

        uint16_t            response;       /* Indicates whether or not the */
		                                    /* search was successful */
        uint16_t            err_code;       /* Error code if one has occurred*/
        uint16_t            size_err_info;  /* size of the err_info */
        uint8_t             *err_info;      /* Error-specific information - */
		                                    /* not used in current version */
		                                    /* of spec */
} SDC_SERVICE_ATTRIBUTE_CFM_T;


/*----------------------------------------------------------------------------*
 * PURPOSE
 *       Submits a search attribute request to the SDC sub-system, which is the 
 *       same as a combination of SDC_SRCH_REQ and SDC_ATTR_REQ.  The device
 *       specified by bd_addr is searched using the search pattern 
 *       specified by *srch_pttrn.  The results of the search are the
 *       attribute values of the attributes specified in *attr_list.
 *
 *---------------------------------------------------------------------------*/
typedef struct
{
        sdc_prim_t          type;            /* Always */
		                                 /* SDC_SERVICE_SEARCH_ATTRIBUTE_REQ */
        phandle_t           phandle;         /* routing handle */
        BD_ADDR_T           bd_addr;         /* Address of Bluetooth device */
		                                     /* to be searched */
        uint16_t            size_srch_pttrn; /* size of the search pattern - bytes */
        uint8_t             *srch_pttrn;     /* Pointer to search pattern */
		                                     /* described as data element */
		                                     /* sequence */
        uint16_t            size_attr_list;  /* size of the attr list - bytes */
        uint8_t             *attr_list;      /* Pointer to attribute list */
		                                     /* described as data element */
		                                     /* sequence */

        /* The maximum number of attribute bytes to be returned in each response
         * packet. If multiple responses are required, they may each contain
         * up to this amount of attribute data.
         */
        uint16_t            max_num_attr;
} SDC_SERVICE_SEARCH_ATTRIBUTE_REQ_T;


/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Returns the results of a service search attribute request. One primitive
 *      is issued for each service record received from the server, containing
 *      the attribute list in 'attr_list' and its size in 'size_attr_list'.
 *      The total size of the entire response (over all services and primitives)
 *      is contained in 'total_response_size'.
 *
 *      NOTE - The initial Data Element Size descriptor, which contains the
 *          overall size of the service record, is stripped off by SDP and is
 *          not included in the 'attr_list' data.
 *
 *      SDP collates the responses arriving from the server and sends only
 *      complete service records to the application. If a service record turns
 *      out to be too large for the available memory, the issued primitive will
 *      have 'size_attr_list = 0' and 'attr_list = NULL', and 'response' will
 *      contain the error SDC_RESPONSE_OUT_OF_MEMORY. SDP will discard the
 *      large service record, and will continue to handle subsequent records.
 *      Therefore the application will continue to receive primitives for the
 *      other service records, memory permitting.
 *
 *      The 'more_to_come' flag indicates to the application whether there are
 *      more service records to come. When the last service record is sent,
 *      'more_to_come' is set to FALSE.
 *
 *      Note that SDP may stop issuing service records if it detects an error,
 *      this is indicated to the application by setting 'more_to_come = FALSE'
 *      and setting the 'response' and 'err_code' fields accordingly. The
 *      'response' field indicates locally-detected error conditions, such as
 *      invalid responses from the server; the 'err_code' field indicates errors
 *      reported from the server.
 *
 *---------------------------------------------------------------------------*/
typedef struct
{
        sdc_prim_t          type;           /* Always */
		                                 /* SDC_SERVICE_SEARCH_ATTRIBUTE_CFM */
        phandle_t           phandle;        /* as supplied in REQ */

        /* The total size of the entire response in bytes. Note that SDP will
         * strip out the initial DES descriptor of each service record, so
         * applications should not expect to receive this many bytes. This
         * value is purely informational.
         */
        uint16_t            total_response_size;

        /* The attribute list and size for a single service, described as a data
         * element sequence. For multiple services, this primitive will be
         * issued multiple times, each one containing the attribute data for
         * one service.
         */
        uint16_t            size_attr_list;
        uint8_t             *attr_list;

        /* Non-zero if there are still some more service records to come. FALSE
         * means that the query has finished.
         */
        bool_t              more_to_come;

        uint16_t            response;       /* Indicates whether or not the */
		                                    /* search was successful */
        uint16_t            err_code;       /* Error code if one has occurred*/
        uint16_t            size_err_info;  /* size of the error info */
        uint8_t             *err_info;      /* Error-specific information  */
		                                    /* not used in current version */
		                                    /* of spec */
} SDC_SERVICE_SEARCH_ATTRIBUTE_CFM_T;

/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Terminates the currently active search primtive. If a search is in
 *      progress then the corresponding search cfm primitive will be issued
 *      with its response set to SDC_RESPONSE_TERMINATED.
 *
 *---------------------------------------------------------------------------*/
typedef struct
{
        sdc_prim_t          type;    /* Always SDC_TERMINATE_PRIMITIVE_REQ */
        phandle_t           phandle; /* routing handle */
} SDC_TERMINATE_PRIMITIVE_REQ_T;


/*----------------------------------------------------------------------------*
 * PURPOSE
 *        Submits the opening of a permanent connection request.  
 *
 *---------------------------------------------------------------------------*/
typedef struct
{
        sdc_prim_t          type;     /* Always SDC_OPEN_SEARCH_REQ */
        phandle_t           phandle;  /* routing handle */
        BD_ADDR_T           bd_addr;  /* Address of Bluetooth device to be */
		                              /* searched */
} SDC_OPEN_SEARCH_REQ_T;


/*----------------------------------------------------------------------------*
 * PURPOSE
 *       Submits the response to the opening of a permanent connection 
 *       request.  
 *
 *---------------------------------------------------------------------------*/
typedef struct
{
        sdc_prim_t          type;     /* Always SDC_OPEN_SEARCH_CFM */
        phandle_t           phandle;  /* routing handle */
        uint8_t             result;   /* result from open search req */
} SDC_OPEN_SEARCH_CFM_T;


/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Requests closure of the permanent connection previously opened with
 *      SDC_OPEN_SEARCH_REQ. When the connection has closed, an
 *      SDC_CLOSE_SEARCH_IND primitive is issued, indicating to the application
 *      that it may now begin another search.
 *
 *---------------------------------------------------------------------------*/
typedef struct
{
        sdc_prim_t          type;     /* Always SDC_CLOSE_SEARCH_REQ */
        phandle_t           phandle;  /* routing handle */
} SDC_CLOSE_SEARCH_REQ_T;


/*----------------------------------------------------------------------------*
 * PURPOSE
 *       Configure L2CAP parameters
 *
 *			This currently only applies to the MTU size.
 *			Note that this request has no confirmation
 *			(if the value supplied is invalid, the DEFAULT_MTU value is used).
 *
 *---------------------------------------------------------------------------*/
typedef struct
{
        sdc_prim_t          type;   /* Always SDC_CONFIG_REQ */
        uint16_t			mtu;	/* L2CAP MTU value */
} SDC_CONFIG_REQ_T;


/*----------------------------------------------------------------------------*
 * PURPOSE
 *      Indicates that the connection previously opened with SDC_OPEN_SEARCH_REQ
 *      has now closed. Applications calling SDC_CLOSE_SEARCH_REQ should wait
 *      for SDC_CLOSE_SEARCH_IND before attempting to open another search.
 *
 *---------------------------------------------------------------------------*/
typedef struct
{
        sdc_prim_t          type;     /* Always SDC_CLOSE_SEARCH_IND */
        phandle_t           phandle;  /* routing handle */
        uint8_t             result;   /* result connection error */
} SDC_CLOSE_SEARCH_IND_T;

/*----------------------------------------------------------------------------*
 * PURPOSE
 *		Common elements of "upstream" SDC primitives
 *
 *---------------------------------------------------------------------------*/
typedef struct
{
	    sdc_prim_t			type;         /* Whatever */
	    phandle_t           phandle;      /* routing handle */
} SDC_UPSTREAM_COMMON_T;

/*------------------------------------------------
 Union of the primitives 
-------------------------------------------------*/

typedef union
{
    sdc_prim_t                            type;
	SDC_UPSTREAM_COMMON_T                 sdc_upstream_common;

    SDC_SERVICE_SEARCH_REQ_T              sdc_service_search_req;
    SDC_SERVICE_SEARCH_CFM_T              sdc_service_search_cfm;
    SDC_SERVICE_ATTRIBUTE_REQ_T           sdc_service_attribute_req;
    SDC_SERVICE_ATTRIBUTE_CFM_T           sdc_service_attribute_cfm;
    SDC_SERVICE_SEARCH_ATTRIBUTE_REQ_T    sdc_service_search_attribute_req;
    SDC_SERVICE_SEARCH_ATTRIBUTE_CFM_T    sdc_service_search_attribute_cfm;
    SDC_TERMINATE_PRIMITIVE_REQ_T         sdc_terminate_primitive_req;
    SDC_OPEN_SEARCH_REQ_T                 sdc_open_search_req;
    SDC_OPEN_SEARCH_CFM_T                 sdc_open_search_cfm;
    SDC_CLOSE_SEARCH_REQ_T                sdc_close_search_req;
    SDC_CONFIG_REQ_T					  sdc_config_req;
    SDC_CLOSE_SEARCH_IND_T                sdc_close_search_ind;

} SDC_UPRIM_T;

/*============================================================================*
 *                               PUBLIC FUNCTIONS
 *===========================================================================*/
/* None */


/*=============================== END OF FILE ===============================*/
#endif /* ndef _SDC_PRIM_H */

