/****************************************************************************
FILE
	io_digital.h  -  low-level I/O definitions required for accessing ps

DESCRIPTION
	The values in this file are defined as part of the BlueCore digital
	design.  Most of these files live in the file io_defs.h in the
	io module, but the ones in this file are necessary for setting
	the values of persistent store keys from host side tools and so
	are more widely visible.

	Unlike io_defs.h itself, enums here are declared unconditionally;
	if we need different values for BlueCore 1 and 2, they are defined
	in different enums.

MODIFICATION HISTORY
	1.1  11:jan:02  pws     Created.
	1.2  21:jan:02  pws     Move more stuff from io_defs.h for XAP tools.
	1.3  25:nov:02  pws     H17.70: copied from mm01 and committed.
	#4   11:dec:03  pws     B-1522: support 32 kHz external slow clock.
*/

#ifndef IO_DIGITAL_H
#define IO_DIGITAL_H

#ifdef RCS_STRINGS
static char io_digital_h_id[]
  = "$Id: //depot/bc/main/interface/host/io/io_digital.h#5 $";
#endif

/*
 * AMUX definitions.  Some of these aren't useful on BC01.  However, they
 * are unconditionally defined to make life easier for code which parses
 * psbc_data, which has to be usable for both bc01 and bc02.
 */

enum ana_amux_sel_enum
{
   ANA_AMUX_SEL_VSS_ANA                     = 0,
   ANA_AMUX_SEL_VREF                        = 1,
   ANA_AMUX_SEL_RSSI                        = 2,
   ANA_AMUX_SEL_XTAL_PLL_TUNE_MON           = 3,
   ANA_AMUX_SEL_XTAL_PLL_TUNE_DRV           = 4,
   ANA_AMUX_SEL_VCO_DC                      = 5,
   ANA_AMUX_SEL_VCO_PK                      = 6,
   ANA_AMUX_SEL_TUNE_IF                     = 7,
   ANA_AMUX_SEL_BIASF                       = 8,
   ANA_AMUX_SEL_PHSET                       = 9,
   ANA_AMUX_SEL_GCTRL_I                     = 10,
   ANA_AMUX_SEL_GCTRL_Q                     = 11,
   ANA_AMUX_SEL_TX_PAD_PK                   = 12,
   ANA_AMUX_SEL_TUNE_LO_MON                 = 13,
   ANA_AMUX_SEL_RX_IF_BIASV_I               = 14,
   ANA_AMUX_SEL_RX_IF_BIASV_Q               = 15,
   ANA_AMUX_SEL_IF2_MUX_IN                  = 16,
   ANA_AMUX_SEL_IF2_MUX_OUT                 = 17,
   ANA_AMUX_SEL_TX_BB_I_MON                 = 18,
   ANA_AMUX_SEL_TX_BB_Q_MON                 = 19,
   ANA_AMUX_SEL_BIASL                       = 20,
   ANA_AMUX_SEL_BIASH                       = 21,
   ANA_AMUX_SEL_AUDIO_IN_MON                = 22,
   ANA_AMUX_SEL_IQ_CMSET                    = 23,
   ANA_AMUX_SEL_AUDIO_OUT_TEST_AMP          = 24,
   ANA_AMUX_SEL_REF_RTOP                    = 25,
   ANA_AMUX_SEL_TX_PA_GCTRL                 = 26,
   ANA_AMUX_SEL_XTAL_PLL_DELAY              = 27
};
typedef enum ana_amux_sel_enum ana_amux_sel;
 
enum ana_amux_a_sel_enum
{
   ANA_AMUX_A_SEL_VSS_ANA                   = 0,
   ANA_AMUX_A_SEL_TEST_ADC                  = 1,
   ANA_AMUX_A_SEL_RSSI                      = 2,
   ANA_AMUX_A_SEL_BIASV_M                   = 3,
   ANA_AMUX_A_SEL_VREF                      = 4,
   ANA_AMUX_A_SEL_VCO_DC                    = 5,
   ANA_AMUX_A_SEL_PHSET                     = 6,
   ANA_AMUX_A_SEL_TUNE_IF                   = 7,
   ANA_AMUX_A_SEL_PRE_SSUP                  = 8,
   ANA_AMUX_A_SEL_IF1_I_MUX_IN              = 9,
   ANA_AMUX_A_SEL_IF2_MUX_OUT               = 10,
   ANA_AMUX_A_SEL_TX_BB_I_OUT               = 11,
   ANA_AMUX_A_SEL_SPARE_A                   = 12,
   ANA_AMUX_A_SEL_CLK_1K0_TEST              = 13,
   ANA_AMUX_A_SEL_CLK_250K_TEST             = 14,
   ANA_AMUX_A_SEL_CLK_16M_DIG               = 15,

   /* bc02 definitions */
   ANA_AMUX_A_SEL_DIG_IN_SPARE_28           = 28,
   ANA_AMUX_A_SEL_DIG_IN_SPARE_29           = 29,
   /* The next setting is available from bc3 */
   ANA_AMUX_A_SEL_DIG_IN_CLK_32K_EXT	    = 29,
   ANA_AMUX_A_SEL_DIG_IN_CLK_LO_DIV_TEST    = 30,
   ANA_AMUX_A_SEL_DIG_IN_PIO_12             = 31,
   ANA_AMUX_A_SEL_DIG_OUT_CLK_PRESC_MON     = 28,
   ANA_AMUX_A_SEL_DIG_OUT_CLK_REF_MON       = 29,
   ANA_AMUX_A_SEL_DIG_OUT_CLK_48M_MON       = 30,
   ANA_AMUX_A_SEL_DIG_OUT_PIO_12            = 31
};
typedef enum ana_amux_a_sel_enum ana_amux_a_sel;


enum ana_amux_b_sel_enum
{
   ANA_AMUX_B_SEL_VSS_ANA                   = 0,
   ANA_AMUX_B_SEL_TEST_ADC                  = 1,
   ANA_AMUX_B_SEL_GCTRL_I                   = 2,
   ANA_AMUX_B_SEL_GCTRL_Q                   = 3,
   ANA_AMUX_B_SEL_MON_TX                    = 4,
   ANA_AMUX_B_SEL_BIASL                     = 5,
   ANA_AMUX_B_SEL_BIASH                     = 6,
   ANA_AMUX_B_SEL_BIASF                     = 7,
   ANA_AMUX_B_SEL_IF2_MUX_OUT               = 8,
   ANA_AMUX_B_SEL_IF1_Q_MUX_IN              = 9,
   ANA_AMUX_B_SEL_IF2_MUX_IN                = 10,
   ANA_AMUX_B_SEL_TX_BB_Q_OUT               = 11,
   ANA_AMUX_B_SEL_CLK_16M_TEST              = 12,
   ANA_AMUX_B_SEL_CLK_TEST_LO_SYNTH         = 13,
   ANA_AMUX_B_SEL_CLK_XTAL                  = 14,
   ANA_AMUX_B_SEL_CLK_SYNTH_MON             = 15,

   /* bc02 definitions */
   ANA_AMUX_B_SEL_DIG_IN_SPARE_28           = 28,
   ANA_AMUX_B_SEL_DIG_IN_SPARE_29           = 29,
   ANA_AMUX_B_SEL_DIG_IN_CLK_ADC_TEST       = 30,
   ANA_AMUX_B_SEL_DIG_IN_PIO_13             = 31,
   ANA_AMUX_B_SEL_DIG_OUT_CLK_1K            = 28,
   ANA_AMUX_B_SEL_DIG_OUT_CLK_LO_DIV_MON    = 29,
   ANA_AMUX_B_SEL_DIG_OUT_CLK_16M_ANA_JIT   = 30,
   ANA_AMUX_B_SEL_DIG_OUT_PIO_13            = 31
};
typedef enum ana_amux_b_sel_enum ana_amux_b_sel;

enum ana_amux_c_sel_enum
{
   ANA_AMUX_C_SEL_DIG_IN_SPARE_28           = 28,
   ANA_AMUX_C_SEL_DIG_IN_SPARE_29           = 29,
   ANA_AMUX_C_SEL_DIG_IN_AUDIO_DAC_BITS     = 30,
   ANA_AMUX_C_SEL_DIG_IN_PIO_14             = 31,
   ANA_AMUX_C_SEL_DIG_OUT_CLK_DIV33_MON     = 28,
   ANA_AMUX_C_SEL_DIG_OUT_CLK_XTAL          = 29,
   ANA_AMUX_C_SEL_DIG_OUT_CLK_16M_ANA       = 30,
   ANA_AMUX_C_SEL_DIG_OUT_PIO_14            = 31
};
typedef enum ana_amux_c_sel_enum ana_amux_c_sel;

enum ana_amux_d_sel_enum
{
   ANA_AMUX_D_SEL_DIG_IN_SPARE_28           = 28,
   ANA_AMUX_D_SEL_DIG_IN_SPARE_29           = 29,
   ANA_AMUX_D_SEL_DIG_IN_SPARE_30           = 30,
   ANA_AMUX_D_SEL_DIG_IN_PIO_15             = 31,
   ANA_AMUX_D_SEL_DIG_OUT_VSS_ANA           = 28,
   ANA_AMUX_D_SEL_DIG_OUT_CLK_PHC_REF_MON   = 29,
   ANA_AMUX_D_SEL_DIG_OUT_AUDIO_ADC_BITS    = 30,
   ANA_AMUX_D_SEL_DIG_OUT_PIO_15            = 31
};
typedef enum ana_amux_d_sel_enum ana_amux_d_sel;


enum uart_rate_enum
{
   UART_RATE_9K6                            = 39,
   UART_RATE_19K2                           = 79,
   UART_RATE_38K4                           = 157,
   UART_RATE_57K6                           = 236,
   UART_RATE_115K2                          = 472,
   UART_RATE_230K4                          = 944,
   UART_RATE_460K8                          = 1887,
   UART_RATE_921K6                          = 3775,
   UART_RATE_1382K4                         = 5662
};
typedef enum uart_rate_enum uart_rate;


enum uart_config_posn_enum
{
   UART_CONFIG_TWO_STOP_BITS_POSN           = 0,
   UART_CONFIG_ONE_PARITY_BIT_POSN          = 1,
   UART_CONFIG_EVEN_PARITY_POSN             = 2,
   UART_CONFIG_FLOW_CTRL_EN_POSN            = 3,
   UART_CONFIG_RTS_AUTO_EN_POSN             = 4,
   UART_CONFIG_RTS_POSN                     = 5,
   UART_CONFIG_TX_ZERO_EN_POSN              = 6,
   UART_CONFIG_NON_BCSP_EN_POSN             = 7,
   UART_CONFIG_RX_RATE_DELAY_LSB_POSN       = 8,
   UART_CONFIG_RX_RATE_DELAY_MSB_POSN       = 10,
   UART_CONFIG_CTS_FILTER_EN_POSN	    = 11,
   UART_CONFIG_H5_EN_POSN                   = 12
};
typedef enum uart_config_posn_enum uart_config_posn;


enum uart_config_mask_enum
{
   UART_CONFIG_TWO_STOP_BITS_MASK           = (int)0x0001,
   UART_CONFIG_ONE_PARITY_BIT_MASK          = (int)0x0002,
   UART_CONFIG_EVEN_PARITY_MASK             = (int)0x0004,
   UART_CONFIG_FLOW_CTRL_EN_MASK            = (int)0x0008,
   UART_CONFIG_RTS_AUTO_EN_MASK             = (int)0x0010,
   UART_CONFIG_RTS_MASK                     = (int)0x0020,
   UART_CONFIG_TX_ZERO_EN_MASK              = (int)0x0040,
   UART_CONFIG_NON_BCSP_EN_MASK             = (int)0x0080,
   UART_CONFIG_RX_RATE_DELAY_LSB_MASK       = (int)0x0100,
   UART_CONFIG_RX_RATE_DELAY_MSB_MASK       = (int)0x0400,
   UART_CONFIG_CTS_FILTER_EN_MASK           = (int)0x0800,
   UART_CONFIG_H5_EN_MASK                   = (int)0x1000
};
typedef enum uart_config_mask_enum uart_config_mask;


enum rst_enable_posn_enum
{
   RST_WATCHDOG_EN_POSN                     = 0,
   RST_UART_EN_POSN                         = 1
};
typedef enum rst_enable_posn_enum rst_enable_posn;


enum rst_enable_mask_enum
{
   RST_WATCHDOG_EN_MASK                     = (int)0x0001,
   RST_UART_EN_MASK                         = (int)0x0002
};
typedef enum rst_enable_mask_enum rst_enable_mask;



enum spi_emu_cmd_posn_enum
{
   SPI_EMU_CMD_XAP_STEP_POSN                = 0,
   SPI_EMU_CMD_XAP_RUN_B_POSN               = 1,
   SPI_EMU_CMD_XAP_BRK_POSN                 = 2,
   SPI_EMU_CMD_XAP_WAKEUP_POSN              = 3
};
typedef enum spi_emu_cmd_posn_enum spi_emu_cmd_posn;


enum spi_emu_cmd_mask_enum
{
   SPI_EMU_CMD_XAP_STEP_MASK                = (int)0x0001,
   SPI_EMU_CMD_XAP_RUN_B_MASK               = (int)0x0002,
   SPI_EMU_CMD_XAP_BRK_MASK                 = (int)0x0004,
   SPI_EMU_CMD_XAP_WAKEUP_MASK              = (int)0x0008
};
typedef enum spi_emu_cmd_mask_enum spi_emu_cmd_mask;

#endif /* IO_DIGITAL_H */
