/****************************************************************************

  Copyright (C) Cambridge Silicon Radio April 2000

FILE
    Win32BCSP.h  -  windows wrapper around bcsp stack

MODIFICATION HISTORY
	1.3  27-apr-00 npm  - added boilerplating, removed defaultconfigureport
	1.4  11-may-00 npm  - added packets array to prevent memory leak
	1.5  22-june-00 npm - added lastXmitTime field for nasty '98 WriteFile hack
	1.7  16-nov-00 at   - added ability to use port opened and closed externally
	1.8  12-dec-00 at   - removed start method's useLinkEstablishment parameter
	1.9  12-mar-01 at   - changed type of mStackThreadId
	1.10 20-mar-01 dm	- added changes for Windows CE _WINCE compile option

*/



#ifndef _WIN32BCSP_H_
#define _WIN32BCSP_H_

#include "common/linkage.h"

#ifdef	RCS_STRINGS
static  char    win32bcsp_h_id[]
  = "$Id: //depot/bc/main/devHost/HostStack/BCSP/Win32/Win32UART/Win32BCSP.h#17 $";
#endif



#include "BCSPStack.h"
#include "WinEnvironment.h"

class BCSP_API_CLASS Win32BCSP
{
private: 
	HANDLE mComport ;
	HANDLE mStackThread ;
#ifdef _WINCE
	ULONG mStackThreadId ;
#else
	UINT mStackThreadId ;
	OVERLAPPED mXMITOverlapped ;
	OVERLAPPED mRCVOverlapped ;
#endif
	
	HANDLE mEvents[4] ;

	BCSPStack * mStack ;
	BCSPEnvironment mBCSPenv ;
	WinEnvironmentState mEnvState ;

	uint8 mXMITBuffer[BYTE_BUFFERSIZE] ;
	uint8 mRCVBuffer[BYTE_BUFFERSIZE] ;
	bool mXMITBusy ;
	bool mRCVBusy ;
	uint32 mXMITBytesAvailable ;
	ULONG mRCVBytesAvailable ;
	DWORD mXMITbytesWritten ;
	bool mStackDead ;
	bool mStackMustDie ;

	uint8 startIO() ;
	void wait(uint32 wakeupTime) ;

	void createBCSPStack() ;
	void runStack();
	void spawnStack(void) ;
	bool setTimeouts() ;
	void kill() ;

	void (* userOnLinkFail)(void*) ;
	void * userOnLinkFailParam ;
	Packet * Packets[10] ;

#ifdef _WINCE
	ULONG mRxWorkerThreadId;
	HANDLE mRxWorkerHandle;
	void createRxWorker (void);
	static void RxworkerThread (void * aObject); /* thread which just detects rx busy */
	void killRxWorker ();
    void waitForRxWork( void );
	bool mOpenedAsOtherInstance;
#endif		

	//<HACK>
	DWORD mLastXmitTime ;
	ichar * mSavedPortName ;
	ichar * mSavedConfigStr ;
	//</HACK>

public :
	Win32BCSP() ;
	~Win32BCSP() ;
	bool open(const ichar * portName, const ichar * parameters) ;
	bool open(HANDLE comport) ;
	BCSPStack * getBCSPStack() {return mStack;}
	bool connect(uint32 timeout) ;
	void start() ;
	bool ConfigurePort(ichar * cfgStr) ;

	friend UINT callRunStack(LPVOID * p) ;
	virtual void onLinkFail(void) ;
	void setOnLinkFail(void (*fn)(void*),void* param) ;
} ;


#endif
