// Simple reference counted pointer
// JBS 10 Oct 2000

#ifndef COUNTEDPOINTER_H

#define COUNTEDPOINTER_H

class Counter
{
  mutable int count;

public:
  Counter()
  { count = 0; }
  Counter(const Counter &)
  { }
  virtual ~Counter()
  { }
  Counter &operator=(const Counter &)
  { return *this; }

  void inc() const
  { ++count; }
  void dec() const
  { if(--count == 0) delete this; }
};

template<class T> class Counted
{
  mutable int count;

public:
  Counted()
  { count = 0; }
  Counted(const Counted<T> &)
  { }
  Counted &operator=(const Counted<T> &)
  { return *this; }

  void inc() const
  { ++count; }
  void dec() const
  { if(--count == 0) delete (T *) this; }
};

template<class T> class CountedPointer
{
  T *ptr;
  void set(T *p)
  { ptr = p; if(ptr) ptr->inc(); }
  void clr()
  { if(ptr) ptr->dec(); ptr = 0; }

public:
  CountedPointer(T *p = 0)
  { set(p); }
  CountedPointer(const CountedPointer<T> &x)
  { set(x.ptr); }
  ~CountedPointer()
  { clr(); }
  CountedPointer<T> &operator=(const CountedPointer<T> &x)
  { if(ptr != x.ptr) { clr(); set(x.ptr); } return *this; }

  T &operator*() const
  { return *ptr; }
  T *operator->() const
  { return ptr; }
  operator bool() const
  { return ptr != 0; }
};

#endif
