/****************************************************************************
**
**  File: HCIPacker.h
**
**            (C) CSR 2000
**
**  Purpose:  advertise the services of the HCIPacker library
**
**  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/Packer/HCIPacker/HCIPacker.h#10 $
**
****************************************************************************/

/****************************************************************************
**  Modification history
***************************************************************************** 
**  1.1   8:may:00	ajh	Created.
*/

#ifndef	__HCIPACKETISER_H__
#define	__HCIPACKETISER_H__
 
#include "common/types.h"
#include "common/linkage.h"

#include "HCI/HCILinkage.h"

#include "driver_types.h"
#include "packer_parameters.h"




/****************************************************************************
NAME
	generateHCICommand  -  generates wire format commands for hostside hci

PARAMETERS
        a pointer to a place to write the address of a buffer, and a
        parameter array containing the HCI OpCode, and its parameters.

RETURNS
        the size of the buffer written.  Returns zero on failure to create
        the command pdu successfully.
****************************************************************************/

DEVICECONTROLLER_API uint32 generateHCICommand( uint8** buffer, PARAMETER_ARRAY parameters );

/****************************************************************************
NAME
	analyseHCIEvent  -  analyses wire format events for hostside hci

PARAMETERS
        a buffer to be analysed, and the nominal size of that buffer.

RETURNS
	The array of event parameters.  NULL is a failure.
****************************************************************************/

DEVICECONTROLLER_API PARAMETER_ARRAY analyseHCIEvent ( uint8 * pdu_buffer , uint32 length );

/****************************************************************************
NAME
	writeHCIDataHeader  -  generates wire format data header

RETURNS
        true (1) if successful, false (0) if something goes wrong.
****************************************************************************/

DEVICECONTROLLER_API void writeHCIDataHeader ( uint8 *write_point ,
                                  uint16 aConnectionHandle,
                                  uint8  aPacketBoundaryFlag,
                                  uint8  aBroadcastType ,
                                  uint16 aDataLength,
                                  enum CONNECTION_TYPE aConnectionType);

/****************************************************************************
NAME
	hci_data_header_analyse  -  analyses wire format data headers.

RETURNS
	a pointer to some data.  Or null if the header is corrupt.
****************************************************************************/

DEVICECONTROLLER_API uint8 * analyseHCIDataHeader ( uint8  *aPointerToBufferContainingApdu,
                                       uint32 aLengthOfBufferContainingApdu,
                                       uint8   SCO, /* interpreted as boolean */
                                       uint16 *aConnectionHandlePointer,
                                       uint8  *aPacketBoundaryFlagPointer,
                                       uint8  *aBroadcastTypePointer,
                                       uint16 *aDataLength_pointer);

#endif	/* __HCIPACKETISER_H__ */
