///////////////////////////////////////////////////////////////////////
//
//  FILE   :  Win32BCSPTransport.h
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  Win32BCSPTransport
//
//  PURPOSE:  transparent class inheriting top API from the
//            HCITransport class, and owning a BCSPStack which does
//            all the work.
//
//            This Class also provides BCSP-specific private channels
//            L2CAP etc and puts the debug channel messages to a file
//            in the debug build version.
//
//            Start-up sequence:
//            new Win32BCSPTransport;
//            setupCallBacks (HCITransport functions);
//            start stack;
//            
//            If the stack is started with out the callbacks in place,
//            then data arriving in the intervening time will be lost.
//            Since this is likely to be the HCI initialisation packet,
//            that would be bad...
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/Win32/BCSP/Win32BCSPTransport.h#25 $
//
//  MODIFICATION HISTORY:
//  1.7   7:Sep:00  at  Added modification history.
//                      Removed the inappropriate duplication of
//                      BlueCoreTransport::recvData().
//  1.11  26:Oct:00 at  Added setLinkEstablishment() method to enable
//                      link establishment to be disabled.
//  1.12  27:Oct:00 at  Added methods to modify the BCSP stack
//                      configuration.
//  1.14  16:Nov:00 at  Added ability to open and close COM port
//                      externally.
//	1.15  23:Mar:01 at  Added sub-directory to BCSP includes.
//	1.16  28:Mar:01 ckl Removed stdafx.h
//
///////////////////////////////////////////////////////////////////////

#ifndef __WIN32BCSPTRANSPORT_H__
#define __WIN32BCSPTRANSPORT_H__

#include "bcsp/BCSPStack.h"
#include "bcsp/Win32BCSP.h"
#include "bcsp/StackConfiguration.h"
#include "blueCoreTransport.h"


class DEVICECONTROLLER_API_CLASS Win32BCSPTransport : public BlueCoreTransport
{
public:
    Win32BCSPTransport( ichar* comPort , uint32 dataRate );
    Win32BCSPTransport( ichar* comPort , ichar * configString );
    Win32BCSPTransport( HANDLE comPort, uint32 baud = 0);
    virtual ~Win32BCSPTransport();

    static void onPacketFromChip ( BCTransferRequest * req, BCTransferStatus status, void * info );
    static void onPacketToChip ( BCTransferRequest * req, BCTransferStatus status, void * info );
    void processDataFromChip ( BCTransferRequest * req, BLUECORE_CHANNELS ch );

    startTransportReturnValue start( void );
    bool ready ( uint32 timeout );
    void stop();

    void getStackConfiguration(StackConfiguration &cfg);
    void setStackConfiguration(const StackConfiguration &cfg);

    uint16 getBufferSize();

    void resetStack ();
// LOOK
	bool ConfigurePort(ichar * ConfigString);
	
private:
    void creationInit ();
    void comportPrep  ( ichar* comPort , uint32 dataRate , ichar * configString );


    uint16 mBufferSize;

    ichar * mComportName;
    ichar * mConfigString;
    HANDLE mComportHandle;
    
    Win32BCSP * mStack;
    BCSPStack * mBCSPStack;
    StackConfiguration mStackCfg;
    BCSPChannel       channels[BLUECORE_CHANNEL_COUNT];
    BCTransferRequest recvReqs[BLUECORE_CHANNEL_COUNT];
    uint8*            buffers [BLUECORE_CHANNEL_COUNT];
    void sendpdu     ( BLUECORE_CHANNELS channel, uint8 *apdu, uint32 aLength , CallBack *aCallBack ) ;
	void set_sco_bandwidth(int bytes_per_second, void *additional_info, int additional_info_size);
};

typedef struct
{
    BLUECORE_CHANNELS channelNumber;
    Win32BCSPTransport * object;
    CallBack * callBack;
}
CallBackInfo;

#endif // __WIN32BCSPTRANSPORT_H__
