///////////////////////////////////////////////////////////////////////
//
//  FILE   :  Win32USBWidTransport.h
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  Win32USBWidTransport
//
//  PURPOSE:  transparent class inheriting top API from the
//            HCITransport class, and owning a USBWidStack which does
//            all the work.
//
//            Start-up sequence:
//            new Win32USBWidTransport;
//            setupCallBacks (HCITransport functions);
//            start stack;
//            
//            If the stack is started with out the callbacks in place,
//            then data arriving in the intervening time will be lost.
//            Since this is likely to be the HCI initialisation packet,
//            that would be bad...
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/Win32/USB/Win32USBWidTransport.h#8 $
//
//  MODIFICATION HISTORY:
//  1.7   7:Sep:00  at  Added modification history.
//                      Removed the inappropriate duplication of
//                      BlueCoreTransport::recvData().
//	1.3  28:Mar:01 ckl Removed stdafx.h
//  1.6   4:Nov:02 at  Added ability to use an existing handle.
//  1.7   6:Nov:02 at  Device name needed even if existing handle.
//
///////////////////////////////////////////////////////////////////////

#ifndef __Win32USBWidTransport_H__
#define __Win32USBWidTransport_H__

#include "blueCoreTransport.h"

class USBWidStack;

class DEVICECONTROLLER_API_CLASS Win32USBWidTransport : public BlueCoreTransport
{
public:
    Win32USBWidTransport( ichar* deviceName , uint32 dataRate = 0 );
    Win32USBWidTransport( ichar* deviceName , ichar * configString );
    Win32USBWidTransport( ichar* deviceName , HANDLE deviceHandle );
    virtual ~Win32USBWidTransport();

    static void onLinkFail(void * aObject);

    static void onEventPacketFromChip ( void * aData , unsigned long aLength , void * info );
    static void onDataPacketFromChip  ( void * aData , unsigned long aLength , void * info );
    void processDataFromChip ( enum BLUECORE_CHANNELS channel , void * aData, unsigned long aLength );

    startTransportReturnValue start( void );
    bool ready ( uint32 timeout );
    void stop();

private:
    void creationInit ( ichar* comPort , uint32 dataRate , ichar * configString );

    USBWidStack * mStack;
    ichar * mDeviceName;
    HANDLE mDeviceHandle;
    ichar * mConfigString;
    
    void sendpdu     ( BLUECORE_CHANNELS channel, uint8 *apdu, uint32 aLength , CallBack *aCallBack ) ;
	void set_sco_bandwidth(int bytes_per_second, void *additional_info, int additional_info_size);

    bool linkFailed;
};

typedef struct
{
    BLUECORE_CHANNELS channelNumber;
    Win32USBWidTransport * object;
    CallBack * callBack;
}
USBWidCallBackInfo;

#endif // __Win32USBWidTransport_H__
