///////////////////////////////////////////////////////////////////////
//
//  FILE   :  bc01DeviceController.h
//
//            (C) CSR 2000
//
//  AUTHOR :  Adam Hughes
//
//  CLASS  :  bc01DeviceController
//
//  PURPOSE:  Provide a C++ object API.
//
//  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/DeviceDriver/bc01/bc01DeviceController.h#22 $
//
///////////////////////////////////////////////////////////////////////

#ifndef __BC01DEVICECONTROLLER_H__
#define __BC01DEVICECONTROLLER_H__


extern "C"
{
#include "BCPacker.h"
}



#include "HCIDeviceController.h"

#include "blueCoreTransport.h"

class PCHandler;
//  JB
class DMHandler;
class RFCOMMHandler;
class SDPHandler;
class L2CAPHandler;

class DEVICECONTROLLER_API_CLASS BlueCoreDeviceController : public HCIDeviceController
{
public:
    BlueCoreDeviceController (BlueCoreTransport *aTransport = 0 , bool withBlueStack = false );

    virtual ~BlueCoreDeviceController ();

    BlueCoreTransport * getTransport( void );

    void setTransport ( BlueCoreTransport * aTransport );

    void setupOnTransportFailure ( void (*onTransportFailFn) ( void * ),
                                   void * onTransportFailParameters );

    void setupRecvHostToChipResponses ( void (*hostToChipCallBackFn)
                                          ( PARAMETER_ARRAY eventParameters , 
                                            void * parameters),
                                        void * hostToChipCallBackParameters);

    void setupRecvChipToHostRequests  ( void (*chipToHostCallBackFn)
                                          ( PARAMETER_ARRAY eventParameters , 
                                            void * parameters),
                                        void * chipToHostCallBackParameters );

    void setupRecvDebug ( void (*DebugDataFn) ( uint8 * buffer ,
                                                uint32 length ,
                                                void * parameters),
                          void * DebugDataParameters );

	// JB callback fun for DM prims from chip
	void setupRecvDMPrimsFromChip(void (*DMprimsFromChipCallBackFn)
                                          (uint16 type, 
										   void *prim,
										   void *parameters),
                                  void *DMprimChipToHostCallBackParameters);

	void setupRecvRFCOMMPrimsFromChip(void (*RFCOMMprimsFromChipCallBackFn)
                                          (uint16 type, 
										   void *prim,
										   void *parameters),
									  void *RFCOMMprimChipToHostCallBackParameters);

	void setupRecvSDPPrimsFromChip(void (*SDPprimsFromChipCallBackFn)
                                          (uint16 prim_type, 
										   void *prim,
										   void *parameters),
                                   void *SDPprimChipToHostCallBackParameters);

	void setupRecvL2CAPPrimsFromChip(void (*L2CAPprimsFromChipCallBackFn)
                                          (uint16 type, 
										   void *prim,
										   void *parameters),
                                   void *L2CAPprimChipToHostCallBackParameters);

    //  However the threading is implemented, calling "doWork" gives
    //  processor time to the object to allow it to send, receive etc.
    //
    //  doWork will return when it has done any pending work it can do.
    void doWork( void );

    //  Wait for something to happen and then do it and wait some more...
    //  createWorker will create a worker thread which calls workerThread
    //  which calls waitForWork, which will return only when the object
    //  is deleted, but it will do all background processing that would
    //  have to be implemented by repeated calls to doWork otherwise.
    void createWorker ( void );
    static void workerThread ( void * aObject );
    //  or just give wait for work a thread...
    void waitForWork( void );

    bool sendChipMgtHostToChip       ( PARAMETER_ARRAY parameters );

    void onChipMgtHostToChipResponse ( PARAMETER_ARRAY parameters );
    void onChipMgtChipToHostRequest  ( PARAMETER_ARRAY parameters );
    static void onDebugData     ( uint8* aBuffer , uint32 aLength , void * aThis );

	// JB BlueStack support
	bool sendDMprim(uint16 prim_type, void *prim);
	bool sendRFCOMMprim(uint16 prim_type, void *prim);
	bool sendSDPprim(uint16 prim_type, void *prim);
	bool sendL2CAPprim(uint16 prim_type, void *prim);

private:
    void creationInit ( void );
    BlueCoreTransport * mBCSP;
    bool withBlueStack;   //  JB support for BlueStack

    PCHandler &mhcHandler; //  bccmd
    PCHandler &mchHandler; //  hq

#ifndef NOPRIMCONVERTER
	DMHandler &dmHandler;  //  JB support for BlueStack
	RFCOMMHandler &rfcommHandler;  // JB 
	SDPHandler &sdpHandler;
	L2CAPHandler &l2capHandler;
#endif

    //SignalBox::Item * mHCSentSignal ; //  bccmd
    SignalBox::Item * mHCRecvSignal ;
    SignalBox::Item * mCHSentSignal ; //  hq
    SignalBox::Item * mCHRecvSignal ;
    // JB stuff added to support BlueStack
	SignalBox::Item * mDMSentSignal ;		//  DM channel
    SignalBox::Item * mDMRecvSignal ;
	SignalBox::Item * mRFCOMMSentSignal ;	//  RFCOMM channel
    SignalBox::Item * mRFCOMMRecvSignal ;
	SignalBox::Item * mSDPSentSignal ;		//  SDP channel
    SignalBox::Item * mSDPRecvSignal ;
	SignalBox::Item * mL2CAPSentSignal ;	//  L2CAP channel
    SignalBox::Item * mL2CAPRecvSignal ;

    void checkQueues ( void );
	// JB added to support BlueStack
	void checkDMqueues(void);   // check the queues for the DM handler for pdus
	void checkRFCOMMqueues(void);   
	void checkSDPqueues(void);   
	void checkL2CAPqueues(void);   

    void (*mOnTransportFailFn) ( void * );
    void * mOnTransportFailParameters;

    void (*mChipToHostCallBackFn) (PARAMETER_ARRAY eventParameters , void * parameters);
    void * mChipToHostCallBackParameters;

    void (*mHostToChipCallBackFn) (PARAMETER_ARRAY eventParameters , void * parameters);
    void * mHostToChipCallBackParameters;

    void (*mDebugDataCallBackFn) ( uint8 * buffer , uint32 length  , void * parameters);
    void * mDebugDataCallBackParameters;

	// JB callback fun for DM prims from chip
	void (*mDMprimsFromChipCallBackFn) (uint16 type, void *prim, void *parameters);
    void *mDMprimChipToHostCallBackParameters;

	void (*mRFCOMMprimsFromChipCallBackFn) (uint16 type, void *prim, void *parameters);
    void *mRFCOMMprimChipToHostCallBackParameters;
	
	void (*mSDPprimsFromChipCallBackFn) (uint16 prim_type, void *prim, void *parameters);
    void *mSDPprimChipToHostCallBackParameters;

	void (*mL2CAPprimsFromChipCallBackFn) (uint16 type, void *prim, void *parameters);
    void *mL2CAPprimChipToHostCallBackParameters;
};

#endif // __BC01DEVICECONTROLLER_H__
