/******************************************************************************
**
**   File:	packer.h
**
**            (C) CSR 2000
**
**   Purpose:	Hold the structure and local procedure definitions for HCI
**              and BC packing common code.
**
**  $Id: //depot/bc/main/devHost/HostStack/DeviceDriver/generic/Packer/packer.h#13 $
**
******************************************************************************/

/****************************************************************************
**  Modification history
***************************************************************************** 
**  1.1  22:june:00	ajh	Created from common code in HCI and BC
**                              packers.
**  1.2   9:aug:00      ajh     Carl and Neil have agreed a data parameter
**                              format.  Time for a rewrite.
*/

#ifndef __PACKER_H__
#define __PACKER_H__

#include "common/types.h"
#include "common/bool.h"
#include "common/linkage.h"

#include "packer_parameters.h"
#include "packer_inline.h"
/*
**  constant for the size of the pdu headers.
*/
#define BC_COMMAND_HEADER_SIZE 18
#define BC_EVENT_HEADER_SIZE 18
#define HCI_COMMAND_HEADER_SIZE 3
#define HCI_EVENT_HEADER_SIZE 2
#define HCI_DATA_HEADER_SIZE 4
#define HCI_VOICE_HEADER_SIZE 3

/*
**  Define the table structs
*/

typedef struct
{
    uint16 code;
    uint32 format;
}
PACKER_PDU_TBL;

/*
**  Define a structure to hold pointers to a pdu and an array of parameters,
**  to make writing them somewhat easier. We just pass around the one pointer,
**  rather than 5 or 6 things.
*/
typedef struct
{
    PARAMETER_ARRAY parameters;
    PARAMETER_ARRAY parameterIndex;
    uint32 parameterCount;
    uint8 *pdu;
    uint8 *pduIndex;
    uint32 pduLength;
}
PDU_AND_PARAMETER_HOLDER;

/*
**  And in the best object oriented style a creator and destructor...
*/
PDU_AND_PARAMETER_HOLDER * createHolder ( uint8 *apdu ,
                                          PARAMETER_ARRAY aParameters );

void deleteHolder ( PDU_AND_PARAMETER_HOLDER * aHolder );

/*
**  Declare Local procedures:
**
**  Common procedures:
*/
bool buildBuffer ( uint32 aFormat ,
                   PDU_AND_PARAMETER_HOLDER * aHolder );

uint8 analyseParameters ( uint32 aFormat ,
                          PDU_AND_PARAMETER_HOLDER * aHolder );

/*
**  what do we do if there is a problem with the pdu?
*/
void setBadPDUCallback ( void (*onBadPDUFunction) ( BadPDUReason , uint8* , uint32 , void * ) , void * onBadPDUParameters );
void onBadPDU ( BadPDUReason aWhy , uint8 * apdu , uint32 aLength );

/*
**  BC specific local procedures
*/
/****************************************************************************
NAME		pcAnalyse

PARAMETERS	a pdu and a boolean, true for bccmb, false for hq.

RETURNS		PARAMETER_ARRAY of data from pdu.
****************************************************************************/
PARAMETER_ARRAY pcAnalyse( uint8 * pdu_buffer , uint32 length , bool out);

/****************************************************************************
NAME		pcResponseSize

PARAMETERS	The varid of a chip management command, and the size of the pdu.

RETURNS		The size of PARAMETER_ARRAY needed to hold all the data
		sent back.
****************************************************************************/
uint8 pcResponseSize( uint16 varid , uint16 apduLength , bool bccmd );

/****************************************************************************
NAMES		bccmdSize
        	bccmdArraySize

PARAMETERS	The varid, with data for the array version.

RETURNS		The number of bytes in the command pdu generated by
		bccmdGenerate().
****************************************************************************/

uint16 bccmdSize (uint16 varid);
uint16 bccmdArraySize(uint16 varid, PARAMETER_ARRAY data, uint16 *extra_words);

/*
**  HCI specific local procedures.
*/
/****************************************************************************
NAME		hci_evt_size

PARAMETERS	The HCI evtcode and the length of the pdu.

RETURNS		The number of PARAMETER_ARRAY elements in the reply
		for the event.
****************************************************************************/
uint32 hciEventSize(uint8 aEventCode , uint32 aLength , uint32 aEventFormat );

/****************************************************************************
NAME		hci_cmd_size
		hci_cmd_array size

PARAMETERS	The HCI op-code, with data for the array version.

RETURNS		The number of bytes in the command pdu generated by
		hci_cmd_generate().
****************************************************************************/

uint32 hciCommandSize      ( uint16 hci_opcode , uint32 format );
uint32 hciCommandArraySize ( uint16 hci_opcode , PARAMETER_ARRAY data );

/****************************************************************************
NAME		hci_event_filter_cmd - create a special case pdu.

PARAMETERS	pointer to the buffer, and a pointer to the arguments.

RETURNS		true if it successfully creates a pdu, false if it gets
		confused.
****************************************************************************/
bool generateHCIEventFilterCommand ( PDU_AND_PARAMETER_HOLDER * aHolder );

/****************************************************************************
NAME		analyseHCICommandCompleteEvent

PARAMETERS	the command opcode, a pointer to the buffer, the
		remaining length of pdu, and a pointer to the answer array.

RETURNS		true if the analysis is successful, false otherwise.
****************************************************************************/

bool analyseHCICommandCompleteEvent ( uint16 Opcode,
                                      PDU_AND_PARAMETER_HOLDER * aHolder );
#endif /* __PACKER_H__ */
