// psserver.h 
// interface definition that all servers of persistent store will implement.
//
// 01:nov:2000    ckl   Created
// 16:jan:2001    ckl   Removed MAX_PS_VALUE_LEN
// 23:feb:01  Chris Lowe 	Now supports firmware versions.

#ifndef  PSSERVER_H_INCLUDED
#define  PSSERVER_H_INCLUDED

#include "pshelp_types.h"

#include "csrhci/bccmd_trans.h"
#include "assert.h"

// pure abstract base class defining the interface for the persistent store 
// This function uses data stored in spi format.

// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the PSHELP_EXPORTS
// symbol defined on the command line. this symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// PSHELP_API functions as being imported from a DLL, wheras this DLL sees symbols
// defined with this macro as being exported.
#ifdef PSHELP_LIB
#define PSHELP_API
#else
#ifdef PSHELP_EXPORTS
#define PSHELP_API __declspec(dllexport)
#else
#define PSHELP_API __declspec(dllimport)
#endif
#endif



enum ps_error_code {
	PS_ERROR_KEY_UNKNOWN = BCCMD_TRANS_ERROR_LAST
};

class PSHELP_API PsServer: public IMemoryReader
{
public:

	// construction
	PsServer() : stores_(0), known_key_list_(NULL), keys_known_(false) {;}
	// destruction
	virtual ~PsServer();
	virtual bool psbc_size(const pskey key, uint16 * const len) = 0;
	// TRUE if the data item was found in the ps, else FALSE.  
	virtual bool psbc_set(const pskey key, const uint16 * const data, const uint16 len ) = 0;
	virtual bool psbc_get(const pskey key, uint16 * const data, uint16 maxlen,
		uint16 * const len ) = 0;
	virtual bool psbc_next(const pskey key, pskey *next_key) = 0;
	virtual bool psbc_clr(const pskey key ) = 0;
	virtual bool psbc_clrs(const  pskey key ) = 0;
	virtual bool psbc_known_key(const pskey key ) = 0;
	virtual bool psbc_factory_set() = 0;
	virtual bool psbc_factory_restore(const bool all ) = 0;
	virtual bool supports_stores() = 0;
	virtual bool inquire_known_keys();
	virtual bccmd_trans_error_handler* get_error_handler() = 0;
	virtual bool reset_bc(reset_t reset_type) = 0; 
	virtual bool open() = 0;
	virtual bool close() = 0;
	virtual void set_error_handler(bccmd_trans_error_handler *error_handler) = 0;
	virtual bool get_firmware_id(int *const id, uint16 * nameBuffer, const size_t sNameBuffer) = 0;
	virtual bool read_mem(uint16 addr, uint16 len, uint16 * data) = 0;
	bool key_exists(pskey key);
	bool keys_known() {return keys_known_;};

	virtual uint16 GetStores() {return stores_;};
	virtual void SetStores(uint16 stores) {stores_ = stores;};

protected:
	uint16 stores_; // the stores to be accessed.  By default, this is 0, indicating the default store.
	class KeySet;
	KeySet * known_key_list_;
	bool keys_known_;
};


class PSHELP_API PsBccmd: public PsServer
{
public:

	// construction
	PsBccmd(bccmd_trans *trans) : trans_(trans), PsServer() {;}
	// destruction
	virtual ~PsBccmd() {delete trans_; trans_ = 0;};
//	virtual int check_ids(const ichar *filename = "pstool.ids");
	bool psbc_size(const pskey key, uint16 * const len);
	// TRUE if the data item was found in the ps, else FALSE.  
	bool psbc_set(const pskey key, const uint16 * const data, const uint16 len );
	bool psbc_get(const pskey key, uint16 * const data, uint16 maxlen,
		uint16 * const len );
	bool psbc_next(const pskey key, pskey *next_key);
	bool psbc_clr(const pskey key );
	bool psbc_clrs(const  pskey key );
	bool psbc_known_key(const pskey key );
	bool psbc_factory_set(); 
	bool psbc_factory_restore(const bool all );
	bool supports_stores();
	bccmd_trans_error_handler* get_error_handler() {assert(trans_); return trans_->get_error_handler();};
	bool reset_bc(reset_t reset_type) {assert(trans_); return trans_->reset_bc(reset_type);};; 
	bool open() {assert(trans_); return trans_->open();};
	bool close() {assert(trans_); return trans_->close();};
	void set_error_handler(bccmd_trans_error_handler *error_handler) {assert(trans_); trans_->set_error_handler(error_handler);};
	bool get_firmware_id(int *const id, uint16 * nameBuffer, const size_t sNameBuffer) {assert(trans_); return trans_->get_firmware_id(id, nameBuffer, sNameBuffer);};
	virtual bool read_mem(uint16 addr, uint16 len, uint16 * data) {assert(trans_); return trans_->read_mem(addr, len, data);};

	bccmd_trans *GetTrans() {return trans_;};
	void SetTrans(bccmd_trans *trans) {trans_ = trans;};
protected:
	bccmd_trans *trans_;
};

#endif // PSSERVER_H_INCLUDED

