/***
COPYRIGHT
		Cambridge Consultants Ltd 2000...

FILE
		Emulators\SPIFNS\SPIFNS.c

AUTHOR
		Nick Marley

DESCRIPTION
		Small library of SPI read/write functions with various semantics.  These
		all basically read/write across the SPI interface but with various return
		types and error checking.

MODIFICATION DETAILS
   $Log: spifns.h,v $
   Revision 1.14  2003/08/08 10:01:53  cl01
   Comments in header

   Revision 1.13  2003/06/17 14:19:35  cl01
   Added functions for SPI command bits

   Revision 1.12  2003/05/02 14:31:41  cl01
   bug - .dsp defined different SPIDLL_EXPORT to header.

   Revision 1.11  2003/03/21 12:21:46  cl01
   Stub functions for 5GH Command Bits.

   Revision 1.10  2003/03/04 17:55:19  cl01
   Added spifns_get_chip_num function

   Revision 1.9  2002/07/18 14:21:36  cl01
   Added export option for spilib

   Revision 1.8  2002/06/19 13:08:06  cl01
   Made modifications to the Mutex functionality

   Revision 1.7  2002/06/18 19:43:25  cl01
   Added timeout to ACQUIRE_SPI

   Revision 1.6  2002/05/29 15:13:10  cl01
   Added acquire_spi and release_spi

   Revision 1.5  2001/09/21 19:29:06  cl01
   Added reiliable read functions

   Revision 1.4  2001/09/10 14:24:11  mm01
   Added ''spifns_multiplex_select'' - Talk to a MulitSpi thing

   Revision 1.3  2001/07/10 17:25:20  cp01
   Merged modifications based on CCL's code
   Migrated API to spifns (can use SIF)

   Revision 1.2  2001/03/20 13:07:23  jrrk
   Integrate Xap2Emu source code

   Revision 1.1  2001/03/09 19:16:22  cl01
   Merged JRRK developments

// 
// 4     28/04/00 3:40p Nm2
// Add block write functions.
// 
// 3     22/04/00 2:25p Nm2
// Use types from global.h
// 
// 2     19/04/00 6:33p Nm2
// First working version.
 */

#ifndef SPIFNS_H_INCLUDED
#define SPIFNS_H_INCLUDED

#ifndef LINT
static const char SPIFNS_h_vss_id[] = "$Id: //depot/bc/main/devHost/spi/spifnscsr/spifns.h#17 $";
#endif

#ifdef __cplusplus
extern "C" {
#endif

#ifdef SPI_LIB
#define SPIFNSDLL_API
#else
#ifdef SPIFNS_EXPORTS
#define SPIFNSDLL_API __declspec(dllexport)
#else
#define SPIFNSDLL_API __declspec(dllimport)
#endif
#endif

#define SPIFNS_SUCCESS (0)
#define SPIFNS_FAILURE (1)

// these are for backward compatibility only:
#define SUCCESS (0)
#define FAILURE (1)




/* Basic SPI read / write values.  Wrappers on spi_read() and spi_write() */
SPIFNSDLL_API int spifns_read(unsigned long addr, unsigned short *value);
SPIFNSDLL_API int spifns_write(unsigned long addr, unsigned short data);

SPIFNSDLL_API int spifns_read_verify(unsigned long addr, unsigned short *value);
SPIFNSDLL_API int spifns_write_verify(unsigned long addr, unsigned short data);

SPIFNSDLL_API int spifns_write_block(unsigned long addr, unsigned long num_dwords, unsigned long *data);
SPIFNSDLL_API int spifns_read_block(unsigned long addr, unsigned long num_dwords, unsigned long *data);

SPIFNSDLL_API int spifns_id(int id); /* replacement for sif_set_sif_id */

/**** CALL THIS FIRST ****/
/* spifns_use_port, port = 1 - n selects LPT(port).  port = 0 selects LPT(1).  port = -1 auto-selects from LPT(1..4) */
SPIFNSDLL_API int spifns_use_port(int port); /* replacement for hardware_test */

SPIFNSDLL_API int spifns_set_config_ex ( void * sc, unsigned long config_size); /* replacement for sif_set_config_ex */
SPIFNSDLL_API int spifns_query_config_ex(void *sc, unsigned long config_size); /* replacement for sif_query_config_ex */
SPIFNSDLL_API int spifns_query_device( void *sd ); /* replacement for sif_query_device */
SPIFNSDLL_API int spifns_set_shift_period(int shift_period); /* replacement for sif_set_shift_period */
SPIFNSDLL_API int spifns_query_shift_period(int *shift_period); /* replacement for sif_query_shift_period */

SPIFNSDLL_API void spifns_set_cmd_bits(int cmd_bits); /* set the otherwise-unused command bits in the spi_read and spi_write wire commands */
SPIFNSDLL_API int spifns_query_cmd_bits(void);  /* default is zero */

#define CHIP_SELECT_XILINX		(0)
#define CHIP_SELECT_SPARTAN		(1)
#define CHIP_SELECT_NONE		(-1)

/* Replacement for the spifns_config argument to xap_ functions */
SPIFNSDLL_API void spifns_chip_select(int which);
SPIFNSDLL_API void spifns_multiplex_select(int which);
SPIFNSDLL_API int spifns_get_chip_num();
SPIFNSDLL_API int spifns_xap_stopped(void);

// return non-zero if spi was acquired within timeout (or was already acquired)
// use INFINITE as timeout, why don't ya.
SPIFNSDLL_API int spifns_acquire_spi(DWORD timeout);
SPIFNSDLL_API void spifns_release_spi();

extern SPIFNSDLL_API const char *SpiAppMutexName;


#ifdef __cplusplus
}
#endif

#ifdef __cplusplus
// Simple class to acquire and release spi;
class SpiAcquireLock
{
private:
	int acquired_;
public:
	SpiAcquireLock(DWORD timeout = INFINITE) {acquired_ = spifns_acquire_spi(timeout);};
	~SpiAcquireLock() {if (acquired_) spifns_release_spi();};
	//  Will return true if acquire suceeds or if already acquired.
	bool Acquire(DWORD timeout = INFINITE) {return acquired_ || (acquired_ = spifns_acquire_spi(timeout));}
};
#endif





#endif // SPIFNS_H_INCLUDED
