///////////////////////////////////////////////////////////////////////
//
//  FILE   :  thread.h
//
//            (C) CSR 2001
//
//  AUTHOR :  Mark Marshall
//
//  PURPOSE:  the header file for all the machine dependent
//            definitions of the Threadable class.
//
//  $Id: //depot/bc/main/devHost/util/thread/thread.h#9 $
//
///////////////////////////////////////////////////////////////////////

#ifndef THREADABLE_H
#define THREADABLE_H

#include "common/nocopy.h"

class Threadable : private NoCopy
{
private:
    // This function is overridden by the derived class, it does
    // all of the work.
    virtual int ThreadFunc() = 0;

    // This is where we will hide all of the platform specific thread data.
    class ThreadData;
    friend ThreadData;

    ThreadData* m_data;

public:

    Threadable();

    virtual ~Threadable();

    // Start the thread running
    bool Start();

    // Returns true if the thread is still running
    bool IsActive();
    
    // This gets the exit code of the thread
    int GetExitCode();

    // This should be checked by the 'ThreadFunc' function every so often.
    // If it returns false, then the 'ThreadFunc' function should exit,
    // but should close down all of its stuff first.
    bool KeepGoing();

    // Call this if you want the thread to close down.
    // This only sets a flag, that should be read by the 'ThreadFunc'
    // function.  If you need other action to take place to signal a
    // stop, then they should be placed in here.  (If for example
    // the ThreadFunc function could be paused in a blocking system call,
    // this should be overriden so that it gets the thread out of
    // that function)
    virtual void Stop();

    // This is the nasty way to bring down a thread.  You should have
    // called Stop already, and not had any response from that.
    // It will make sure that the thread is dead.
    void Kill(unsigned millisec = 1000);

    // Lower the threads priority a couple of points
    void LowPriority(void);

    // Set priority back to normal
    void NormalPriority(void);

    // Make the thread have more priority
    void HighPriority(void);
};

#endif // def THREADABLE_H
