////////////////////////////////////////////////////////////////////////////////
//
//  FILE    :    dichar_ptr.h
//
//  Copyright CSR 2001
//
//  CLASS   :    dichar_ptr
//
//  PURPOSE :    to define a "const ichar*"-a-like type which is dually char
//               and wchar_t, both at the same time in terms of typing,
//               but not data.  Effectively it contains two strings one of
//               which is alway empty.
//
//               dichar_ptr is named for this dimorphism.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef _DICHAR_PTR_H_
#define _DICHAR_PTR_H_
#include <stdlib.h>

// dichar_ptr asserts that every pointer passed in or out is not null.
class dichar_ptr
{
public:
    dichar_ptr ( const char * const aString );
    dichar_ptr ( const wchar_t * const aString );
    virtual ~dichar_ptr ();
    dichar_ptr ( const dichar_ptr & );

    const char * getA () const;
    const wchar_t * getW () const;
    void get ( const char *& aString ) const;
    void get ( const wchar_t *& aString ) const;

    bool isA() const { return ( mStringA != 0 ); }
    bool isW() const { return ( mStringW != 0 ); }
private:
    dichar_ptr & operator= ( const dichar_ptr & );
    char * mStringA;
    wchar_t * mStringW;
};

#endif//_DICHAR_PTR_H_
